/*
	Copyright (C) 2012-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "private.h"



JTFUNC_HEADER ( jtf_load )
{
	if ( cuf_file_load ( CEDCLI_FILE, args[0] ) )
	{
		fprintf ( stderr, "Unable to load '%s'\n\n", args[0] );

		return 1;
	}

	return 0;
}

JTFUNC_HEADER ( jtf_save )
{
	if ( CEDCLI_FILE->name == NULL )
	{
		fprintf ( stderr, "Filename not set.\n\n" );

		return 1;
	}

	if ( cuf_file_save ( CEDCLI_FILE, CEDCLI_FILE->name, CEDCLI_FILE->type
		) )
	{
		fprintf ( stderr, "Unable to save.\n\n" );

		return 1;
	}

	return 0;
}

static int get_file_type (
	char		* txt,
	CedFileType	* dest
	)
{
	static charINT	types[] = {
		{ "tsv_book",		CED_FILE_TYPE_TSV_BOOK },
		{ "tsv_value_book",	CED_FILE_TYPE_TSV_VAL_BOOK },
		{ "ledger_book",	CED_FILE_TYPE_LEDGER_BOOK },
		{ "ledger_value_book",	CED_FILE_TYPE_LEDGER_VAL_BOOK },
		{ NULL }
		};

	int		res;


	if ( match_chint ( txt, types, &res ) )
	{
		fprintf ( stderr, "Unknown file type: '%s'\n\n", txt );

		return 1;
	}

	dest[0] = res;

	return 0;
}

JTFUNC_HEADER ( jtf_save_as )
{
	if ( args[1] )
	{
		if ( args[2] )
		{
			fprintf ( stderr, "Too many arguments.\n\n" );

			return 1;
		}

		if ( get_file_type ( args[1], &CEDCLI_FILE->type ) )
		{
			return 1;
		}
	}

	if ( cuf_file_save ( CEDCLI_FILE, args[0], CEDCLI_FILE->type ) )
	{
		fprintf ( stderr, "Unable to save.\n\n" );

		return 1;
	}

	return 0;
}

JTFUNC_HEADER ( jtf_import_book )
{
	CedSheet	* sheet;
	CedUI_file	* uifile;
	int		sheet_tot,
			sheet_fail,
			file_tot,
			file_fail,
			res;


	uifile = cuf_file_new ();
	if (	! uifile ||
		cuf_file_load ( uifile, args[0] ) ||
		! uifile->cubook ||
		! uifile->cubook->book )
	{
		cuf_file_free ( uifile );
		fprintf ( stderr, "Unable to load file.\n\n" );

		return 1;
	}

	res = cuf_book_merge ( CEDCLI_FILE->cubook, uifile->cubook->book,
		&sheet_tot, &sheet_fail, &file_tot, &file_fail );

	cuf_file_free ( uifile );

	undo_report_updates ( res );

	printf ( "%i sheets imported.\n"
		"%i sheets not imported due to identical names.\n"
		"%i graphs/files imported.\n"
		"%i graphs/files not imported due to identical names.\n\n",
		sheet_tot, sheet_fail, file_tot, file_fail );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return 1;
	}

	// Update sheet/graph names if they were empty before
	sheet = cedcli_get_sheet ( state );
	cedcli_get_graph ( state );

	update_changes_chores ( state, sheet );

	return 0;	// Success
}

JTFUNC_HEADER ( jtf_import_graph )
{
	char		* buf;
	int		buf_size;


	if ( ced_graph_get ( CEDCLI_FILE->cubook->book, args[0] ) )
	{
		fprintf ( stderr, "Graph name already exists.\n\n" );

		return 1;
	}

	buf = mtkit_file_load ( args[1], &buf_size, 0, 0 );
	if ( ! buf )
	{
		fprintf ( stderr, "Unable to load '%s'.\n\n", args[1] );

		return 1;
	}

	if ( ! ced_graph_new ( CEDCLI_FILE->cubook->book, buf, buf_size,
		args[0] )
		)
	{
		free ( buf );
		fprintf ( stderr, "Unable to create graph.\n\n" );

		return 1;
	}

	mtkit_strfreeswap ( &CEDCLI_FILE->cubook->book->prefs.active_graph,
		args[0] );

	return 0;
}

JTFUNC_HEADER ( jtf_export_graph )
{
	CedBookFile	* graph;


	graph = cedcli_get_graph ( state );
	if ( ! graph )
	{
		return 1;
	}

	if ( mtkit_file_save ( args[0], graph->mem, graph->size, 0 ) )
	{
		fprintf ( stderr, "Error exporting graph.\n\n" );

		return 1;
	}

	return 0;
}

JTFUNC_HEADER ( jtf_export_output_graph )
{
	CedBookFile	* graph;
	int		filetype;
	static charINT	graph_types[] = {
			{ "eps",	CED_GRAPH_TYPE_EPS },
			{ "pdf",	CED_GRAPH_TYPE_PDF },
			{ "png",	CED_GRAPH_TYPE_PNG },
			{ "ps",		CED_GRAPH_TYPE_PS },
			{ "svg",	CED_GRAPH_TYPE_SVG },
			{ NULL }
			};


	graph = cedcli_get_graph ( state );
	if ( ! graph )
	{
		return 1;
	}

	if ( match_chint ( args[1], graph_types, &filetype ) )
	{
		fprintf ( stderr, "Unknown graph type.\n\n" );

		return 1;
	}

	if ( ced_graph_render_file ( CEDCLI_FILE->cubook->book,
		CEDCLI_FILE->cubook->book->prefs.active_graph,
		args[0], filetype, NULL, 1 )
		)
	{
		fprintf ( stderr, "Error exporting graph output.\n\n" );

		return 1;
	}

	return 0;
}

JTFUNC_HEADER ( jtf_export_output_sheet )
{
	CedSheet	* sheet;
	int		filetype;
	static charINT	sheet_types[] = {
			{ "eps",	CED_SHEET_EXPORT_EPS },
			{ "html",	CED_SHEET_EXPORT_HTML },
			{ "pdf",	CED_SHEET_EXPORT_PDF },
			{ "pdf_paged",	CED_SHEET_EXPORT_PDF_PAGED },
			{ "png",	CED_SHEET_EXPORT_PNG },
			{ "ps",		CED_SHEET_EXPORT_PS },
			{ "svg",	CED_SHEET_EXPORT_SVG },
			{ "tsv",	CED_SHEET_EXPORT_TSV },
			{ "tsv_q",	CED_SHEET_EXPORT_TSV_QUOTED },
			{ NULL }
			};


	sheet = cedcli_get_sheet ( state );
	if ( ! sheet )
	{
		return 1;
	}

	if ( match_chint ( args[1], sheet_types, &filetype ) )
	{
		fprintf ( stderr, "Unknown sheet type.\n\n" );

		return 1;
	}

	if ( ced_export_output ( state->prefs, sheet, args[0],
		CEDCLI_FILE->name, filetype) )
	{
		fprintf ( stderr, "Error exporting sheet output.\n\n" );

		return 1;
	}

	return 0;
}

JTFUNC_HEADER ( jtf_export_sheet )
{
	CedSheet	* sheet;
	int		filetype;
	static charINT	sheet_types[] = {
		{ "tsv",		CED_FILE_TYPE_TSV_CONTENT },
		{ "tsv_gz",		CED_FILE_TYPE_TSV_CONTENT_GZ },
		{ "tsv_noq",		CED_FILE_TYPE_TSV_CONTENT_NOQ },
		{ "tsv_val",		CED_FILE_TYPE_TSV_VALUE },
		{ "tsv_val_gz",		CED_FILE_TYPE_TSV_VALUE_GZ },
		{ "tsv_val_noq",	CED_FILE_TYPE_TSV_VALUE_NOQ },
		{ "csv",		CED_FILE_TYPE_CSV_CONTENT },
		{ "csv_noq",		CED_FILE_TYPE_CSV_CONTENT_NOQ },
		{ "csv_val",		CED_FILE_TYPE_CSV_VALUE },
		{ "csv_val_noq",	CED_FILE_TYPE_CSV_VALUE_NOQ },
		{ "ledger",		CED_FILE_TYPE_LEDGER },
		{ "ledger_gz",		CED_FILE_TYPE_LEDGER_GZ },
		{ "ledger_val",		CED_FILE_TYPE_LEDGER_VAL },
		{ "ledger_val_gz",	CED_FILE_TYPE_LEDGER_VAL_GZ },
		{ NULL }
		};


	sheet = cedcli_get_sheet ( state );
	if ( ! sheet )
	{
		return 1;
	}

	if ( match_chint ( args[1], sheet_types, &filetype ) )
	{
		fprintf ( stderr, "Unknown sheet type.\n\n" );

		return 1;
	}

	if ( ced_sheet_save ( sheet, args[0], filetype ) )
	{
		fprintf ( stderr, "Error exporting sheet.\n\n" );

		return 1;
	}

	return 0;
}

