/*
	Copyright (C) 2012-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "private.h"



typedef struct
{
	char		* name;
	void		* ip;
} charVOIDp;



static int match_chvoidp (
	char		* txt,
	charVOIDp	* chvoidp,
	void		** result
	)
{
	if ( ! txt || ! chvoidp || ! result )
	{
		return 1;
	}

	for ( ; chvoidp->name; chvoidp ++ )
	{
		if ( strcmp ( chvoidp->name, txt ) == 0 )
		{
			result[0] = chvoidp->ip;

			return 0;
		}
	}

	return 1;	// Not found
}



JTFUNC_HEADER ( jtf_set_prefs_book )
{
	CedBookPrefs	* bp = &CEDCLI_FILE->cubook->book->prefs;
	int		num,
			* ip;
	void		* vp;
	charVOIDp	table_s[] = {
			{ "active_sheet", &bp->active_sheet },
			{ "active_graph", &bp->active_graph },
			{ "author", &bp->author },
			{ "comment", &bp->comment },
			{ NULL }
			},
			table_i[] = {
			{ "disable_locks", &bp->disable_locks },
			{ "auto_recalc", &bp->auto_recalc },
			{ NULL }
			};


	// Try strings first
	if ( match_chvoidp ( args[0], table_s, &vp ) == 0 )
	{
		mtkit_strfreeswap ( (char **)vp, args[1] );

		return 0;
	}

	// Its not a string so it must be an int
	if ( match_chvoidp ( args[0], table_i, &vp ) != 0 )
	{
		fprintf ( stderr, "Unable to set book preference\n\n" );

		return 1;
	}

	if ( cedcli_get_integer ( args[1], &num, 0, -1 ) )
	{
		return 1;
	}

	ip = (int *)vp;
	ip[0] = num;

	return 0;
}

JTFUNC_HEADER ( jtf_set_prefs_cell )
{
	CedSheet	* sheet,
			* tmp_sheet = NULL;
	char		* charp = NULL;
	int		r1, r2, c1, c2,
			pref_id,
			num = 0,
			res;
	static charINT	ctab[] = {
		{ "align_horizontal",	CEDUI_CELLPREFS_align_horizontal },
		{ "color_background",	CEDUI_CELLPREFS_color_background },
		{ "color_foreground",	CEDUI_CELLPREFS_color_foreground },
		{ "format",		CEDUI_CELLPREFS_format },
		{ "width",		CEDUI_CELLPREFS_width },
		{ "num_decimal_places",	CEDUI_CELLPREFS_num_decimal_places },
		{ "num_zeros",		CEDUI_CELLPREFS_num_zeros },
		{ "bold_text",		CEDUI_CELLPREFS_bold_text },
		{ "locked",		CEDUI_CELLPREFS_locked },
		{ "border_type",	CEDUI_CELLPREFS_border_type },
		{ "border_color",	CEDUI_CELLPREFS_border_color },
		{ "format_datetime",	CEDUI_CELLPREFS_format_datetime },
		{ "num_thousands",	CEDUI_CELLPREFS_num_thousands },
		{ "text_prefix",	CEDUI_CELLPREFS_text_prefix },
		{ "text_suffix",	CEDUI_CELLPREFS_text_suffix },
		{ NULL }
		};


	sheet = cedcli_get_sheet ( state );
	if ( ! sheet )
	{
		return 1;
	}

	if ( match_chint ( args[0], ctab, &pref_id ) )
	{
		fprintf ( stderr, "No such cell preference name" );

		return 1;
	}

	if (	pref_id == CEDUI_CELLPREFS_format_datetime ||
		pref_id == CEDUI_CELLPREFS_num_thousands ||
		pref_id == CEDUI_CELLPREFS_text_prefix ||
		pref_id == CEDUI_CELLPREFS_text_suffix
		)
	{
		charp = args[1];
	}
	else
	{
		if ( cedcli_get_integer ( args[1], &num, 0, -1 ) )
		{
			return 1;
		}

//printf ( "Integer = %i\n", num );
	}

	if ( cedui_cellprefs_init ( sheet, &r1, &c1, &r2, &c2, &tmp_sheet ) )
	{
		fprintf ( stderr, "Error setting up temp sheet\n\n" );

		return 1;
	}

	res = cedui_cellprefs_change ( CEDCLI_FILE->cubook, sheet,
		r1, c1, r2, c2, tmp_sheet, pref_id, charp, num );

	ced_sheet_destroy ( tmp_sheet );

	undo_report_updates ( res );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return 1;
	}

	if ( res > 0 )
	{
		fprintf ( stderr, "Error setting cell preferences\n\n" );

		return 1;
	}

	return 0;
}


JTFUNC_HEADER ( jtf_set_prefs_cellborder )
{
	int		num,
			res;


	if ( ! cedcli_get_sheet ( state ) )
	{
		return 1;
	}

	if ( cedcli_get_integer ( args[0], &num, CEDUI_CELLBORD_MIN,
		CEDUI_CELLBORD_MAX )
		)
	{
		return 1;
	}

	res = cedui_cellprefs_border ( CEDCLI_FILE, num );
	undo_report_updates ( res );
	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return 1;
	}

	if ( res > 0 )
	{
		fprintf ( stderr, "Error setting cell border\n\n" );

		return 1;
	}

	return 0;
}

JTFUNC_HEADER ( jtf_set_prefs_sheet )
{
	int		num,
			* ip;
	void		* vp;
	CedSheet	* sheet;


	sheet = cedcli_get_sheet ( state );
	if ( ! sheet )
	{
		return 1;
	}

	// The sheet prefs are purely int
	if ( cedcli_get_integer ( args[1], &num, 0, -1 ) )
	{
		return 1;
	}
	else
	{
		CedSheetPrefs	* sp = &sheet->prefs;
		charVOIDp	table[] = {
				{ "cursor_r1", &sp->cursor_r1 },
				{ "cursor_c1", &sp->cursor_c1 },
				{ "cursor_r2", &sp->cursor_r2 },
				{ "cursor_c2", &sp->cursor_c2 },
				{ "split_r1", &sp->split_r1 },
				{ "split_r2", &sp->split_r2 },
				{ "split_c1", &sp->split_c1 },
				{ "split_c2", &sp->split_c2 },
				{ "start_row", &sp->start_row },
				{ "start_col", &sp->start_col },
				{ "locked", &sp->locked },
				{ NULL }
				};


		if ( match_chvoidp ( args[0], table, &vp ) )
		{
			fprintf ( stderr,
				"Unable to set sheet preference\n\n" );

			return 1;
		}
	}

	ip = (int *)vp;
	ip[0] = num;

	return 0;
}

JTFUNC_HEADER ( jtf_set_prefs_state )
{
	int	num;


	// The state is purely ints or options with no limits
	if ( cedcli_get_integer ( args[1], &num, 0, -1 ) )
	{
		return 1;
	}

	if ( mtkit_prefs_set_int ( state->prefs, args[0], num ) )
	{
		fprintf ( stderr, "Error setting state preference\n\n" );
	}

	return 0;
}

JTFUNC_HEADER ( jtf_print_prefs_book )
{
	CedBookPrefs	* bp;


	bp = &CEDCLI_FILE->cubook->book->prefs;

	if ( bp->active_sheet )
	{
		printf ( "active_sheet = '%s'\n", bp->active_sheet );
	}

	if ( bp->active_graph )
	{
		printf ( "active_graph = '%s'\n", bp->active_graph );
	}

	if ( bp->author )
	{
		printf ( "author = '%s'\n", bp->author );
	}

	if ( bp->comment )
	{
		printf ( "comment = '%s'\n", bp->comment );
	}

	printf ( "disable_locks = %i\n", bp->disable_locks );
	printf ( "auto_recalc = %i\n", bp->auto_recalc );

	return 0;
}

JTFUNC_HEADER ( jtf_print_prefs_sheet )
{
	CedSheetPrefs	* sp;
	CedSheet	* sheet;


	sheet = cedcli_get_sheet ( state );
	if ( ! sheet )
	{
		return 1;
	}

	sp = &sheet->prefs;

	printf ( "cursor_r1 = %i\n", sp->cursor_r1 );
	printf ( "cursor_c1 = %i\n", sp->cursor_c1 );
	printf ( "cursor_r2 = %i\n", sp->cursor_r2 );
	printf ( "cursor_c2 = %i\n", sp->cursor_c2 );
	printf ( "split_r1 = %i\n", sp->split_r1 );
	printf ( "split_r2 = %i\n", sp->split_r2 );
	printf ( "split_c1 = %i\n", sp->split_c1 );
	printf ( "split_c2 = %i\n", sp->split_c2 );
	printf ( "start_row = %i\n", sp->start_row );
	printf ( "start_col = %i\n", sp->start_col );
	printf ( "locked = %i\n", sp->locked );

	return 0;
}

static int cb_scan_state (
	mtTreeNode	* node,
	void		* user_data
	)
{
	mtPrefItemValue	* iv = node->data;


	printf ( "%s = '%s'\n", (char *)node->key, iv->value );

	return 0;	// continue
}

JTFUNC_HEADER ( jtf_print_prefs_state )
{
	if ( mtkit_tree_scan ( mtkit_prefs_get_tree ( state->prefs ),
		cb_scan_state, NULL, 0 ) )
	{
		fprintf ( stderr, "Problem scanning state preferences\n\n" );
	}

	return 0;
}

int cedcli_prefs_init (
	CedCli_STATE	* state
	)
{
	static mtPrefItem	prefs_table[] = {
{ CED_INIFILE_PAGE_FONT_SIZE,	MTKIT_PREF_TYPE_INT, "12" },
{ CED_INIFILE_PAGE_WIDTH,	MTKIT_PREF_TYPE_INT, "297" },
{ CED_INIFILE_PAGE_HEIGHT,	MTKIT_PREF_TYPE_INT, "210" },
{ CED_INIFILE_PAGE_MARGIN_X,	MTKIT_PREF_TYPE_INT, "10" },
{ CED_INIFILE_PAGE_MARGIN_Y,	MTKIT_PREF_TYPE_INT, "10" },
{ CED_INIFILE_PAGE_HEADER_LEFT, MTKIT_PREF_TYPE_OPTION, "2" },
{ CED_INIFILE_PAGE_HEADER_CENTRE, MTKIT_PREF_TYPE_OPTION, "0" },
{ CED_INIFILE_PAGE_HEADER_RIGHT, MTKIT_PREF_TYPE_OPTION, "3" },
{ CED_INIFILE_PAGE_FOOTER_LEFT, MTKIT_PREF_TYPE_OPTION, "6" },
{ CED_INIFILE_PAGE_FOOTER_CENTRE, MTKIT_PREF_TYPE_OPTION, "0" },
{ CED_INIFILE_PAGE_FOOTER_RIGHT, MTKIT_PREF_TYPE_OPTION, "4" },
{NULL}
};


	state->prefs = mtkit_prefs_new ( prefs_table );
	if ( ! state->prefs )
	{
		fprintf ( stderr, "Unable to create state preferences.\n\n" );

		return 1;
	}

	return 0;
}

int cedcli_prefs_free (
	CedCli_STATE	* state
	)
{
	mtkit_prefs_destroy ( state->prefs );
	state->prefs = NULL;

	return 0;
}

