/*
	Copyright (C) 2012-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "private.h"



JTFUNC_HEADER ( jtf_delete_graph )
{
	CedBook		* book = CEDCLI_FILE->cubook->book;


	if ( ! cedcli_get_graph ( state ) )
	{
		return 1;
	}

	if ( ced_graph_destroy ( book, book->prefs.active_graph ) )
	{
		fprintf ( stderr, "Unable to delete graph\n\n" );

		return 1;
	}

	cuf_file_get_graph ( CEDCLI_FILE );	// Sets active_graph for us

	return 0;
}

JTFUNC_HEADER ( jtf_delete_sheet )
{
	int		res;
	CedSheet	* sheet;


	sheet = cedcli_get_sheet ( state );
	if ( ! sheet )
	{
		return 1;
	}

	res = cuf_book_destroy_sheet ( CEDCLI_FILE->cubook,
		sheet->book_tnode->key );
	undo_report_updates ( res );

	if ( res )
	{
		return 1;
	}

	return 0;
}

JTFUNC_HEADER ( jtf_duplicate_sheet )
{
	int		res;
	CedSheet	* sheet,
			* new;


	sheet = cedcli_get_sheet ( state );
	if ( ! sheet )
	{
		return 1;
	}

	res = cuf_book_duplicate_sheet ( CEDCLI_FILE->cubook, sheet, &new );
	undo_report_updates ( res );

	if ( res )
	{
		return 1;
	}

	mtkit_strfreeswap ( &CEDCLI_FILE->cubook->book->prefs.active_sheet,
		new->book_tnode->key );

	return 0;
}

static int prep_insert (
	CedCli_STATE	* state,
	char		** args,
	CedSheet	* sheet,
	int		* r1,
	int		* c1,
	int		* r2,
	int		* c2
	)
{
	ced_sheet_cursor_max_min ( sheet, r1, c1, r2, c2 );

	if ( args[0] )
	{
		if ( strcmp ( "clip", args[0] ) != 0 )
		{
			fprintf ( stderr, "Argument not recognised.\n\n" );

			return 1;
		}

		if ( ! state->clipboard->sheet )
		{
			fprintf ( stderr, "No clipboard available.\n\n" );

			return 1;
		}

		r2[0] = r1[0] + state->clipboard->rows - 1;
		c2[0] = c1[0] + state->clipboard->cols - 1;
	}

	return 0;
}

JTFUNC_HEADER ( jtf_insert_column )
{
	CedSheet	* sheet;
	int		r1, c1, r2, c2,
			res;


	sheet = cedcli_get_sheet ( state );
	if ( ! sheet )
	{
		return 1;
	}

	if ( prep_insert ( state, args, sheet, &r1, &c1, &r2, &c2 ) )
	{
		return 1;
	}

	res = cuf_sheet_insert_column ( CEDCLI_FILE->cubook, sheet,
		c1, c2 - c1 + 1 );
	undo_report_updates ( res );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return 1;
	}

	update_changes_chores ( state, sheet );

	return 0;
}

JTFUNC_HEADER ( jtf_insert_row )
{
	CedSheet	* sheet;
	int		r1, c1, r2, c2,
			res;


	sheet = cedcli_get_sheet ( state );
	if ( ! sheet )
	{
		return 1;
	}

	if ( prep_insert ( state, args, sheet, &r1, &c1, &r2, &c2 ) )
	{
		return 1;
	}

	res = cuf_sheet_insert_row ( CEDCLI_FILE->cubook, sheet,
		r1, r2 - r1 + 1 );
	undo_report_updates ( res );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return 1;
	}

	update_changes_chores ( state, sheet );

	return 0;
}

JTFUNC_HEADER ( jtf_new_book )
{
	if ( cuf_file_book_new ( CEDCLI_FILE ) )
	{
		fprintf ( stderr, "Unable to create new book.\n\n" );

		return 1;
	}

	return 0;
}

JTFUNC_HEADER ( jtf_new_sheet )
{
	int		res;


	res = cuf_file_sheet_add ( CEDCLI_FILE );
	if ( res == 1 )
	{
		fprintf ( stderr, "Unable to create a new sheet.\n\n" );
		return 1;
	}

	undo_report_updates ( res );

	if ( res )
	{
		return 1;
	}

	return 0;
}

JTFUNC_HEADER ( jtf_rename_graph )
{
	CedBookFile	* old;
	CedBook		* book = CEDCLI_FILE->cubook->book;


	old = cedcli_get_graph ( state );
	if ( ! old )
	{
		return 1;
	}

	if ( ced_graph_get ( book, args[0] ) )
	{
		fprintf ( stderr, "Graph name already exists.\n\n" );

		return 1;
	}

	if ( ! ced_graph_new ( book, old->mem, old->size, args[0] ) )
	{
		fprintf ( stderr, "Unable to rename graph" );

		return 1;
	}

	old->mem = NULL;
	old->size = 0;

	ced_graph_destroy ( book, book->prefs.active_graph );
	mtkit_strfreeswap ( &book->prefs.active_graph, args[0] );

	return 0;
}

JTFUNC_HEADER ( jtf_rename_sheet )
{
	int		res;
	CedSheet	* sheet;


	sheet = cedcli_get_sheet ( state );
	if ( ! sheet )
	{
		return 1;
	}

	res = cuf_book_page_rename ( CEDCLI_FILE->cubook, sheet, args[0] );
	undo_report_updates ( res );

	if ( res )
	{
		return 1;
	}

	return 0;
}

JTFUNC_HEADER ( jtf_set_2dyear )
{
	CedSheet	* sheet;
	int		res,
			year;


	sheet = cedcli_get_sheet ( state );
	if ( ! sheet )
	{
		return 1;
	}

	if ( args[0] )
	{
		if ( cedcli_get_integer ( args[0], &year,
			MTKIT_DDT_MIN_DATE_YEAR, MTKIT_DDT_MAX_DATE_YEAR ) )
		{
			return 1;
		}
	}
	else
	{
		struct tm	* tm_now;
		time_t		now;


		now = time ( NULL );
		tm_now = localtime ( &now );

		year = 1900 + tm_now->tm_year - 50;
		if (	year < MTKIT_DDT_MIN_DATE_YEAR ||
			year > MTKIT_DDT_MAX_DATE_YEAR )
		{
			// Should never happen, but just in case
			fprintf ( stderr, "Unexpected error in system date.\n"
				);

			return 1;
		}
	}

	res = cuf_sheet_2dyear ( CEDCLI_FILE, year );

	if ( res > 0 )
	{
		fprintf ( stderr, "Unable to fix years.\n" );
	}

	undo_report_updates ( res );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_LOCKED_SHEET ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return 1;			// Nothing changed
	}

	update_changes_chores ( state, sheet );

	return 0;
}

