/*
	Copyright (C) 2012-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#ifndef MAIN_H
#define MAIN_H

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include <unistd.h>
#include <time.h>
#include <errno.h>
#include <ctype.h>

#include <mtkit.h>
#include <mtcelledit.h>
#include <mtcedui.h>



#define CEDCLI_FILE_MIN		0
#define CEDCLI_FILE_MAX		4
#define CEDCLI_FILE		( state->file_list[ state->booknum ] )
#define CEDCLI_USER_QUIT	-1

#define ABOUT_PRINTF \
"%s\n" \
"Copyright (C) 2012-2013 Mark Tyler\n" \
"Type 'help' for command hints.  Read the manual for more specific info.\n" \
"\n" \
, VERSION



typedef struct
{
	char		* name;
	int		num;
} charINT;

typedef struct
{
	int		exit,		// Set to nonzero to exit program
					// -1 = return 0, else return (exit)
			booknum		// Current active book
			;

	CedUI_file	* file_list[CEDCLI_FILE_MAX + 1];
	CedUI_clip	* clipboard;

	mtPrefs		* prefs;
	mtCliTab	* clitab;

} CedCli_STATE;



typedef int (* jtFunc) (
	CedCli_STATE	* state,
	char		** args		// NULL terminated
					// Points to next unparsed item
	);
	// 0 = success



#define JTFUNC_HEADER(X)	int X ( CedCli_STATE * state, char ** args )



// Return 0 = success, NULL = fail unless otherwise stated

int cedcli_jumptable_init (
	CedCli_STATE	* state
	);

int cedcli_jumptable_free (
	CedCli_STATE	* state
	);

int cedcli_parse (
	char		* input,
	CedCli_STATE	* state
	);

JTFUNC_HEADER ( jtf_clip_flip_h );
JTFUNC_HEADER ( jtf_clip_flip_v );
JTFUNC_HEADER ( jtf_clip_rotate_a );
JTFUNC_HEADER ( jtf_clip_rotate_c );
JTFUNC_HEADER ( jtf_clip_transpose );
JTFUNC_HEADER ( jtf_clip_load );
JTFUNC_HEADER ( jtf_clip_save );
JTFUNC_HEADER ( jtf_copy );
JTFUNC_HEADER ( jtf_copy_output );
JTFUNC_HEADER ( jtf_copy_values );
JTFUNC_HEADER ( jtf_cut );
JTFUNC_HEADER ( jtf_clear );
JTFUNC_HEADER ( jtf_clear_content );
JTFUNC_HEADER ( jtf_clear_prefs );
JTFUNC_HEADER ( jtf_paste );
JTFUNC_HEADER ( jtf_paste_content );
JTFUNC_HEADER ( jtf_paste_prefs );

JTFUNC_HEADER ( jtf_load );
JTFUNC_HEADER ( jtf_save );
JTFUNC_HEADER ( jtf_save_as );

JTFUNC_HEADER ( jtf_import_book );
JTFUNC_HEADER ( jtf_import_graph );

JTFUNC_HEADER ( jtf_export_graph );
JTFUNC_HEADER ( jtf_export_output_graph );
JTFUNC_HEADER ( jtf_export_output_sheet );
JTFUNC_HEADER ( jtf_export_sheet );

JTFUNC_HEADER ( jtf_about );
JTFUNC_HEADER ( jtf_help );
JTFUNC_HEADER ( jtf_info );
JTFUNC_HEADER ( jtf_quit );

JTFUNC_HEADER ( jtf_list_files );
JTFUNC_HEADER ( jtf_list_graphs );
JTFUNC_HEADER ( jtf_list_sheets );

JTFUNC_HEADER ( jtf_print_cell_num );
JTFUNC_HEADER ( jtf_print_cell_text );
JTFUNC_HEADER ( jtf_print_cell_type );
JTFUNC_HEADER ( jtf_print );

JTFUNC_HEADER ( jtf_print_prefs_cell );

int match_chint (
	char		* txt,
	charINT		* chint,
	int		* result
	);
	// 0 = Found
	// 1 = Not found

JTFUNC_HEADER ( jtf_delete_column );
JTFUNC_HEADER ( jtf_delete_row );
JTFUNC_HEADER ( jtf_find );
JTFUNC_HEADER ( jtf_set_book );
JTFUNC_HEADER ( jtf_set_cell );
JTFUNC_HEADER ( jtf_select );
JTFUNC_HEADER ( jtf_set_graph );
JTFUNC_HEADER ( jtf_set_sheet );
JTFUNC_HEADER ( jtf_set_width );
JTFUNC_HEADER ( jtf_undo );
JTFUNC_HEADER ( jtf_redo );
JTFUNC_HEADER ( jtf_recalc_book );
JTFUNC_HEADER ( jtf_recalc_sheet );

int undo_report_updates (
	int		error
	);

void update_changes_chores (		// Update volatile cells
	CedCli_STATE	* state,	// after a change
	CedSheet	* sheet
	);

CedSheet * cedcli_get_sheet (		// Get active sheet + report error
	CedCli_STATE	* state
	);

CedBookFile * cedcli_get_graph (	// Get active graph + report error
	CedCli_STATE	* state
	);

int cedcli_get_integer (
	char		* input,
	int		* output,
	int		min,		// If max < min don't check bounds
	int		max
	);

JTFUNC_HEADER ( jtf_delete_graph );
JTFUNC_HEADER ( jtf_delete_sheet );
JTFUNC_HEADER ( jtf_duplicate_sheet );
JTFUNC_HEADER ( jtf_insert_column );
JTFUNC_HEADER ( jtf_insert_row );
JTFUNC_HEADER ( jtf_new_book );
JTFUNC_HEADER ( jtf_new_sheet );
JTFUNC_HEADER ( jtf_rename_graph );
JTFUNC_HEADER ( jtf_rename_sheet );
JTFUNC_HEADER ( jtf_set_2dyear );

JTFUNC_HEADER ( jtf_sort_column );
JTFUNC_HEADER ( jtf_sort_row );

JTFUNC_HEADER ( jtf_set_prefs_book );
JTFUNC_HEADER ( jtf_set_prefs_cell );
JTFUNC_HEADER ( jtf_set_prefs_cellborder );
JTFUNC_HEADER ( jtf_set_prefs_sheet );
JTFUNC_HEADER ( jtf_set_prefs_state );

JTFUNC_HEADER ( jtf_print_prefs_book );
JTFUNC_HEADER ( jtf_print_prefs_sheet );
JTFUNC_HEADER ( jtf_print_prefs_state );

int cedcli_prefs_init (
	CedCli_STATE	* state
	);

int cedcli_prefs_free (
	CedCli_STATE	* state
	);



#endif		// MAIN_H
