#!/bin/sh


OUT=output
CHECK=results
SHEXE=scripts
LOGFILE=output/log.txt
VALG_LOGFILE=valg_log.txt
CEDCOM="mtcedcli -q -t"

if [ "$VALG" = "Y" ]
then
	VALG="valgrind --leak-check=full --show-possibly-lost=no"
else
	unset VALG
fi


run_sh_head()
{
	echo ----------- $FILE_SH -----------
	echo ----------- $FILE_SH ----------- >> $1
	echo >> $1
}

run_sh_tail()
{
	echo
	echo >> $1
	echo >> $1
}

run_sh()
{
	while [ -n "$1" ]
	do
		FILE_SH=$SHEXE/$1.txt

		if [ -n "$VALG" ]
		then
			run_sh_head $VALG_LOGFILE
			$VALG $CEDCOM < $FILE_SH >> $VALG_LOGFILE 2>&1
			run_sh_tail $VALG_LOGFILE
		fi

		run_sh_head $LOGFILE
		$CEDCOM < $FILE_SH >> $LOGFILE 2>&1
		run_sh_tail $LOGFILE

		shift
	done
}


export LANG=C

# On error exit
set -e

> $LOGFILE
> $VALG_LOGFILE


# Creative
run_sh 11 12 13


# Destructive
run_sh 21


# Transformative
run_sh 31


# Reflective
run_sh 41


# Misc
run_sh 51


# Hand checked output
LOGFILE=tmp/91.log.txt
> $LOGFILE
run_sh 91



if [ -n "$VALG" ]
then
	# Valgrind summary
	# Using Valgrind creates numerical differences in formula results
	# on my system which renders a 'diff' test useless.

	echo
	grep "definitely lost" $VALG_LOGFILE

	echo
	grep "ERROR SUMMARY" $VALG_LOGFILE
fi

# Check results
echo
echo Comparing with checked results ...
echo

diff -Nurp $CHECK $OUT

