/*
	Copyright (C) 2011-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#ifndef _mtcedui_h
#define _mtcedui_h



#include <mtcelledit.h>
#include <mtpixel.h>



typedef struct _CedRenPage	CedRenPage;
typedef struct _CedRender	CedRender;
typedef struct _CedUI_clip	CedUI_clip;
typedef struct _CedUI_file	CedUI_file;
typedef struct _CuBook		CuBook;
typedef struct _CuUndo		CuUndo;
typedef struct _CuUndoStep	CuUndoStep;



#define CEDUI_CLIPBOARD_TIMESTAMP_SIZE		32



struct _CedRenPage
{
	int		font_size,
			width,
			height,
			margin_x,
			margin_y,
			footer[3],
			header[3]
			;
};

struct _CedRender
{
	CedSheet	* sheet;	// Sheet currently being edited

	mtFont		* font;
	int		font_width,	// Average width of font for numbers,
					// i.e. "01234567890"
			row_header_width, // Used to cetralize the
					// rendering of row numbers
			row_pad		// Pixels above and below the row
			;
};

struct _CedUI_clip
{
	CedSheet	* sheet;	// Current clipboard
	char		* ced,		// Dynamic description for clip from/to
					// other instance
			* tsv,		// TSV data created for others
			timestamp[CEDUI_CLIPBOARD_TIMESTAMP_SIZE]
			;
	int		temp,		// Flag 1 = temp file exists
			rows,		// Size of clipboard - needed in case of
					// blank.
			cols		// trailing rows/cols.
			;
};

struct _CedUI_file
{
	CuBook		* cubook;
	char		* name;		// OS filename loaded/saved
	CedFileType	type;
};

struct _CuUndo
{
	int		max,		// Maximum allowable undo/redo steps
			redo_tot,	// Total steps currently stored
			undo_tot	// Total steps currently stored
			;
	CuUndoStep	* redo_step,	// Next redo step
			* undo_step	// Next undo step
			;
};

struct _CuBook
{
	CedBook		* book;
	CuUndo		undo;
};



typedef void (* CedRenCB) (
	int		x,
	int		y,
	int		w,
	int		h,
	unsigned char	* rgb,
	void		* user_data
	);

typedef int (* CedGraphScan) (
	CedBook		* book,		// Don't add/remove any graph/file from
					// the book during this callback
	char		* graph_name,
	CedBookFile	* bookfile,
	void		* user_data
	);
	// 0 = continue
	// 1 = stop



enum
{
	CED_SHEET_EXPORT_EPS,
	CED_SHEET_EXPORT_HTML,
	CED_SHEET_EXPORT_PDF,
	CED_SHEET_EXPORT_PDF_PAGED,
	CED_SHEET_EXPORT_PNG,
	CED_SHEET_EXPORT_PS,
	CED_SHEET_EXPORT_SVG,
	CED_SHEET_EXPORT_TSV,
	CED_SHEET_EXPORT_TSV_QUOTED,

	CED_SHEET_EXPORT_TOTAL
};

// Unless otherwise stated functions cuf_* () return these values:
enum
{
	CU_ERROR_NONE		= 0,	// No errors, nothing to report

	CU_ERROR_NO_CHANGES	= -1,	// The sheet/book was not changed
	CU_ERROR_UNDO_OP	= -2,	// Error in undo system, operation not
					// completed
	CU_ERROR_UNDO_LOST	= -3,	// Undo history lost, but operation
					// completed
	CU_ERROR_CHANGES	= -4,	// The sheet/book may have changed
	CU_ERROR_LOCKED_CELL	= -5,	// This operation was blocked as a cell
					// was locked
	CU_ERROR_LOCKED_SHEET	= -6	// This operation was blocked as a
					// sheet was locked
};

enum
{
	CED_GRAPH_TYPE_EPS,
	CED_GRAPH_TYPE_PDF,
	CED_GRAPH_TYPE_PNG,
	CED_GRAPH_TYPE_PS,
	CED_GRAPH_TYPE_SVG,

	CED_GRAPH_TYPE_TOTAL
};

enum
{
CEDUI_CELLPREFS_align_horizontal = 0,
CEDUI_CELLPREFS_color_background,
CEDUI_CELLPREFS_color_foreground,
CEDUI_CELLPREFS_format,
CEDUI_CELLPREFS_width,
CEDUI_CELLPREFS_num_decimal_places,
CEDUI_CELLPREFS_num_zeros,
CEDUI_CELLPREFS_bold_text,
CEDUI_CELLPREFS_locked,
CEDUI_CELLPREFS_border_type,
CEDUI_CELLPREFS_border_color,
CEDUI_CELLPREFS_format_datetime,
CEDUI_CELLPREFS_num_thousands,
CEDUI_CELLPREFS_text_prefix,
CEDUI_CELLPREFS_text_suffix,

CEDUI_CELLPREFS_TOTAL
};

enum
{
CEDUI_CELLBORD_NONE =		-1,
CEDUI_CELLBORD_OUT_THIN =	-2,
CEDUI_CELLBORD_OUT_THICK =	-3,
CEDUI_CELLBORD_OUT_DOUBLE =	-4,
CEDUI_CELLBORD_TB_THIN =	-5,
CEDUI_CELLBORD_TB_THICK =	-6,
CEDUI_CELLBORD_TB_DOUBLE =	-7,

CEDUI_CELLBORD_H_CLEAR_TOP =	0,
CEDUI_CELLBORD_H_CLEAR_MID =	1,
CEDUI_CELLBORD_H_CLEAR_BOT =	2,
CEDUI_CELLBORD_H_THIN_TOP =	3,
CEDUI_CELLBORD_H_THIN_MID =	4,
CEDUI_CELLBORD_H_THIN_BOT =	5,
CEDUI_CELLBORD_H_THICK_TOP =	6,
CEDUI_CELLBORD_H_THICK_MID =	7,
CEDUI_CELLBORD_H_THICK_BOT =	8,
CEDUI_CELLBORD_H_DOUB_TOP =	9,
CEDUI_CELLBORD_H_DOUB_MID =	10,
CEDUI_CELLBORD_H_DOUB_BOT =	11,

CEDUI_CELLBORD_V_CLEAR_L =	12,
CEDUI_CELLBORD_V_CLEAR_C =	13,
CEDUI_CELLBORD_V_CLEAR_R =	14,
CEDUI_CELLBORD_V_THIN_L =	15,
CEDUI_CELLBORD_V_THIN_C =	16,
CEDUI_CELLBORD_V_THIN_R =	17,
CEDUI_CELLBORD_V_THICK_L =	18,
CEDUI_CELLBORD_V_THICK_C =	19,
CEDUI_CELLBORD_V_THICK_R =	20,
CEDUI_CELLBORD_V_DOUB_L =	21,
CEDUI_CELLBORD_V_DOUB_C =	22,
CEDUI_CELLBORD_V_DOUB_R =	23,

CEDUI_CELLBORD_MIN =		-7,
CEDUI_CELLBORD_MAX =		23
};



#define CED_DEFAULT_CELLWIDTH_CHARS	10
#define CED_ROWHEIGHT( REN )		(REN->font->height + 2 * REN->row_pad)
#define CED_DEFAULT_CELLWIDTH( REN )	(CED_DEFAULT_CELLWIDTH_CHARS * REN->font_width)

#define CED_INIFILE_FONT_FILENAME	"main_font_filename"
#define CED_INIFILE_FONT_SIZE		"main_font_size"
#define CED_INIFILE_FONT_TYPE		"main_font_type"
#define CED_INIFILE_ROW_PAD		"main_row_pad"
#define CED_INIFILE_LAST_DIR		"main_last_dir"
#define CED_INIFILE_MAIN_WINDOW		"main_window"
#define CED_INIFILE_PREFS_WINDOW	"prefs.window"
#define CED_INIFILE_SHARED_WINDOW	"~/shared/window"

#define CED_INIFILE_HELP_FILE		"help_file"
#define CED_INIFILE_HELP_BROWSER	"help_browser"

#define CED_INIFILE_RECENT_FILENAME_LEN	"main_recent_filename_len"
#define CED_INIFILE_MAIN_FILE		"main_file"
#define CED_INIFILE_MAIN_FPICK		"main_fpick"
#define CED_INIFILE_MAIN_FPICK_PRE	"main_"

#define CED_INIFILE_GRAPH_PANE_POS	"graph_pane_pos"
#define CED_INIFILE_FIND_PANE_POS	"find_pane_pos"
#define CED_INIFILE_FIND_ALL_SHEETS	"find_all_sheets"
#define CED_INIFILE_FIND_WILCARDS	"find_wilcards"
#define CED_INIFILE_FIND_CASE_SENSITIVE "find_case_sensitive"
#define CED_INIFILE_FIND_VALUE		"find_value"
#define CED_INIFILE_FIND_TEXT		"find_text"


#define CED_INIFILE_PAGE_MARGIN_X	"page_margin_x"
#define CED_INIFILE_PAGE_MARGIN_Y	"page_margin_y"
#define CED_INIFILE_PAGE_FONT_SIZE	"page_font_size"
#define CED_INIFILE_PAGE_FOOTER_LEFT	"page_footer_left"
#define CED_INIFILE_PAGE_FOOTER_CENTRE	"page_footer_centre"
#define CED_INIFILE_PAGE_FOOTER_RIGHT	"page_footer_right"
#define CED_INIFILE_PAGE_HEADER_LEFT	"page_header_left"
#define CED_INIFILE_PAGE_HEADER_CENTRE	"page_header_centre"
#define CED_INIFILE_PAGE_HEADER_RIGHT	"page_header_right"
#define CED_INIFILE_PAGE_WIDTH		"page_mm_width"
#define CED_INIFILE_PAGE_HEIGHT		"page_mm_height"

#define CED_INIFILE_SHEET_PREFS_PERSIST	"sheet_prefs_persist"

#define CED_INIFILE_GRAPH_SCALE		"graph_scale"
#define CED_INIFILE_2DYEAR_START	"date.2digit_year_start"


#define CED_HEADER_OPTIONS		_("None	Filename (long)	Filename (short)	Sheet Name	Page #	Date	Date and Time")

// Default number of max undo/redo steps
#define CU_DEFAULT_MAX_STEPS		100

// Sanity limits for the size of sheets when saving TSV/CSV

#define CEDUI_SHEET_MAX_ROW		1000000
#define CEDUI_SHEET_MAX_COL		1000000
#define CEDUI_SHEET_MAX_AREA		1000000000
#define CEDUI_SHEET_MAX_NAME		10000

#define CEDUI_CLIPBOARD_SHEET_NAME	"clipboard"

#define CEDUI_GRAPH_NAME_PREFIX		"graph/"
#define CEDUI_GRAPH_NAME_PREFIX_LEN	6



// 0 = success, NULL = fail unless otherwise stated

/*
	GEOMETRY FUNCTIONS
*/

int ced_ren_column_width_array (	// Create arrays of pixel coords/widths
					// for each column
	int		col_start,	// <= c1 <= c2
	CedRender	* viewren,
	int		x,		// Pixel X
	int		w,		// Pixel Width
	int		* c1,		// Put c1 column here
	int		* c2,		// Put c2 column here
	int		** col_x,	// Place array of column X coords here
	int		** col_w	// Place array of column widths here
	);

int ced_ren_x_from_column (
	int		col_start,
	CedRender	* viewren,
	int		column
	);
	// X

int ced_ren_y_from_row (
	int		row_start,
	CedRender	* viewren,
	int		row
	);
	// Y

int ced_ren_column_from_x_backwards (
	int		col_start,
	CedRender	* viewren,
	int		x
	);
	// Column

int ced_ren_column_from_x (
	int		col_start,
	CedRender	* viewren,
	int		x
	);
	// Column

int ced_ren_row_from_y (
	int		row_start,
	CedRender	* viewren,
	int		y
	);
	// Row

int ced_ren_cellwidth (
	CedRender	* viewren,
	int		column
	);
	// Pixel width of this column

/*
	PAGE EXPORT FUNCTIONS
*/

int ced_export_output (
	mtPrefs		* prefs_file,
	CedSheet	* sheet,
	char		* filename,
	char		* gui_filename,
	int		filetype
	);

/*
	PIXEL EXPOSURE FUNCTIONS


	Before calling these *_expose_* functions 'view' must contain validated
	'sheet' and 'font' data.
*/

int ced_ren_expose_main (
	int		row_start,
	int		col_start,
	CedRender	* viewren,
	int		x,
	int		y,
	int		w,
	int		h,
	CedRenCB	callback,
	void		* callback_data
	);

int ced_ren_expose_row_header (
	int		row_start,
	CedRender	* viewren,
	int		x,
	int		y,
	int		w,
	int		h,
	CedRenCB	callback,
	void		* callback_data
	);

int ced_ren_expose_column_header (
	int		col_start,
	CedRender	* viewren,
	int		x,
	int		y,
	int		w,
	int		h,
	CedRenCB	callback,
	void		* callback_data
	);

/*
	BOOK FUNCTIONS
*/

CuBook * cuf_book_new (
	void
	);

int cuf_book_destroy (			// Destroy book, undo, and the ubook
					// that holds them
	CuBook		* ub
	);

int cuf_book_undo_set_max (		// Set maximum undo/redo steps,
					// clearing any excess
	CuBook		* ub,
	int		max_steps	// 0..
	);

int cuf_book_undo_flush (		// Remove all undo/redo steps (but keep
					// the current max_steps)
	CuBook		* ub
	);

int cuf_book_undo_step (		// Undo the last step
	CuBook		* ub
	);

int cuf_book_redo_step (		// Redo the next step
	CuBook		* ub
	);

int cuf_check_sheet_lock (		// Is this sheet locked?
	CedSheet	* sheet
	);
	// 1 = locked

int cuf_check_cell_lock (		// Are any of these cells locked?
	CedSheet	* sheet,
	int		r,
	int		c,
	int		rowtot,
	int		coltot
	);
	// 1 = locked

char * cuf_error_str (			// Get the error string to report to
					// user (when cells locked)
	void
	);

/*
	--------------------------------------------
	Undoable wrapper functions for libmtcelledit
	--------------------------------------------

All arguments are required to be valid on entry unless otherwise stated.
*/

int cuf_book_add_sheet (
	CuBook		* ub,
	CedSheet	* new_sheet,
	char		* page
	);

int cuf_book_destroy_sheet (
	CuBook		* ub,
	char		* page
	);

int cuf_book_page_rename (
	CuBook		* ub,
	CedSheet	* a_sheet,
	char		* name
	);

int cuf_book_merge (
	CuBook		* ub,
	CedBook		* book,
	int		* sheet_tot,
	int		* sheet_fail,
	int		* file_tot,
	int		* file_fail
	);

int cuf_sheet_set_cell (
	CuBook		* ub,
	CedSheet	* sheet,
	unsigned int	row,
	unsigned int	column,
	char		* text
	);

int cuf_sheet_set_column_width (
	CuBook		* ub,
	CedSheet	* sheet,
	unsigned int	column,
	unsigned int	coltot,
	int		width
	);

int cuf_sheet_set_column_width_list (
	CuBook		* ub,
	CedSheet	* sheet,
	unsigned int	column,
	unsigned int	coltot,
	int		* width_list
	);

int cuf_sheet_set_column_width_auto (
	CuBook		* ub,
	CedSheet	* sheet,
	unsigned int	column,
	unsigned int	coltot
	);

int cuf_sheet_sort_rows (
	CuBook		* ub,
	CedSheet	* sheet,
	unsigned int	row,
	unsigned int	rowtot,
	unsigned int	cols[],
	int		mode,
	int		mode_list[]
	);

int cuf_sheet_sort_columns (
	CuBook		* ub,
	CedSheet	* sheet,
	unsigned int	column,
	unsigned int	coltot,
	unsigned int	rows[],
	int		mode,
	int		mode_list[]
	);

int cuf_sheet_insert_row (
	CuBook		* ub,
	CedSheet	* sheet,
	unsigned int	row,
	unsigned int	rowtot
	);

int cuf_sheet_delete_row (
	CuBook		* ub,
	CedSheet	* sheet,
	unsigned int	row,
	unsigned int	rowtot
	);

int cuf_sheet_insert_column (
	CuBook		* ub,
	CedSheet	* sheet,
	unsigned int	col,
	unsigned int	coltot
	);

int cuf_sheet_delete_column (
	CuBook		* ub,
	CedSheet	* sheet,
	unsigned int	col,
	unsigned int	coltot
	);

int cuf_sheet_paste_area (
	CuBook		* ub,
	CedSheet	* sheet,
	CedSheet	* paste,
	unsigned int	row,
	unsigned int	column,
	unsigned int	rowtot,		// Must be >= paste_rowtot
	unsigned int	coltot,		// Must be >= paste_coltot
	unsigned int	paste_rowtot,
	unsigned int	paste_coltot,
	int		mode
	);

int cuf_sheet_clear_area (
	CuBook		* ub,
	CedSheet	* sheet,
	unsigned int	row,
	unsigned int	column,
	unsigned int	rowtot,
	unsigned int	coltot,
	int		mode
	);

int cuf_book_duplicate_sheet (
	CuBook		* cubook,
	CedSheet	* sheet,
	CedSheet	** new_sh	// Optional: new sheet
	);

/*
	FILE FUNCTIONS

The general idea is that a UI will have access to at least one file and
at least one clipboard.  Exotic implementations may have several of each.
*/

CedUI_file * cuf_file_new (		// Create new empty file state
	void
	);

int cuf_file_free (			// Release this structure, and its
					// contents
	CedUI_file	* file
	);

int cuf_file_load (			// Load a sheet or book as a new CuBook
	CedUI_file	* file,		// On success set filetype and real
					// filename in state
	char		* filename	// Filename to load
	);
	// 0 = success
	// 1 = arg error
	// 2 = file error

int cuf_file_save (
/*
	Save a sheet or book using a temp file to avoid destroying file data
	if a fail occurs mid-way. TSV/CSV sheets must be within limits
	outlined above.
*/
	CedUI_file	* file,		// On success set filetype and real
					// filename in state
	char		* filename,
	CedFileType	filetype
	);
	// -1 = error
	//  0 = success
	//  1 = TSV/CSV sheet too large

int cuf_file_sheet_add (		// Add a new sheet "Sheet 1", 2, 3, etc.
					// via undo system
	CedUI_file	* file
	);
	// 0 = success
	// 1 = error creating sheet, else CU_ERROR_*

int cuf_file_book_new (			// Remove old book, filename, type, and
					// start afresh
	CedUI_file	* file
	);

CedSheet * cuf_file_get_sheet (
/*
	Get the currently active sheet (if there is one)
	If active name is dangling, it gets cleared.
	If active name is empty but sheets exist, root is chosen.
*/
	CedUI_file	* file
	);
	// NULL = fail or no active sheet

CedBookFile * cuf_file_get_graph (
/*
	Get the currently active graph (if there is one)
	If active name is dangling, it gets cleared.
	If active name is empty but graphs exist, 1st is chosen.
*/
	CedUI_file	* file
	);
	// NULL = fail or no active graph

int cuf_sheet_check_geometry (		// Is this sheet too large to save?
	CedSheet	* sheet
	);
	// 0 = ok to save
	// 1 = too large
	// 2 = error

int cuf_sheet_2dyear (			// Change century of 2 digit years
	CedUI_file	* file,
	int year_start			// MTKIT_DDT_MIN_DATE_YEAR..
					// (MTKIT_DDT_MAX_DATE_YEAR - 99)
	);
	// 1 = Arg Error, else CU_ERROR_*

/*
	CLIPBOARD FUNCTIONS
*/

CedUI_clip * cuf_clip_new (		// Create new empty clipboard
	void
	);

int cuf_clip_free (			// Release this structure, and its
					// contents
	CedUI_clip	* clipboard
	);

int cuf_clip_flush (			// Empty clipboard contents
	CedUI_clip	* clipboard
	);

int cuf_clip_import_text (		// Import text from another program via
					// TSV text block
	CedUI_clip	* clipboard,
	char		* text		// NUL terminated text to import as a
					// sheet
	);
	// 0 = success

int cuf_clip_export_text (		// Create the file->tsv text for export
	CedUI_clip	* clipboard
	);

int cuf_clip_save_temp (		// Save current clipboard to filesystem
	CedUI_clip	* clipboard
	);

int cuf_clip_save_temp_file (		// Save current clipboard to filesystem
	CedUI_clip	* clipboard,
	char		* filename
	);

int cuf_clip_load_temp (		// Load clipboard from filesystem
	CedUI_clip	* clipboard
	);
	// 0 = success

int cuf_clip_load_temp_file (		// Load clipboard from filesystem
	CedUI_clip	* clipboard,
	char		* filename
	);

int cuf_clip_set_timestamp (		// Set the timestamp
	CedUI_clip	* clipboard,
	char		* txt		// If NULL set to current time,
					// else use this NUL terminate string
	);

int cuf_clip_paste (			// Paste the clipboard onto the current
					// cursor area
	CedUI_file	* file,
	CedUI_clip	* clipboard,
	int		paste_mode
	);
	// 1 = arg error else CU_ERROR_*

int cuf_clip_copy (			// Copy current selection to the
					// clipboard
	CedUI_file	* file,
	CedUI_clip	* clipboard
	);

/*
	GRAPH FUNCTIONS
*/

CedBookFile * ced_graph_new (
/*
	Create a new graph in this book (possibly destroying same named graph)
*/
	CedBook		* book,
	char		* mem,
	int		memsize,
	char		* graph_name
	);

CedBookFile * ced_graph_get (
	CedBook		* book,
	char		* graph_name
	);

int ced_graph_destroy (
	CedBook		* book,
	char		* graph_name
	);

char * ced_graph_get_filename (		// Get the full filename for this graph
					// name, i.e. "graph/graph_name"
	char		* graph_name
	);
	// Return pointer to newly allocated string - caller must free ()

int ced_graph_scan (			// Scan a book and use callback for each
					// graph found
	CedBook		* book,
	CedGraphScan	callback,
	void		* user_data
	);
	// 0 = success
	// 1 = error
	// 2 = user termination

mtImage * ced_graph_render_mtimage (
	CedBook		* book,
	char		* graph_name,
	int		* breakpoint,
/*
	Store point of error here (i.e. valid characters) (NULL = don't)
	-1 = Error during rendering (not parsing)
*/
	double		scale		// 1 = 100% 2 = 200% (10..1000%)
	);

int ced_graph_render_file (
	CedBook		* book,
	char		* graph_name,
	char		* filename,
	int		filetype,	// CED_GRAPH_TYPE_*
	int		* breakpoint,
/*
	Store point of error here (i.e. valid characters) (NULL = don't)
	-1 = Error during rendering (not parsing)
*/
	double		scale		// 1 = 100% 2 = 200% (10..1000%)
	);

int ced_graph_duplicate (
	CuBook		* cubook,
	char		* graph_name,
	char		** new_name	// Optional - put allocated string to
					// new name here
	);

/*
	CELLPREFS FUNCTIONS

All function returns: 0 = success, 1 = arg error, CU_ERROR_*
*/

int cedui_cellprefs_init (
	CedSheet	* sheet,
	int		* r1,
	int		* c1,
	int		* r2,
	int		* c2,
	CedSheet	** tmp_sheet	// Put temp sheet here
	);

int cedui_cellprefs_change (
	CuBook		* cubook,
	CedSheet	* sheet,
	int		r1,		// Must be same values as sent to
	int		c1,		// cedui_cellprefs_init ()
	int		r2,
	int		c2,
	CedSheet	* tmp_sheet,	// From cedui_cellprefs_init ()
	int		pref_id,	// Pref to change CEDUI_CELLPREF_*
	char		* pref_charp,	// Points to new value (if string)
	int		pref_int	// Used if pref_id requires it
	);
	// After final use, caller must destroy tmp_sheet

// The following functions don't require cedui_cellprefs_init ()

int cedui_cellprefs_bold (		// Toggles the bold in selected cells
	CedUI_file	* uifile
	);

int cedui_cellprefs_border (		// Adds a border type to selected cells.
	CedUI_file	* uifile,
	int		border_type	// CEDUI_CELLBORD_*
	);



#endif		// _mtcedui_h

