/*
	Copyright (C) 2011-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "private.h"



typedef int (* utFunc) ();



typedef struct
{
	char		* name;
	utFunc		func;
	char		* help;
} utComRec;

typedef struct
{
	char		* name;
	int		id;
} utFTRec;



GLOBAL			global = { 0 };

static	utComRec	* comrec;	// Current command record
					// (NULL = cedutils)



#define CEDUT_HELP_APPEND \
	"cedappend [[OPTION] [FILE]]...\n" \
	"\n" \
	"Data from files is appended, vertically or horizontally.\n" \
	"\n" \
	"Options:\n" \
	"\n" \
	"  -csv                 Following input file is CSV.\n" \
	"  -col                 Place following files horizontally.\n" \
	"  -o STRING            Save current sheet to this file (immediately).\n" \
	"  -otype STRING        Output format of file (use 'list' to list types, default=TSV content).\n" \
	"  -recalc INTEGER      Recalculate the sheet this many times (top-bottom, bottom-top).\n" \
	"  -tsv                 Following input file is TSV.\n" \
	"  -row                 Place following files vertically (default).\n" \
	"\n" \
	""

#define CEDUT_HELP_CLEAR \
	"cedclear [[OPTION] [FILE]]...\n" \
	"\n" \
	"Clear cells.\n" \
	"If FILE==\"\" then action the current sheet in memory.\n" \
	"\n" \
	"Options:\n" \
	"\n" \
	"  -csv                 Following input file is CSV.\n" \
	"  -o STRING            Save current sheet to this file (immediately).\n" \
	"  -otype STRING        Output format of file (use 'list' to list types, default=TSV content).\n" \
	"  -range STRING        Clear these cells (default = r1c1:r_c_ ).\n" \
	"  -recalc INTEGER      Recalculate the sheet this many times (top-bottom, bottom-top).\n" \
	"  -tsv                 Following input file is TSV.\n" \
	"\n" \
	""

#define CEDUT_HELP_CUT \
	"cedcut [[OPTION] [FILE]]...\n" \
	"\n" \
	"Cut rows or columns.\n" \
	"If FILE==\"\" then action the current sheet in memory.\n" \
	"\n" \
	"Options:\n" \
	"\n" \
	"  -col                 Cut column(s).\n" \
	"  -csv                 Following input file is CSV.\n" \
	"  -o STRING            Save current sheet to this file (immediately).\n" \
	"  -otype STRING        Output format of file (use 'list' to list types, default=TSV content).\n" \
	"  -recalc INTEGER      Recalculate the sheet this many times (top-bottom, bottom-top).\n" \
	"  -row                 Cut row(s) (default).\n" \
	"  -start INTEGER       Start cutting at this row or column (default = 1).\n" \
	"  -total INTEGER       Total number of rows or columns to cut (default = 1).\n" \
	"  -tsv                 Following input file is TSV.\n" \
	"\n" \
	""

#define CEDUT_HELP_EVAL \
	"cedeval [[OPTION] [EXP]]...\n" \
	"\n" \
	"Evaluate a spreadsheet expression.\n" \
	"\n" \
	"Options:\n" \
	"\n" \
	"  -csv                 Following input file is CSV.\n" \
	"  -i STRING            Input file.\n" \
	"  -tsv                 Following input file is TSV.\n" \
	"\n" \
	""

#define CEDUT_HELP_FIND \
	"cedfind [[OPTION] [EXP]]...\n" \
	"\n" \
	"Find all cells that match the given expression.\n" \
	"\n" \
	"Options:\n" \
	"\n" \
	"  -case                Case sensitive search.\n" \
	"  -csv                 Following input file is CSV.\n" \
	"  -i STRING            Input file.\n" \
	"  -num                 Find numerical value.\n" \
	"  -range STRING        Search in this range (default = r1c1:r_c_ ).\n" \
	"  -tsv                 Following input file is TSV.\n" \
	"  -wildcard            * and ? characters are wilcards.\n" \
	"  -v                   Verbose.  Output the cell content as well as the address.\n" \
	"\n" \
	""

#define CEDUT_HELP_FLIP \
	"cedflip [[OPTION] [FILE]]...\n" \
	"\n" \
	"Flips the sheet by row or column.\n" \
	"If FILE==\"\" then action the current sheet in memory.\n" \
	"\n" \
	"Options:\n" \
	"\n" \
	"  -col                 Flip columns horizontally.\n" \
	"  -csv                 Following input file is CSV.\n" \
	"  -o STRING            Save current sheet to this file (immediately).\n" \
	"  -otype STRING        Output format of file (use 'list' to list types, default=TSV content).\n" \
	"  -recalc INTEGER      Recalculate the sheet this many times (top-bottom, bottom-top).\n" \
	"  -row                 Flip rows vertically (default).\n" \
	"  -tsv                 Following input file is TSV.\n" \
	"\n" \
	""

#define CEDUT_HELP_INSERT \
	"cedinsert [[OPTION] [FILE]]...\n" \
	"\n" \
	"Insert rows or columns.\n" \
	"If FILE==\"\" then action the current sheet in memory.\n" \
	"\n" \
	"Options:\n" \
	"\n" \
	"  -col                 Insert column(s).\n" \
	"  -csv                 Following input file is CSV.\n" \
	"  -o STRING            Save current sheet to this file (immediately).\n" \
	"  -otype STRING        Output format of file (use 'list' to list types, default=TSV content).\n" \
	"  -recalc INTEGER      Recalculate the sheet this many times (top-bottom, bottom-top).\n" \
	"  -row                 Insert row(s) (default).\n" \
	"  -start INTEGER       Start inserting at this row or column (default = 1).\n" \
	"  -total INTEGER       Total number of rows or columns to insert (default = 1).\n" \
	"  -tsv                 Following input file is TSV.\n" \
	"\n" \
	""

#define CEDUT_HELP_LS \
	"cedls [[OPTION] [FILE]]...\n" \
	"\n" \
	"List file information.\n" \
	"\n" \
	"Options:\n" \
	"\n" \
	"  -csv                 Following input file is CSV.\n" \
	"  -tsv                 Following input file is TSV.\n" \
	"  -v                   Verbose.\n" \
	"\n" \
	""

#define CEDUT_HELP_PASTE \
	"cedpaste [[OPTION] [FILE]]...\n" \
	"\n" \
	"Load first file, and paste subsequent files onto the sheet.\n" \
	"\n" \
	"Options:\n" \
	"\n" \
	"  -csv                 Following input file is CSV.\n" \
	"  -dest STRING         Paste to this section of the sheet (default = r1c1:r1c1 ).\n" \
	"                       When a single cell, paste whole of input.  Otherwise truncate\n" \
	"                       or repeat input data to fill the dest range.\n" \
	"  -o STRING            Save current sheet to this file (immediately).\n" \
	"  -otype STRING        Output format of file (use 'list' to list types, default=TSV content).\n" \
	"  -range STRING        Paste from this section of the input file (default = r1c1:r_c_ ).\n" \
	"  -recalc INTEGER      Recalculate the sheet this many times (top-bottom, bottom-top).\n" \
	"  -tsv                 Following input file is TSV.\n" \
	"\n" \
	""

#define CEDUT_HELP_ROTATE \
	"cedrotate [[OPTION] [FILE]]...\n" \
	"\n" \
	"Rotates a sheet clockwise or anticlockwise.\n" \
	"If FILE==\"\" then action the current sheet in memory.\n" \
	"\n" \
	"Options:\n" \
	"\n" \
	"  -anti                Rotate anticlockwise.\n" \
	"  -clock               Rotate clockwise (default).\n" \
	"  -csv                 Following input file is CSV.\n" \
	"  -o STRING            Save current sheet to this file (immediately).\n" \
	"  -otype STRING        Output format of file (use 'list' to list types, default=TSV content).\n" \
	"  -recalc INTEGER      Recalculate the sheet this many times (top-bottom, bottom-top).\n" \
	"  -tsv                 Following input file is TSV.\n" \
	"\n" \
	""

#define CEDUT_HELP_SET \
	"cedset [[OPTION] [EXP]]...\n" \
	"\n" \
	"Set cell contents to the given expression.\n" \
	"Expression \"r5c5 'Hello\" sets 'Hello as the contents of cell r5c5.\n" \
	"\n" \
	"Options:\n" \
	"\n" \
	"  -csv                 Following input file is CSV.\n" \
	"  -i STRING            Input file.\n" \
	"  -o STRING            Save current sheet to this file (immediately).\n" \
	"  -otype STRING        Output format of file (use 'list' to list types, default=TSV content).\n" \
	"  -recalc INTEGER      Recalculate the sheet this many times (top-bottom, bottom-top).\n" \
	"  -tsv                 Following input file is TSV.\n" \
	"\n" \
	""

#define CEDUT_HELP_SORT \
	"cedsort [[OPTION] [EXP]]...\n" \
	"\n" \
	"Sort a sheet according to the given expression.\n" \
	"Expression 1,ac,2,d sorts by row 1 (ascending, case sensitive), then row 2 (descending).\n" \
	"Up to 10 rows or columns may be specified in the expression.\n" \
	"\n" \
	"Options:\n" \
	"\n" \
	"  -col                 Sort columns.\n" \
	"  -csv                 Following input file is CSV.\n" \
	"  -i STRING            Input file.\n" \
	"  -o STRING            Save current sheet to this file (immediately).\n" \
	"  -otype STRING        Output format of file (use 'list' to list types, default=TSV content).\n" \
	"  -recalc INTEGER      Recalculate the sheet this many times (top-bottom, bottom-top).\n" \
	"  -row                 Sort rows (default).\n" \
	"  -start INTEGER       First row/column to sort from (default = 1).\n" \
	"  -total INTEGER       Total rows/colums to sort (default = 1 = all).\n" \
	"  -tsv                 Following input file is TSV.\n" \
	"  -v                   Verbose.\n" \
	"\n" \
	""

#define CEDUT_HELP_TRANSPOSE \
	"cedtranspose [[OPTION] [FILE]]...\n" \
	"\n" \
	"Transpose sheet cells.\n" \
	"If FILE==\"\" then action the current sheet in memory.\n" \
	"\n" \
	"Options:\n" \
	"\n" \
	"  -csv                 Following input file is CSV.\n" \
	"  -o STRING            Save current sheet to this file (immediately).\n" \
	"  -otype STRING        Output format of file (use 'list' to list types, default=TSV content).\n" \
	"  -recalc INTEGER      Recalculate the sheet this many times (top-bottom, bottom-top).\n" \
	"  -tsv                 Following input file is TSV.\n" \
	"\n" \
	""



//	Command table is strictly alpha sorted for quick binary searching
static utComRec	comtab[] = {
	{ "append",	cedut_append,		CEDUT_HELP_APPEND },
	{ "clear",	cedut_clear,		CEDUT_HELP_CLEAR },
	{ "cut",	cedut_cut,		CEDUT_HELP_CUT },
	{ "eval",	cedut_eval,		CEDUT_HELP_EVAL },
	{ "find",	cedut_find,		CEDUT_HELP_FIND },
	{ "flip",	cedut_flip,		CEDUT_HELP_FLIP },
	{ "insert",	cedut_insert,		CEDUT_HELP_INSERT },
	{ "ls",		cedut_ls,		CEDUT_HELP_LS },
	{ "paste",	cedut_paste,		CEDUT_HELP_PASTE },
	{ "rotate",	cedut_rotate,		CEDUT_HELP_ROTATE },
	{ "set",	cedut_set,		CEDUT_HELP_SET },
	{ "sort",	cedut_sort,		CEDUT_HELP_SORT },
	{ "transpose",	cedut_transpose,	CEDUT_HELP_TRANSPOSE }
	};

//	File type table is strictly alpha sorted for quick binary searching
static utFTRec ftypes[] = {
	{ "csv_content",	CED_FILE_TYPE_CSV_CONTENT },
	{ "csv_content_noq",	CED_FILE_TYPE_CSV_CONTENT_NOQ },
	{ "csv_value",		CED_FILE_TYPE_CSV_VALUE },
	{ "csv_value_noq",	CED_FILE_TYPE_CSV_VALUE_NOQ },
	{ "input",		CED_FILE_TYPE_NONE },
	{ "ledger",		CED_FILE_TYPE_LEDGER },
	{ "ledger_gz",		CED_FILE_TYPE_LEDGER_GZ },
	{ "ledger_val",		CED_FILE_TYPE_LEDGER_VAL },
	{ "ledger_val_gz",	CED_FILE_TYPE_LEDGER_VAL_GZ },
	{ "output_html",	CED_FILE_TYPE_OUTPUT_HTML },
	{ "output_tsv",		CED_FILE_TYPE_OUTPUT_TSV },
	{ "output_tsv_q",	CED_FILE_TYPE_OUTPUT_TSV_QUOTED },
	{ "tsv_content",	CED_FILE_TYPE_TSV_CONTENT },
	{ "tsv_content_gz",	CED_FILE_TYPE_TSV_CONTENT_GZ },
	{ "tsv_content_noq",	CED_FILE_TYPE_TSV_CONTENT_NOQ },
	{ "tsv_value",		CED_FILE_TYPE_TSV_VALUE },
	{ "tsv_value_gz",	CED_FILE_TYPE_TSV_VALUE_GZ },
	{ "tsv_value_noq",	CED_FILE_TYPE_TSV_VALUE_NOQ }
	};

static char * ff_errtab[] = {
	"Unknown",				// 0
	"Unable to load file",			// 1
	"Unexpected libmtcelledit failure"	// 2
	};

#define COMTAB_LEN	( sizeof ( comtab ) / sizeof ( comtab[0] ) )
#define FTYPES_LEN	( sizeof ( ftypes ) / sizeof ( ftypes[0] ) )
#define FF_ERRTAB_LEN	( sizeof ( ff_errtab ) / sizeof ( ff_errtab[0] ) )



static utComRec * get_comrec (
	char		* command
	)
{
	int		a, b, c, cmp;


	a = 0;
	b = COMTAB_LEN - 1;

	while ( a <= b )
	{
		c = (a + b) / 2;
		cmp = strcmp ( command, comtab[c].name );

		if ( cmp == 0 )
		{
			return &comtab[c];	// Found
		}

		if ( cmp < 0 )
		{
			b = c - 1;
		}
		else	// cmp > 0
		{
			a = c + 1;
		}
	}


	return NULL;			// Not found
}

static int get_filetype ( void )
{
	int		a, b, c, cmp;


	a = 0;
	b = FTYPES_LEN - 1;

	while ( a <= b )
	{
		c = (a + b) / 2;
		cmp = strcmp ( global.s_arg, ftypes[c].name );

		if ( cmp == 0 )
		{
			global.i_ftype_out = ftypes[c].id;

			return 0;	// Found
		}

		if ( cmp < 0 )
		{
			b = c - 1;
		}
		else	// cmp > 0
		{
			a = c + 1;
		}
	}


	return 1;			// Not found
}

static int file_func (
	char		* filename
	)
{
	if ( comrec && comrec->func )
	{
		int		res;


		global.s_arg = filename;

		res = comrec->func ();

		if ( res > 0 )
		{
			if ( res > (FF_ERRTAB_LEN - 1) )
			{
				res = 0;
			}

			fprintf ( stderr, "%s error: %s. arg = '%s'\n",
				comrec->name, ff_errtab[res], filename );

			global.i_error = 1;	// Terminate ASAP
		}
	}

	return global.i_error;		// Keep parsing if no errors encountered
}

static int error_func (
	int		error,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	fprintf ( stderr, "error_func: Argument ERROR! - num = %i arg = %i/%i",
		error, arg, argc );

	if ( arg < argc )
	{
		fprintf ( stderr, " '%s'", argv[arg] );
	}

	fprintf ( stderr, "\n" );

	return 0;			// Keep parsing
}

static void select_command ( void )
{
	if (	global.s_arg[0] == 'c' &&
		global.s_arg[1] == 'e' &&
		global.s_arg[2] == 'd'
		)
	{
		global.s_arg += 3;
	}

	comrec = get_comrec ( global.s_arg );
}

static int print_version (
	mtArg		* mtarg,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	if ( comrec )
	{
		printf ( "ced%s - Part of %s\n\n", comrec->name, VERSION );
	}
	else
	{
		printf ( "%s\n\n", VERSION );
	}

	return 1;			// Stop parsing
}

static int print_help (
	mtArg		* mtarg,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	if ( comrec )
	{
		printf ( "ced%s - Part of %s\n"
			"Usage:\n"
			"%s\n"
			, comrec->name, VERSION, comrec->help );
	}
	else
	{
		int		i;


		printf ( "%s\n\n"
			"Usage: %s [OPTION]...\n"
			"\n"
			"Options:\n"
			"  --help               Output this help.\n"
			"  --version            Output version information.\n"
			"  -v                   Verbose.\n"
			"  -com STRING          Change command (with or without"
					" ced prefix).\n"
			"\n"
			"Available commands:\n\n\n"
			, VERSION, "cedutils" );

		for ( i = 0; i < COMTAB_LEN; i++ )
		{
			printf ( "ced%s\n", comtab[i].name );

			if ( global.i_verbose )
			{
				printf ( "Usage: %s"
					"\n"
					, comtab[i].help );
			}
		}

		printf ( "\n" );
	}

	return 1;			// Stop parsing
}

static int init_globals ()
{
	ced_init ();

	memset ( &global, 0, sizeof ( global ) );

	global.i_range[0] = 1;
	global.i_range[1] = CED_MAX_ROW;
	global.i_range[2] = 1;
	global.i_range[3] = CED_MAX_COLUMN;
	global.i_dest[0] = 1;
	global.i_dest[1] = 1;
	global.i_dest[2] = 1;
	global.i_dest[3] = 1;
	global.i_start = 1;
	global.i_total = 1;
	global.i_ftype_in = CED_FILE_TYPE_TSV_CONTENT;
	global.i_ftype_out = CED_FILE_TYPE_TSV_CONTENT;

	global.sheet = ced_sheet_new ();
	if ( ! global.sheet )
	{
		global.i_error = 1;

		return 1;
	}

	return 0;			// Success
}

static void cleanup_globals ()
{
	ced_sheet_destroy ( global.sheet );
	global.sheet = NULL;
}

static int argcb_com (
	mtArg		* mtarg,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	comrec = get_comrec ( global.s_arg );
	if ( ! comrec )
	{
		// User error, so stop program here (loudly)
		fprintf ( stderr, "Error: No such command '%s'\n",
			global.s_arg );
		global.i_error = 1;

		return 1;
	}

	return 0;			// Continue parsing args
}

static int parse_cellrange (
	int		idata[4]
	)
{
	CedCellRef	r1, r2;


	if ( strchr ( global.s_arg, ':' ) )
	{
		// User has entered a range

		if ( ced_strtocellrange ( global.s_arg, &r1, &r2, NULL, 1 ) )
		{
			goto error;
		}
	}
	else
	{
		// User has entered a cell reference

		if ( ced_strtocellref ( global.s_arg, &r1, NULL, 1 ) )
		{
			goto error;
		}

		r2 = r1;
	}

	// Disallow relative r1/r2
	if (	r1.row_m ||
		r1.col_m ||
		r2.row_m ||
		r2.col_m
		)
	{
		goto error;
	}

	// Lowest row/col first for deterministic calculations later
	idata[0] = MIN ( r1.row_d, r2.row_d );
	idata[1] = MAX ( r1.row_d, r2.row_d );
	idata[2] = MIN ( r1.col_d, r2.col_d );
	idata[3] = MAX ( r1.col_d, r2.col_d );

	return 0;

error:

	fprintf ( stderr, "Error: Bad cell range or reference '%s'\n",
		global.s_arg );
	global.i_error = 1;

	return 1;
}

static int argcb_dest (
	mtArg		* mtarg,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	return parse_cellrange ( global.i_dest );
}

static int argcb_range (
	mtArg		* mtarg,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	return parse_cellrange ( global.i_range );
}

int ut_load_file ( void )
{
	CedFileType	new_type;
	CedSheet	* new_sheet;


	if ( ! global.s_arg )
	{
		goto error;
	}

	if ( global.s_arg[0] == 0 )
	{
		// User sent us "" which means action the current sheet

		return 0;
	}

	if ( global.i_csv )
	{
		new_sheet = ced_sheet_load_csv ( global.s_arg, NULL );
		new_type = CED_FILE_TYPE_CSV_CONTENT;
	}
	else
	{
		new_sheet = ced_sheet_load ( global.s_arg, NULL, &new_type );
	}

	if ( ! new_sheet )
	{
		goto error;
	}

	ced_sheet_destroy ( global.sheet );
	global.sheet = new_sheet;
	global.i_ftype_in = new_type;

	return 0;			// Success

error:
	global.i_error = 1;

	return 1;			// Fail
}

static int argcb_i (
	mtArg		* mtarg,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	if ( ut_load_file () )
	{
		fprintf ( stderr, "Error: Unable to load file '%s'\n",
			global.s_arg );

		return 1;		// Failure, so stop parsing
	}

	return 0;			// Continue parsing args
}

static int argcb_o (
	mtArg		* mtarg,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	int		ftype;


	if ( global.i_ftype_out == CED_FILE_TYPE_NONE )
	{
		ftype = global.i_ftype_in;
	}
	else
	{
		ftype = global.i_ftype_out;
	}

	if ( ced_sheet_save ( global.sheet, global.s_arg, ftype ) )
	{
		fprintf ( stderr, "Error: Unable to save file '%s'\n",
			global.s_arg );

		return 1;		// Failure, so stop parsing
	}

	return 0;			// Continue parsing args
}

static int argcb_otype (
	mtArg		* mtarg,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	if ( strcmp ( global.s_arg, "list" ) == 0 )
	{
		int		i;

		printf ( "Valid file types:\n\n" );

		for ( i = 0; i < FTYPES_LEN ; i++ )
		{
			printf ( "%s\n", ftypes[i].name );
		}
	}
	else if ( get_filetype () )
	{
		// User error, so stop program here (loudly)
		fprintf ( stderr, "Error: No such file format '%s'\n",
			global.s_arg );
		global.i_error = 1;

		return 1;
	}

	return 0;			// Continue parsing args
}

static int argcb_recalc (
	mtArg		* mtarg,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	int		i;


	for ( i = 0; i < global.i_tmp; i++ )
	{
		ced_sheet_recalculate ( global.sheet, NULL, 0 );
		ced_sheet_recalculate ( global.sheet, NULL, 1 );
	}

	return 0;			// Continue parsing args
}



static mtArg	arg_list[] = {
{ "-help",	MTKIT_ARG_SWITCH,	&global.i_tmp, 1, print_help },
{ "-version",	MTKIT_ARG_SWITCH,	&global.i_tmp, 2, print_version },

{ "anti",	MTKIT_ARG_SWITCH,	&global.i_clock, 1 },
{ "case",	MTKIT_ARG_SWITCH,	&global.i_case, 1 },
{ "clock",	MTKIT_ARG_SWITCH,	&global.i_clock, 0 },
{ "col",	MTKIT_ARG_SWITCH,	&global.i_rowcol, 1 },
{ "com",	MTKIT_ARG_STRING,	&global.s_arg, 1, argcb_com },
{ "csv",	MTKIT_ARG_SWITCH,	&global.i_csv, 1 },
{ "dest",	MTKIT_ARG_STRING,	&global.s_arg, 0, argcb_dest },
{ "i",		MTKIT_ARG_STRING,	&global.s_arg, 0, argcb_i },
{ "num",	MTKIT_ARG_SWITCH,	&global.i_num, 1 },
{ "o",		MTKIT_ARG_STRING,	&global.s_arg, 0, argcb_o },
{ "otype",	MTKIT_ARG_STRING,	&global.s_arg, 0, argcb_otype },
{ "range",	MTKIT_ARG_STRING,	&global.s_arg, 0, argcb_range },
{ "recalc",	MTKIT_ARG_INT,		&global.i_tmp, 0, argcb_recalc },
{ "row",	MTKIT_ARG_SWITCH,	&global.i_rowcol, 0 },
{ "start",	MTKIT_ARG_INT,		&global.i_start },
{ "total",	MTKIT_ARG_INT,		&global.i_total },
{ "tsv",	MTKIT_ARG_SWITCH,	&global.i_csv, 0 },
{ "v",		MTKIT_ARG_SWITCH,	&global.i_verbose, 1 },
{ "wildcard",	MTKIT_ARG_SWITCH,	&global.i_wildcard, 1 },

{ NULL }
};



int main (
	int		argc,
	char		* argv[]
	)
{
	if ( init_globals () )
	{
		return 1;
	}

	// Establish what command has been used to call this program
	global.s_arg = strrchr ( argv[0], MTKIT_DIR_SEP );

	if ( ! global.s_arg )
	{
		global.s_arg = argv[0];
	}
	else
	{
		global.s_arg ++;
	}

	select_command ();

	// Parse & action the command line arguments
	mtkit_arg_parse ( argc, argv, arg_list, file_func, error_func );

	cleanup_globals ();

	return global.i_error;
}
