#!/bin/sh


OUT=output
CHECK=results



if [ "$VALG" = "Y" ]
then
	VALG="valgrind --leak-check=full --show-possibly-lost=no"
fi


export LANG=C

# On error exit
set -e



# Creative

FILE_11=$OUT/11_set.tsv
FILE_12=$OUT/12_append.tsv
FILE_13=$OUT/13_paste.tsv

echo $FILE_11
$VALG cedset	"r1c1 qwert" "r1c2 Qwert" "r1c3 589.23" \
		"r2c1 13-4-2011 13:59:01" "r2c2 -1.2" "r2c3 $%^" \
		"r3c4 =sum( r1c1:r[-1]c[-1] )" \
		-o $FILE_11

echo $FILE_12
$VALG cedappend $FILE_11 $FILE_11 -col $FILE_11 -o $FILE_12

echo $FILE_13
$VALG cedpaste	$FILE_11 \
		-dest r8c8:r20c15 $FILE_11 \
		-dest r25c2 $FILE_11 \
		-dest r30c3:r35c8 -range r2c2:r_c_ $FILE_11 \
		-o $FILE_13


# Destructive

FILE_21=$OUT/21_clear.tsv
FILE_22=$OUT/22_cut.tsv
FILE_23=$OUT/23_insert.tsv

echo $FILE_21
$VALG cedclear -range r10c10:r17c13 $FILE_13 -o $FILE_21

echo $FILE_22
$VALG cedcut -col -start 5 -total 2 $FILE_21 -o $FILE_22

echo $FILE_23
$VALG cedinsert -start 13 -total 2 $FILE_22 -o $FILE_23


# Transformative

FILE_31=$OUT/31_flip.tsv
FILE_32=$OUT/32_sort.tsv
FILE_33=$OUT/33_transpose.tsv
FILE_34=$OUT/34_rotate.tsv

echo $FILE_31
$VALG cedflip $FILE_23 -o $FILE_31

echo $FILE_32
$VALG cedsort -i $FILE_31 -col -start 6 -total 7 "16,ac" -o $FILE_32

echo $FILE_33
$VALG cedtranspose $FILE_32 -o $FILE_33

echo $FILE_34
$VALG cedrotate $FILE_33 -o $FILE_34




# Reflective

FILE_41=$OUT/41_eval.tsv
FILE_42=$OUT/42_find.tsv
FILE_43=$OUT/43_ls.tsv

echo $FILE_41
$VALG cedeval -i $FILE_34 "sum( r1c1:r_c_ )" > $FILE_41

echo $FILE_42
$VALG cedfind -v -i $FILE_34 "qwert" > $FILE_42

echo $FILE_43
$VALG cedls -v $OUT/[1-3]*.tsv > $FILE_43





# Misc
FILE_51=$OUT/51_ensemble.tsv
FILE_52=$OUT/52_csv.csv
FILE_53=$OUT/53_tsv_output.tsv
FILE_54=$OUT/54_tsv_output_noq.tsv

echo $FILE_51
$VALG cedutils -i $FILE_34 -com set "r9c3 Hello World" "r9c4 -951.357" -com insert -start 4 "" -o $FILE_51

echo $FILE_52
$VALG cedutils -i $FILE_51 -otype csv_content -o $FILE_52

echo $FILE_53
$VALG cedutils -i $FILE_51 -otype output_tsv_q -o $FILE_53

echo $FILE_54
$VALG cedutils -i $FILE_51 -otype output_tsv -o $FILE_54




# Check results
echo
echo Comparing with checked results ...
echo

diff -Nurp $CHECK $OUT
