/*
	Copyright (C) 2008-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "gui.h"



void pressed_select_all (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		r,
			c;


	mtMAX ( r, 1, global.cedview->sheet_rows )
	mtMAX ( c, 1, global.cedview->sheet_cols )

	set_cursor_range ( 1, 1, r, c, 0, 0 );
}

void pressed_edit_cell (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	grab_focus_entry ();
}


static int calc_selection_row_extent (
	int		* row,
	int		* rowtot
	)
{
	CedSheet	* sheet = global.cedview->ren.sheet;


	if ( ! sheet )
	{
		return 1;
	}

	if ( sheet->prefs.cursor_r1 > sheet->prefs.cursor_r2 )
	{
		row[0] = sheet->prefs.cursor_r2;
		rowtot[0] = sheet->prefs.cursor_r1 - sheet->prefs.cursor_r2 + 1;
	}
	else
	{
		row[0] = sheet->prefs.cursor_r1;
		rowtot[0] = sheet->prefs.cursor_r2 - sheet->prefs.cursor_r1 + 1;
	}

	return 0;
}

static int calc_selection_col_extent (
	int		* col,
	int		* coltot
	)
{
	CedSheet	* sheet = global.cedview->ren.sheet;


	if ( ! sheet )
	{
		return 1;
	}

	if ( sheet->prefs.cursor_c1 > sheet->prefs.cursor_c2 )
	{
		col[0] = sheet->prefs.cursor_c2;
		coltot[0] = sheet->prefs.cursor_c1 - sheet->prefs.cursor_c2 + 1;
	}
	else
	{
		col[0] = sheet->prefs.cursor_c1;
		coltot[0] = sheet->prefs.cursor_c2 - sheet->prefs.cursor_c1 + 1;
	}

	return 0;
}


void pressed_row_insert (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		row,
			rowtot,
			res;


// We check here to tell user action is invalid before wasting any CPU/user time

	if ( undo_report_updates ( cuf_check_sheet_lock (
		global.cedview->ren.sheet ) )
		)
	{
		return;
	}

	if ( calc_selection_row_extent ( &row, &rowtot ) )
	{
		return;
	}

	res = cuf_sheet_insert_row ( global.file->cubook,
		global.cedview->ren.sheet, row, rowtot );
	undo_report_updates ( res );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return;
	}

	update_changes_chores ( 1, 0 );
}

void pressed_row_delete (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		row,
			rowtot,
			res;


// We check here to tell user action is invalid before wasting any CPU/user time

	if ( undo_report_updates ( cuf_check_sheet_lock (
		global.cedview->ren.sheet ) )
		)
	{
		return;
	}

	if ( calc_selection_row_extent ( &row, &rowtot ) )
	{
		return;
	}

	res = cuf_sheet_delete_row ( global.file->cubook,
		global.cedview->ren.sheet, row, rowtot );
	undo_report_updates ( res );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return;
	}

	update_changes_chores ( 1, 0 );
}

void pressed_column_insert (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		col,
			coltot,
			res;


// We check here to tell user action is invalid before wasting any CPU/user time

	if ( undo_report_updates ( cuf_check_sheet_lock (
		global.cedview->ren.sheet ) )
		)
	{
		return;
	}

	if ( calc_selection_col_extent ( &col, &coltot ) )
	{
		return;
	}

	res = cuf_sheet_insert_column ( global.file->cubook,
		global.cedview->ren.sheet, col, coltot );
	undo_report_updates ( res );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return;
	}

	update_splits ();
	update_changes_chores ( 1, 0 );
}

void pressed_column_delete (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		col,
			coltot,
			res;


// We check here to tell user action is invalid before wasting any CPU/user time

	if ( undo_report_updates ( cuf_check_sheet_lock (
		global.cedview->ren.sheet ) )
		)
	{
		return;
	}

	if ( calc_selection_col_extent ( &col, &coltot ) )
	{
		return;
	}

	res = cuf_sheet_delete_column ( global.file->cubook,
		global.cedview->ren.sheet, col, coltot );
	undo_report_updates ( res );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return;
	}

	update_splits ();
	update_changes_chores ( 1, 0 );
}



//	SIMPLE DIALOGS

static gboolean simple_dialog_close (
	GtkWidget	* widget,
	gpointer	data
	)
{
	gtk_widget_destroy ( GTK_WIDGET ( data ) );

	return TRUE;
}



//	COLUMN WIDTH DIALOG

static gboolean column_width_ok (
	GtkWidget	* widget,
	gpointer	data
	)
{
	GtkWidget	* window;
	int		c,
			ctot,
			w,
			res;
	CedSheet	* sheet;


	sheet = global.cedview->ren.sheet;

	if (	! widget	||
		! data		||
		! sheet
		)
	{
		return FALSE;
	}

	gtk_spin_button_update ( GTK_SPIN_BUTTON ( data ) );

	window = gtk_object_get_user_data ( GTK_OBJECT ( widget ) );
	w = gtk_spin_button_get_value_as_int ( GTK_SPIN_BUTTON ( data ) );

	c = MIN ( sheet->prefs.cursor_c1, sheet->prefs.cursor_c2 );
	ctot = MAX ( sheet->prefs.cursor_c1, sheet->prefs.cursor_c2 ) - c + 1;

	gtk_widget_destroy ( window );

	res = cuf_sheet_set_column_width ( global.file->cubook, sheet, c,
		ctot, w );
	undo_report_updates ( res );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return TRUE;
	}

	ced_view_set_split ( global.table, sheet->prefs.split_r1,
		sheet->prefs.split_r2, sheet->prefs.split_c1,
		sheet->prefs.split_c2 );

	update_changes_chores ( 1, 1 );

	return TRUE;
}

void pressed_column_width_auto (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		res,
			c,
			ctot;
	CedSheet	* sheet;


	sheet = global.cedview->ren.sheet;
	if ( ! sheet )
	{
		return;
	}

	c = MIN ( sheet->prefs.cursor_c1, sheet->prefs.cursor_c2 );
	ctot = MAX ( sheet->prefs.cursor_c1, sheet->prefs.cursor_c2 ) - c + 1;

	res = cuf_sheet_set_column_width_auto ( global.file->cubook, sheet, c,
		ctot );

	undo_report_updates ( res );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return;
	}

	ced_view_set_split ( global.table, sheet->prefs.split_r1,
		sheet->prefs.split_r2, sheet->prefs.split_c1,
		sheet->prefs.split_c2 );

	update_changes_chores ( 0, 1 );
}

void pressed_column_width (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	CedSheet	* sheet;
	CedCell		* cell;
	GtkWidget	* window,
			* spin,
			* button,
			* vbox,
			* hbox;
	GtkAccelGroup	* ag;
	int		w;


	sheet = global.cedview->ren.sheet;
	if ( ! sheet )
	{
		return;
	}

// We check here to tell user action is invalid before wasting any CPU/user time

	if ( undo_report_updates ( cuf_check_sheet_lock ( sheet ) ) )
	{
		return;
	}

	window = mtgex_add_a_window ( GTK_WINDOW_TOPLEVEL,
		_("Set Column Width"), GTK_WIN_POS_CENTER, TRUE );
	ag = gtk_accel_group_new ();

	vbox = gtk_vbox_new ( FALSE, 0 );
	gtk_widget_show ( vbox );
	gtk_container_add ( GTK_CONTAINER ( window ), vbox );

	spin = gtk_spin_button_new_with_range ( 0, 100, 1 );
	gtk_widget_show ( spin );
	gtk_box_pack_start ( GTK_BOX ( vbox ), spin, FALSE, FALSE, 5 );

	cell = ced_sheet_get_cell ( sheet, 0, sheet->prefs.cursor_c1 );

	if ( cell && cell->prefs )
	{
		w = cell->prefs->width;
	}
	else
	{
		w = 0;
	}

	gtk_spin_button_set_value ( GTK_SPIN_BUTTON ( spin ), w );

	mtgex_add_hseparator ( vbox, -2, 10 );

	hbox = gtk_hbox_new ( FALSE, 0 );
	gtk_widget_show ( hbox );
	gtk_box_pack_start ( GTK_BOX ( vbox ), hbox, FALSE, FALSE, 5 );

	button = gtk_button_new_with_label ( _("Close") );
	gtk_widget_set_usize ( button, 100, -2 );
	gtk_widget_show ( button );
	gtk_box_pack_start ( GTK_BOX ( hbox ), button, TRUE, TRUE, 5 );
	gtk_widget_add_accelerator ( button, "clicked", ag, GDK_Escape, 0,
		(GtkAccelFlags) 0 );
	g_signal_connect ( button, "clicked",
		G_CALLBACK ( simple_dialog_close ), (gpointer)window );

	button = gtk_button_new_with_label ( _("OK") );
	gtk_object_set_user_data ( GTK_OBJECT ( button ), (gpointer)window );
	gtk_widget_set_usize ( button, 100, -2 );
	gtk_widget_show ( button );
	gtk_box_pack_start ( GTK_BOX ( hbox ), button, TRUE, TRUE, 5 );
	gtk_widget_add_accelerator ( button, "clicked", ag, GDK_KP_Enter, 0,
		(GtkAccelFlags) 0 );
	gtk_widget_add_accelerator ( button, "clicked", ag, GDK_Return, 0,
		(GtkAccelFlags) 0 );
	g_signal_connect ( button, "clicked",
		G_CALLBACK ( column_width_ok ), (gpointer)spin );

	gtk_window_set_transient_for ( GTK_WINDOW ( window ),
		GTK_WINDOW ( global.main_window ) );
	gtk_widget_show ( window );
	gtk_window_add_accel_group ( GTK_WINDOW ( window ), ag );
}



enum
{
	CHANGE_author,
	CHANGE_comment,
	CHANGE_disable_locks,
	CHANGE_auto_recalc
};



static void book_pref_changed (
	mtPrefItemValue	* piv,
	int		callback_data
	)
{
	CedBook		* book;


	book = global.file->cubook->book;

	switch ( callback_data )
	{
	case CHANGE_author:
		mtkit_strfreeswap ( &book->prefs.author, piv->value );
		break;

	case CHANGE_comment:
		mtkit_strfreeswap ( &book->prefs.comment, piv->value );
		break;

	case CHANGE_disable_locks:
		mtkit_strtoi ( piv->value, &book->prefs.disable_locks, NULL,
			0 );
		update_entry_celltext ();
		break;

	case CHANGE_auto_recalc:
		mtkit_strtoi ( piv->value, &book->prefs.auto_recalc, NULL, 0 );
		break;

	default:
		return;
	}

	update_changes_chores ( 0, 0 );
}

static void book_prefs_closure (
	mtPrefItemValue	* piv,
	int		callback_data
	)
{
	mtPrefs		* prefs = (mtPrefs *)piv;


	save_pref_window_prefs ( prefs );
	mtkit_prefs_destroy ( prefs );
}

void pressed_edit_book_prefs (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	CedBook		* book = global.file->cubook->book;
	mtPrefs		* prefs;
	static mtPrefItem item_table[] = {
	{ CED_FILE_PREFS_BOOK_AUTHOR, MTKIT_PREF_TYPE_STR_MULTI, "", _("Book Author"), book_pref_changed, CHANGE_author },
	{ CED_FILE_PREFS_BOOK_COMMENT, MTKIT_PREF_TYPE_STR_MULTI, "", _("Book Comment"), book_pref_changed, CHANGE_comment },
	{ CED_FILE_PREFS_BOOK_DISABLE_LOCKS, MTKIT_PREF_TYPE_BOOL, "0", _("Disable all cell locks"), book_pref_changed, CHANGE_disable_locks },
	{ CED_FILE_PREFS_BOOK_AUTO_RECALC, MTKIT_PREF_TYPE_OPTION, "1", _("Automatically recalculate the sheet or book after changes"), book_pref_changed, CHANGE_auto_recalc, _("None	Sheet	Book") },
	{ NULL }
	};


	// Load window prefs from disk
	prefs = load_pref_window_prefs ( item_table );

	// Populate with current settings
	mtkit_prefs_block_callback ( prefs );
	mtkit_prefs_set_str ( prefs, CED_FILE_PREFS_BOOK_AUTHOR,
		book->prefs.author );
	mtkit_prefs_set_str ( prefs, CED_FILE_PREFS_BOOK_COMMENT,
		book->prefs.comment );
	mtkit_prefs_set_int ( prefs, CED_FILE_PREFS_BOOK_DISABLE_LOCKS,
		book->prefs.disable_locks );
	mtkit_prefs_set_int ( prefs, CED_FILE_PREFS_BOOK_AUTO_RECALC,
		book->prefs.auto_recalc );
	mtkit_prefs_unblock_callback ( prefs );

	mtgex_prefs_window ( prefs, global.main_window, _("Book Preferences"),
		book_prefs_closure, (mtPrefItemValue *)prefs, 0 );
}

void pressed_fix_2dyears (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		res,
			year = -1;


	mtkit_prefs_get_int ( global.prefs_file, CED_INIFILE_2DYEAR_START,
		&year );

	if ( year < 0 )
	{
		struct tm	* tm_now;
		time_t		now;


		now = time ( NULL );
		tm_now = localtime ( &now );

		year = 1900 + tm_now->tm_year - 50;
		if (	year < MTKIT_DDT_MIN_DATE_YEAR ||
			year > MTKIT_DDT_MAX_DATE_YEAR
			)
		{
			mtgex_alert_box ( _("Error"),
				_("Unexpected error in system date."),
				_("OK"), NULL, NULL, global.main_window );

			return;
		}
	}

	res = cuf_sheet_2dyear ( global.file, year );

	if ( res > 0 )
	{
		mtgex_alert_box ( _("Error"),
			_("Unable to fix years."),
			_("OK"), NULL, NULL, global.main_window );
	}

	undo_report_updates ( res );

	if (	res == CU_ERROR_LOCKED_CELL	||
		res == CU_ERROR_LOCKED_SHEET	||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return;			// Nothing changed
	}

	update_changes_chores ( 1, 0 );
}
