/*
	Copyright (C) 2008-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "gui.h"



static void error_message_file (
	char		* message,
	char		* filename
	)
{
	char		* msg,
			* f8;


	f8 = mtgex_gtkuncpy ( NULL, filename, 0 );
	msg = g_strdup_printf ( _("Unable to %s file: %s"), message, f8 );
	mtgex_alert_box ( _("Error"), msg, _("OK"), NULL, NULL,
		global.main_window );
	g_free ( msg );
	g_free ( f8 );
}



/*
	File operations - return 0 on success
*/

static void remember_last_dir (
	char		* filename
	)
{
	char		* c,
			c2;

	c = strrchr ( filename, MTKIT_DIR_SEP );
	if ( c )
	{
		c2 = c[0];
		c[0] = 0;		// Strip off filename
		mtkit_prefs_set_str ( global.prefs_file, CED_INIFILE_LAST_DIR,
			filename );
		c[0] = c2;
	}
}

static void register_project (
	char		* filename,
	int		change_project_filename
	)
{
	char		txt[64],
			txt1[64],
			* c;
	int		i,
			f;


	if ( ! filename )
	{
		return;
	}

	if (	change_project_filename &&
		global.file->name != filename
		)
	{
		mtkit_strfreeswap ( &global.file->name, filename );
	}

	remember_last_dir ( filename );
	update_titlebar ();

/*
Is it already in used file list?  If so shift relevant filenames down and put
at top.
*/
	i = 1;
	f = 0;

	while ( i < MAX_RECENT && f == 0 )
	{
		sprintf ( txt, CED_INIFILE_MAIN_FILE"%i", i );
		c = "";
		mtkit_prefs_get_str ( global.prefs_file, txt, &c );
		if ( strcmp ( filename, c ) == 0 )
		{
			f = 1;		// Filename found in list
		}
		else
		{
			i++;
		}
	}

	// If file is already most recent, do nothing
	if ( i > 1 )
	{
		while ( i > 1 )
		{
			sprintf ( txt, CED_INIFILE_MAIN_FILE"%i", i - 1 );
			sprintf ( txt1, CED_INIFILE_MAIN_FILE"%i", i );

			c = "";
			mtkit_prefs_get_str ( global.prefs_file, txt, &c );
			mtkit_prefs_set_str ( global.prefs_file, txt1, c );

			i--;
		}

		mtkit_prefs_set_str ( global.prefs_file,
			CED_INIFILE_MAIN_FILE"1", filename );
					// Strip off filename
	}

	update_recent_files ();
}

int project_load (
	char		* filename
	)
{
	CedSheet	* sheet;
	CedSheetPrefs	* tmp_sheet_prefs = NULL;


	sheet = global.cedview->ren.sheet;
	if ( sheet )
	{
		ced_view_set_sheet ( global.table, NULL );

		if ( ced_file_type_class ( global.file->type ) == 1 )
		{
			int		restore = 0;


			mtkit_prefs_get_int ( global.prefs_file,
				CED_INIFILE_SHEET_PREFS_PERSIST, &restore );

			if ( restore )
			{
				tmp_sheet_prefs = ced_sheet_prefs_new ();
				ced_sheet_prefs_copy ( tmp_sheet_prefs,
					&sheet->prefs );
			}
		}
	}

	if ( cuf_file_load ( global.file, filename ) )
	{
		ced_view_set_sheet ( global.table, sheet );
		error_message_file ( _("load project"), filename );
		ced_sheet_prefs_free ( tmp_sheet_prefs );

		return 1;
	}

	if ( tmp_sheet_prefs )
	{
		if ( ced_file_type_class ( global.file->type ) == 1 )
		{
			sheet = cuf_file_get_sheet ( global.file );
			if ( sheet )
			{
				ced_sheet_prefs_copy ( &sheet->prefs,
					tmp_sheet_prefs );
			}
		}

		ced_sheet_prefs_free ( tmp_sheet_prefs );
		tmp_sheet_prefs = NULL;
	}

	sheet_selector_populate ();
	graph_gui_update ( global.file->cubook->book->prefs.active_graph );

	if (	global.file->cubook->book->prefs.auto_recalc ||
		! ( global.file->type == CED_FILE_TYPE_TSV_VAL_BOOK ||
			global.file->type == CED_FILE_TYPE_LEDGER_VAL_BOOK )
		)
	{
		// Books with values don't need recalculating
		recalc_book_core ();
	}

	update_menus ();
	global.mem_changed = 0;
	register_project ( global.file->name, 1 );

	if ( global.file->cubook->book->prefs.disable_locks )
	{
		mtgex_alert_box ( _("Warning"),
			_("This book has the disable_locks flag set, "
			"which is potentially dangerous."),
			_("OK"), NULL, NULL, global.main_window );
	}

	return 0;
}

int project_import (
	char		* filename
	)
{
	CedUI_file	* uifile;
	int		sheet_tot,
			sheet_fail,
			file_tot,
			file_fail,
			res;
	char		buf[MTGEX_PATHBUF];


	uifile = cuf_file_new ();
	if ( ! uifile )
	{
		goto fail;
	}

	if (	cuf_file_load ( uifile, filename )	||
		! uifile->cubook			||
		! uifile->cubook->book
		)
	{
		cuf_file_free ( uifile );

		goto fail;
	}

	res = cuf_book_merge ( global.file->cubook, uifile->cubook->book,
		&sheet_tot, &sheet_fail, &file_tot, &file_fail );

	cuf_file_free ( uifile );

	undo_report_updates ( res );

	sheet_selector_populate ();	// Done here as need for success/fail

	// Update graph selector
	graph_gui_update ( global.file->cubook->book->prefs.active_graph );

	snprintf ( buf, MTGEX_PATHBUF,
		"%i sheets imported.  "
		"%i sheets not imported due to identical names.\n"
		"%i graphs/files imported.  "
		"%i graphs/files not imported due to identical names.",
		sheet_tot, sheet_fail, file_tot, file_fail );

	buf[MTGEX_PATHBUF - 1] = 0;
	mtgex_alert_box ( _("Information"), buf, _("OK"), NULL, NULL,
		global.main_window );

	if (	res == CU_ERROR_LOCKED_CELL	||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return 1;
	}

	if ( sheet_tot > 0 )
	{
		update_file_to_book ();
	}

	update_changes_chores ( 0, 1 );

	return 0;			// Success

fail:
	error_message_file ( _("load project (for import)"), filename );

	return 1;
}

static void report_large_tsv ( void )
{
	mtgex_alert_box ( _("Error"),
		_("The sheet geometry is too large to save in this file"
			" format."),
		_("OK"), NULL, NULL, global.main_window );
}


int project_save (
	char		* filename,
	int		filetype
	)
{
	int		res;


	if (	filetype == CED_FILE_TYPE_TSV_BOOK	||
		filetype == CED_FILE_TYPE_TSV_VAL_BOOK	||
		filetype == CED_FILE_TYPE_LEDGER_BOOK	||
		filetype == CED_FILE_TYPE_LEDGER_VAL_BOOK
		)
	{
		graph_gui_store_changes ();
	}

	res = cuf_file_save ( global.file, filename, filetype );
	if ( res == 1 )
	{
		report_large_tsv ();

		return 1;
	}

	if ( res )
	{
		goto error;
	}

	global.mem_changed = 0;
	register_project ( filename, 1 );

	return 0;

error:
	error_message_file ( _("save project"), filename );

	return 1;
}

int project_export (
	char		* filename,
	int		filetype
	)
{
	if (	filetype == CED_FILE_TYPE_TSV_BOOK	||
		filetype == CED_FILE_TYPE_TSV_VAL_BOOK	||
		filetype == CED_FILE_TYPE_LEDGER_BOOK	||
		filetype == CED_FILE_TYPE_LEDGER_VAL_BOOK
		)
	{
		int		res;
		CedBook		* book;
		CedSheet	* sheet;


		book = ced_book_new ();
		if ( ! book )
		{
			goto error;
		}

		sheet = ced_sheet_duplicate ( global.cedview->ren.sheet );
		if ( ! sheet )
		{
			ced_book_destroy ( book );

			goto error;
		}

		if ( ced_book_add_sheet ( book, sheet,
			global.cedview->ren.sheet->book_tnode->key )
			)
		{
			ced_sheet_destroy ( sheet );
			ced_book_destroy ( book );

			goto error;
		}

		res = ced_book_save ( book, filename, filetype );
		ced_book_destroy ( book );

		if ( res )
		{
			goto error;
		}
	}
	else
	{
		if ( ced_sheet_save ( global.cedview->ren.sheet, filename,
			filetype ) )
		{
			goto error;
		}
	}

	register_project ( filename, 0 );

	return 0;

error:
	error_message_file ( _("export sheet"), filename );

	return 1;
}

int project_export_output (
	char		* filename,
	int		filetype
	)
{
	if ( ced_export_output ( global.prefs_file, global.cedview->ren.sheet,
		filename, global.file->name, filetype ) )
	{
		error_message_file ( _("export sheet output"), filename );

		return 1;
	}

	remember_last_dir ( filename );

	return 0;
}

int project_export_graph (
	char		* filename,
	int		filetype
	)
{
	double		scale = 1;


	mtkit_prefs_get_double ( global.prefs_file, CED_INIFILE_GRAPH_SCALE,
		&scale );

	if ( ced_graph_render_file ( global.file->cubook->book,
		global.file->cubook->book->prefs.active_graph,
		filename, filetype, NULL, scale ) )
	{
		error_message_file ( _("export graph"), filename );

		return 1;
	}

	remember_last_dir ( filename );

	return 0;
}

void pressed_new_project (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		i;


	i = check_for_changes ();

	if ( i != -10 && i != 2 )
	{
		return;
	}

	clear_all ();
}

void pressed_open_project (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		i;


	i = check_for_changes ();
	if ( i != -10 && i != 2 )
	{
		return;
	}

	file_selector ( FS_LOAD_PROJECT, global.main_window, global.prefs_file,
		global.file->name );
}

void pressed_save_project_as (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	file_selector ( FS_SAVE_PROJECT, global.main_window, global.prefs_file,
		global.file->name );
}

void pressed_save_project (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	if (	! global.file->name		||
		global.file->name[0] == 0
		)
	{
		file_selector ( FS_SAVE_PROJECT, global.main_window,
			global.prefs_file, global.file->name );
	}
	else
	{
		project_save ( global.file->name, global.file->type );
	}
}

void pressed_recent (
	GtkMenuItem	* menu_item,
	gpointer	user_data,
	gint		item
	)
{
	int		i;
	char		txt[64],
			* c;


	i = check_for_changes ();
	if ( i != -10 && i != 2 )
	{
		return;
	}

	sprintf ( txt, CED_INIFILE_MAIN_FILE"%i", item );
	c = ".";
	mtkit_prefs_get_str ( global.prefs_file, txt, &c );
	project_load ( c );
}

void pressed_sheet_export (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	if ( ! global.cedview->ren.sheet )
	{
		return;			// Nothing to export
	}

	file_selector ( FS_EXPORT_SHEET, global.main_window, global.prefs_file,
		NULL );
}

void pressed_sheet_export_output (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	if ( ! global.cedview->ren.sheet )
	{
		return;			// Nothing to export
	}

	if ( cuf_sheet_check_geometry ( global.cedview->ren.sheet ) )
	{
		report_large_tsv ();

		return;
	}

	file_selector ( FS_EXPORT_SHEET_OUTPUT, global.main_window,
		global.prefs_file, NULL );
}

void pressed_import_project (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	file_selector ( FS_IMPORT_PROJECT, global.main_window,
		global.prefs_file, NULL );
}
