/*
	Copyright (C) 2011-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "gui.h"



enum
{
	GR_MENU_TOP,

	GR_MENU_DUPLICATE,
	GR_MENU_RENAME,
	GR_MENU_DELETE,
	GR_MENU_REDRAW,
	GR_MENU_EXPORT,
	GR_MENU_SELECTION,

	GR_MENU_TOTAL
};



static int		graph_total;
static GtkWidget	* graph_bmenu,
			* graph_image,
			* graph_menu_widgets[GR_MENU_TOTAL],
			* gredit_view;
static GtkTextBuffer	* gredit_buffer;

// WARNING - This string should not get anywhere near 2048 bytes
static char		* graph_text_default =
"/*\n"
"	mtCellEdit Graph Template\n"
"\n"
"	Created		%i-%i-%i %02i:%02i:%02i\n"
"*/\n"
"\n"
"\n"
"{ page		width=\"640\"	height=\"320\"\n"
"			line_color=\"-1\"	fill_color=\"-1\"\n"
"}\n"
"\n"
"{ graph		fill_color=\"0xffffff\"	line_color=\"0\"\n"
"			x_pad=\"10\"	y_pad=\"5\"\n"
"			antialias=\"0\"\n"
"}\n"
"{ plot		fill_color=\"0xdddddd\"\n"
"			x_pad=\"5\"	y_pad=\"5\"	line_color=\"0\"\n"
"			antialias=\"0\"\n"
"}\n"
"\n"
"\n"
"{ x_axis		min=\"0\"		max=\"100\"	}\n"
"{ y_axis		min=\"10\"		max=\"-10\"	}\n"
"\n"
"\n"
"{ plot_x_axis_grid	gap=\"1\"	line_color=\"rgb(200,200,200)\"	antialias=\"0\"	}\n"
"{ plot_y_axis_grid	gap=\"2\"	line_color=\"rgb(200,200,200)\"	antialias=\"0\"	}\n"
"\n"
"{ plot_x_axis_grid	gap=\"5\"	line_color=\"rgb(150,150,150)\"	antialias=\"0\"	}\n"
"{ plot_y_axis_grid	gap=\"10\"	line_color=\"rgb(150,150,150)\"	antialias=\"0\"	}\n"
"\n"
"\n"
"{ plot_x_axis_top	text=\"Graph Title\"	text_size=\"20\"\n"
"				x_justify=\"0.5\"		y_justify=\"0.5\"	y_pad=\"10\"\n"
"}\n"
"\n"
"{ plot_x_axis	size=\"5\"	gap=\"10\"	text_size=\"10\"\n"
"			x_justify=\"0\"		y_pad=\"5\"\n"
"			antialias=\"0\"\n"
"}\n"
"{ plot_y_axis	size=\"5\"	gap=\"2\"	text_size=\"10\"\n"
"			y_justify=\"0\"		x_pad=\"5\"\n"
"			antialias=\"0\"\n"
"}\n"
"\n"
"{ plot_x_axis	text=\"Input\"		text_size=\"14\"\n"
"			y_pad=\"0\"\n"
"}\n"
"{ plot_y_axis	text=\"Output\"		text_size=\"14\"\n"
"			x_pad=\"0\"\n"
"}\n"
"\n"
"\n"
;


static void redraw_graph ( void )
{
	int		err	= -1;
	mtImage		* im;
	double		scale	= 1;
	CedBook		* book;


	mtkit_prefs_get_double ( global.prefs_file, CED_INIFILE_GRAPH_SCALE,
		&scale );

	book = global.file->cubook->book;
	im = ced_graph_render_mtimage ( book, book->prefs.active_graph, &err,
		scale );

	mtgex_image_set_image ( graph_image, im, 1 );

	if ( err >= 0 )
	{
		GtkTextIter	start, end;


		gtk_text_buffer_get_start_iter ( gredit_buffer, &start );
		gtk_text_buffer_get_end_iter ( gredit_buffer, &end );
		gtk_text_iter_set_offset ( &start, err );

		gtk_text_buffer_select_range ( gredit_buffer, &start, &end );
	}
}

void graph_gui_store_changes ( void )
{
	// Checks for changes, if so save them to the current graph
	// (active_graph).

	if ( gtk_text_buffer_get_modified ( gredit_buffer ) )
	{
		char		* txt = NULL;
		GtkTextIter	start,
				end;
		CedBookFile	* bookfile;


		bookfile = cuf_file_get_graph ( global.file );
		if ( ! bookfile )
		{
			return;
		}

		gtk_text_buffer_get_start_iter ( gredit_buffer, &start );
		gtk_text_buffer_get_end_iter ( gredit_buffer, &end );
		txt = gtk_text_buffer_get_text ( gredit_buffer, &start, &end,
			FALSE );

		// Free old memory and use newly allocated text
		free ( bookfile->mem );
		bookfile->mem = txt;
		bookfile->size = strlen ( txt );
		ced_book_timestamp_file ( bookfile );

		update_changes_chores ( 0, 1 );
		gtk_text_buffer_set_modified ( gredit_buffer, FALSE );

		redraw_graph ();
	}
}

static void graph_bmenu_changed (
	GtkWidget	* widget,
	void		* user_data,
	int		new_value
	)
{
	char		* txt = "",
			* graph_name;
	int		txt_len = 0;
	CedBookFile	* bookfile;
	GtkTextIter	iter;
	CedBook		* book;


	graph_gui_store_changes ();

	graph_name = mtgex_bmenu_get_value_text ( widget, new_value );

	book = global.file->cubook->book;
	mtkit_strfreeswap ( &book->prefs.active_graph, graph_name );

	redraw_graph ();		// Draw the new graph

	bookfile = ced_graph_get ( book, graph_name );
	if ( bookfile && bookfile->mem )
	{
		txt = bookfile->mem;
		txt_len = bookfile->size;
	}

	gtk_text_buffer_set_text ( GTK_TEXT_BUFFER ( gredit_buffer ), txt,
		txt_len );
	gtk_text_buffer_set_modified ( gredit_buffer, FALSE );

	// Move the cursor back to the top
	gtk_text_buffer_get_start_iter ( gredit_buffer, &iter );
	gtk_text_buffer_place_cursor ( gredit_buffer, &iter );
}

void show_hide_graphname ( void )
{
	int		graph_visible;


	graph_visible = GTK_WIDGET_VISIBLE ( global.pane_graph );

	if ( graph_visible )
	{
		gtk_widget_show ( graph_menu_widgets[GR_MENU_TOP] );
	}
	else
	{
		gtk_widget_hide ( graph_menu_widgets[GR_MENU_TOP] );
	}

	if ( graph_total > 0 && graph_visible )
	{
		gtk_widget_show ( graph_bmenu );
		gtk_widget_set_sensitive ( global.pane_graph, TRUE );
	}
	else
	{
		gtk_widget_hide ( graph_bmenu );
		gtk_widget_set_sensitive ( global.pane_graph, FALSE );
	}
}

void pressed_graph (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	if ( GTK_WIDGET_VISIBLE ( global.pane_graph ) )
	{
		pane_pair_set ( PANE_PAIR_HIDE_GRAPH );
	}
	else
	{
		pane_pair_set ( PANE_PAIR_SHOW_GRAPH );
	}
}

static int populate_graph_list_cb (
	CedBook		* book,
	char		* graph_name,
	CedBookFile	* bookfile,
	void		* user_data
	)
{
	mtgex_bmenu_add_item ( graph_bmenu, graph_name );

	graph_total ++;

	return 0;			// Continue
}

void graph_gui_update (
	char		* graph_name
	)
{
	int		i;

/*
NOTE: this function is called after loading a new book/sheet so we can't save
data at this point as we might be storing an old graph script to a new file
with the same graph name.
*/

	// Clear the bmenu, remove the current graph and its text
	mtgex_bmenu_add_item ( graph_bmenu, NULL );
	mtgex_image_set_image ( graph_image, NULL, 1 );
	gtk_text_buffer_set_text ( GTK_TEXT_BUFFER ( gredit_buffer  ), "", 0 );
	gtk_text_buffer_set_modified ( gredit_buffer, FALSE );
	graph_total = 0;

	// Turn off callbacks for now
	mtgex_bmenu_set_callback ( graph_bmenu, NULL, NULL );

	ced_graph_scan ( global.file->cubook->book, populate_graph_list_cb,
		NULL );

	// Turn on callbacks again
	mtgex_bmenu_set_callback ( graph_bmenu, graph_bmenu_changed, NULL );

	if ( graph_total > 0 )
	{
		int		item = 0;


		if ( graph_name )
		{
			item = mtgex_bmenu_get_value_from_text ( graph_bmenu,
				graph_name );

			if ( item < 0 )
			{
				// Not on list, so select first
				item = 0;
			}
		}

		mtgex_bmenu_set_value ( graph_bmenu, item );
	}

	for ( i = GR_MENU_TOP + 1; i < GR_MENU_TOTAL; i++ )
	{
		gtk_widget_set_sensitive ( graph_menu_widgets[i],
			graph_total ? TRUE : FALSE );
	}

	show_hide_graphname ();
}



#define MAX_GRAPH_ID		10000



static void pressed_graph_new (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	char		graph_name[256],
			* text,
			txt[2048];
	int		i;
	time_t		now;
	struct tm	* now_tm;
	CedBook		* book;


	now = time ( NULL );
	now_tm = localtime ( &now );
	book = global.file->cubook->book;

	for ( i = 1; i < MAX_GRAPH_ID; i++ )
	{
		sprintf ( graph_name, "Graph %i", i );
		if ( ! ced_graph_get ( book, graph_name ) )
		{
			break;
		}

		// Keep looping until we find an unused name
	}

	graph_gui_store_changes ();

	sprintf ( txt, graph_text_default, now_tm->tm_year + 1900,
		now_tm->tm_mon + 1, now_tm->tm_mday, now_tm->tm_hour,
		now_tm->tm_min, now_tm->tm_sec );

	text = strdup ( txt );

	if (	! text			||
		i == MAX_GRAPH_ID	||
		ced_graph_new ( book, text, strlen ( text ), graph_name ) ==
			NULL
		)
	{
		free ( text );
		mtgex_alert_box ( _("Error"),
			_("Unable to create a new graph."), _("OK"),
			NULL, NULL, global.main_window );

		return;
	}

	mtkit_strfreeswap ( &book->prefs.active_graph, graph_name );
	graph_gui_update ( graph_name );

	update_changes_chores ( 0, 1 );
}

static void pressed_graph_duplicate (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	char		* name,
			* newname = NULL;
	int		i;


	i = mtgex_bmenu_get_value ( graph_bmenu );
	if ( i < 0 )
	{
		goto error;
	}

	name = mtgex_bmenu_get_value_text ( graph_bmenu, i );
	if ( ! name )
	{
		goto error;
	}

	graph_gui_store_changes ();

	if ( ced_graph_duplicate ( global.file->cubook, name, &newname ) )
	{
		goto error;
	}

	graph_gui_update ( newname );
	free ( newname );

	update_changes_chores ( 0, 1 );

	return;				// Success

error:
	mtgex_alert_box( _("Error"),
		_("Unable to duplicate the current graph."),
		_("OK"), NULL, NULL, global.main_window );
}

static int rename_graph (
	char		* old_name,
	char		* new_name
	)
{
	char		* msg = NULL;
	CedBookFile	* old;
	CedBook		* book = global.file->cubook->book;


	if ( ! new_name || new_name[0] == 0 )
	{
		msg = _("Bad graph name");
	}
	else if ( ced_graph_get ( book, new_name ) )
	{
		msg = _("Graph name already exists");
	}

	if ( msg )
	{
		mtgex_alert_box ( _("Error"), msg, _("OK"), NULL, NULL,
			global.main_window );

		return 1;
	}

	old = ced_graph_get ( book, old_name );
	if (	! old ||
		! ced_graph_new ( book, old->mem, old->size, new_name )
		)
	{
		msg = _("Unable to rename graph");
		mtgex_alert_box ( _("Error"), msg, _("OK"), NULL, NULL,
			global.main_window );

		return 1;
	}
	else
	{
		old->mem = NULL;
		ced_graph_destroy ( book, old_name );
	}

	graph_gui_update ( new_name );
	update_changes_chores ( 0, 1 );

	return 0;
}

static void pressed_graph_rename (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		res;
	char		* old_name,
			* new_name;


	old_name = global.file->cubook->book->prefs.active_graph;

	if ( ! old_name )
	{
		return;
	}

	graph_gui_store_changes ();

	for ( res = 1; res; )
	{
		new_name = dialog_text_entry ( _("Rename Graph"), old_name );

		if ( new_name )
		{
			res = rename_graph ( old_name, new_name );
			free ( new_name );
		}
		else
		{
			res = 0;
		}
	}
}

static void pressed_graph_delete (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	char		* graph_name;
	int		i,
			res;
	CedBook		* book;


	res = mtgex_alert_box ( _("Question"),
		_("Do you really want to delete this graph?"),
		_("No"), _("Yes"), NULL, global.main_window );

	if ( res != 2 )
	{
		return;
	}

	i = mtgex_bmenu_get_value ( graph_bmenu ) - 1;
	if ( i < 0 )
	{
		i = 0;
	}

	book = global.file->cubook->book;
	if ( ced_graph_destroy ( book, book->prefs.active_graph ) )
	{
		mtgex_alert_box ( _("Error"), _("Unable to delete this graph"),
			_("OK"), NULL, NULL, global.main_window );

		return;
	}

	graph_name = mtgex_bmenu_get_value_text ( graph_bmenu, i );
	if ( graph_name )
	{
		graph_name = strdup ( graph_name );

/*
		NOTE: we need to strdup here because the original
		graph_name is free'd in graph_gui_update ()
*/
	}

	graph_gui_update ( graph_name );
	update_changes_chores ( 0, 1 );
	free ( graph_name );
}

void pressed_graph_redraw (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	if ( gtk_text_buffer_get_modified ( gredit_buffer ) )
	{
		graph_gui_store_changes ();
	}
	else
	{
		redraw_graph ();
	}
}

static void pressed_graph_export (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	file_selector ( FS_EXPORT_GRAPH, global.main_window,
		global.prefs_file, NULL );
}



#define TXT_SIZE 2000

static void pressed_graph_plot_sel (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	GtkClipboard	* clip;
	char		txt[TXT_SIZE] = { 0 };


	if (	global.cedview->ren.sheet &&
		global.cedview->ren.sheet->book_tnode->key
		)
	{
		int		r1,
				c1,
				r2,
				c2;
		char		sheetname[TXT_SIZE],
				* smax,
				* dest,
				* src,
				cellrange[TXT_SIZE] = {0},
				* template =
"{ plot_graph_bar	fill_color=\"0x9999FF\"	gap=\"1\"\n"
"				sheet=\"%s\"		data=\"%s\"\n"
"				antialias=\"0\"\n"
"}\n";


		// Convert sheet name to valid utree string
		src = global.cedview->ren.sheet->book_tnode->key;
		smax = sheetname + TXT_SIZE - 3;

		for ( dest = sheetname; src[0] != 0 && dest < smax; dest ++ )
		{
			if (	src[0] == '\\' ||
				src[0] == '"'
				)
			{
				*dest++ = '\\';
			}

			dest[0] = *src++;
		}

		dest[0] = 0;

		ced_sheet_cursor_max_min ( global.cedview->ren.sheet, &r1, &c1,
			&r2, &c2 );

		snprintf ( cellrange, TXT_SIZE, "r%ic%i:r%ic%i", r1, c1, r2,
			c2 );
		cellrange[TXT_SIZE - 1] = 0;

		snprintf ( txt, TXT_SIZE, template, sheetname, cellrange );
		txt[TXT_SIZE - 1] = 0;
	}
	else
	{
		sprintf ( txt, _("No sheet available") );
	}

	clip = gtk_clipboard_get ( GDK_SELECTION_CLIPBOARD );
	gtk_clipboard_set_text ( clip, txt, strlen ( txt ) );
}

void graph_gui_build (
	GtkWidget	* vbox,
	GtkWidget	* hb,
	GtkItemFactory	* itemfactory
	)
{
	int		i;
	char		* menu_text[GR_MENU_TOTAL] = {
			_("/Graph"),
			_("/Graph/Duplicate"),
			_("/Graph/Rename ..."),
			_("/Graph/Delete ..."),
			_("/Graph/Redraw"),
			_("/Graph/Export ..."),
			_("/Graph/Sheet Selection to Clipboard")
			};
	GtkWidget	* pane,
			* sw;
	GtkItemFactoryEntry menu_items[] = {
{ _("/_Graph"),			NULL, NULL,0, "<Branch>" },
{ _("/Graph/tear"),		NULL, NULL,0, "<Tearoff>" },
{ _("/Graph/New"),		"", pressed_graph_new,0, "<StockItem>", GTK_STOCK_NEW },
{ menu_text[GR_MENU_DUPLICATE],	"", pressed_graph_duplicate, 0, "<StockItem>", GTK_STOCK_COPY },
{ _("/Graph/sep1"),		NULL, NULL,0, "<Separator>" },
{ menu_text[GR_MENU_RENAME],	NULL, pressed_graph_rename,0, "<StockItem>", GTK_STOCK_EDIT },
{ menu_text[GR_MENU_DELETE],	NULL, pressed_graph_delete,0, "<StockItem>", GTK_STOCK_DELETE },
{ _("/Graph/sep2"),		NULL, NULL,0, "<Separator>" },
{ menu_text[GR_MENU_REDRAW],	"<control>F5", pressed_graph_redraw,0, "<StockItem>", GTK_STOCK_REFRESH },
{ menu_text[GR_MENU_EXPORT],	NULL, pressed_graph_export,0, "<StockItem>", GTK_STOCK_SAVE_AS },
{ menu_text[GR_MENU_SELECTION],	NULL, pressed_graph_plot_sel }
	};


///	MENU

	gtk_item_factory_create_items_ac ( itemfactory,
		sizeof ( menu_items ) / sizeof ( ( menu_items[0] ) ),
		menu_items, NULL, 2 );

	for ( i = 0; i < GR_MENU_TOTAL; i++ )
	{
		graph_menu_widgets[i] = gtk_item_factory_get_item (
			itemfactory, menu_text[i] );
	}

	graph_bmenu = mtgex_bmenu_new ();
	gtk_box_pack_start ( GTK_BOX ( hb ), graph_bmenu, TRUE, FALSE, 0 );
	mtgex_bmenu_set_callback ( graph_bmenu, graph_bmenu_changed, NULL );

	g_signal_connect ( G_OBJECT ( vbox ), "key_press_event",
		G_CALLBACK ( key_event_escape ), NULL );

	pane = gtk_vpaned_new ();
	gtk_widget_show ( pane );
	gtk_box_pack_start ( GTK_BOX ( vbox ), pane, TRUE, TRUE, 0 );

	graph_image = mtgex_image_new ();
	gtk_paned_pack1 ( GTK_PANED ( pane ), graph_image, FALSE, TRUE );

	sw = gtk_scrolled_window_new ( NULL, NULL );
	gtk_widget_show ( sw );
	gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW ( sw ),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );

	gtk_paned_pack2 ( GTK_PANED ( pane ), sw, FALSE, TRUE );

	gredit_view = gtk_text_view_new ();
	gtk_widget_show ( gredit_view );
	gtk_text_view_set_wrap_mode ( GTK_TEXT_VIEW ( gredit_view ),
		GTK_WRAP_WORD );
	gtk_container_add ( GTK_CONTAINER ( sw ), gredit_view );

	gredit_buffer = gtk_text_view_get_buffer (
		GTK_TEXT_VIEW ( gredit_view ) );

	gtk_widget_hide ( graph_menu_widgets[GR_MENU_TOP] );
}
