/*
	Copyright (C) 2008-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "gui.h"


static int project_set_font (
	char		* filename,
	int		size
	)
{
	mtFont		* newfont;
	CedSheet	* sheet;


	newfont = mtpixel_font_new ( filename, filename ? 0 : size );
	if ( ! newfont )
	{
		char		* msg,
				* f8;


		f8 = mtgex_gtkuncpy ( NULL, filename, 0 );
		msg = g_strdup_printf (
			_("Unable to open font file %s size %i."), f8, size );

		mtgex_alert_box ( _("Error"), msg, _("OK"), NULL, NULL,
			global.main_window );

		g_free ( msg );
		g_free ( f8 );

		return 1;
	}

	if ( filename )
	{
		mtpixel_font_set_size ( newfont, size );
	}

	ced_view_set_font ( global.table, newfont );
	mtpixel_font_destroy ( global.font );
	global.font = newfont;

	sheet = global.cedview->ren.sheet;
	if ( sheet )
	{
		ced_view_set_split ( global.table,
			sheet->prefs.split_r1, sheet->prefs.split_r2,
			sheet->prefs.split_c1, sheet->prefs.split_c2 );
	}

	return 0;
}

void pref_change_font (
	mtPrefItemValue	* piv,
	int		callback_data )
{
	char		* filename = NULL;
	int		size,
			type;


	if ( mtkit_prefs_get_int ( global.prefs_file, CED_INIFILE_FONT_TYPE,
		&type ) )
	{
		return;
	}

	switch ( type )
	{
	case 0:
		size = 8;
		break;

	case 2:
		if ( mtkit_prefs_get_str ( global.prefs_file,
			CED_INIFILE_FONT_FILENAME, &filename ) )
		{
			return;
		}

		if ( mtkit_prefs_get_int ( global.prefs_file,
			CED_INIFILE_FONT_SIZE, &size ) )
		{
			return;
		}
		break;

	default:
		size = 16;
		break;
	}

	if ( project_set_font ( filename, size ) )
	{
		project_set_font ( NULL, 16 );
	}
}

void pref_init_row_pad ( void )
{
	int		size = 0;
	CedSheet	* sheet;


	mtkit_prefs_get_int ( global.prefs_file, CED_INIFILE_ROW_PAD, &size );

	global.cedview->ren.row_pad = size;

	sheet = global.cedview->ren.sheet;
	if ( sheet )
	{
		ced_view_set_split ( global.table,
			sheet->prefs.split_r1, sheet->prefs.split_r2,
			sheet->prefs.split_c1, sheet->prefs.split_c2 );
	}

	ced_view_redraw_view ( global.cedview );
}

static void pref_change_row_pad (
	mtPrefItemValue	* piv,
	int		callback_data
	)
{
	pref_init_row_pad ();
}

static void pref_change_graph_scale (
	mtPrefItemValue	* piv,
	int		callback_data )
{
	pressed_graph_redraw ( NULL, NULL );
}

static void pref_change_recent_filename_len (
	mtPrefItemValue	* piv,
	int		callback_data
	)
{
	update_recent_files ();
}



static mtPrefItem	prefs_table[] = {

{ CED_INIFILE_LAST_DIR,	MTKIT_PREF_TYPE_DIR },

{ CED_INIFILE_MAIN_WINDOW"_state", MTKIT_PREF_TYPE_INT, "0" },
{ CED_INIFILE_MAIN_WINDOW"_x", MTKIT_PREF_TYPE_INT, "0" },
{ CED_INIFILE_MAIN_WINDOW"_y", MTKIT_PREF_TYPE_INT, "0" },
{ CED_INIFILE_MAIN_WINDOW"_w", MTKIT_PREF_TYPE_INT, "400" },
{ CED_INIFILE_MAIN_WINDOW"_h", MTKIT_PREF_TYPE_INT, "400" },

{ CED_INIFILE_SHARED_WINDOW"_x", MTKIT_PREF_TYPE_INT, "0" },
{ CED_INIFILE_SHARED_WINDOW"_y", MTKIT_PREF_TYPE_INT, "0" },
{ CED_INIFILE_SHARED_WINDOW"_w", MTKIT_PREF_TYPE_INT, "400" },
{ CED_INIFILE_SHARED_WINDOW"_h", MTKIT_PREF_TYPE_INT, "400" },

{ CED_INIFILE_HELP_FILE, MTKIT_PREF_TYPE_FILE, "/usr/share/doc/mtcelledit/en_GB/chap_00.html" },
{ CED_INIFILE_HELP_BROWSER, MTKIT_PREF_TYPE_STR, "" },

{ CED_INIFILE_MAIN_FILE"1", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"2", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"3", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"4", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"5", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"6", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"7", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"8", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"9", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"10",MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"11", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"12", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"13", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"14", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"15", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"16", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"17", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"18", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"19", MTKIT_PREF_TYPE_FILE },
{ CED_INIFILE_MAIN_FILE"20",MTKIT_PREF_TYPE_FILE },

{ CED_INIFILE_FONT_TYPE, MTKIT_PREF_TYPE_OPTION, "1", NULL, pref_change_font, 0, "8x8	8x16	Custom" },
{ CED_INIFILE_FONT_FILENAME, MTKIT_PREF_TYPE_FILE, NULL, NULL, pref_change_font },
{ CED_INIFILE_FONT_SIZE,	MTKIT_PREF_TYPE_INT, "16", NULL, pref_change_font, 0, "0	128" },

{ CED_INIFILE_ROW_PAD,	MTKIT_PREF_TYPE_INT, "0", NULL, pref_change_row_pad, 0, "0	4" },

{ CED_INIFILE_RECENT_FILENAME_LEN,	MTKIT_PREF_TYPE_INT, "80", NULL, pref_change_recent_filename_len, 0, "50	500" },

{ CED_INIFILE_2DYEAR_START,	MTKIT_PREF_TYPE_INT, "-1", "Resets two digit years to between this and 99 years later.  -1 = 50 years ago to 49 years time.", NULL, 0, "-1	5879510" },

{ CED_INIFILE_GRAPH_PANE_POS,	MTKIT_PREF_TYPE_INT, "-1" },
{ CED_INIFILE_FIND_PANE_POS,	MTKIT_PREF_TYPE_INT, "-1" },
{ CED_INIFILE_FIND_WILCARDS,	MTKIT_PREF_TYPE_BOOL, "0" },
{ CED_INIFILE_FIND_CASE_SENSITIVE, MTKIT_PREF_TYPE_BOOL, "0" },
{ CED_INIFILE_FIND_VALUE,	MTKIT_PREF_TYPE_BOOL, "0" },
{ CED_INIFILE_FIND_ALL_SHEETS,	MTKIT_PREF_TYPE_BOOL, "0" },

{ CED_INIFILE_FIND_TEXT,	MTKIT_PREF_TYPE_STR, "" },

{ CED_INIFILE_PAGE_FONT_SIZE,	MTKIT_PREF_TYPE_INT, "12" },
{ CED_INIFILE_PAGE_WIDTH,	MTKIT_PREF_TYPE_INT, "297" },
{ CED_INIFILE_PAGE_HEIGHT,	MTKIT_PREF_TYPE_INT, "210" },
{ CED_INIFILE_PAGE_MARGIN_X,	MTKIT_PREF_TYPE_INT, "10" },
{ CED_INIFILE_PAGE_MARGIN_Y,	MTKIT_PREF_TYPE_INT, "10" },

{ CED_INIFILE_PAGE_HEADER_LEFT, MTKIT_PREF_TYPE_OPTION, "2", NULL, NULL, 0, CED_HEADER_OPTIONS },
{ CED_INIFILE_PAGE_HEADER_CENTRE, MTKIT_PREF_TYPE_OPTION, "0", NULL, NULL, 0, CED_HEADER_OPTIONS },
{ CED_INIFILE_PAGE_HEADER_RIGHT, MTKIT_PREF_TYPE_OPTION, "3", NULL, NULL, 0, CED_HEADER_OPTIONS },
{ CED_INIFILE_PAGE_FOOTER_LEFT, MTKIT_PREF_TYPE_OPTION, "6", NULL, NULL, 0, CED_HEADER_OPTIONS },
{ CED_INIFILE_PAGE_FOOTER_CENTRE, MTKIT_PREF_TYPE_OPTION, "0", NULL, NULL, 0, CED_HEADER_OPTIONS },
{ CED_INIFILE_PAGE_FOOTER_RIGHT, MTKIT_PREF_TYPE_OPTION, "4", NULL, NULL, 0, CED_HEADER_OPTIONS },

{ CED_INIFILE_GRAPH_SCALE,	MTKIT_PREF_TYPE_DOUBLE, "1", NULL, pref_change_graph_scale, 0, "0.1	1000" },

{ CED_INIFILE_SHEET_PREFS_PERSIST, MTKIT_PREF_TYPE_BOOL, "0", "Keep sheet preferences (cursor area, frozen panes) when loading a new text file after a previous text file" },

{NULL}
};



#define PREFS_NAME_SIZE		2048



static char	st_prefs_file[PREFS_NAME_SIZE];



static mtPrefTrans
	winprefs_main_main[] = {
		{ CED_INIFILE_PREFS_WINDOW"_x", CED_INIFILE_PREFS_WINDOW"_x" },
		{ CED_INIFILE_PREFS_WINDOW"_y", CED_INIFILE_PREFS_WINDOW"_y" },
		{ CED_INIFILE_PREFS_WINDOW"_w", CED_INIFILE_PREFS_WINDOW"_w" },
		{ CED_INIFILE_PREFS_WINDOW"_h", CED_INIFILE_PREFS_WINDOW"_h" },
		{ NULL }
		},
	winprefs_shared_main[] = {
		{ CED_INIFILE_SHARED_WINDOW"_x", CED_INIFILE_PREFS_WINDOW"_x" },
		{ CED_INIFILE_SHARED_WINDOW"_y", CED_INIFILE_PREFS_WINDOW"_y" },
		{ CED_INIFILE_SHARED_WINDOW"_w", CED_INIFILE_PREFS_WINDOW"_w" },
		{ CED_INIFILE_SHARED_WINDOW"_h", CED_INIFILE_PREFS_WINDOW"_h" },
		{ NULL }
		},
	winprefs_main_shared[] = {
		{ CED_INIFILE_PREFS_WINDOW"_x", CED_INIFILE_SHARED_WINDOW"_x" },
		{ CED_INIFILE_PREFS_WINDOW"_y", CED_INIFILE_SHARED_WINDOW"_y" },
		{ CED_INIFILE_PREFS_WINDOW"_w", CED_INIFILE_SHARED_WINDOW"_w" },
		{ CED_INIFILE_PREFS_WINDOW"_h", CED_INIFILE_SHARED_WINDOW"_h" },
		{ NULL }
		}
	;



void save_pref_window_prefs (
	mtPrefs		* prefs
	)
{
	mtkit_prefs_value_copy ( global.prefs_file, prefs, winprefs_shared_main
		);
	mtkit_prefs_value_copy ( prefs, global.prefs_file, winprefs_main_main );
	mtgex_prefs_window_mirror_prefs ( global.prefs_file, prefs );
}

mtPrefs * load_pref_window_prefs (
	mtPrefItem	* table
	)
{
	mtPrefs		* prefs;


	prefs = mtkit_prefs_new ( table );
	mtgex_prefs_init_prefs ( prefs );

	if ( mtgex_prefs_window_mirror_prefs ( prefs, global.prefs_file ) )
	{
		goto fail;
	}

	mtkit_prefs_value_copy ( prefs, global.prefs_file, winprefs_main_shared
		);

	return prefs;

fail:
	mtkit_prefs_destroy ( prefs );

	return NULL;
}

void open_prefs (
	char		* filename
	)
{
	if ( filename )
	{
		strncpy ( st_prefs_file, filename, PREFS_NAME_SIZE );
	}
	else
	{
		snprintf ( st_prefs_file, PREFS_NAME_SIZE, "%s/.%s",
			mtkit_file_home (), BIN_NAME );
	}

	st_prefs_file[PREFS_NAME_SIZE - 1] = 0;

	global.prefs_file = mtkit_prefs_new ( prefs_table );

	mtgex_fpick_init_prefs ( global.prefs_file,
		CED_INIFILE_MAIN_FPICK_PRE );

	mtgex_prefs_init_prefs ( global.prefs_file );

	mtkit_prefs_load ( global.prefs_file, st_prefs_file );
}

void close_prefs ( void )
{
	mtkit_prefs_save ( global.prefs_file, st_prefs_file );
	mtkit_prefs_destroy ( global.prefs_file );
	global.prefs_file = NULL;
}

void pressed_preferences (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	mtgex_prefs_window ( global.prefs_file, global.main_window,
		_("Program Preferences"), NULL, NULL, 0 );
}

void pressed_set_fullscreen(
	GtkMenuItem	* menu_item,
	gpointer	user_data,
	gint		item
	)
{
	if ( gtk_check_menu_item_get_active (
		GTK_CHECK_MENU_ITEM ( menu_item ) )
		)
	{
		gtk_window_fullscreen ( GTK_WINDOW ( global.main_window ) );
	}
	else
	{
		gtk_window_unfullscreen ( GTK_WINDOW ( global.main_window ) );
	}
}



static CedSheet		* pref_paste_sheet;
static int		pref_r1,
			pref_c1,
			pref_r2,
			pref_c2;
static unsigned int	old_cursor_r1,
			old_cursor_c1,
			old_cursor_r2,
			old_cursor_c2;



static int bulk_import_prefs (
	mtPrefs		* prefs,
	CedCellPrefs	* cell_prefs
	)
{
	mtBulkInt	b_int[] = {
		{ CED_FILE_PREFS_CELL_ALIGN_HORIZONTAL,		&cell_prefs->align_horizontal },
		{ CED_FILE_PREFS_CELL_COLOR_BACKGROUND,		&cell_prefs->color_background },
		{ CED_FILE_PREFS_CELL_COLOR_FOREGROUND,		&cell_prefs->color_foreground },
		{ CED_FILE_PREFS_CELL_FORMAT,			&cell_prefs->format },
		{ CED_FILE_PREFS_CELL_NUM_DECIMAL_PLACES,	&cell_prefs->num_decimal_places },
		{ CED_FILE_PREFS_CELL_NUM_ZEROS,		&cell_prefs->num_zeros },
		{ CED_FILE_PREFS_CELL_BOLD_TEXT,		&cell_prefs->bold_text },
		{ CED_FILE_PREFS_CELL_LOCKED,			&cell_prefs->locked },
		{ CED_FILE_PREFS_CELL_BORDER_TYPE,		&cell_prefs->border_type },
		{ CED_FILE_PREFS_CELL_BORDER_COLOR,		&cell_prefs->border_color },
		{ NULL }
		};
	mtBulkStr	b_str[] = {
		{ CED_FILE_PREFS_CELL_FORMAT_DATETIME,		&cell_prefs->format_datetime },
		{ CED_FILE_PREFS_CELL_NUM_THOUSANDS,		&cell_prefs->num_thousands },
		{ CED_FILE_PREFS_CELL_TEXT_PREFIX,		&cell_prefs->text_prefix },
		{ CED_FILE_PREFS_CELL_TEXT_SUFFIX,		&cell_prefs->text_suffix },
		{ NULL }
		};

	if ( ! prefs || ! cell_prefs )
	{
		return 1;
	}

	mtkit_prefs_block_callback ( prefs );
	mtkit_prefs_bulk_set ( prefs, b_int, NULL, b_str );
	mtkit_prefs_unblock_callback ( prefs );

	return 0;
}

static void cleanup_prefs_set ( void )
{
	ced_sheet_destroy ( pref_paste_sheet );
	pref_paste_sheet = NULL;
}

static void cell_prefs_closure (
	mtPrefItemValue	* piv,
	int		callback_data
	)
{
	mtPrefs		* prefs = (mtPrefs *)piv;
	CedCellPrefs	* cell_prefs = ced_cell_prefs_default ();
	CedSheet	* sheet;


	if ( prefs && cell_prefs )
	{
		bulk_import_prefs ( prefs, cell_prefs );

		save_pref_window_prefs ( prefs );
		mtkit_prefs_destroy ( prefs );
	}

	cleanup_prefs_set ();

	// Restore the cursor to its original position
	sheet = global.cedview->ren.sheet;
	if ( sheet )
	{
		sheet->prefs.cursor_r1 = old_cursor_r1;
		sheet->prefs.cursor_r2 = old_cursor_r2;
		sheet->prefs.cursor_c1 = old_cursor_c1;
		sheet->prefs.cursor_c2 = old_cursor_c2;

		ced_view_redraw_view ( global.cedview );
	}
}

static void commit_prefs_set (
	int		pref_id,
	int		pref_num,
	char		* pref_charp,
	int		change_cursor
	)
{
	int		res;
	CedSheet	* sheet;


	sheet = global.cedview->ren.sheet;
	if ( sheet )
	{
		res = cedui_cellprefs_change ( global.file->cubook, sheet,
			pref_r1, pref_c1, pref_r2, pref_c2,
			pref_paste_sheet, pref_id, pref_charp, pref_num );

		undo_report_updates ( res );

		if (	res == CU_ERROR_LOCKED_CELL ||
			res == CU_ERROR_NO_CHANGES
			)
		{
			return;
		}

		if (	change_cursor &&
			(	pref_id == CEDUI_CELLPREFS_color_background ||
				pref_id == CEDUI_CELLPREFS_color_foreground
				)
			)
		{
/*
Set the cursor to invisible, i.e. 0,0 -> 0,0 so that the user can see
background colour updates
*/
			sheet->prefs.cursor_r1 = 0;
			sheet->prefs.cursor_r2 = 0;
			sheet->prefs.cursor_c1 = 0;
			sheet->prefs.cursor_c2 = 0;
		}

		update_changes_chores ( 1, 1 );
	}
}

static void pref_changed (
	mtPrefItemValue	* piv,
	int		callback_data
	)
{
	int		num = 0;
	char		* charp = NULL;


	switch ( callback_data )
	{
	case CEDUI_CELLPREFS_align_horizontal:
	case CEDUI_CELLPREFS_color_background:
	case CEDUI_CELLPREFS_color_foreground:
	case CEDUI_CELLPREFS_format:
	case CEDUI_CELLPREFS_num_decimal_places:
	case CEDUI_CELLPREFS_num_zeros:
	case CEDUI_CELLPREFS_bold_text:
	case CEDUI_CELLPREFS_locked:
	case CEDUI_CELLPREFS_border_type:
	case CEDUI_CELLPREFS_border_color:
		mtkit_strtoi ( piv->value, &num, NULL, 0 );
		break;

	case CEDUI_CELLPREFS_format_datetime:
	case CEDUI_CELLPREFS_num_thousands:
	case CEDUI_CELLPREFS_text_prefix:
	case CEDUI_CELLPREFS_text_suffix:
		charp = piv->value;
		break;

	default:
		return;
	}

	commit_prefs_set ( callback_data, num, charp, 1 );
}

static int prepare_prefs_set ( void )
{
	CedSheet	* sheet;


	sheet = global.cedview->ren.sheet;
	if ( ! sheet )
	{
		return 1;
	}

	if ( pref_paste_sheet )
	{
		cleanup_prefs_set ();
	}

	cedui_cellprefs_init ( sheet, &pref_r1, &pref_c1, &pref_r2, &pref_c2,
		&pref_paste_sheet );
	if ( ! pref_paste_sheet )
	{
		return 1;
	}

	// NOTE: we have to do this as well as pref_*
	old_cursor_r1 = sheet->prefs.cursor_r1;
	old_cursor_r2 = sheet->prefs.cursor_r2;
	old_cursor_c1 = sheet->prefs.cursor_c1;
	old_cursor_c2 = sheet->prefs.cursor_c2;

	return 0;			// Success
}

void pressed_edit_cell_prefs (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	CedCell			* cell = NULL;
	CedCellPrefs		* cell_prefs;
	mtPrefs			* prefs;
	static mtPrefItem	item_table[] = {
	{ CED_FILE_PREFS_CELL_ALIGN_HORIZONTAL, MTKIT_PREF_TYPE_OPTION, "0", _("Horizontal text alignment"), pref_changed, CEDUI_CELLPREFS_align_horizontal, "None	Left	Centre	Right" },
	{ CED_FILE_PREFS_CELL_COLOR_BACKGROUND, MTKIT_PREF_TYPE_RGB, "16777215", _("Cell background colour"), pref_changed, CEDUI_CELLPREFS_color_background },
	{ CED_FILE_PREFS_CELL_COLOR_FOREGROUND, MTKIT_PREF_TYPE_RGB, "0", _("Text foreground colour"), pref_changed, CEDUI_CELLPREFS_color_foreground },
	{ CED_FILE_PREFS_CELL_FORMAT, MTKIT_PREF_TYPE_OPTION, "0", _("Cell format"), pref_changed, CEDUI_CELLPREFS_format, "General	Text	Fixed Decimal	Hexadecimal	Binary	Scientific	Percentage	Datetime" },
	{ CED_FILE_PREFS_CELL_FORMAT_DATETIME, MTKIT_PREF_TYPE_STR, "", _("Datetime format e.g. d/m/y HH:MM:SS"), pref_changed, CEDUI_CELLPREFS_format_datetime, "32" },
	{ CED_FILE_PREFS_CELL_NUM_DECIMAL_PLACES, MTKIT_PREF_TYPE_INT, "0", _("Number of decimal places for fixed type"), pref_changed, CEDUI_CELLPREFS_num_decimal_places, "0	99" },
	{ CED_FILE_PREFS_CELL_NUM_THOUSANDS, MTKIT_PREF_TYPE_STR, "", _("Thousands separator character"), pref_changed, CEDUI_CELLPREFS_num_thousands, "1" },
	{ CED_FILE_PREFS_CELL_NUM_ZEROS, MTKIT_PREF_TYPE_INT, "0", _("Leading zeros"), pref_changed, CEDUI_CELLPREFS_num_zeros, "0	32" },
	{ CED_FILE_PREFS_CELL_BOLD_TEXT, MTKIT_PREF_TYPE_BOOL, "0", _("Bold"), pref_changed, CEDUI_CELLPREFS_bold_text },
	{ CED_FILE_PREFS_CELL_LOCKED, MTKIT_PREF_TYPE_BOOL, "0", _("Cell locked - read only"), pref_changed, CEDUI_CELLPREFS_locked },
	{ CED_FILE_PREFS_CELL_BORDER_TYPE, MTKIT_PREF_TYPE_INT, "0", _("Border type"), pref_changed, CEDUI_CELLPREFS_border_type },
	{ CED_FILE_PREFS_CELL_BORDER_COLOR, MTKIT_PREF_TYPE_RGB, "0", _("Border colour"), pref_changed, CEDUI_CELLPREFS_border_color },
	{ CED_FILE_PREFS_CELL_TEXT_PREFIX, MTKIT_PREF_TYPE_STR, "", _("Cell text prefix"), pref_changed, CEDUI_CELLPREFS_text_prefix },
	{ CED_FILE_PREFS_CELL_TEXT_SUFFIX, MTKIT_PREF_TYPE_STR, "", _("Cell text suffix"), pref_changed, CEDUI_CELLPREFS_text_suffix },
	{ NULL }
	};


	if ( undo_report_updates ( cuf_check_sheet_lock (
		global.cedview->ren.sheet ) ) )
	{
		return;
	}

	if ( prepare_prefs_set () )
	{
		return;
	}

	cell = ced_sheet_get_cell ( global.cedview->ren.sheet,
		old_cursor_r1, old_cursor_c1 );

	if ( cell && cell->prefs )
	{
		cell_prefs = cell->prefs;
	}
	else
	{
		cell_prefs = ced_cell_prefs_default ();
	}

	// Load window prefs from disk, cell prefs from sheet/default
	prefs = load_pref_window_prefs ( item_table );
	if ( prefs && cell_prefs )
	{
		bulk_import_prefs ( prefs, cell_prefs );
	}

	mtgex_prefs_window ( prefs, global.main_window, _("Cell Preferences"),
		cell_prefs_closure, (mtPrefItemValue *)prefs, 0 );
}

void pressed_bold (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		res;


	if ( prepare_prefs_set () )
	{
		return;
	}

	res = cedui_cellprefs_bold ( global.file );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return;
	}

	undo_report_updates ( res );
	update_changes_chores ( 1, 1 );
	cleanup_prefs_set ();
}



#define COLOR_SWATCH_SIZE	24
#define COLOR_SWATCH_ROWS	5
#define COLOR_SWATCH_COLS	8
#define COLOR_SWATCH_TOTAL	40
#define COLOR_SWATCH_DEFAULT	-100
#define COLOR_SWATCH_TERMINATE	-10



static int swatch_color_table[COLOR_SWATCH_TOTAL + 1] = {
	MTPIXEL_RGB_2_INT (   0,   0,   0 ),
	MTPIXEL_RGB_2_INT (  90,   0,   0 ),
	MTPIXEL_RGB_2_INT ( 100,  72,   0 ),
	MTPIXEL_RGB_2_INT ( 100, 100,   0 ),
	MTPIXEL_RGB_2_INT (   0,  90,   0 ),
	MTPIXEL_RGB_2_INT (   0,  90,  90 ),
	MTPIXEL_RGB_2_INT (   0,   0, 100 ),
	MTPIXEL_RGB_2_INT (  90,   0,  90 ),

	MTPIXEL_RGB_2_INT ( 100, 100, 100 ),
	MTPIXEL_RGB_2_INT ( 170,   0,   0 ),
	MTPIXEL_RGB_2_INT ( 180, 106,   0 ),
	MTPIXEL_RGB_2_INT ( 170, 170,   0 ),
	MTPIXEL_RGB_2_INT (   0, 160,   0 ),
	MTPIXEL_RGB_2_INT (   0, 160, 160 ),
	MTPIXEL_RGB_2_INT (   0, 100, 180 ),
	MTPIXEL_RGB_2_INT ( 150,   0, 150 ),

	MTPIXEL_RGB_2_INT ( 180, 180, 180 ),
	MTPIXEL_RGB_2_INT ( 230, 140, 140 ),
	MTPIXEL_RGB_2_INT ( 230, 150,   0 ),
	MTPIXEL_RGB_2_INT ( 220, 220, 100 ),
	MTPIXEL_RGB_2_INT ( 120, 220, 120 ),
	MTPIXEL_RGB_2_INT ( 100, 230, 230 ),
	MTPIXEL_RGB_2_INT ( 100, 200, 255 ),
	MTPIXEL_RGB_2_INT ( 230, 140, 230 ),

	MTPIXEL_RGB_2_INT ( 220, 220, 220 ),
	MTPIXEL_RGB_2_INT ( 255, 200, 200 ),
	MTPIXEL_RGB_2_INT ( 255, 216, 160 ),
	MTPIXEL_RGB_2_INT ( 255, 255, 160 ),
	MTPIXEL_RGB_2_INT ( 180, 255, 180 ),
	MTPIXEL_RGB_2_INT ( 180, 255, 255 ),
	MTPIXEL_RGB_2_INT ( 180, 230, 255 ),
	MTPIXEL_RGB_2_INT ( 255, 200, 255 ),

	MTPIXEL_RGB_2_INT ( 255, 255, 255 ),
	MTPIXEL_RGB_2_INT ( 255,   0,   0 ),
	MTPIXEL_RGB_2_INT ( 255, 200,   0 ),
	MTPIXEL_RGB_2_INT ( 255, 255,   0 ),
	MTPIXEL_RGB_2_INT (   0, 255,   0 ),
	MTPIXEL_RGB_2_INT (   0, 255, 255 ),
	MTPIXEL_RGB_2_INT (   0,   0, 255 ),
	MTPIXEL_RGB_2_INT ( 255,   0, 255 ),

	0				// This is for the result
	};



static gboolean click_swatch (
	GtkWidget	* widget,
	gpointer	data
	)
{
	if ( swatch_color_table[COLOR_SWATCH_TOTAL] != COLOR_SWATCH_DEFAULT )
	{
		// Already set once so take the first selection

		return TRUE;
	}

	swatch_color_table[COLOR_SWATCH_TOTAL] = (intptr_t)data;

	return TRUE;
}

static gboolean expose_swatch (
	GtkWidget	* widget,
	GdkEventExpose	* event,
	gpointer	user_data
	)
{
	int		i,
			px,
			py,
			pw,
			ph,
			col = 0,
			r,
			g,
			b;
	unsigned char	* rgb,
			* dest;


	if ( ! widget || ! event )
	{
		return TRUE;
	}

	px = event->area.x;
	py = event->area.y;
	pw = event->area.width;
	ph = event->area.height;

	if ( pw < 1 || ph < 1 )
	{
		return TRUE;
	}

	col = (intptr_t)user_data;
	r = MTPIXEL_INT_2_R ( col );
	g = MTPIXEL_INT_2_G ( col );
	b = MTPIXEL_INT_2_B ( col );

	rgb = malloc ( pw * 3 );
	if ( ! rgb )
	{
		return FALSE;
	}

	for ( dest = rgb, i = 0; i < pw; i++ )
	{
		*dest++ = r;
		*dest++ = g;
		*dest++ = b;
	}

	gdk_draw_rgb_image ( widget->window, widget->style->black_gc,
		px, py, pw, ph, GDK_RGB_DITHER_NONE, rgb, 0 );

	free ( rgb );

	return TRUE;
}

static int swatch_color_dialog (
	char		* title
	)
{
	int		r,
			c,
			i;
	GtkWidget	* window,
			* table,
			* button,
			* vbox,
			* area;
	GtkAccelGroup	* ag;


	// Revert to default
	swatch_color_table[COLOR_SWATCH_TOTAL] = COLOR_SWATCH_DEFAULT;

	window = mtgex_add_a_window ( GTK_WINDOW_TOPLEVEL, title,
		GTK_WIN_POS_MOUSE, TRUE );
	ag = gtk_accel_group_new ();

	g_signal_connect ( window, "destroy", G_CALLBACK ( click_swatch ),
		(gpointer)COLOR_SWATCH_TERMINATE );

	vbox = gtk_vbox_new ( FALSE, 0 );
	gtk_widget_show ( vbox );
	gtk_container_add ( GTK_CONTAINER ( window ), vbox );

	table = gtk_table_new ( COLOR_SWATCH_ROWS, COLOR_SWATCH_COLS, FALSE );
	gtk_widget_show ( table );
	gtk_box_pack_start ( GTK_BOX ( vbox ), table, FALSE, FALSE, 0 );

	for ( r = 0; r < COLOR_SWATCH_ROWS; r++ )
	{
		for ( c = 0; c < COLOR_SWATCH_COLS; c++ )
		{
			i = r * COLOR_SWATCH_COLS + c;

			button = gtk_button_new ();
			gtk_widget_show ( button );
			g_signal_connect ( button, "clicked",
				G_CALLBACK ( click_swatch ),
				(gpointer)(intptr_t)swatch_color_table[i] );
			gtk_table_attach ( GTK_TABLE ( table ), button, c,
				c + 1, r, r + 1, GTK_FILL, GTK_FILL, 0, 0 );

			area = gtk_drawing_area_new ();
			gtk_drawing_area_size ( GTK_DRAWING_AREA ( area ),
				COLOR_SWATCH_SIZE, COLOR_SWATCH_SIZE );
			mtgex_fix_darea ( area );
			gtk_widget_show ( area );
			g_signal_connect ( area, "expose_event",
				G_CALLBACK ( expose_swatch ),
				(gpointer)(intptr_t)swatch_color_table[i] );

			gtk_container_add ( GTK_CONTAINER ( button ), area );
		}
	}

	button = gtk_button_new_with_label ( _("Cancel") );
	gtk_box_pack_start ( GTK_BOX ( vbox ), button, FALSE, FALSE, 0 );
	gtk_widget_show ( button );
	g_signal_connect ( button, "clicked",
		G_CALLBACK ( click_swatch ), (gpointer)-1 );
	gtk_widget_add_accelerator ( button, "clicked", ag, GDK_Escape, 0,
		(GtkAccelFlags) 0 );
	gtk_widget_add_accelerator ( button, "clicked", ag, GDK_F8, 0,
		(GtkAccelFlags) 0 );
	gtk_widget_add_accelerator ( button, "clicked", ag, GDK_F9, 0,
		(GtkAccelFlags) 0 );

	gtk_widget_show ( window );
	gtk_window_set_transient_for ( GTK_WINDOW ( window ),
		GTK_WINDOW ( global.main_window ) );
	gtk_window_add_accel_group ( GTK_WINDOW ( window ), ag );

	while ( swatch_color_table[COLOR_SWATCH_TOTAL] == COLOR_SWATCH_DEFAULT )
	{
		gtk_main_iteration ();
	}

	if ( swatch_color_table[COLOR_SWATCH_TOTAL] != COLOR_SWATCH_TERMINATE )
	{
		gtk_widget_destroy ( window );
	}

	return swatch_color_table[COLOR_SWATCH_TOTAL];
}

void pressed_change_color (
	GtkMenuItem	* menu_item,
	gpointer	user_data,
	gint		item
	)
{
	char		* title = "?";
	int		pref_id,
			num;


	switch ( item )
	{
	case 0:
		pref_id = CEDUI_CELLPREFS_color_background;
		title = _("Background Colour");
		break;

	case 1:
		pref_id = CEDUI_CELLPREFS_color_foreground;
		title = _("Foreground Colour");
		break;

	case 2:
		pref_id = CEDUI_CELLPREFS_border_color;
		title = _("Border Colour");
		break;

	default:
		return;
	}

	if ( prepare_prefs_set () )
	{
		return;
	}

	// Get the colour from the user driven swatch
	num = swatch_color_dialog ( title );

	if ( num >= 0 )
	{
		commit_prefs_set ( pref_id, num, NULL, 0 );
	}

	cleanup_prefs_set ();
}

void pressed_border_type (
	GtkMenuItem	* menu_item,
	gpointer	user_data,
	gint		item
	)
{
	int		res;


	if ( prepare_prefs_set () )
	{
		return;
	}

	res = cedui_cellprefs_border ( global.file, item );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return;
	}

	undo_report_updates ( res );
	update_changes_chores ( 1, 1 );
	cleanup_prefs_set ();
}
