/*
	Copyright (C) 2008-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "gui.h"



void update_main_area ( void )
{
	ced_view_redraw ( global.table );
}

void update_menus ( void )		// Update undo/redo menus
{
	char		* txt;
	int		i,
			menu_list[] = {
			_MENU_CUT,
			_MENU_COPY,
			_MENU_PASTE,
			_MENU_CLEAR,
			_MENU_SELECT_ALL,
			_MENU_CLEAR_CONTENT,
			_MENU_CLEAR_PREFS,
			_MENU_ROW_INSERT,
			_MENU_ROW_DELETE,
			_MENU_COL_INSERT,
			_MENU_COL_DELETE,
			-1
			};
	gboolean	on;
	GtkWidget	* focus;
	CedSheet	* sheet;


	sheet = global.cedview->ren.sheet;

	focus = GTK_WINDOW ( global.main_window )->focus_widget;

	if (	GTK_WIDGET_HAS_FOCUS ( global.entry_celltext ) ||
		GTK_WIDGET_HAS_FOCUS ( global.entry_cellref ) ||
		! focus ||
		gtk_widget_is_ancestor ( focus, global.pane_find ) ||
		gtk_widget_is_ancestor ( focus, global.pane_graph )
		)
	{
		on = FALSE;
	}
	else
	{
		on = TRUE;
	}

	// This stops the shortcuts interfering with the entries when they
	// have focus

	for ( i = 0; menu_list[i] >= 0; i++ )
	{
		gtk_widget_set_sensitive ( global.menu_widgets[ menu_list[i] ],
			on );
	}

	if ( ! global.file->cubook->book )
	{
		return;
	}

	// Enable/disable undo/redo according to their availability
	if ( global.file->cubook->undo.undo_step )
	{
		gtk_widget_set_sensitive ( global.menu_widgets[_MENU_UNDO],
			TRUE );
	}
	else
	{
		gtk_widget_set_sensitive ( global.menu_widgets[_MENU_UNDO],
			FALSE );
	}

	if ( global.file->cubook->undo.redo_step )
	{
		gtk_widget_set_sensitive ( global.menu_widgets[_MENU_REDO],
			TRUE );
	}
	else
	{
		gtk_widget_set_sensitive ( global.menu_widgets[_MENU_REDO],
			FALSE );
	}

	// Set the lock/unlock option for the sheet
	if ( sheet && sheet->prefs.locked )
	{
		txt = _("Unlock");
	}
	else
	{
		txt = _("Lock");
	}

	gtk_label_set_text ( GTK_LABEL ( GTK_MENU_ITEM (
		global.menu_widgets[_MENU_SHEET_LOCKED] )->item.bin.child ),
		txt );

	if (	sheet &&
		(sheet->prefs.split_r1 != 0 ||
			sheet->prefs.split_c1 != 0)
		)
	{
		txt = _("Unfreeze Panes");
	}
	else
	{
		txt = _("Freeze Panes");
	}

	gtk_label_set_text ( GTK_LABEL ( GTK_MENU_ITEM (
		global.menu_widgets[_MENU_SHEET_FREEZE_PANES] )->item.bin.child
		), txt );
}

void update_titlebar ( void )
{
	char		txt[MTGEX_PATHBUF],
			txt2[MTGEX_PATHBUF],
			* fn,
			* fd = NULL;


	if ( global.file->name )
	{
		mtgex_gtkuncpy ( txt2, global.file->name, MTGEX_PATHBUF );
	}
	else
	{
		txt2[0] = 0;
	}

	if ( txt2[0] )
	{
		fn = strrchr ( txt2, MTKIT_DIR_SEP );

		if ( ! fn )
		{
			fn = txt2;
		}
		else
		{
			fn ++;
		}

		fd = strdup ( txt2 );
	}
	else
	{
		fn = _("Untitled");
	}

	mtkit_strncpy0 ( txt, fn, MTGEX_PATHBUF );

	if ( global.mem_changed )
	{
		mtkit_strnncat ( txt, _(" (Modified)"), MTGEX_PATHBUF );
	}

	mtkit_strnncat ( txt, " [", MTGEX_PATHBUF );
	mtkit_strnncat ( txt, ced_file_type_text ( global.file->type),
				MTGEX_PATHBUF );
	mtkit_strnncat ( txt, "] ", MTGEX_PATHBUF );

	if ( fd )
	{
		fn = strrchr ( fd, MTKIT_DIR_SEP );
		if ( fn )
		{
			fn[1] = 0;
		}

		mtkit_strnncat ( txt, " - ", MTGEX_PATHBUF );
		mtkit_strnncat ( txt, fd, MTGEX_PATHBUF );
		free ( fd );
	}

	mtkit_strnncat ( txt, "    ", MTGEX_PATHBUF );
	mtkit_strnncat ( txt, VERSION, MTGEX_PATHBUF );

	gtk_window_set_title ( GTK_WINDOW ( global.main_window ), txt );

	if (	global.file->type == CED_FILE_TYPE_TSV_BOOK	||
		global.file->type == CED_FILE_TYPE_TSV_VAL_BOOK	||
		global.file->type == CED_FILE_TYPE_LEDGER_BOOK	||
		global.file->type == CED_FILE_TYPE_LEDGER_VAL_BOOK
		)
	{
		gtk_widget_show ( global.bmenu_sheet );
	}
	else
	{
		gtk_widget_hide ( global.bmenu_sheet );
	}
}

void update_entry_celltext ( void )
{
	CedCell		* cell = NULL;
	GtkEntry	* entry;
	CedSheet	* sheet;


	sheet = global.cedview->ren.sheet;

	if ( sheet )
	{
		cell = ced_sheet_get_cell ( sheet, sheet->prefs.cursor_r1,
			sheet->prefs.cursor_c1 );
	}

	entry = GTK_ENTRY ( global.entry_celltext );

	gtk_entry_set_text ( entry, "" );

	if ( cell && cell->text )
	{
		if ( cell->type == CED_CELL_TYPE_TEXT_EXPLICIT )
		{
			gtk_entry_append_text ( entry, "'" );
		}

		gtk_entry_append_text ( entry, cell->text );

		if ( cell->type == CED_CELL_TYPE_ERROR )
		{
			int		e = cell->value / 1000;


			gtk_editable_select_region ( GTK_EDITABLE ( entry ),
				0, e );
		}
	}

	if (	sheet &&
		( sheet->prefs.locked ||
			( cell && cell->prefs && cell->prefs->locked ) ) &&
		! global.file->cubook->book->prefs.disable_locks
		)
	{
		gtk_widget_set_sensitive ( GTK_WIDGET ( entry ), FALSE );
	}
	else
	{
		gtk_widget_set_sensitive ( GTK_WIDGET ( entry ), TRUE );
	}
}

void update_entry_cellref ( void )
{
	char		txt[128] = {0};
	CedSheet	* sheet;


	sheet = global.cedview->ren.sheet;
	if ( sheet )
	{
		if (	sheet->prefs.cursor_c1 == sheet->prefs.cursor_c2 &&
			sheet->prefs.cursor_r1 == sheet->prefs.cursor_r2
			)
		{
			sprintf ( txt, "r%uc%u", sheet->prefs.cursor_r1,
				sheet->prefs.cursor_c1 );
		}
		else
		{
			int		r1,
					c1,
					r2,
					c2;


			ced_sheet_cursor_max_min ( sheet, &r1, &c1, &r2, &c2 );

			sprintf ( txt, "r%uc%u:r%uc%u", r1, c1, r2, c2 );
		}
	}

	gtk_entry_set_text ( GTK_ENTRY ( global.entry_cellref ), txt );
}

/*
NOTE

lim_edge must be many bytes less than half of lim_tot in order to
easily insert the 'snip' characters without having to allocate dynamic
strings or some other hack.
*/

static void trim_string (
	char		* buf,
	int		bufchars,	// UTF characters in buf
	int		lim_tot
	)
{
	static char	* snip = " ... ";
	char		* spa,
			* spb;
	int		res,
			lim_edge = lim_tot / 2 - 5;


	res = mtkit_utf8_offset ( (unsigned char *)buf, MTGEX_PATHBUF,
		lim_edge );

	if ( res < 0 )
	{
		return;
	}

	// Place snip text here later
	spa = buf + res;

	res = mtkit_utf8_offset ( (unsigned char *)spa, MTGEX_PATHBUF,
		bufchars - 2 * lim_edge );

	if ( res < 0 )
	{
		return;
	}

	// Points to the last lim_edge chars in buf string
	spb = spa + res;

	// Add snip text
	strcpy ( spa, snip );

	spa += strlen ( snip );

	// Move remaining chars and NUL terminator
	memmove ( spa, spb, strlen ( spb ) + 1 );
}

void update_recent_files ( void )	// Update the menu items
{
	char		txt[64],
			* t,
			txt2[ MTGEX_PATHBUF ];
	int		i,
			slen,
			count = 0,
			lim_tot = 80;	// Max total chars


	mtkit_prefs_get_int ( global.prefs_file,
		CED_INIFILE_RECENT_FILENAME_LEN, &lim_tot );

	if ( lim_tot < 50 || lim_tot > 500 )
	{
		// FIXME - push limit validation into mtKit by
		// enforcing limits inside mtkit_prefs_set_value ()
		lim_tot = 80;
	}

	for ( i = 1; i <= MAX_RECENT; i++ )
	{
		sprintf ( txt, CED_INIFILE_MAIN_FILE"%i", i );

		t = "";
		mtkit_prefs_get_str ( global.prefs_file, txt, &t );
		if ( strlen ( t ) < 2 )
		{
			// Hide if empty
			gtk_widget_hide ( global.menu_widgets[i + _MENU_REC_SEP]
				);

			continue;
		}

		mtgex_gtkuncpy ( txt2, t, MTGEX_PATHBUF );

		slen = mtkit_utf8_len ( (unsigned char *)txt2,
			strlen ( txt2 ) );

		if ( slen > lim_tot )
		{
			trim_string ( txt2, slen, lim_tot );
		}

		gtk_label_set_text ( GTK_LABEL ( GTK_MENU_ITEM (
			global.menu_widgets[i + _MENU_REC_SEP]
			)->item.bin.child), txt2 );
		gtk_widget_show ( global.menu_widgets[i + _MENU_REC_SEP] );
		count ++;
	}

	for ( ; i <= MAX_RECENT; i++ )
	{
		// Hide extra items
		gtk_widget_hide ( global.menu_widgets[i + _MENU_REC_SEP] );
	}

	// Hide separator if not needed
	if ( count )
	{
		gtk_widget_show ( global.menu_widgets[_MENU_REC_SEP] );
	}
	else
	{
		gtk_widget_hide ( global.menu_widgets[_MENU_REC_SEP] );
	}
}

static void set_sheet_cb (
	GtkWidget	* widget,
	void		* user_data,
	int		new_value
	)
{
	char		* name;
	CedSheet	* sheet;
	CedBook		* book = global.file->cubook->book;


	name = mtgex_bmenu_get_value_text ( widget, new_value );
	sheet = ced_book_get_sheet ( book, name );
	ced_view_set_sheet ( global.table, sheet );

	mtkit_strfreeswap ( &book->prefs.active_sheet, name );

	update_entry_celltext ();
	update_entry_cellref ();
	update_quicksum_label ();
	update_menus ();
	grab_focus_sheet ();
}

void enable_sheet_selector_cb ( void )
{
	mtgex_bmenu_set_callback ( global.bmenu_sheet, set_sheet_cb, NULL );
}

void disable_sheet_selector_cb ( void )
{
	mtgex_bmenu_set_callback ( global.bmenu_sheet, NULL, NULL );
}

static int sselpop_cb (
	CedSheet	* sheet,
	char		* name,
	void		* user_data
	)
{
	int		new,
			* check = (int *)user_data;


	new = mtgex_bmenu_add_item ( global.bmenu_sheet, name );
	if ( check[0] )
	{
		// Select this item from the list if its the current sheet
		if ( ! strcmp ( name,
			global.file->cubook->book->prefs.active_sheet ) )
		{
			check[0] = 0;
			enable_sheet_selector_cb ();
			mtgex_bmenu_set_value ( global.bmenu_sheet, new );
		}
	}

	return 0;			// Continue
}

void sheet_selector_populate ( void )
{
	int		check = 1;


	if ( ! global.file->cubook->book->prefs.active_sheet )
	{
		check = 0;
	}
	else
	{
		disable_sheet_selector_cb ();
	}

	mtgex_bmenu_add_item ( global.bmenu_sheet, NULL );
	ced_book_scan ( global.file->cubook->book, sselpop_cb, &check );

	if ( check )
	{
		enable_sheet_selector_cb ();
		mtgex_bmenu_set_value ( global.bmenu_sheet, 0 );
	}
}

int check_for_changes ( void )
{
	int		i = -10;
	char		* warning = _("This project contains changes that have not been saved.  Do you really want to lose these changes?");


	graph_gui_store_changes ();

	if ( global.mem_changed )
	{
		i = mtgex_alert_box ( _("Warning"), warning,
			_("Cancel Operation"), _("Lose Changes"), NULL,
			global.main_window );
	}

	return i;
}

void grab_focus_sheet ( void )
{
	ced_view_grab_focus ( global.table );
}

void grab_focus_entry ( void )
{
	gtk_widget_grab_focus ( global.entry_celltext );
	gtk_entry_set_position ( GTK_ENTRY ( global.entry_celltext ), -1 );
}


#define QUICKSUM_FUNCS	9

void update_quicksum_label ( void )
{
	CedSheet	* sheet;
	char		txt[256] = "0",
			* funcs[QUICKSUM_FUNCS] =
			{	"none",
				"sum",
				"min",
				"max",
				"",
				"average",
				"median",
				"count",
				"counta"
			};
	int		r1,
			c1,
			r2,
			c2;

	sheet = global.cedview->ren.sheet;

	if (	global.sum_label_op < 0 ||
		global.sum_label_op >= QUICKSUM_FUNCS
		)
	{
		global.sum_label_op = 0;
	}

	if ( global.sum_label_op == 0 )
	{
		gtk_widget_hide ( global.label_sum );

		return;
	}

	gtk_widget_show ( global.label_sum );

	if ( sheet )
	{
		CedParser	parser;


		ced_sheet_cursor_max_min ( sheet, &r1, &c1, &r2, &c2 );

		if ( global.sum_label_op == 4 )
		{
			sprintf ( txt,
			"max( r%ic%i : r%ic%i ) - min( r%ic%i : r%ic%i )",
				r1, c1, r2, c2, r1, c1, r2, c2 );
		}
		else
		{
			sprintf ( txt, "%s( r%ic%i : r%ic%i )",
				funcs[global.sum_label_op], r1, c1, r2, c2 );
		}

		parser = ced_sheet_parse_text ( sheet, 1, 1, txt, NULL );

		if (	parser.ced_errno ||
			( parser.flag & CED_PARSER_FLAG_ERROR )
			)
		{
			sprintf ( txt, "err:%i", parser.ced_errno );
		}
		else
		{
			sprintf ( txt, "%.10g", parser.data );
		}

	}

	gtk_label_set_text ( GTK_LABEL ( global.label_sum ), txt );
}

void update_file_to_book ( void )
{
	int		new;


	switch ( global.file->type )
	{
	case CED_FILE_TYPE_TSV_CONTENT:
	case CED_FILE_TYPE_TSV_CONTENT_GZ:
	case CED_FILE_TYPE_TSV_CONTENT_NOQ:
	case CED_FILE_TYPE_TSV_VALUE:
	case CED_FILE_TYPE_TSV_VALUE_GZ:
	case CED_FILE_TYPE_TSV_VALUE_NOQ:
	case CED_FILE_TYPE_CSV_CONTENT:
	case CED_FILE_TYPE_CSV_CONTENT_NOQ:
	case CED_FILE_TYPE_CSV_VALUE:
	case CED_FILE_TYPE_CSV_VALUE_NOQ:
		new = CED_FILE_TYPE_TSV_VAL_BOOK;
		break;

	case CED_FILE_TYPE_LEDGER:
	case CED_FILE_TYPE_LEDGER_GZ:
	case CED_FILE_TYPE_LEDGER_VAL:
	case CED_FILE_TYPE_LEDGER_VAL_GZ:
		new = CED_FILE_TYPE_LEDGER_VAL_BOOK;
		break;

	default:
		new = 0;
	}

	if ( new )
	{
		global.file->type = new;
		mtkit_strfreeswap ( &global.file->name, NULL );
	}
}

void set_cursor_range (
	unsigned int	r1,
	unsigned int	c1,
	unsigned int	r2,
	unsigned int	c2,
	int		cursor_visible,
	int		force_update
	)
{
	if ( ced_view_set_cursor_area ( global.table, r1, c1, r2, c2,
		cursor_visible, force_update ) )
	{
		return;
	}

	update_entry_cellref ();
	update_entry_celltext ();
	update_quicksum_label ();
}

void clear_all ( void )
{
	ced_view_set_sheet ( global.table, NULL );

	cuf_file_book_new ( global.file );

	update_menus ();
	global.mem_changed = 0;
	sheet_selector_populate ();
	update_titlebar ();
	update_entry_celltext ();
	set_cursor_range ( 1, 1, 1, 1, 1, 1 );
	graph_gui_update ( NULL );
}


void recalc_sheet_core ( void )
{
	ced_sheet_recalculate ( global.cedview->ren.sheet, NULL, 0 );
	ced_sheet_recalculate ( global.cedview->ren.sheet, NULL, 1 );
}

void pressed_recalculate (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	recalc_sheet_core ();
	update_changes_chores ( 0, 1 );
}

void recalc_book_core ( void )
{
	ced_book_recalculate ( global.file->cubook->book, 0 );
	ced_book_recalculate ( global.file->cubook->book, 1 );
}

void pressed_recalculate_book (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	recalc_book_core ();
	update_changes_chores ( 0, 1 );
}

gboolean view_focus_out (
	GtkWidget	* widget,
	GdkEventFocus	* event,
	gpointer	user_data
	)
{
	update_menus ();

	return FALSE;
}

gboolean view_focus_in (
	GtkWidget	* widget,
	GdkEventFocus	* event,
	gpointer	user_data
	)
{
	update_menus ();

	return FALSE;
}

gboolean cellref_entry_focus_in (
	GtkWidget	* widget,
	GdkEventFocus	* event,
	gpointer	user_data
	)
{
	update_menus ();

	return FALSE;
}

gboolean cellref_entry_focus_out (
	GtkWidget	* widget,
	GdkEventFocus	* event,
	gpointer	user_data
	)
{
	update_entry_cellref ();
	update_menus ();

	return FALSE;
}

gboolean celltext_entry_focus_in (
	GtkWidget	* widget,
	GdkEventFocus	* event,
	gpointer	user_data
	)
{
	update_menus ();

	return FALSE;
}

gboolean celltext_entry_focus_out (
	GtkWidget	* widget,
	GdkEventFocus	* event,
	gpointer	user_data
	)
{
	update_menus ();

	return FALSE;
}

void pressed_undo (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		res;


	// This is needed in case we remove the last sheet
	ced_view_set_sheet ( global.table, NULL );

	res = cuf_book_undo_step ( global.file->cubook );
	undo_report_updates ( res );

	if ( res == CU_ERROR_NO_CHANGES )
	{
		return;
	}

	sheet_selector_populate ();
	update_splits ();
	update_changes_chores ( 1, 0 );	// Update menus/titles
}

void pressed_redo (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		res;


	// This is needed in case we remove the last sheet
	ced_view_set_sheet ( global.table, NULL );

	res = cuf_book_redo_step ( global.file->cubook );
	undo_report_updates ( res );

	if ( res == CU_ERROR_NO_CHANGES )
	{
		return;
	}

	sheet_selector_populate ();
	update_splits ();
	update_changes_chores ( 1, 0 );	// Update menus/titles
}

void update_changes_chores (
	int		new_geometry,
	int		block_sheet_recalcs
	)
{
	if (	! block_sheet_recalcs &&
		global.cedview->ren.sheet &&
		global.file->cubook->book->prefs.auto_recalc
		)
	{
		// Always update sheet (must be done before book updates)
		recalc_sheet_core ();

		switch ( global.file->cubook->book->prefs.auto_recalc )
		{
		case 2:
			recalc_book_core ();
			break;
		}
	}

	global.mem_changed = 1;
	update_main_area ();
	update_menus ();
	update_titlebar ();
	update_entry_celltext ();
	update_quicksum_label ();
	if ( new_geometry )
	{
		ced_view_update_geometry ( global.cedview );
	}
}

void update_splits ( void )
{
	ced_view_set_sheet ( global.table, global.cedview->ren.sheet );
	// Update splits/scrollbars
}

int undo_report_updates (
	int		error
	)
{
	if ( error < 0 )
	{
		static char	* mes[] = { "?",
		_("Error during operation."),
		_("Unable to begin operation due to problem with undo system."),
		_("Undo history lost."),
		_("Undo history lost.  Possible data corruption."),
		_("cell locked.  Operation aborted."),
		_("Sheet locked.  Operation aborted.")
				};
		char		* msg;
		int		mi;


		mi = -error;

		if ( mi >= sizeof ( mes ) / sizeof ( msg[0] ) )
		{
			mi = 0;
		}

		msg = mes[mi];

		if ( error == CU_ERROR_LOCKED_CELL )
		{
			static char	buf[2048];


			sprintf ( buf, "%s %s", cuf_error_str (), msg );
			msg = buf;
		}

		// Report this error to the user
		mtgex_alert_box ( _("Error"), msg, _("OK"), NULL, NULL,
			global.main_window );
	}

	return error;
}

