/*
	Copyright (C) 2009-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "eleana.h"



int eleana_table_total_sum (
	double		* table,
	int		items,
	int		first,
	int		last
	)
{
	double		tot;
	int		col,
			row;


	if ( ! table || items < 1 )
	{
		return 1;
	}

	for ( col = first; col <= last; col ++ )
	{
		tot = 0;

		for ( row = 0; row < items; row ++ )
		{
			tot += EL_TABLE_VALUE ( table, row, col );
		}

		EL_TABLE_VALUE ( table, items, col ) = tot;
	}

	return 0;
}

static int largest_other (
	double		* table,
	int		row
	)
{
	int		i,
			j,
			maxo;
	double		maxv;


	maxo = 3;			// Find the largest other party
	maxv = 0;

	for ( i = 3; i < 7; i++ )
	{
		j = EL_TABLE_VALUE ( table, row, i + EL_TAB_A );
		if ( j > maxv )
		{
			maxo = i;
			maxv = j;
		}
	}

	return maxo;
}

static void recalc_row_projection (
	double		* table,
	int		row
	)
{
	double		mx;


	EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_O ) =
		EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_O1 ) +
		EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_O2 ) +
		EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_O3 ) +
		EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_O4 )
		;

	mx = EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_A );
	mx = MAX ( mx, EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_B ) );
	mx = MAX ( mx, EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_C ) );
	mx = MAX ( mx, EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_O1 ) );
	mx = MAX ( mx, EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_O2 ) );
	mx = MAX ( mx, EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_O3 ) );
	mx = MAX ( mx, EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_O4 ) );

	EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_A_SEAT ) = 0;
	EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_B_SEAT ) = 0;
	EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_C_SEAT ) = 0;
	EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_O_SEAT ) = 0;

	if ( EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_A ) == mx )
	{
		EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_WINNER ) = 1;
		EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_A_SEAT ) = 1;
	}
	else if ( EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_B ) == mx )
	{
		EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_WINNER ) = 2;
		EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_B_SEAT ) = 1;
	}
	else if ( EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_C ) == mx )
	{
		EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_WINNER ) = 3;
		EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_C_SEAT ) = 1;
	}
	else
	{
		EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_WINNER ) = 4;
		EL_TABLE_VALUE ( table, row, EL_TAB_PROJ_O_SEAT ) = 1;
	}
}

static void record_seat_totals (
	int		party_seats[],
	double		* table,
	int		items
	)
{
	party_seats[0] = EL_TABLE_VALUE ( table, items, EL_TAB_PROJ_A_SEAT );
	party_seats[1] = EL_TABLE_VALUE ( table, items, EL_TAB_PROJ_B_SEAT );
	party_seats[2] = EL_TABLE_VALUE ( table, items, EL_TAB_PROJ_C_SEAT );
	party_seats[3] = EL_TABLE_VALUE ( table, items, EL_TAB_PROJ_O_SEAT );
}

int eleana_calculate_uniform_swing (
	double		* table,
	int		items,
	double		swing[],
	int		party_seats[]
	)
{
	double		tswings[8],
			gs[EL_SWING_TOTAL];
	int		row,
			i,
			maxo;


	if (	! table		||
		items < 1	||
		! swing
		)
	{
		return 1;
	}

	for ( i = 0; i < EL_SWING_TOTAL; i++ )
	{
		gs[i] = swing[i] / 100;
	}

	tswings[0] = -gs[EL_SWING_AB] - gs[EL_SWING_AC] - gs[EL_SWING_AO];
	tswings[1] = gs[EL_SWING_AB] - gs[EL_SWING_BC] - gs[EL_SWING_BO];
	tswings[2] = gs[EL_SWING_AC] + gs[EL_SWING_BC] - gs[EL_SWING_CO];
	tswings[3] = tswings[4] = tswings[5] = tswings[6] = tswings[7] =
		gs[EL_SWING_AO] + gs[EL_SWING_BO] + gs[EL_SWING_CO];

	for ( row = 0; row < items; row ++ )
	{
		for ( i = 0; i < 3; i++ )
		{
			EL_TABLE_VALUE ( table, row, i + EL_TAB_PROJ_A ) =
				EL_TABLE_VALUE ( table, row, i + EL_TAB_A ) +
				tswings[i] * EL_TABLE_VALUE ( table, row,
					EL_TAB_TURNOUT );
		}

		for ( i = 3; i < 7; i++ )
		{
			EL_TABLE_VALUE ( table, row, i + EL_TAB_PROJ_A ) =
				EL_TABLE_VALUE ( table, row, i + EL_TAB_A );
		}

		maxo = largest_other ( table, row );

		EL_TABLE_VALUE ( table, row, maxo + EL_TAB_PROJ_A ) =
			EL_TABLE_VALUE ( table, row, maxo + EL_TAB_A ) +
			tswings[3] * EL_TABLE_VALUE ( table, row,
				EL_TAB_TURNOUT );

		recalc_row_projection ( table, row );
	}

	eleana_table_total_sum ( table, items, EL_TAB_PROJ_A,
		EL_TAB_PROJ_O_SEAT );

	if (	party_seats	&&
		tswings[0] == 0	&&
		tswings[1] == 0	&&
		tswings[2] == 0	&&
		tswings[3] == 0
		)
	{
		record_seat_totals ( party_seats, table, items );
	}

	return 0;
}

int eleana_calculate_eleana_swing (
	double		* table,
	int		items,
	double		swing[],
	int		party_seats[]
	)
{
	double		gs[EL_SWING_TOTAL], pot;
	int		row,
			i,
			src,
			dest,
			maxo,
			xfer[EL_SWING_TOTAL][2] = {
			{ EL_TAB_A, EL_TAB_B },
			{ EL_TAB_A, EL_TAB_C },
			{ EL_TAB_A, EL_TAB_O },
			{ EL_TAB_B, EL_TAB_C },
			{ EL_TAB_B, EL_TAB_O },
			{ EL_TAB_C, EL_TAB_O },
			};


	if (	! table		||
		items < 1	||
		! swing
		)
	{
		return 1;
	}

	for ( i = 0; i < EL_SWING_TOTAL; i++ )
	{
		gs[i] = swing[i] / 100;
	}

	for ( row = 0; row < items; row ++ )
	{
		for ( i = 0; i < 7; i++ )	// Default to current votes
		{
			EL_TABLE_VALUE ( table, row, i + EL_TAB_PROJ_A ) =
				EL_TABLE_VALUE ( table, row, i + EL_TAB_A );
		}

		maxo = largest_other ( table, row );

		for ( i = 0; i < EL_SWING_TOTAL; i++ )
		{
			if ( gs[i] < 0 )
			{
				src = 1;
				dest = 0;
			}
			else
			{
				src = 0;
				dest = 1;
			}

			pot = gs[i] * EL_TABLE_VALUE ( table, row,
				xfer[i][src] );

			if ( pot < 0 )
			{
				pot = -pot;
			}

			if ( pot == 0 )
			{
				// No votes in the swing pot so nothing to do

				continue;
			}

			// Add swing pot to new party
			if ( xfer[i][dest] == EL_TAB_O )
			{
				// Add the pot to the largest other party
				EL_TABLE_VALUE ( table, row,
					EL_TAB_PROJ_A + maxo ) += pot;
			}
			else
			{
				EL_TABLE_VALUE ( table, row,
					EL_TAB_PROJ_A +
					( xfer[i][dest] - EL_TAB_A ) ) += pot;
			}

			// Subtract swing pot from old party
			if ( xfer[i][src] == EL_TAB_O )
			{
				// Subract the pot from the largest other party
				EL_TABLE_VALUE ( table, row,
					EL_TAB_PROJ_A + maxo ) -= pot;
			}
			else
			{
				EL_TABLE_VALUE ( table, row,
					EL_TAB_PROJ_A +
					( xfer[i][src] - EL_TAB_A ) ) -= pot;
			}
		}

		recalc_row_projection ( table, row );
	}

	eleana_table_total_sum ( table, items, EL_TAB_PROJ_A,
		EL_TAB_PROJ_O_SEAT );

	for ( i = 0; i < EL_SWING_TOTAL; i++ )
	{
		if ( gs[i] != 0 )
		{
			break;
		}
	}

	if ( party_seats && i == EL_SWING_TOTAL )
	{
		record_seat_totals ( party_seats, table, items );
	}

	return 0;
}

int eleana_calculate_diagram (
	double		* table,
	int		items,
	int		max_x,
	int		may_y
	)
{
	double		x,
			y,
			z,
			co[4][2] = {
				{ 1, 1 },
				{ 0, 0 },
				{ 1, 0 },
				{ 0, 1 }
				},
			v[4],
			vp
			;
	int		i,
			winner,
			codel[4][4] = {
			// Winner, Diagonal, vertical, horizontal
		{ EL_TAB_PROJ_A, EL_TAB_PROJ_B, EL_TAB_PROJ_C, EL_TAB_PROJ_O },
		{ EL_TAB_PROJ_B, EL_TAB_PROJ_A, EL_TAB_PROJ_O, EL_TAB_PROJ_C },
		{ EL_TAB_PROJ_C, EL_TAB_PROJ_O, EL_TAB_PROJ_A, EL_TAB_PROJ_B },
		{ EL_TAB_PROJ_O, EL_TAB_PROJ_C, EL_TAB_PROJ_B, EL_TAB_PROJ_A },
			}
			;

	if ( ! table )
	{
		return 1;
	}

	for ( i = 0; i < items; i++ )
	{
		winner = EL_TABLE_VALUE ( table, i, EL_TAB_PROJ_WINNER ) - 1;

		if ( winner < 0 || winner > 3 )
		{
			// Should never happen
			continue;
		}

		x = co[winner][0];	// Start in the corner of the winner
		y = co[winner][1];

		v[0] = EL_TABLE_VALUE ( table, i, codel[winner][0] );
		v[1] = EL_TABLE_VALUE ( table, i, codel[winner][1] );
		v[2] = EL_TABLE_VALUE ( table, i, codel[winner][2] );
		v[3] = EL_TABLE_VALUE ( table, i, codel[winner][3] );

		if ( v[0] <= 0 )
		{
			continue;	// Watch out for empty table
		}

		vp = v[0] / (v[0] + v[3]);	// 0.5 to 1.0 (horizontal)
		vp = (vp - 0.5) * 2;		// 0.0 to 1.0
						// ( 1 => no opposing votes,
						// 0 => dead heat)
		x = vp * x + (1 - vp) * 0.5;

		vp = v[0] / (v[0] + v[2]);	// 0.5 to 1.0 (vertical)
		vp = (vp - 0.5) * 2;		// 0.0 to 1.0
						// ( 1 => no opposing votes,
						// 0 => dead heat)
		y = vp * y + (1 - vp) * 0.5;

		z = v[0] / (v[0] + v[1]);	// 0.5 to 1.0 (diagonal)
		z = (z - 0.5) * 2;		// 0.0 to 1.0
						// ( 1 => no diagonal votes,
						// 0 => dead heat)

		x = z * x + (1 - z) * 0.5;	// Move to centre according to
						// diagonal shift
		y = z * y + (1 - z) * 0.5;

		EL_TABLE_VALUE ( table, i, EL_TAB_DIAGRAM_X ) = x * max_x;
		EL_TABLE_VALUE ( table, i, EL_TAB_DIAGRAM_Y ) = y * may_y;
	}

	return 0;
}

int eleana_swingset_prepare (
	double		* table,
	int		items,
	double		from[],
	double		to[]
	)
{
	int		i,
			k;
	double		party_p[4],
			swing_val[EL_SWING_TOTAL][2] = {
			{ 1, 0 },
			{ 2, 0 },
			{ 3, 0 },
			{ 2, 1 },
			{ 3, 1 },
			{ 3, 2 },
			};


	if (	! table		||
		items < 1	||
		! to
		)
	{
		return 1;
	}

	party_p[0] = 100 * EL_TABLE_VALUE ( table, items, EL_TAB_A ) /
		EL_TABLE_VALUE ( table, items, EL_TAB_TURNOUT );

	party_p[1] = 100 * EL_TABLE_VALUE ( table, items, EL_TAB_B ) /
		EL_TABLE_VALUE ( table, items, EL_TAB_TURNOUT );

	party_p[2] = 100 * EL_TABLE_VALUE ( table, items, EL_TAB_C ) /
		EL_TABLE_VALUE ( table, items, EL_TAB_TURNOUT );

	party_p[3] = 100 * EL_TABLE_VALUE ( table, items, EL_TAB_O ) /
		EL_TABLE_VALUE ( table, items, EL_TAB_TURNOUT );

	for ( i = 0; i < EL_SWING_TOTAL; i++ )
	{
		if ( from[i] < 0 )
		{
			k = swing_val[i][0];
		}
		else
		{
			k = swing_val[i][1];
		}

		if ( party_p[k] == 0 )
		{
			to[i] = 0;
		}
		else
		{
			to[i] = party_p[k];
		}
	}

	return 0;
}

int eleana_swingset_eleana_to_uniform (
	double		from[],
	double		to[]
	)
{
	int		i;


	if ( ! from || ! to )
	{
		return 1;
	}

	for ( i = 0; i < EL_SWING_TOTAL; i++ )
	{
		if ( to[i] != 0 )
		{
			to[i] = from[i] * to[i] / 100;
		}
	}

	return 0;
}

int eleana_swingset_uniform_to_eleana (
	double		from[],
	double		to[]
	)
{
	int		i;


	if ( ! from || ! to )
	{
		return 1;
	}

	for ( i = 0; i < EL_SWING_TOTAL; i++ )
	{
		if ( to[i] != 0 )
		{
			to[i] = from[i] * 100 / to[i];
		}
	}

	return 0;
}



#define INDEX_HEADERS 5

static int invalid_index (
	CedSheet	* sheet
	)
{
	int		i;
	char		* heads[INDEX_HEADERS] = {
			"Year",
			"Result Summary",
			"Full Result",
			"Cartogram",
			"Polymap"
			};
	CedCell		* cell;


	for ( i = 0; i < INDEX_HEADERS; i++ )
	{
		cell = ced_sheet_get_cell ( sheet, 1, i + 1 );

		if ( ! cell )
		{
			return 1;
		}

		if ( strcmp ( cell->text, heads[i] ) )
		{
			return 1;
		}
	}

	return 0;
}

eleanaINDEX * eleana_index_open (
	char		* filename
	)
{
	int		i,
			j;
	char		* s,
			* saj[3] = { NULL };
	eleanaINDEX	* index;
	CedCell		* cell;


	if ( ! filename )
	{
		return NULL;
	}

	index = calloc ( sizeof ( eleanaINDEX ), 1 );
	if ( ! index )
	{
		return NULL;
	}

	index->path = strdup ( filename );
	if ( ! index->path )
	{
		goto fail;
	}

	s = strrchr ( index->path, MTKIT_DIR_SEP );
	if ( s )
	{
		s[1] = 0;
	}

	index->book = ced_book_load ( filename, NULL, NULL );
	if ( ! index->book )
	{
		goto fail;
	}

	index->sheet_index = ced_book_get_sheet ( index->book,
		ELEANA_BOOK_SHEET_INDEX );

	if (	! index->sheet_index ||
		invalid_index ( index->sheet_index )
		)
	{
		goto fail;
	}

	for ( i = ELEANA_INDEX_ROW_START; ; i++ )
	{
		cell = ced_sheet_get_cell ( index->sheet_index, i,
			ELEANA_INDEX_COL_YEAR );

		if ( ! cell )
		{
			break;
		}

		for (	j = ELEANA_INDEX_COL_FULL;
			j <= ELEANA_INDEX_COL_POLYMAP;
			j++
			)
		{
			cell = ced_sheet_get_cell ( index->sheet_index, i, j );
			if ( ! cell )
			{
				continue;
			}

			saj[0] = index->path;
			saj[1] = cell->text;
			s = mtkit_strjoinv ( NULL, saj );

			if ( s )
			{
				ced_sheet_set_cell_text ( index->sheet_index,
					i, j, s );

				free ( s );
			}
		}
	}

	index->records = i - ELEANA_INDEX_ROW_START;

	return index;

fail:
	eleana_index_close ( index );

	return NULL;
}

int eleana_index_close (
	eleanaINDEX	* index
	)
{
	if ( ! index )
	{
		return 1;
	}

	ced_book_destroy ( index->book );
	free ( index->path );
	free ( index );

	return 0;
}

char * eleana_index_get (
	eleanaINDEX	* index,
	unsigned int	record,
	unsigned int	field
	)
{
	CedCell		* cell;


	if (	! index				||
		! index->sheet_index		||
		record >= index->records	||
		field < 1			||
		field >= ELEANA_INDEX_COL_TOTAL
		)
	{
		return NULL;
	}

	cell = ced_sheet_get_cell ( index->sheet_index,
		record + ELEANA_INDEX_ROW_START, field );

	if ( ! cell )
	{
		return NULL;
	}

	return cell->text;
}



typedef struct
{
	CedSheet	* results_sheet;
	CedCell		* cell;
	double		* table;
	int		row,
			turnout,
			next_other,
			i,
			party,
			votes;

	CedIndex	* idx_regions,
			* idx_counties;
	CedIndexItem	* index_item;
	char		* txt_party[3];
	mtTree		** seat_names;
	int		* seats;
} populSTATE;



static int scrape_seat_cb (
	CedSheet	* sheet,
	CedCell		* cell,
	unsigned int	row,
	unsigned int	col,
	void		* user_data
	)
{
	populSTATE	* state = (populSTATE *)user_data;


	// Electorate
	state->cell = ced_sheet_get_cell ( sheet, row, FULL_COL_ELECTORATE );
	if ( state->cell )
	{
		EL_TABLE_VALUE ( state->table, state->row, EL_TAB_ELECTORATE ) =
			state->cell->value;
	}

	// County
	state->cell = ced_sheet_get_cell ( sheet, row, FULL_COL_COUNTY );
	if ( state->cell )
	{
		if ( ced_index_query ( state->idx_counties, 0,
			state->cell->text, &state->index_item ) == 1 )
		{
			EL_TABLE_VALUE ( state->table, state->row,
				EL_TAB_COUNTY ) = state->index_item->row;
		}
	}

	// Region
	state->cell = ced_sheet_get_cell ( sheet, row, FULL_COL_REGION );
	if ( state->cell )
	{
		if ( ced_index_query ( state->idx_regions, 0,
			state->cell->text, &state->index_item ) == 1 )
		{
			EL_TABLE_VALUE ( state->table, state->row,
				EL_TAB_REGION ) = state->index_item->row;
		}

	}

	// Find all votes/parties and put into table
	state->turnout = 0;
	state->next_other = EL_TAB_O1;
	state->i = row;

	while ( 1 )
	{
		state->cell = ced_sheet_get_cell ( sheet, state->i,
			FULL_COL_VOTES );

		if ( ! state->cell )
		{
			break;
		}

		state->votes = state->cell->value;

		// Default to other
		state->party = state->next_other;

		// Is this cell a main 3 party?
		state->cell = ced_sheet_get_cell ( sheet, state->i,
			FULL_COL_PARTY );

		if ( state->cell && state->cell->text )
		{
			if ( mtkit_strmatch ( state->cell->text,
				state->txt_party[0], 1 ) >= 0 )
			{
				state->party = EL_TAB_A;
			}
			else if ( mtkit_strmatch ( state->cell->text,
				state->txt_party[1], 1 ) >= 0 )
			{
				state->party = EL_TAB_B;
			}
			else if ( mtkit_strmatch ( state->cell->text,
				state->txt_party[2], 1 ) >= 0 )
			{
				state->party = EL_TAB_C;
			}
		}

		// Add these votes to the party total
		EL_TABLE_VALUE ( state->table, state->row, state->party ) +=
			state->votes;

		if (	state->party == state->next_other &&
			state->next_other < EL_TAB_O4
			)
		{
			state->next_other ++;
		}

		state->i ++;
	}

	EL_TABLE_VALUE ( state->table, state->row, EL_TAB_ELECTORATE ) =
		state->turnout;

	EL_TABLE_VALUE ( state->table, state->row, EL_TAB_O ) =
		EL_TABLE_VALUE ( state->table, state->row, EL_TAB_O1 ) +
		EL_TABLE_VALUE ( state->table, state->row, EL_TAB_O2 ) +
		EL_TABLE_VALUE ( state->table, state->row, EL_TAB_O3 ) +
		EL_TABLE_VALUE ( state->table, state->row, EL_TAB_O4 );

	EL_TABLE_VALUE ( state->table, state->row, EL_TAB_TURNOUT ) =
		EL_TABLE_VALUE ( state->table, state->row, EL_TAB_O ) +
		EL_TABLE_VALUE ( state->table, state->row, EL_TAB_A ) +
		EL_TABLE_VALUE ( state->table, state->row, EL_TAB_B ) +
		EL_TABLE_VALUE ( state->table, state->row, EL_TAB_C );

	state->row ++;

	return 0;
}

static int count_seat_cb (
	CedSheet	* sheet,
	CedCell		* cell,
	unsigned int	row,
	unsigned int	col,
	void		* user_data
	)
{
	char		* new;
	populSTATE	* state = (populSTATE *)user_data;


	if ( ! cell->text )
	{
		return 0;
	}

	// Add this seat name to the indexed list
	new = strdup ( cell->text );
	if (	! new ||
		mtkit_tree_node_add ( state->seat_names[0],
			(void *)(intptr_t)state->seats[0], new ) <= 0 )
	{
		free ( new );
		state->seats[0] = -1;	// Flag failure

		return 1;		// STOP
	}

	state->seats[0] = state->seats[0] + 1;

	return 0;			// Continue
}

static int seat_name_cmp (
	void		* k1,
	void		* k2
	)
{
	if ( k1 < k2 )
	{
		return -1;
	}

	if ( k1 > k2 )
	{
		return 1;
	}

	return 0;
}

static void seat_name_del (
	mtTreeNode	* node
	)
{
	free ( node->data );
}

static int populate_table (
	eleanaINDEX	* index,
	unsigned int	record,
	CedSheet	* sheet,
	int		* seats,
	double		** table_out,
	mtTree		** seat_names
	)
{
	populSTATE	state = { sheet };
	CedSheet	* sh;
	int		i,
			res = 1;


	seat_names[0] = mtkit_tree_new ( seat_name_cmp, seat_name_del );

	if ( ! seat_names[0] )
	{
		return 1;
	}

	state.seat_names = seat_names;

	// Count up seat names - put into 'row'
	state.seats = seats;
	seats[0] = 0;

	ced_sheet_scan_area ( sheet, FULL_ROW_PARTY1, FULL_COL_SEAT_NAME, 0, 1,
		count_seat_cb, &state );

	if ( seats < 0 )
	{
		goto error;
	}

	state.table = calloc ( sizeof ( double ),
		EL_TAB_TOTAL * (seats[0] + 1) );

	if ( ! state.table )
	{
		goto error;
	}

	table_out[0] = state.table;

	// Set up 2 indexes for names of regions and counties - use row as
	// reference in table

	sh = ced_book_get_sheet ( index->book, ELEANA_BOOK_SHEET_COUNTIES );
	state.idx_counties = ced_index_new ( CED_INDEX_TYPE_TEXT );

	if ( ! sh || ! state.idx_counties )
	{
		goto error;
	}

	ced_index_add_items ( state.idx_counties, sh, 1, 1, 0, 1 );

	sh = ced_book_get_sheet ( index->book, ELEANA_BOOK_SHEET_REGIONS );
	state.idx_regions = ced_index_new ( CED_INDEX_TYPE_TEXT );

	if ( ! sh || ! state.idx_regions )
	{
		goto error;
	}

	ced_index_add_items ( state.idx_regions, sh, 1, 1, 0, 1 );

	// Set up 3 pointers to names of 3 main parties
	for ( i = 0; i < 3; i++ )
	{
		state.cell = ced_sheet_get_cell ( index->sheet_index, record,
			ELEANA_INDEX_COL_PARTY_A + i );

		if ( ! state.cell || ! state.cell->text )
		{
			goto error;
		}

		state.txt_party[i] = state.cell->text;
	}

// Do second scan to scrape the data for each seat and put it into the table

	state.row = 0;
	ced_sheet_scan_area ( sheet, FULL_ROW_PARTY1, FULL_COL_SEAT_NAME, 0, 1,
		scrape_seat_cb, &state );

	eleana_table_total_sum ( state.table, seats[0], EL_TAB_A,
		EL_TAB_ELECTORATE );

	res = 0;			// Success

error:
	ced_index_destroy ( state.idx_regions );
	ced_index_destroy ( state.idx_counties );

	if ( res )
	{
		free ( state.table );
		mtkit_tree_destroy ( seat_names[0] );
		seat_names[0] = NULL;
	}

	return res;
}

static int get_cxy (
	unsigned char	c,
	int		* idx,
	unsigned char	* mem,
	int		* x,
	int		* y,
	int		w,
	int		idxmax,
	int		validate
	)
{
	do
	{
		if ( mem[ idx[c] ] == c )
		{
			idx[c] ++;

			x[0] = idx[c] % w;
			y[0] = idx[c] / w;

			return 0;
		}

		idx[c] ++;
	}
	while ( idx[c] < idxmax );

	if ( validate )
	{
		printf ( "ERROR - Unable to find %i\n", (int)c );
	}

	return 1;
}

static void create_map_data (
	double		* table,
	mtImage		* image,
	int		items,
	int		validate
	)
{
	int		i,
			w,
			h,
			bpp,
			idx[256] = {0},
			target[256] = {0},
			idxmax,
			x,
			y;
	unsigned char	c,
			* mem;


	bpp = image->type;
	w = image->width;
	h = image->height;

	if ( bpp != 1 )
	{
		if ( validate )
		{
			printf ( "create_map_data: invalid cartogram image\n" );
		}

		return;
	}

	idxmax = w * h - 1;
	mem = image->channel[MTPIXEL_CHANNEL_IMAGE];

	for ( i = 0; i < items; i++ )
	{
		c = 200 + (unsigned char)( EL_TABLE_VALUE ( table, i,
			EL_TAB_REGION) );

		target[c] ++;

		c = (unsigned char)( EL_TABLE_VALUE ( table, i,
			EL_TAB_COUNTY ) );

		target[c] ++;

		if (	idx[c] < idxmax &&
			! get_cxy ( c, idx, mem, &x, &y, w, idxmax, validate )
			)
		{
			EL_TABLE_VALUE ( table, i, EL_TAB_MAP_X ) = (double)x;
			EL_TABLE_VALUE ( table, i, EL_TAB_MAP_Y ) = (double)y;
		}
	}

	memset ( idx, 0, 256 * sizeof ( int ) );
	x = 0;
	for ( i = 0; i < idxmax; i++ )
	{
		c = mem[i];
		idx[c] ++;

		if ( c >= 1 && c <= 150 )
		{
			x++;
		}
	}

	if ( validate )
	{
		if ( x != items )
		{
			printf ( "Total seats found: %i (should be %i)\n\n",
				x, items );
		}

		for ( i = 1; i < 256; i++ )
		{
			if ( idx[i] != target[i] )
			{
				printf ( "%3i png = %5i   tsv = %5i  %i\n",
					i, idx[i], target[i],
					target[i] - idx[i] );
			}
		}
	}
}

int eleana_index_record_load (
	eleanaINDEX	* index,
	unsigned int	record,
	int		validate,
	CedSheet	** results,
	mtTree		** polymap,
	double		** table,
	int		* seats,
	mtTree		** seat_names,
	CedIndex	** seat_index
	)
{
	mtImage		* cartogram;
	char		* filename;
	mtTree		* new_seat_names;


	if (	! index			||
		! index->sheet_index	||
		record >= index->records ||
		! results		||
		! polymap		||
		! table			||
		! seats
		)
	{
		return 1;
	}

	filename = eleana_index_get ( index, record, ELEANA_INDEX_COL_FULL );
	if ( ! filename )
	{
		return 1;
	}

	results[0] = ced_sheet_load ( filename, "ISO-8859-1", NULL );
	if ( ! results[0] )
	{
		// Fail here as we must have the summary to do anything

		return 1;
	}

	if ( populate_table ( index, record + ELEANA_INDEX_ROW_START,
			results[0], seats, table, &new_seat_names ) )
	{
		ced_sheet_destroy ( results[0] );
		results[0] = NULL;

		return 1;
	}

	filename = eleana_index_get ( index, record, ELEANA_INDEX_COL_CARTOGRAM
		);

	if ( filename )
	{
		cartogram = mtpixel_image_load ( filename );
		if ( cartogram )
		{
			create_map_data ( table[0], cartogram, seats[0],
				validate );

			mtpixel_image_destroy ( cartogram );
		}
		else
		{
			if ( validate )
			{
				printf ( "Unable to load cartogram %s\n",
					filename );
			}
		}
	}

	filename = eleana_index_get ( index, record, ELEANA_INDEX_COL_POLYMAP );
	if ( filename )
	{
		polymap[0] = eleana_polymap_load ( filename, index, validate,
			seats[0], results[0] );
	}
	else
	{
		polymap[0] = NULL;
	}

	if ( seat_names )
	{
		seat_names[0] = new_seat_names;
	}
	else
	{
		mtkit_tree_destroy ( new_seat_names );
	}

	if ( seat_index )
	{
		seat_index[0] = ced_index_new ( CED_INDEX_TYPE_TEXT );
		if ( seat_index[0] )
		{
			ced_index_add_items ( seat_index[0], results[0],
				FULL_ROW_PARTY1, FULL_COL_SEAT_NAME, 0, 1 );
		}
	}

	return 0;
}

void eleana_index_record_close (
	CedSheet	* results,
	mtTree		* polymap,
	double		* table,
	mtTree		* seat_names,
	CedIndex	* seat_index
	)
{
	ced_sheet_destroy ( results );
	mtkit_tree_destroy ( polymap );
	free ( table );
	mtkit_tree_destroy ( seat_names );
	ced_index_destroy ( seat_index );
}

