/*
	Copyright (C) 2009-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/


#ifndef ELEANA_H
#define ELEANA_H

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <ctype.h>

#include <stdint.h>

#include <mtcelledit.h>
#include <mtpixel.h>



typedef struct
{
	char		* path;		// Dynamic string pointing to index path
	int		records;	// Number of records (rows) in the index

	CedBook		* book;		// Whole book containing all index
					// sheets
	CedSheet	* sheet_index;	// Index sheet
} eleanaINDEX;



#define ROW_DATA_PARTY_NAME		5
#define ROW_DATA_PARTY1			6

#define COL_DATA_SEAT_NAME		1
#define COL_MP				2
#define COL_DATA_REGION_NUM		3
#define COL_DATA_COUNTY_NAME		4
#define COL_ELECTORATE			5
#define COL_RES_A			6
#define COL_RES_B			7
#define COL_RES_C			8
#define COL_RES_O1			9
#define COL_RES_O2			10
#define COL_RES_O3			11
#define COL_RES_O4			12
#define COL_DATA_MAP_X			15
#define COL_DATA_MAP_Y			16
#define COL_DATA_COUNTY_NUM		17

#define COL_MAP_POLY			30

#define FULL_ROW_TITLES			2
#define FULL_ROW_PARTY1			4

#define FULL_COL_SEAT_NAME		1
#define FULL_COL_ELECTORATE		2
#define FULL_COL_MP_NAME		3
#define FULL_COL_PARTY			4
#define FULL_COL_VOTES			5
#define FULL_COL_COUNTY			6
#define FULL_COL_REGION			7

#define POLYMAP_WIDTH			640
#define POLYMAP_HEIGHT			1234
#define POLYMAP_ZOOM_MIN		0.3
#define POLYMAP_ZOOM_MAX		25.0

// row = 0..seats
// col = as EL_TAB_*
// table = pointer to table array
#define EL_TABLE_VALUE( table, row, col )	table[EL_TAB_TOTAL * row + col]

#ifndef MAX
#define MAX(a, b)  (((a) > (b)) ? (a) : (b))
#endif

#ifndef MIN
#define MIN(a, b)  (((a) < (b)) ? (a) : (b))
#endif


#define ELEANA_INDEX_ROW_START	2

// Sheet names in the index CED file
#define ELEANA_BOOK_SHEET_INDEX		"Index"
#define ELEANA_BOOK_SHEET_COUNTIES	"Counties"
#define ELEANA_BOOK_SHEET_REGIONS	"Regions"

enum
{
	ELEANA_INDEX_COL_YEAR =		1,
	ELEANA_INDEX_COL_SUMMARY =	2,
	ELEANA_INDEX_COL_FULL =		3,
	ELEANA_INDEX_COL_CARTOGRAM =	4,
	ELEANA_INDEX_COL_POLYMAP =	5,
	ELEANA_INDEX_COL_PARTY_A =	6,
	ELEANA_INDEX_COL_PARTY_B =	7,
	ELEANA_INDEX_COL_PARTY_C =	8,

	ELEANA_INDEX_COL_TOTAL
};

enum
{
	EL_SWING_AB,
	EL_SWING_AC,
	EL_SWING_AO,
	EL_SWING_BC,
	EL_SWING_BO,
	EL_SWING_CO,

	EL_SWING_TOTAL
};

enum
{
	EL_TAB_A,
	EL_TAB_B,
	EL_TAB_C,
	EL_TAB_O1,
	EL_TAB_O2,
	EL_TAB_O3,
	EL_TAB_O4,
	EL_TAB_O,

	EL_TAB_TURNOUT,
	EL_TAB_ELECTORATE,
	EL_TAB_REGION,
	EL_TAB_COUNTY,
	EL_TAB_MAP_X,
	EL_TAB_MAP_Y,
	EL_TAB_DIAGRAM_X,
	EL_TAB_DIAGRAM_Y,

	EL_TAB_PROJ_A,
	EL_TAB_PROJ_B,
	EL_TAB_PROJ_C,
	EL_TAB_PROJ_O1,
	EL_TAB_PROJ_O2,
	EL_TAB_PROJ_O3,
	EL_TAB_PROJ_O4,
	EL_TAB_PROJ_O,

	EL_TAB_PROJ_A_SEAT,
	EL_TAB_PROJ_B_SEAT,
	EL_TAB_PROJ_C_SEAT,
	EL_TAB_PROJ_O_SEAT,
	EL_TAB_PROJ_WINNER,

	EL_TAB_TOTAL,
};



// Functions return 0 on success, unless otherwise stated



int eleana_table_total_sum (
	double		* table,
	int		items,
	int		first,
	int		last
	);

int eleana_calculate_uniform_swing (
	double		* table,
	int		items,
	double		swing[],
	int		party_seats[]
	);

int eleana_calculate_eleana_swing (
	double		* table,
	int		items,
	double		swing[],
	int		party_seats[]
	);

int eleana_calculate_diagram (
	double		* table,
	int		items,
	int		max_x,
	int		max_y
	);

int eleana_swingset_prepare (		// Use this before
					// eleana_swingset_*_to_*
	double		* table,
	int		items,
	double		from[],
	double		to[]
	);

int eleana_swingset_eleana_to_uniform (
	double		from[],
	double		to[]
	);

int eleana_swingset_uniform_to_eleana (
	double		from[],
	double		to[]
	);



// Polygon key & data items for an mtTree

typedef struct
{
	double		x,
			y;
} mapPOLYpoint;

typedef struct
{
	int		seat;		// Seat index for global.table[?]
					// position
	double		minx,
			maxx,		// bounding
			miny,
			maxy;		// Sorted by miny first, then seat
} mapPOLYkey;

typedef struct
{
	int		points;		// Number of points in this polygon
	mapPOLYpoint	* array;	// Array of points
} mapPOLYdata;

typedef int (* elRenderFunc) (
	int		seat,
	mtBrush		** brush_fill,	// Put fill brush here -
					// NULL => don't fill
	mtBrush		** brush_outline, // Put outline brush here -
					// NULL => don't outline
	void		* user_data
	);
	// 0 = render seat
	// 1 = don't render seat
	// 2 = terminate



mtTree * eleana_poly_new (		// Creates a new tree of polygons
	CedSheet	* sheet,
	int		seats
	);

int eleana_poly_render (
	mtImage		* image,	// Image to render on
	int		x,		// Offset that the image origin starts
					// at
	int		y,		// Offset that the image origin starts
					// at
	mtTree		* tree,		// Polygon tree
	double		* table,	// Table of results
	double		zoom,		// Zoom factor for the polygons
	int		cols[5]		// Packed RGB colours for 4 parties +
					// ink
	);

int eleana_poly_render_custom (
	mtImage		* image,	// Image to render on
	int		x,		// Offset that the image origin starts
					// at
	int		y,		// Offset that the image origin starts
					// at
	mtTree		* tree,		// Polygon tree
	double		* table,	// Table of results
	double		zoom,		// Zoom factor for the polygons
	elRenderFunc	callback,	// Function that determines the fill
					// brush for a given seat
	void		* user_data
	);

mtTree * eleana_polymap_load (
	char		* filename,	// SVG file
	eleanaINDEX	* index,	// Used for matching names
	int		validate,	// Validate all data and output errors
					// to stdout
	int		tot_seats,	// Total number of seats
	CedSheet	* results	// Full results sheet
	);

eleanaINDEX * eleana_index_open (
	char		* filename	// Index file (CED book)
	);
	// NULL = fail

int eleana_index_close (		// Close the index, release all memory
	eleanaINDEX	* index
	);

char * eleana_index_get (		// Get index contents
	eleanaINDEX	* index,
	unsigned int	record,		// 0 = first record (row)
	unsigned int	field		// ELEANA_INDEX_COL_* (column)
	);
	// NULL = no data

char * eleana_get_seat_name (
	mtTree		* seat_names,	// List of seat names
	int		index		// Number of the seating, from 0
	);				// NULL = not found

int eleana_index_record_load (		// Load a set of files into memory from
					// this record
	eleanaINDEX	* index,
	unsigned int	record,		// 0 = first record (row)
	int		validate,	// Validate data and output errors to
					// stdout if = 1
	CedSheet	** results,	// Put full result sheet here
	mtTree		** polymap,	// Load SVG polymap file here as tree
					// of polygons
	double		** table,	// Large data table
	int		* seats,	// Put the total number of seats here
	mtTree		** seat_names,	// Put list of seat names here
					// (NULL = don't)
	CedIndex	** seat_index	// Put seat name index here
					// (NULL = don't)
	);
/*
Note that a NULL can still be returned in one of the result slots (cartogram,
polymap) to indicate an individual failure to load that data set in.
*/

void eleana_index_record_close (	// Close a set of data
	CedSheet	* results,
	mtTree		* polymap,
	double		* table,
	mtTree		* seat_names,
	CedIndex	* seat_index
	);



#endif		// ELEANA_H
