/*
	Copyright (C) 2008-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.

*/

#ifndef GUI_H
#define GUI_H

#include <unistd.h>
#include <time.h>
#include <math.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include <mtkit.h>
#include <mtpixel.h>
#include <mtcelledit.h>
#include <mtgex.h>

#include "eleana.h"


#ifdef U_NLS
	#include <libintl.h>
	#define _(text) gettext(text)
	#define __(text) gettext(text)
#else
	#define _(text) text
	#define __(text) text
#endif



#define _GW_LABEL_RES_COL_V	0
#define _GW_LABEL_RES_COL_VP	1
#define _GW_LABEL_RES_COL_VPS	2
#define _GW_LABEL_RES_COL_S	3
#define _GW_LABEL_RES_COL_SC	4
#define _GW_LABEL_RES_COLS	5

#define _INIFILE_FONT_FILENAME	"main.font_filename"
#define _INIFILE_FONT_SIZE	"main.font_size"
#define _INIFILE_LAST_DIR	"main.last_dir"
#define _INIFILE_MAIN_WINDOW	"main.window"
#define _INIFILE_FPICK_PRE	"main."
#define _INIFILE_FPICK_WINDOW	"main.fpick_window"

#define _INIFILE_LAST_YEAR	"main.last_year"

#define _INIFILE_GRAPH_COL_A	"graph.col_a"
#define _INIFILE_GRAPH_COL_B	"graph.col_b"
#define _INIFILE_GRAPH_COL_C	"graph.col_c"
#define _INIFILE_GRAPH_COL_D	"graph.col_d"
#define _INIFILE_GRAPH_COL_INK	"graph.col_ink"
#define _INIFILE_GRAPH_COL_PAGE	"graph.col_page"
#define _INIFILE_GRAPH_COL_G1	"graph.col_g1"
#define _INIFILE_GRAPH_COL_G2	"graph.col_g2"

#define _INIFILE_MAP_COL_A	"map.col_a"
#define _INIFILE_MAP_COL_B	"map.col_b"
#define _INIFILE_MAP_COL_C	"map.col_c"
#define _INIFILE_MAP_COL_D	"map.col_d"
#define _INIFILE_MAP_COL_INK	"map.col_ink"

#define	MAP_WIDTH		40
#define MAP_HEIGHT		48

// row = 0..seats
// col = as EL_TAB_*
#define EL_TAB_VALUE( row, col )	global.table[EL_TAB_TOTAL * row + col]

#define PART_NAME_SIZE	16

#define FS_SELECT_FILE 15
#define FS_SELECT_DIR 16

#define DATA_DIR		DATA_INSTALL "/" DATA_NAME "/"


enum
{
	_GW_SDRAW_MAP,
	_GW_SDRAW_AREA_MAP,

	_GW_GIMAGE_GRAPH,

	_GW_SPINSLIDER_SWING_AB,
	_GW_SPINSLIDER_SWING_AC,
	_GW_SPINSLIDER_SWING_AO,
	_GW_SPINSLIDER_SWING_BC,
	_GW_SPINSLIDER_SWING_BO,
	_GW_SPINSLIDER_SWING_CO,

	_GW_SPINSLIDER_MAPZOOM,

	_GW_BMENU_PICK_YEAR,

	_GW_LABEL_SWING_HEAD_AB,
	_GW_LABEL_SWING_HEAD_AC,
	_GW_LABEL_SWING_HEAD_AO,
	_GW_LABEL_SWING_HEAD_BC,
	_GW_LABEL_SWING_HEAD_BO,
	_GW_LABEL_SWING_HEAD_CO,

	_GW_LABEL_RESULTS_HEAD_A,
	_GW_LABEL_RESULTS_HEAD_B,
	_GW_LABEL_RESULTS_HEAD_C,
	_GW_LABEL_RESULTS_HEAD_O,
	_GW_LABEL_RESULTS_HEAD_TOTAL,

	_GW_LABEL_RES_A_V,
	_GW_LABEL_RES_A_VP,
	_GW_LABEL_RES_A_VPS,
	_GW_LABEL_RES_A_S,
	_GW_LABEL_RES_A_SC,

	_GW_LABEL_RES_B_V,
	_GW_LABEL_RES_B_VP,
	_GW_LABEL_RES_B_VPS,
	_GW_LABEL_RES_B_S,
	_GW_LABEL_RES_B_SC,

	_GW_LABEL_RES_C_V,
	_GW_LABEL_RES_C_VP,
	_GW_LABEL_RES_C_VPS,
	_GW_LABEL_RES_C_S,
	_GW_LABEL_RES_C_SC,

	_GW_LABEL_RES_O_V,
	_GW_LABEL_RES_O_VP,
	_GW_LABEL_RES_O_VPS,
	_GW_LABEL_RES_O_S,
	_GW_LABEL_RES_O_SC,

	_GW_LABEL_RES_T_V,
	_GW_LABEL_RES_T_VP,
	_GW_LABEL_RES_T_VPS,
	_GW_LABEL_RES_T_S,
	_GW_LABEL_RES_T_SC,

	_GW_TOTAL
};

enum	// File selector quarks
{
	_FS_XTRA_HBOX		= 1,
	_FS_SPIN_FONT_SIZE
};

enum
{
	FS_LOAD_PROJECT		= 1,
	FS_SAVE_GRAPH,
	FS_SAVE_POLYMAP,
	FS_PICK_FONT,
};



typedef struct
{
	mtPrefs		* prefs_file;
	char		** argv,
			mem_filename [ 2048 ],
//			* arg_filename,
			party_name [ 4 ][ PART_NAME_SIZE ]
			;
	int		argc
			;
	GtkWidget	* main_window,
			* gw [ _GW_TOTAL ],
			* cursor_left_table,
			* notebook_left
			;

	gexImage	* gimage_carto,
			* gimage_diagram;

	mtFont		* font;
	unsigned int	font_size,
			font_w,
			font_h;

	double		swing [ EL_SWING_TOTAL ],
			map_zoom;
	unsigned char	* map;
	mtImage		* diagram_image;
	int		seats,
			party_seats [ 4 ],
			eleana_swing,
			carto_zoom,
			diag_zoom;

	eleanaINDEX	* eleana_index;
	CedSheet	* sheet_full;
	double		* table;
	mtTree		* polymap_tree,	// Tree containing all of the map
					// polygons
			*seat_names;	// List of seat names indexed by number
					// 0..(seats-1)
	CedIndex	*seat_index;	// Index of seat coords by name
} GUI_Global;



GUI_Global	global;



int project_load (
	char * filename
	);

int project_save_graph (
	char * filename
	);

int project_save_polymap (
	char * filename
	);

int project_set_font (
	char * filename,
	int size
	);

void pressed_help (
	GtkMenuItem	* menu_item,
	gpointer	user_data,
	gint		item
	);

GtkTextBuffer * swing_report_window_init (
	char		* txt
	);

void file_selector (
	int		action_type,
	GtkWidget	* main_window,
	mtPrefs		* inifile,
	char		* filename
	);

void pressed_plot_graph (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	);


#endif		// GUI_H

