/*
	Copyright (C) 2009-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "gui.h"



static int help_window_active = 0;



static gboolean click_help_end (
	GtkWidget	* widget,
	GdkEvent	* event,
	gpointer	data
	)
{
	gtk_widget_destroy ( widget );
	help_window_active = 0;

	return FALSE;
}

static GtkTextBuffer * setup_help_report_window (
	char		* title
	)
{
	GtkWidget	* window,
			* vbox,
			* button,
			* textview,
			* sw;
	GtkAccelGroup	* ag;
	GtkTextBuffer	* buffer;


	ag = gtk_accel_group_new ();

	window = mtgex_add_a_window ( GTK_WINDOW_TOPLEVEL, title,
		GTK_WIN_POS_CENTER, TRUE );
	gtk_window_set_transient_for ( GTK_WINDOW ( window ),
		GTK_WINDOW ( global.main_window ) );

	vbox = gtk_vbox_new ( FALSE, 0 );
	gtk_container_add ( GTK_CONTAINER ( window ), vbox );
	gtk_widget_show ( vbox );

	sw = gtk_scrolled_window_new ( NULL, NULL );
	gtk_widget_show ( sw );
	gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW ( sw ),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
	gtk_box_pack_start ( GTK_BOX ( vbox ), sw, TRUE, TRUE, 0 );

	textview = gtk_text_view_new ();
	gtk_widget_show ( textview );
	gtk_container_add ( GTK_CONTAINER ( sw ), textview );
	gtk_text_view_set_editable ( GTK_TEXT_VIEW ( textview ), FALSE );
	gtk_text_view_set_cursor_visible ( GTK_TEXT_VIEW ( textview ), FALSE );
	gtk_text_view_set_wrap_mode ( GTK_TEXT_VIEW ( textview ),
		GTK_WRAP_WORD );

	buffer = gtk_text_view_get_buffer ( GTK_TEXT_VIEW ( textview ) );

	mtgex_add_hseparator ( vbox, -2, 10 );

	button = mtgex_pack ( vbox, gtk_button_new_with_label ( _("Close") ) );
	gtk_widget_add_accelerator ( button, "clicked", ag, GDK_KP_Enter, 0,
		(GtkAccelFlags) 0 );
	gtk_widget_add_accelerator ( button, "clicked", ag, GDK_Return, 0,
		(GtkAccelFlags) 0 );
	gtk_widget_add_accelerator ( button, "clicked", ag, GDK_Escape, 0,
		(GtkAccelFlags) 0 );
	GTK_WIDGET_SET_FLAGS ( button, GTK_CAN_DEFAULT );

	gtk_signal_connect_object ( GTK_OBJECT ( button ), "clicked",
		GTK_SIGNAL_FUNC ( click_help_end ), GTK_OBJECT ( window ) );
	gtk_signal_connect ( GTK_OBJECT ( window ), "delete_event",
		GTK_SIGNAL_FUNC ( click_help_end ), NULL);
	gtk_widget_show ( button );

	gtk_window_set_default_size ( GTK_WINDOW ( window ), 512, 400 );

	gtk_widget_show ( window );
	gtk_window_add_accel_group ( GTK_WINDOW ( window ), ag );
	help_window_active = 1;

	return buffer;
}

void pressed_help (
	GtkMenuItem	* menu_item,
	gpointer	user_data,
	gint		item
	)
{
	GtkTextBuffer	* buffer;
	gchar		* about_text =

	VERSION"\n"
	"\n"
	"Copyright (C) 2009-2013 Mark Tyler.\n"
	"\n"
	"This program is free software: you can redistribute it and/or modify "
	"it under the terms of the GNU General Public License as published by "
	"the Free Software Foundation, either version 3 of the License, or "
	"(at your option) any later version.\n"
	"\n"
	"This program is distributed in the hope that it will be useful, "
	"but WITHOUT ANY WARRANTY; without even the implied warranty of "
	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
	"GNU General Public License for more details.\n"
	"\n"
	"You should have received a copy of the GNU General Public License "
	"along with this program.  If not, see http://www.gnu.org/licenses/\n"
			;

	buffer = setup_help_report_window ( _("About") );
	gtk_text_buffer_set_text ( buffer, about_text, -1 );
}

GtkTextBuffer * swing_report_window_init (
	char		* txt
	)
{
	return setup_help_report_window ( txt );
}



///	FILE SELECTION WINDOW

// Does file already exist?  Ask if OK to overwrite
static int check_file (
	char		* fname
	)
{
	char		* msg,
			* f8;
	int		res = 0;


	if ( mtkit_file_valid ( fname ) == 0 )
	{
		f8 = mtgex_gtkuncpy ( NULL, fname, 0 );
		msg = g_strdup_printf (
		_("File: %s already exists. Do you want to overwrite it?"),
			f8 );

		res = mtgex_alert_box ( _("File Found"), msg, _("NO"),
			_("YES"), NULL, global.main_window ) != 2;

		g_free ( msg );
		g_free ( f8 );
	}

	return res;
}

static gboolean fs_destroy (
	GtkWidget	* fs
	)
{
	mtgex_store_window_position ( fs, global.prefs_file,
		_INIFILE_FPICK_WINDOW );
	mtgex_fpick_destroy ( fs );

	return FALSE;
}

static void fs_ok (
	GtkWidget	* fs
	)
{
	GtkWidget	* xtra_hbox,
			* w;
	char		fname[MTGEX_PATHBUF],
			* c,
			c2;
	int		action_type,
			s;


	action_type = (intptr_t) gtk_object_get_user_data ( GTK_OBJECT ( fs ) );
	xtra_hbox = (GtkWidget *) g_object_get_qdata ( G_OBJECT ( fs ),
		_FS_XTRA_HBOX );

	/* Needed to show progress in Windows GTK+2 */
	gtk_window_set_modal ( GTK_WINDOW ( fs ), FALSE);

	/* Looks better if no dialog under progressbar */
	mtgex_store_window_position ( fs, global.prefs_file,
		_INIFILE_FPICK_WINDOW );
	gtk_widget_hide ( fs );

	/* File extension */
	mtkit_strncpy0 ( fname, mtgex_fpick_get_filename ( fs, TRUE ),
		MTGEX_PATHBUF );
	mtgex_fpick_set_filename ( fs, fname, TRUE );

	mtkit_strncpy0 ( fname, mtgex_fpick_get_filename ( fs, FALSE ),
		MTGEX_PATHBUF);

	switch (action_type)
	{
	case FS_LOAD_PROJECT:
		if ( project_load ( fname ) )
		{
			goto redo;
		}
		break;

	case FS_SAVE_GRAPH:
		if ( check_file ( fname ) )
		{
			goto redo;
		}

		if ( project_save_graph ( fname ) )
		{
			goto redo;
		}
		break;

	case FS_SAVE_POLYMAP:
		if ( check_file ( fname ) )
		{
			goto redo;
		}

		if ( project_save_polymap ( fname ) )
		{
			goto redo;
		}

		break;

	case FS_PICK_FONT:
		w = (GtkWidget *) g_object_get_qdata ( G_OBJECT ( xtra_hbox ),
			_FS_SPIN_FONT_SIZE );

		if ( w )
		{
			s = (int)gtk_spin_button_get_value (
				GTK_SPIN_BUTTON ( w ) );
		}
		else
		{
			s = 12;
		}

		if ( project_set_font ( fname, s ) )
		{
			goto redo;
		}

		break;
	}

	c = strrchr ( fname, MTKIT_DIR_SEP );
	if ( c )
	{
		c2 = c[0];
		c[0] = 0;		// Strip off filename
		mtkit_prefs_set_str ( global.prefs_file, _INIFILE_LAST_DIR,
			fname );
		c[0] = c2;
	}

	mtgex_fpick_destroy ( fs );

	return;

redo:
	gtk_widget_show ( fs );
	gtk_window_set_modal ( GTK_WINDOW ( fs ), TRUE );
}

static void fs_setup (
	GtkWidget	* fs,
	GtkWidget	* xtra_hbox,
	int		action_type,
	char		* filename
	)
{
	char		txt[MTGEX_PATHBUF],
			* last;


	if (	action_type == FS_PICK_FONT &&
		filename &&
		filename[0]
		)
	{
		strncpy ( txt, filename, MTGEX_PATHBUF );
		// If we have a filename and saving
	}
	else
	{
		last = NULL;
		mtkit_prefs_get_str ( global.prefs_file, _INIFILE_LAST_DIR,
			&last );

		if ( ! last || ! last[0] )
		{
			last = mtkit_file_home ();
		}

		snprintf ( txt, MTGEX_PATHBUF, "%s%c", last, MTKIT_DIR_SEP );
	}

	gtk_window_set_modal ( GTK_WINDOW ( fs ), TRUE);
	mtgex_restore_window_position ( fs, global.prefs_file,
		_INIFILE_FPICK_WINDOW );

	gtk_object_set_user_data ( GTK_OBJECT ( fs ),
		(gpointer)(intptr_t)action_type );
	mtgex_fpick_setup ( fs, xtra_hbox, GTK_SIGNAL_FUNC ( fs_ok ),
		GTK_SIGNAL_FUNC ( fs_destroy ) );
	mtgex_fpick_set_filename ( fs, txt, FALSE );

	if ( xtra_hbox )
	{
		g_object_set_qdata ( G_OBJECT ( fs ), _FS_XTRA_HBOX,
			xtra_hbox );
	}

	gtk_widget_show ( fs );
	gtk_window_set_transient_for ( GTK_WINDOW ( fs ),
		GTK_WINDOW ( global.main_window ) );

	// Needed to ensure window is at the top
	gdk_window_raise ( fs->window );
}

void file_selector (
	int		action_type,
	GtkWidget	* main_window,
	mtPrefs		* prefs_file,
	char		* filename
	)
{
	GtkWidget	* hbox = NULL,
			* label,
			* spin;
	char		* title = NULL,
			* font_filename;
	int		fpick_flags = MTGEX_FPICK_ENTRY,
			num;


	switch ( action_type )
	{
	case FS_LOAD_PROJECT:
		title = _("Load Project File");
		fpick_flags = MTGEX_FPICK_LOAD;
		break;

	case FS_SAVE_GRAPH:
		title = _("Save Graph PNG File");
		break;

	case FS_SAVE_POLYMAP:
		title = _("Save Map PNG File");
		break;

	case FS_PICK_FONT:
		title = _("Choose Font File");

		hbox = gtk_hbox_new ( FALSE, 0 );
		gtk_widget_show ( hbox );

		label = gtk_label_new ( _("Font Size:") );
		gtk_widget_show ( label );
		gtk_box_pack_start ( GTK_BOX ( hbox ), label, FALSE, FALSE, 5 );

		spin = gtk_spin_button_new_with_range ( 6, 72, 1 );
		gtk_spin_button_set_digits ( GTK_SPIN_BUTTON ( spin ), 0 );

		num = 16;
		mtkit_prefs_get_int ( global.prefs_file, _INIFILE_FONT_SIZE,
			&num );
		gtk_spin_button_set_value ( GTK_SPIN_BUTTON ( spin ), num );

		gtk_widget_show ( spin );
		gtk_box_pack_start ( GTK_BOX ( hbox ), spin, FALSE, FALSE, 5 );

		font_filename = NULL;
		mtkit_prefs_get_str ( global.prefs_file, _INIFILE_FONT_FILENAME,
			&font_filename );

		if ( font_filename && ! font_filename[0] )
		{
			font_filename = NULL;
		}

		if ( font_filename )
		{
			filename = font_filename;
		}

		g_object_set_qdata ( G_OBJECT ( hbox ), _FS_SPIN_FONT_SIZE,
			spin );

		break;
	}

	fs_setup ( mtgex_fpick_new ( title, fpick_flags, main_window,
		global.prefs_file, NULL ),
		hbox, action_type, filename );
}
