/*
	Copyright (C) 2009-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "gui.h"



#define GRAPH_SIZE 200

enum
{
	BRUSH_A, BRUSH_B, BRUSH_C, BRUSH_D,
	BRUSH_INK, BRUSH_PAGE, BRUSH_G1, BRUSH_G2,

	BRUSH_TOTAL
};

enum
{
	RULER_X_TEXT_1,
	RULER_X_TEXT_2,
	RULER_X_TEXT_3,
	RULER_X_TEXT_4,

	RULER_X_GRAPH_1,

	RULER_Y_TEXT_1,

	RULER_Y_GRAPH_1,
	RULER_Y_GRAPH_2,
	RULER_Y_GRAPH_3,
	RULER_Y_GRAPH_4,

	RULER_TOTAL
};


typedef struct
{
	mtImage		* image;
	mtBrush		* brush[BRUSH_TOTAL];
	int		ruler[RULER_TOTAL],
			axis,
			x,
			y
			;
} graphSTATE;

typedef struct
{
	double		value;		// > 0 = win < 0 = loss
	int		row;		// Row in global.table of this item
} graphTkey;

typedef struct
{
	int		party;		// Other party who came 1st / 2nd
} graphTdata;



static void graph_finish (
	graphSTATE	* state
	)
{
	int		i;


	mtgex_image_set_image ( global.gw[_GW_GIMAGE_GRAPH], state->image, 1 );

	for ( i = 0; i < BRUSH_TOTAL; i++ )
	{
		mtpixel_brush_destroy ( state->brush[i] );
	}
}

static void graph_setup (
	graphSTATE	* state
	)
{
	char		* pref_names[BRUSH_TOTAL] = {
			_INIFILE_GRAPH_COL_A,
			_INIFILE_GRAPH_COL_B,
			_INIFILE_GRAPH_COL_C,
			_INIFILE_GRAPH_COL_D,
			_INIFILE_GRAPH_COL_INK,
			_INIFILE_GRAPH_COL_PAGE,
			_INIFILE_GRAPH_COL_G1,
			_INIFILE_GRAPH_COL_G2
			};
	int		w,
			h,
			i,
			col,
			x,
			tw;


	tw = global.font->height;
	x = 0;

	state->ruler[RULER_X_TEXT_1] = x = x + 5 * tw;
	state->ruler[RULER_X_TEXT_2] = x = x + 5 * tw;
	state->ruler[RULER_X_TEXT_3] = x = x + 10 * tw;
	state->ruler[RULER_X_TEXT_4] = x = x + 5 * tw;

	state->ruler[RULER_X_GRAPH_1] = state->ruler[RULER_X_TEXT_1];

	state->ruler[RULER_Y_TEXT_1] = tw;
	state->ruler[RULER_Y_GRAPH_1] = tw * 10 + 0 * (GRAPH_SIZE * 2 + 1 + tw);
	state->ruler[RULER_Y_GRAPH_2] = tw * 10 + 1 * (GRAPH_SIZE * 2 + 1 + tw);
	state->ruler[RULER_Y_GRAPH_3] = tw * 10 + 2 * (GRAPH_SIZE * 2 + 1 + tw);
	state->ruler[RULER_Y_GRAPH_4] = tw * 10 + 3 * (GRAPH_SIZE * 2 + 1 + tw);

	w = tw * 10 + global.seats;
	h = tw * 10 + 4 * ( GRAPH_SIZE * 2 + 1 + tw );

	state->image = mtpixel_image_new ( w, h, MTPIXEL_IMAGE_RGB,
		MTPIXEL_CHANNEL_MASK_IMAGE );

	for ( i = 0; i < BRUSH_TOTAL; i++ )
	{
		state->brush[i] = mtpixel_brush_new ();

		if ( ! state->brush[i] )
		{
			continue;
		}

		col = 0;
		mtkit_prefs_get_int ( global.prefs_file, pref_names[i], &col );
		mtpixel_brush_set_RGB ( state->brush[i], col, -1 );
		mtpixel_brush_set_size ( state->brush[i], 0 );
	}

	mtpixel_brush_rectangle ( state->image, state->brush[BRUSH_PAGE], 0, 0,
		w - 1, h - 1 );
}

static int graphT_cmp (
	void		* k1,
	void		* k2
	)
{
	graphTkey	* key1 = k1,
			* key2 = k2;


	if ( key1->value < key2->value )
	{
		return -1;
	}

	if ( key1->value > key2->value )
	{
		return 1;
	}

	if ( key1->row < key2->row )
	{
		return -1;
	}

	if ( key1->row > key2->row )
	{
		return 1;
	}

	return 0;
}

static void graphT_delete (
	mtTreeNode	* node
	)
{
	free ( node->data );
	free ( node->key );
}

static void graph_recurse (
	graphSTATE	* state,
	mtTreeNode	* node
	)
{
	double		v;
	graphTkey	* key;
	graphTdata	* data;


	if ( ! node )
	{
		return;
	}

	key = node->key;
	data = node->data;

	graph_recurse ( state, node->left );

	v = GRAPH_SIZE * 2 * key->value;

	if ( v < -GRAPH_SIZE )
	{
		v = -GRAPH_SIZE;
	}

	if ( v > GRAPH_SIZE )
	{
		v = GRAPH_SIZE;
	}

	mtpixel_brush_rectangle ( state->image,
		state->brush[BRUSH_A + data->party], state->x, state->y,
		state->x, state->y + v );
	mtpixel_brush_rectangle ( state->image,
		state->brush[BRUSH_A + state->axis], state->x, state->y,
		state->x, state->y - v );

	state->x ++;

	graph_recurse ( state, node->right );
}

static void graph_draw_plot (
	graphSTATE	* state
	)
{
	double		valp;
	int		i,
			j,
			best_i,
			best_vote,
			win,
			party[4] = {
				EL_TAB_PROJ_A,
				EL_TAB_PROJ_B,
				EL_TAB_PROJ_C,
				EL_TAB_PROJ_O
				};
	mtTree		* tree;
	graphTkey	* key;
	graphTdata	* data;


	tree = mtkit_tree_new ( graphT_cmp, graphT_delete );
	if ( ! tree )
	{
		return;
	}

	for ( i = 0; i < global.seats; i++ )
	{
		win = EL_TAB_VALUE ( i, EL_TAB_PROJ_WINNER ) - 1;
		if ( win < 0 || win > 3 )
		{
			continue;
		}

		key = calloc ( sizeof ( graphTkey ), 1 );
		data = calloc ( sizeof ( graphTdata ), 1 );
		if ( ! key || ! data )
		{
			free ( key );
			free ( data );

			break;
		}

	// Find winner (if not this party), or second (if this party wins)

		if ( win == state->axis )
		{
			// Find second placed party
			best_i = 0;
			best_vote = 0;

			for ( j = 0; j < 4; j++ )
			{
				if ( j == win )
				{
					continue;
				}

				if ( EL_TAB_VALUE ( i, party[j] ) > best_vote )
				{
					best_vote = EL_TAB_VALUE ( i,
						party[j] );
					best_i = j;
				}
			}
		}
		else
		{
			best_i = win;
		}

		valp = ( EL_TAB_VALUE ( i, party[state->axis] ) -
			EL_TAB_VALUE ( i, party[best_i] ) ) /
			EL_TAB_VALUE ( i, EL_TAB_TURNOUT );

		key->value	= valp;
		key->row	= i;
		data->party	= best_i;

		if ( ! mtkit_tree_node_add ( tree, key, data ) )
		{
			free ( key );
			free ( data );

			break;
		}
	}

	state->x = state->ruler[RULER_X_GRAPH_1];
	state->y = state->ruler[RULER_Y_GRAPH_1 + state->axis] + GRAPH_SIZE;
	graph_recurse ( state, tree->root );

	mtkit_tree_destroy ( tree );
}

static void graph_draw_axis (
	graphSTATE	* state
	)
{
	char		* txt[3] = { "50%", "0%", "-50%" };
	int		i,
			x,
			y,
			x1,
			y1,
			x2,
			y2,
			yy[4] = {
				state->ruler[RULER_Y_GRAPH_1],
				state->ruler[RULER_Y_GRAPH_2],
				state->ruler[RULER_Y_GRAPH_3],
				state->ruler[RULER_Y_GRAPH_4]
			};
	mtFontRender	* ren;
	mtImage		* ti;


	if (	state->axis < 0 ||
		state->axis > 3
		)
	{
		return;
	}

	ren = mtpixel_font_render_new ( global.font, NULL,
		state->brush[BRUSH_INK], MTPIXEL_IMAGE_RGB );

	if ( ! ren )
	{
		return;
	}

	x1 = state->ruler[RULER_X_GRAPH_1];
	y1 = yy[state->axis];
	x2 = x1 + global.seats - 1;
	y2 = y1 + GRAPH_SIZE * 2;

	mtpixel_brush_rectangle ( state->image, state->brush[BRUSH_G1], x1, y1,
		x2, y2 );

	for ( i = 0; i < 12; i++ )
	{
		y = y1 + GRAPH_SIZE / 5 * i;
		mtpixel_brush_rectangle ( state->image, state->brush[BRUSH_G2],
			x1, y, x2, y );
	}

	for ( i = 0; i < 5; i++ )
	{
		x = ( x1 * (4 - i) + x2 * i ) / 4;
		mtpixel_brush_rectangle ( state->image, state->brush[BRUSH_G2],
			x, y1, x, y2 );
	}

	mtpixel_brush_rectangle ( state->image, state->brush[BRUSH_INK], x1 - 8,
		y1, x1 - 8, y2 );

	for ( i = 0; i < 3; i++ )
	{
		y = ( y1 * (2 - i) + y2 * i ) / 2;
		mtpixel_brush_rectangle ( state->image, state->brush[BRUSH_INK],
			x1 - 8, y, x1 - 16, y );

		ren->utf8 = txt[i];
		ti = mtpixel_font_render ( ren );
		if ( ti )
		{
			x = x1 - 16 - ti->width;
			y = y - ti->height / 2;
			mtpixel_image_paste ( state->image, ti,
				state->brush[BRUSH_INK], x, y );
			mtpixel_image_destroy ( ti );
		}
	}

	mtpixel_font_render_destroy ( ren );
}

void pressed_plot_graph (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		i;
	graphSTATE	state;


	if (	! global.gw[_GW_GIMAGE_GRAPH] ||
		! global.font ||
		global.seats < 1
		)
	{
		return;
	}

	memset ( &state, 0, sizeof ( state ) );

	graph_setup ( &state );

	for ( i = 0; i < 4; i++ )
	{
		state.axis = i;

		graph_draw_axis ( &state );
		graph_draw_plot ( &state );
	}

	graph_finish ( &state );
}

