/*
	Copyright (C) 2009-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "gui.h"

#include "graphics/icon.xpm"



// Forward refs
static void map_redraw_ini (
	mtPrefItemValue	* piv,
	int		callback_data
	);



GUI_Global		global = { NULL };


static mtPrefItem	prefs_table[] = {

{ _INIFILE_LAST_DIR,	MTKIT_PREF_TYPE_STR },

{ _INIFILE_LAST_YEAR,	MTKIT_PREF_TYPE_INT, "0" },

{ _INIFILE_MAIN_WINDOW"_x", MTKIT_PREF_TYPE_INT, "0" },
{ _INIFILE_MAIN_WINDOW"_y", MTKIT_PREF_TYPE_INT, "0" },
{ _INIFILE_MAIN_WINDOW"_w", MTKIT_PREF_TYPE_INT, "500" },
{ _INIFILE_MAIN_WINDOW"_h", MTKIT_PREF_TYPE_INT, "500" },

{ _INIFILE_GRAPH_COL_A, MTKIT_PREF_TYPE_RGB, "255" },
{ _INIFILE_GRAPH_COL_B, MTKIT_PREF_TYPE_RGB, "16711680" },
{ _INIFILE_GRAPH_COL_C, MTKIT_PREF_TYPE_RGB, "16776960" },
{ _INIFILE_GRAPH_COL_D, MTKIT_PREF_TYPE_RGB, "11842740" },
{ _INIFILE_GRAPH_COL_INK, MTKIT_PREF_TYPE_RGB, "0" },
{ _INIFILE_GRAPH_COL_PAGE, MTKIT_PREF_TYPE_RGB, "16777215" },
{ _INIFILE_GRAPH_COL_G1, MTKIT_PREF_TYPE_RGB, "14474460" },
{ _INIFILE_GRAPH_COL_G2, MTKIT_PREF_TYPE_RGB, "13158600" },

{ _INIFILE_MAP_COL_A, MTKIT_PREF_TYPE_RGB, "3302655", NULL, map_redraw_ini },
{ _INIFILE_MAP_COL_B, MTKIT_PREF_TYPE_RGB, "16711680", NULL, map_redraw_ini },
{ _INIFILE_MAP_COL_C, MTKIT_PREF_TYPE_RGB, "16776960", NULL, map_redraw_ini },
{ _INIFILE_MAP_COL_D, MTKIT_PREF_TYPE_RGB, "11842740", NULL, map_redraw_ini },
{ _INIFILE_MAP_COL_INK, MTKIT_PREF_TYPE_RGB, "0", NULL, map_redraw_ini },

{ _INIFILE_FONT_FILENAME, MTKIT_PREF_TYPE_FILE },
{ _INIFILE_FONT_SIZE,	MTKIT_PREF_TYPE_INT, "16" },

{NULL}
};



static void reset_map_zoom_pos ( void )
{
	GtkWidget	* area;
	int		w,
			h;


	area = global.gw[ _GW_SDRAW_MAP ];
	w = area->allocation.width;
	h = area->allocation.height;

	mtgex_sdraw_set ( area, w, h + POLYMAP_HEIGHT / 2,
		POLYMAP_WIDTH + 2 * w, POLYMAP_HEIGHT + 2 * h
		);

	// This hack ensures the x / y map position is as I want
	mtgex_spinslide_set_value ( global.gw[_GW_SPINSLIDER_MAPZOOM], 1.01 );
	mtgex_spinslide_set_value ( global.gw[_GW_SPINSLIDER_MAPZOOM], 1.0 );
}

static void init_carto_area ( void )
{
	mtgex_image_set_zoom ( global.gimage_carto->scrolled_window,
		global.carto_zoom );
}

static void init_diagram_area ( void )
{
	mtgex_image_set_zoom ( global.gimage_diagram->scrolled_window,
		global.diag_zoom );
}

static void error_message_file (
	char		* message,
	char		* filename
	)
{
	char		* msg,
			* f8;


	f8 = mtgex_gtkuncpy ( NULL, filename, 0 );
	msg = g_strdup_printf ( _("Unable to %s file: %s"), message, f8 );
	mtgex_alert_box ( _("Error"), msg, _("OK"), NULL, NULL,
		global.main_window );
	g_free ( msg );
	g_free ( f8 );
}

static void update_menus ( void )	// Update undo/redo menus
{
}

// Update filename in titlebar and the menus
static void update_titlebar ( void )
{
	char		txt[300],
			txt2[MTGEX_PATHBUF];


	mtgex_gtkuncpy ( txt2, global.mem_filename, MTGEX_PATHBUF );

	snprintf ( txt, 290, "%s - %s", VERSION,
		txt2[0] ? txt2 : _("Untitled") );

	gtk_window_set_title ( GTK_WINDOW ( global.main_window ), txt );
}

static void purge_index_loaded ( void )
{
	eleana_index_close ( global.eleana_index );
	global.eleana_index = NULL;
}

static void purge_data_loaded ( void )
{
	eleana_index_record_close ( global.sheet_full, global.polymap_tree,
		global.table, global.seat_names, global.seat_index );

	global.sheet_full = NULL;
	global.table = NULL;
	global.polymap_tree = NULL;
	global.seat_names = NULL;
	global.seat_index = NULL;
}

static void delete_event (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	mtgex_store_window_position ( global.main_window, global.prefs_file,
		_INIFILE_MAIN_WINDOW );

	purge_data_loaded ();
	purge_index_loaded ();

	gtk_main_quit ();
}

static void clear_all ( void )
{
	purge_data_loaded ();

	global.mem_filename[0] = 0;
	update_titlebar ();
	update_menus ();
}

static void pressed_file_action (
	GtkMenuItem	* menu_item,
	gpointer	user_data,
	gint		item
	)
{
	file_selector ( item, global.main_window, global.prefs_file,
		global.mem_filename );
}

static void diagram_redraw ( void )
{
	unsigned char	* dest,
			* d1;
	int		i,
			x,
			y,
			w,
			h;


	if (	! global.gimage_diagram		||
		! global.gimage_diagram->image	||
		! global.diagram_image		||
		! global.diagram_image->channel[MTPIXEL_CHANNEL_IMAGE]
		)
	{
		return;
	}

	w = global.gimage_diagram->image->width;
	h = global.gimage_diagram->image->height;
	i = w * h;
	d1 = global.gimage_diagram->image->channel[MTPIXEL_CHANNEL_IMAGE];
	memcpy ( d1, global.diagram_image->channel[MTPIXEL_CHANNEL_IMAGE], i );

	for ( i = 0; i < global.seats; i++ )
	{
		x = EL_TABLE_VALUE ( global.table, i, EL_TAB_DIAGRAM_X ) + 0.5;
		y = EL_TABLE_VALUE ( global.table, i, EL_TAB_DIAGRAM_Y ) + 0.5;

		if (	x < 0	||
			y < 0	||
			x >= w	||
			y >= h
			)
		{
			continue;
		}

		dest = d1 + (x + y * w);
		*dest++ = 0;
	}

	gtk_widget_queue_draw ( global.gimage_diagram->drawing_area );
}

static void polymap_redraw ( void )
{
	gtk_widget_queue_draw ( global.gw[ _GW_SDRAW_AREA_MAP ] );
}

static void map_redraw_ini (
	mtPrefItemValue	* piv,
	int		callback_data
	)
{
	polymap_redraw ();
}

static void map_redraw ( void )
{
	unsigned char	pal[4][3] = {
			{ 0, 0, 255 },
			{ 255, 0, 0 },
			{ 255, 255, 0 },
			{ 200, 200, 200 }
			},
			c,
			* dest,
			* d1;
	int		r,
			x,
			y;


	if (	! global.gimage_carto		||
		! global.gimage_carto->image	||
		! global.table
		)
	{
		return;
	}

	d1 = global.gimage_carto->image->channel[MTPIXEL_CHANNEL_IMAGE];

	// Clear the old rubbish
	memset ( d1, 0, MAP_WIDTH * MAP_HEIGHT * 3 );

	for ( r = 0; r < global.seats; r++ )
	{
		x = EL_TAB_VALUE ( r, EL_TAB_MAP_X );
		y = EL_TAB_VALUE ( r, EL_TAB_MAP_Y );
		c = EL_TAB_VALUE ( r, EL_TAB_PROJ_WINNER ) - 1;

		if ( x < 0 || x >= MAP_WIDTH )
		{
			continue;
		}

		if ( y < 0 || y >= MAP_HEIGHT )
		{
			continue;
		}

		if ( c > 3 )
		{
			continue;
		}

		dest = d1 + 3 * (x + y * MAP_WIDTH);
		*dest++ = pal[c][0];
		*dest++ = pal[c][1];
		*dest++ = pal[c][2];
	}

	gtk_widget_queue_draw ( global.gimage_carto->drawing_area );
	polymap_redraw ();
}

static void swing_sheet_update ( void )
{
	double		p1,
			p2;
	char		txt[128];
	int		i,
			j,
			tab[4][3] = {
			{ EL_TAB_PROJ_A, EL_TAB_PROJ_A_SEAT, EL_TAB_A },
			{ EL_TAB_PROJ_B, EL_TAB_PROJ_B_SEAT, EL_TAB_B },
			{ EL_TAB_PROJ_C, EL_TAB_PROJ_C_SEAT, EL_TAB_C },
			{ EL_TAB_PROJ_O, EL_TAB_PROJ_O_SEAT, EL_TAB_O }
			};


	if ( ! global.table )
	{
		return;
	}

	if ( global.eleana_swing )
	{
		eleana_calculate_eleana_swing ( global.table, global.seats,
			global.swing, global.party_seats );
	}
	else
	{
		eleana_calculate_uniform_swing ( global.table, global.seats,
			global.swing, global.party_seats );
	}

	for ( i = 0; i < 4; i++ )
	{
		j = rint ( EL_TAB_VALUE ( global.seats, tab[i][0] ) );
		sprintf ( txt, "%i", j );
		gtk_label_set_text ( GTK_LABEL ( global.gw[_GW_LABEL_RES_A_V +
			i * _GW_LABEL_RES_COLS + _GW_LABEL_RES_COL_V] ), txt );

		p1 = 100 * ( (double)j ) /
			EL_TAB_VALUE ( global.seats, EL_TAB_TURNOUT );

		sprintf ( txt, "%.1f%%", p1 );
		gtk_label_set_text ( GTK_LABEL ( global.gw[_GW_LABEL_RES_A_V +
			i * _GW_LABEL_RES_COLS + _GW_LABEL_RES_COL_VP]), txt );

		p2 = p1 - 100 * EL_TAB_VALUE ( global.seats, tab[i][2] ) /
			EL_TAB_VALUE ( global.seats, EL_TAB_TURNOUT );

		if ( p2 < 0.001 && p2 > -0.001 )
		{
			p2 = 0;		// Stops irritating -0.0
		}

		sprintf ( txt, "%.1f%%", p2 );
		gtk_label_set_text ( GTK_LABEL ( global.gw[_GW_LABEL_RES_A_V +
			i * _GW_LABEL_RES_COLS + _GW_LABEL_RES_COL_VPS] ),
			txt );

		j = EL_TAB_VALUE ( global.seats, tab[i][1] );

		sprintf ( txt, "%i", j );
		gtk_label_set_text ( GTK_LABEL ( global.gw[_GW_LABEL_RES_A_V +
			i * _GW_LABEL_RES_COLS + _GW_LABEL_RES_COL_S] ), txt );
	}

	sprintf ( txt, "%i", global.seats );
	gtk_label_set_text ( GTK_LABEL ( global.gw[_GW_LABEL_RES_T_S] ), txt );

	j = rint ( EL_TAB_VALUE ( global.seats, EL_TAB_PROJ_A ) +
		EL_TAB_VALUE ( global.seats, EL_TAB_PROJ_B ) +
		EL_TAB_VALUE ( global.seats, EL_TAB_PROJ_C ) +
		EL_TAB_VALUE ( global.seats, EL_TAB_PROJ_O ) );

	sprintf ( txt, "%i", j );
	gtk_label_set_text ( GTK_LABEL ( global.gw[_GW_LABEL_RES_T_V] ), txt );

	i = EL_TAB_VALUE ( global.seats, EL_TAB_TURNOUT );
	sprintf ( txt, "%.1f%%", 100 * ( (double)j ) / i );
	gtk_label_set_text ( GTK_LABEL ( global.gw[_GW_LABEL_RES_T_VP] ), txt );

	for ( i = 0; i < 4; i++ )
	{
		sprintf ( txt, "%i", (int)EL_TAB_VALUE ( global.seats,
			EL_TAB_PROJ_A_SEAT + i ) - global.party_seats[i] );

		gtk_label_set_text ( GTK_LABEL ( global.gw[_GW_LABEL_RES_A_SC +
			i * _GW_LABEL_RES_COLS] ), txt );
	}

	map_redraw ();

	if (	global.gimage_diagram &&
		global.gimage_diagram->image
		)
	{
		int		w, h;


		w = global.gimage_diagram->image->width;
		h = global.gimage_diagram->image->height;

		eleana_calculate_diagram ( global.table, global.seats, w - 1,
			h - 1 );

		diagram_redraw ();
	}
}

static void pressed_reset_swings (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	int		i;


	for ( i = _GW_SPINSLIDER_SWING_AB; i <= _GW_SPINSLIDER_SWING_CO; i++ )
	{
		mtgex_spinslide_set_value ( global.gw[i], 0 );
	}
}

static void map_zoom_changed (
	GtkWidget	* widget,
	gpointer	user_data
	)
{
	int		nv_h,
			nv_v,
			w,
			h;
	double		cx = -1,
			cy = -1,
			aw,
			ah;
	GtkWidget	* area;
	gexSdraw	* sdraw;


	area = global.gw[ _GW_SDRAW_MAP ];

	if ( mtgex_sdraw_get ( area, &sdraw ) || ! sdraw )
	{
		return;
	}

	aw = sdraw->drawing_area->allocation.width;
	ah = sdraw->drawing_area->allocation.height;

	cx = sdraw->x - aw + aw/2;
	cy = sdraw->y - ah + ah/2;
	cx /= (sdraw->w - 2 * aw);
	cy /= (sdraw->h - 2 * ah);

	global.map_zoom = mtgex_spinslide_read_value (
		global.gw[_GW_SPINSLIDER_MAPZOOM] );

	if ( cx > 1 ) cx = 1;
	if ( cx < 0 ) cx = 0;
	if ( cy > 1 ) cy = 1;
	if ( cy < 0 ) cy = 0;

	w = rint ( global.map_zoom * POLYMAP_WIDTH );
	h = rint ( global.map_zoom * POLYMAP_HEIGHT );

	nv_h = rint ( cx * w - aw / 2 );
	nv_v = rint ( cy * h - ah / 2 );

	mtgex_sdraw_set ( area,
		nv_h + aw,
		nv_v + ah,
		w + aw * 2,
		h + ah * 2 );
}



#define UPTAB_SPACE_H	4
#define UPTAB_SPACE_V	1

typedef struct
{
	int		last_seat;
	mtImage		* image;
	mtBrush		* brush;
} uptabSTATE;



static int uptab_cb (
	int		seat,
	mtBrush		** brush_fill,
	mtBrush		** brush_outline,
	void		* user_data
	)
{
	uptabSTATE	* state = (uptabSTATE *)user_data;


	if ( state->image->channel[MTPIXEL_CHANNEL_IMAGE][0] != 255 )
	{
		// Terminate if the last rendering found a seat

		return 2;
	}

	state->last_seat = seat;

	brush_fill[0] = state->brush;
	brush_outline[0] = state->brush;

	return 0;			// Render & continue
}

static void update_table_coord (
	GtkWidget	* table,
	int		x,
	int		y,
	gexSdraw	* sdraw
	)
{
	uptabSTATE	state = { -1 };
	int		i,
			candidates,
			votes;
	char		buf[1024],
			* st_list[10] = { NULL };
	mtTreeNode	* node;
	CedIndexItem	* index_item;
	CedCell		* cell;
	GtkWidget	* label;


	if ( ! global.sheet_full )
	{
		// No data loaded

		return;
	}

	mtgex_table_clear ( table );
	gtk_table_resize ( GTK_TABLE ( table ), 1, 1 );	// Remove old junk

	state.image = mtpixel_image_new_rgb ( 1, 1 );
	if ( ! state.image )
	{
		goto finish;
	}

	state.image->channel[MTPIXEL_CHANNEL_IMAGE][0] = 255;

	state.brush = mtpixel_brush_new ();
	if ( ! state.brush )
	{
		goto finish;
	}

	mtpixel_brush_set_RGB ( state.brush, 0, 0 );
	mtpixel_brush_set_size ( state.brush, 0 );

	eleana_poly_render_custom ( state.image, x + sdraw->x, y + sdraw->y,
		global.polymap_tree, global.table, global.map_zoom, uptab_cb,
		&state );

	if ( state.image->channel[MTPIXEL_CHANNEL_IMAGE][0] == 255 )
	{
		// No seat was found

		mtgex_add_to_table ( NULL, table, 0, 0, UPTAB_SPACE_H,
			UPTAB_SPACE_V );

		goto finish;
	}

	node = mtkit_tree_node_find ( global.seat_names,
		(void *)(intptr_t)state.last_seat );

	if ( ! node )
	{
		mtgex_add_to_table ( "?", table, 0, 0, UPTAB_SPACE_H,
			UPTAB_SPACE_V );

		goto finish;
	}

	st_list[0] = node->data;
	votes = 0;

	if ( ced_index_query ( global.seat_index, 0, st_list[0],
		&index_item ) != 1 ||
		! index_item
		)
	{
		goto finish;
	}

	// Count candidates and the total votes
	for ( candidates = 0; ; candidates ++ )
	{
		cell = ced_sheet_get_cell ( global.sheet_full,
			index_item->row + candidates, FULL_COL_VOTES );

		if ( ! cell )
		{
			break;
		}

		votes += cell->value;
	}

	gtk_table_resize ( GTK_TABLE ( table ), 7 + candidates, 4 );

	cell = ced_sheet_get_cell ( global.sheet_full, index_item->row,
		FULL_COL_COUNTY );

	if ( cell && cell->text )
	{
		st_list[1] = " (";
		st_list[2] = cell->text;
		st_list[3] = ")";
	}

	st_list[9] = mtkit_strjoinv ( NULL, st_list );
	if ( ! st_list[9] )
	{
		goto finish;
	}

	label = gtk_label_new ( st_list[9] );
	gtk_widget_show ( label );
	gtk_table_attach ( GTK_TABLE ( table ), label, 0, 4, 1, 2, GTK_FILL,
		GTK_FILL, UPTAB_SPACE_H, UPTAB_SPACE_V );

	free ( st_list[9] );

	i = mtgex_bmenu_get_value ( global.gw[_GW_BMENU_PICK_YEAR] );
	if ( i > -1 )
	{
		st_list[0] = mtgex_bmenu_get_value_text (
			global.gw[_GW_BMENU_PICK_YEAR], i );

		if ( st_list[0] )
		{
			mtgex_add_to_table ( st_list[0], table, 0, 0,
				UPTAB_SPACE_H, UPTAB_SPACE_V );
		}
	}

	sprintf ( buf, "%i", votes );

	mtgex_add_to_table ( "", table, 2, 0, UPTAB_SPACE_H, UPTAB_SPACE_V );
	mtgex_add_to_table ( "", table, candidates + 3, 0, UPTAB_SPACE_H,
		UPTAB_SPACE_V );
	mtgex_add_to_table ( "", table, candidates + 5, 0, UPTAB_SPACE_H,
		UPTAB_SPACE_V );

	mtgex_add_to_table ( "Turnout", table, candidates + 4, 1, UPTAB_SPACE_H,
		UPTAB_SPACE_V );
	mtgex_add_to_table ( buf, table, candidates + 4, 2, UPTAB_SPACE_H,
		UPTAB_SPACE_V );
	mtgex_add_to_table ( "Electorate", table, candidates + 6, 1,
		UPTAB_SPACE_H, UPTAB_SPACE_V );

	cell = ced_sheet_get_cell ( global.sheet_full, index_item->row,
		FULL_COL_ELECTORATE );

	if ( cell && cell->text )
	{
		mtgex_add_to_table ( cell->text, table, candidates + 6, 2,
			UPTAB_SPACE_H, UPTAB_SPACE_V );
		sprintf ( buf, "%.1f%%", 100 * ( (double)votes ) /
			cell->value );
		mtgex_add_to_table ( buf, table, candidates + 4, 3,
			UPTAB_SPACE_H, UPTAB_SPACE_V );
	}

	// Populate table with candidates
	for ( i = 0; i < candidates; i++ )
	{
		cell = ced_sheet_get_cell ( global.sheet_full,
			index_item->row + i, FULL_COL_MP_NAME );

		if ( cell )
		{
			mtgex_add_to_table ( cell->text, table, i + 3, 0,
				UPTAB_SPACE_H, UPTAB_SPACE_V );
		}

		cell = ced_sheet_get_cell ( global.sheet_full,
			index_item->row + i, FULL_COL_PARTY );

		if ( cell )
		{
			mtgex_add_to_table ( cell->text, table, i + 3, 1,
				UPTAB_SPACE_H, UPTAB_SPACE_V );
		}

		cell = ced_sheet_get_cell ( global.sheet_full,
			index_item->row + i, FULL_COL_VOTES );

		if ( cell )
		{
			mtgex_add_to_table ( cell->text, table, i + 3, 2,
				UPTAB_SPACE_H, UPTAB_SPACE_V );

			sprintf ( buf, "%.1f%%",
				100 * cell->value / ( (double)votes ) );

			mtgex_add_to_table ( buf, table, i + 3, 3,
				UPTAB_SPACE_H, UPTAB_SPACE_V );
		}
	}

finish:
	mtpixel_image_destroy ( state.image );
	mtpixel_brush_destroy ( state.brush );
}

static gboolean map_click_button (
	GtkWidget	* widget,
	GdkEventButton	* event,
	gexSdraw	* sdraw
	)
{
	double		aw,
			ah,
			cx,
			cy,
			zd = 0,
			newzoom;


	if ( ! event || ! sdraw )
	{
		return FALSE;
	}

	aw = sdraw->drawing_area->allocation.width;
	ah = sdraw->drawing_area->allocation.height;

	cx = ( (double)(event->x - aw + sdraw->x) ) / (sdraw->w - 2 * aw );
	cy = ( (double)(event->y - ah + sdraw->y) ) / (sdraw->h - 2 * ah );

	if ( event->state & GDK_CONTROL_MASK )
	{
		gtk_notebook_set_current_page (
			GTK_NOTEBOOK ( global.notebook_left ), 1 );

		update_table_coord ( global.cursor_left_table,
			event->x - aw, event->y - ah, sdraw );

		return TRUE;
	}

	gtk_notebook_set_current_page ( GTK_NOTEBOOK ( global.notebook_left ),
		0 );

	mtMAX ( cx, cx, 0 )
	mtMAX ( cy, cy, 0 )
	mtMIN ( cx, cx, 1 )
	mtMIN ( cy, cy, 1 )

	if ( event->button == 1 )
	{
		zd =  global.map_zoom / 10;
	}

	if ( event->button == 3 )
	{
		zd = -global.map_zoom / 10;
	}

	newzoom = global.map_zoom + zd;

	if ( newzoom < POLYMAP_ZOOM_MIN )
	{
		newzoom = POLYMAP_ZOOM_MIN;
	}

	if ( newzoom > POLYMAP_ZOOM_MAX )
	{
		newzoom = POLYMAP_ZOOM_MAX;
	}

	if ( global.map_zoom != newzoom )
	{
		int		w,
				h,
				nv_h = 0,
				nv_v = 0;	// New positions of scrollbar


		w = rint ( POLYMAP_WIDTH * newzoom );
		h = rint ( POLYMAP_HEIGHT * newzoom );

		nv_h = rint ( w * cx - event->x );
		nv_v = rint ( h * cy - event->y );

		global.map_zoom = newzoom;

		// Block the callback and set the spinslider
		mtgex_spinslide_block ( global.gw[_GW_SPINSLIDER_MAPZOOM],
			GTK_SIGNAL_FUNC ( map_zoom_changed ), NULL );
		mtgex_spinslide_set_value ( global.gw[_GW_SPINSLIDER_MAPZOOM],
			newzoom );
		mtgex_spinslide_unblock ( global.gw[_GW_SPINSLIDER_MAPZOOM],
			GTK_SIGNAL_FUNC ( map_zoom_changed ), NULL );

		mtgex_sdraw_set ( sdraw->table,
			nv_h + aw,
			nv_v + ah,
			w + aw * 2,
			h + ah * 2 );
	}

	return TRUE;
}

static gboolean map_expose_canvas (
	GtkWidget	* widget,
	GdkEventExpose	* event,
	gpointer	data
	)
{
	mtImage		* image;
	int		px,
			py,
			pw,
			ph;
	gexSdraw	* sdraw = (gexSdraw *)data;


	if ( ! widget || ! event )
	{
		return TRUE;
	}

	px = event->area.x;
	py = event->area.y;
	pw = event->area.width;
	ph = event->area.height;

	if ( pw < 1 || ph < 1 )
	{
		return TRUE;
	}

	image = mtpixel_image_new_rgb ( pw, ph );
	if ( ! image )
	{
		return TRUE;
	}

	if ( global.polymap_tree )
	{
		char		* s[5] = {
					_INIFILE_MAP_COL_A,
					_INIFILE_MAP_COL_B,
					_INIFILE_MAP_COL_C,
					_INIFILE_MAP_COL_D,
					_INIFILE_MAP_COL_INK
					};
		int		cols[5] = {0},
				i;


		for ( i = 0; i < 5; i++ )
		{
			mtkit_prefs_get_int ( global.prefs_file, s[i],
				&cols[i] );
		}

		memset ( image->channel[MTPIXEL_CHANNEL_IMAGE], 255,
			pw * ph * 3 );

		eleana_poly_render ( image,
			px + sdraw->x - sdraw->drawing_area->allocation.width,
			py + sdraw->y - sdraw->drawing_area->allocation.height,
			global.polymap_tree, global.table, global.map_zoom,
			cols );
	}

	gdk_draw_rgb_image ( widget->window, widget->style->black_gc,
		px, py, pw, ph, GDK_RGB_DITHER_NONE,
		image->channel[MTPIXEL_CHANNEL_IMAGE], pw * 3 );

	mtpixel_image_destroy ( image );

	return TRUE;
}

static void carto_zoom_changed (
	GtkAdjustment	* adj,
	gpointer	user_data
	)
{
	if ( ! adj )
	{
		return;
	}

	global.carto_zoom = gtk_adjustment_get_value ( adj );
	init_carto_area ();
}

static void diag_zoom_changed (
	GtkAdjustment	* adj,
	gpointer	user_data
	)
{
	if ( ! adj )
	{
		return;
	}

	global.diag_zoom = gtk_adjustment_get_value ( adj );
	init_diagram_area ();
}

static void swing_changed (
	GtkWidget	* w,
	gpointer	user_data
	)
{
	int		item = (intptr_t)user_data;


	if (	! w			||
		item < EL_SWING_AB	||
		item > EL_SWING_CO	||
		! global.gw[_GW_SPINSLIDER_SWING_AB + item]
		)
	{
		return;
	}

	global.swing[item] = mtgex_spinslide_get_value (
		global.gw[_GW_SPINSLIDER_SWING_AB + item] );

	swing_sheet_update ();
}

static void eleana_swing_changed (
	GtkToggleButton	* togglebutton,
	gpointer	user_data
	)
{
	double		new_swing[EL_SWING_TOTAL];
	int		i;


	eleana_swingset_prepare ( global.table, global.seats, global.swing,
		new_swing );

	if ( gtk_toggle_button_get_active ( togglebutton ) )
	{
		global.eleana_swing = 1;

		// Moving from Uniform to Eleana swing
		eleana_swingset_uniform_to_eleana ( global.swing, new_swing );
	}
	else
	{
		global.eleana_swing = 0;

		// Moving from Eleana to Uniform swing
		eleana_swingset_eleana_to_uniform ( global.swing, new_swing );
	}

	if ( global.table )		// Set spinsliders to new values
	{
		for ( i = 0; i < EL_SWING_TOTAL; i++ )
		{
			mtgex_spinslide_set_value (
				global.gw[i + _GW_SPINSLIDER_SWING_AB],
				new_swing[i] );
		}
	}

	swing_sheet_update ();
}

static void pressed_preferences (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	mtgex_prefs_window ( global.prefs_file, global.main_window, NULL, NULL,
		NULL, 0 );
}

static void load_party_names ( void )
{
	char		* names[4] = { "CON", "LAB", "LIB", "OTH" },
			txt[128];
	int		j,
			swlab[EL_SWING_TOTAL][3] = {
				{ 0, 1, _GW_LABEL_SWING_HEAD_AB },
				{ 0, 2, _GW_LABEL_SWING_HEAD_AC },
				{ 0, 3, _GW_LABEL_SWING_HEAD_AO },
				{ 1, 2, _GW_LABEL_SWING_HEAD_BC },
				{ 1, 3, _GW_LABEL_SWING_HEAD_BO },
				{ 2, 3, _GW_LABEL_SWING_HEAD_CO }
				};


	for ( j = 0; j < 4; j++ )
	{
		gtk_label_set_text (
			GTK_LABEL ( global.gw[_GW_LABEL_RESULTS_HEAD_A + j] ),
			names[j] );

		snprintf ( global.party_name[j], PART_NAME_SIZE, "%s",
			names[j] );

		global.party_name[j][PART_NAME_SIZE - 1] = 0;
	}

	for ( j = 0; j < EL_SWING_TOTAL; j++ )
	{
		snprintf ( txt, 128, "%s->%s", names[ swlab[j][0] ],
			names[ swlab[j][1] ] );

		txt[120] = 0;
		gtk_label_set_text ( GTK_LABEL ( global.gw[ swlab[j][2] ] ),
			txt );
	}
}

static void eleana_year_changed (
	GtkWidget	* widget,
	void		* user_data,
	int		new_value
	)
{
	CedSheet	* full;
	mtTree		* polymap,
			* seat_names;
	CedIndex	* seat_index;
	double		* table;
	int		seats;


	if ( ! global.eleana_index )
	{
		return;
	}

	if ( eleana_index_record_load ( global.eleana_index, new_value, 0,
		&full, &polymap, &table, &seats, &seat_names, &seat_index ) )
	{
		mtgex_alert_box ( _("Error"), _("Unable to change year"),
			_("OK"), NULL, NULL, global.main_window );

		return;
	}

	purge_data_loaded ();		// Remove old year's data

	global.sheet_full = full;
	global.table = table;
	global.polymap_tree = polymap;
	global.seats = seats;
	global.seat_names = seat_names;
	global.seat_index = seat_index;

	load_party_names ();

	mtkit_prefs_set_int ( global.prefs_file, _INIFILE_LAST_YEAR,
		new_value );

	swing_sheet_update ();
	mtgex_image_set_image ( global.gw[_GW_GIMAGE_GRAPH], NULL, 1 );
}

static void gui_main_init ( void )
{
	mtImage		* mim;
	int		i,
			j,
			font_size;
	char		* font_name,
			* party_swing_txt[EL_SWING_TOTAL] = {
				"A->B",
				"A->C",
				"A->O",
				"B->C",
				"B->O",
				"C->O"
				},
			* party_txt[] = { "A", "B", "C", "O", "Total" },
			* tab_titles[] = {
				_("Party"),
				_("    Votes    "),
				_("  Share  "),
				_("  Swing  "),
				_("Seats"),
				_("Change") };
			;
	gexSdraw	* sdraw;
	GtkWidget	* menubar,
			* vbox_main,
			* sw,
			* table,
			* hbox,
			* book,
			* label,
			* vbox,
			* frame,
			* slider,
			* vbox2,
			* button,
			* area,
			* combo;
	GtkAdjustment	* adj;
	GtkAccelGroup	* accel_group;
	GtkItemFactory	* item_factory;
	GtkItemFactoryEntry menu_items[] = {
{ _("/_File"),			NULL,		NULL,0, "<Branch>" },
{ _("/File/tear"),		NULL,		NULL,0, "<Tearoff>" },
{ _("/File/Save Graph Image ..."), NULL,	pressed_file_action, FS_SAVE_GRAPH, NULL },
{ _("/File/Save Map Image ..."), NULL,		pressed_file_action, FS_SAVE_POLYMAP, NULL },
{ _("/File/sep3"),		NULL,		NULL,0, "<Separator>" },
{ _("/File/Quit"),		"<control>Q",	delete_event, 0, NULL },

{ _("/_Edit"),			NULL,		NULL,0, "<Branch>" },
{ _("/Edit/tear"),		NULL,		NULL,0, "<Tearoff>" },
{ _("/Edit/Plot Graph"),	"F6",		pressed_plot_graph,0, NULL },
{ _("/Edit/Reset Swings"),	"F9",		pressed_reset_swings,0, NULL },
{ _("/Edit/sep4"),		NULL,		NULL,0, "<Separator>" },
{ _("/Edit/Preferences"),	"<control>P",	pressed_preferences },

{ _("/_Help"),			NULL,		NULL,0, "<LastBranch>" },
{ _("/Help/About ..."),		"F1",		pressed_help, 0, NULL }
	};


	global.carto_zoom = 10;
	global.diag_zoom = 2;
	global.map_zoom = 1;

	// Needed to read non ASCII filenames in GTK+2
	putenv ( "G_BROKEN_FILENAMES=1" );
	gtk_init ( &global.argc, &global.argv );

	accel_group = gtk_accel_group_new ();

///	MAIN WINDOW

	global.main_window = gtk_window_new ( GTK_WINDOW_TOPLEVEL );

	// Set minimum width/height
	gtk_widget_set_usize ( global.main_window, 100, 100 );

	mtgex_restore_window_position ( global.main_window, global.prefs_file,
		_INIFILE_MAIN_WINDOW );
	gtk_window_set_title ( GTK_WINDOW ( global.main_window ), VERSION );

	vbox_main = gtk_vbox_new ( FALSE, 0 );
	gtk_widget_show ( vbox_main );
	gtk_container_add ( GTK_CONTAINER ( global.main_window ), vbox_main );

// we need to realize the window because we use pixmaps for
// items on the toolbar & menus in the context of it
	gtk_widget_realize ( global.main_window );

///	MENU

	item_factory = gtk_item_factory_new ( GTK_TYPE_MENU_BAR,"<main>",
		accel_group);
	gtk_item_factory_create_items_ac ( item_factory,
		( sizeof ( menu_items ) / sizeof ( menu_items[0]) ),
		menu_items, NULL, 2 );

	menubar = gtk_item_factory_get_widget ( item_factory, "<main>" );
	gtk_widget_show ( menubar );
	gtk_box_pack_start ( GTK_BOX ( vbox_main ), menubar, FALSE, FALSE, 0 );

	// Stop dynamic allocation of accelerators during runtime
	gtk_accel_group_lock ( accel_group );

	gtk_window_add_accel_group ( GTK_WINDOW ( global.main_window ),
		accel_group );

	hbox = gtk_hbox_new ( FALSE, 0 );
	gtk_widget_show ( hbox );
	gtk_box_pack_start ( GTK_BOX ( vbox_main ), hbox, TRUE, TRUE, 0 );

	vbox = gtk_vbox_new ( FALSE, 0 );
	gtk_widget_show ( vbox );
	gtk_box_pack_start ( GTK_BOX ( hbox ), vbox, FALSE, FALSE, 0 );

///	YEAR PICKER FRAME

	frame = gtk_frame_new ( "Year" );
	gtk_widget_show ( frame );
	gtk_box_pack_start ( GTK_BOX ( vbox ), frame, FALSE, FALSE, 5 );

	vbox2 = gtk_vbox_new ( FALSE, 0 );
	gtk_widget_show ( vbox2 );
	gtk_container_add ( GTK_CONTAINER ( frame ), vbox2 );

	combo = mtgex_bmenu_new ();
	gtk_widget_show ( combo );
	gtk_box_pack_start ( GTK_BOX ( vbox2 ), combo, TRUE, TRUE, 5 );
	mtgex_bmenu_set_callback ( combo, eleana_year_changed, NULL );

	global.gw[_GW_BMENU_PICK_YEAR] = combo;

///	LEFT NOTEBOOK

	book = gtk_notebook_new ();
	gtk_widget_show ( book );
	gtk_box_pack_start ( GTK_BOX ( vbox ), book, TRUE, TRUE, 0 );
	global.notebook_left = book;

	label = gtk_label_new ( "Election" );
	gtk_widget_show ( label );

	vbox = gtk_vbox_new ( FALSE, 0 );
	gtk_widget_show ( vbox );
	gtk_notebook_append_page ( GTK_NOTEBOOK ( book ), vbox, label );

///	RESULTS FRAME

	frame = gtk_frame_new ( "Results" );
	gtk_widget_show ( frame );
	gtk_box_pack_start ( GTK_BOX ( vbox ), frame, FALSE, FALSE, 5 );

	vbox2 = gtk_vbox_new ( FALSE, 0 );
	gtk_widget_show ( vbox2 );
	gtk_container_add ( GTK_CONTAINER ( frame ), vbox2 );

	table = gtk_table_new ( 6, 6, FALSE );
	gtk_widget_show ( table );
	gtk_box_pack_start ( GTK_BOX ( vbox2 ), table, TRUE, TRUE, 10 );

	for ( i = 0; i <= _GW_LABEL_RES_COLS; i++ )
	{
		label = mtgex_add_to_table ( tab_titles[i], table, 0, i, 4, 4 );
		gtk_misc_set_alignment ( GTK_MISC ( label ), 0.5, 0.5 );
	}

	for ( i = 0; i < 5; i++ )
	{
		label = mtgex_add_to_table ( party_txt[i], table, i + 1, 0, 6,
			4 );
		gtk_misc_set_alignment ( GTK_MISC ( label ), 0.5, 0.5 );
		global.gw[_GW_LABEL_RESULTS_HEAD_A + i] = label;

		for ( j = 1; j <= _GW_LABEL_RES_COLS; j++ )
		{
			label = mtgex_add_to_table ( "", table, i + 1, j, 6, 4
				);
			global.gw[_GW_LABEL_RES_A_V + i * _GW_LABEL_RES_COLS +
				j - 1] = label;
			gtk_misc_set_alignment ( GTK_MISC ( label ), 0.5, 0.5 );
		}
	}

///	SWING FRAME

	frame = gtk_frame_new ( "Swing" );
	gtk_widget_show ( frame );
	gtk_box_pack_start ( GTK_BOX ( vbox ), frame, FALSE, FALSE, 5 );

	vbox2 = gtk_vbox_new ( FALSE, 0 );
	gtk_widget_show ( vbox2 );
	gtk_container_add ( GTK_CONTAINER ( frame ), vbox2 );

	table = gtk_table_new ( 4, 2, FALSE );
	gtk_widget_show ( table );
	gtk_box_pack_start ( GTK_BOX ( vbox2 ), table, TRUE, TRUE, 10 );

	for ( i = 0; i < EL_SWING_TOTAL; i++ )
	{
		label = mtgex_add_to_table ( party_swing_txt[i], table, i, 0,
			4, 4 );

		global.gw[_GW_LABEL_SWING_HEAD_AB + i] = label;

		slider = mtgex_spinslide_new ( -2, -2, 0, -90.0, 90.0,
			2, 1 );
		global.gw[_GW_SPINSLIDER_SWING_AB + i] = slider;

		mtgex_spinslide_connect ( slider,
			GTK_SIGNAL_FUNC ( swing_changed ),
			(gpointer)(intptr_t)i );

		gtk_table_attach ( GTK_TABLE ( table ), slider,
			1, 1 + 1, i, i + 1,
			GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 0 );
	}

	button = gtk_check_button_new_with_label ( "Eleana Swing" );
	gtk_widget_show ( button );
	g_signal_connect ( G_OBJECT ( button ), "toggled",
		G_CALLBACK ( eleana_swing_changed ), NULL );
	gtk_box_pack_start ( GTK_BOX ( vbox2 ), button, FALSE, FALSE, 0 );

	label = gtk_label_new ( "Seat" );
	gtk_widget_show ( label );

	vbox = gtk_vbox_new ( FALSE, 0 );
	gtk_widget_show ( vbox );
	gtk_notebook_append_page ( GTK_NOTEBOOK ( book ), vbox, label );

	frame = gtk_frame_new ( "Cursor (CTRL+left)" );
	gtk_widget_show ( frame );
	gtk_box_pack_start ( GTK_BOX ( vbox ), frame, TRUE, TRUE, 0 );

	sw = gtk_scrolled_window_new ( NULL, NULL );
	gtk_widget_show ( sw );
	gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW ( sw ),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
	gtk_container_add ( GTK_CONTAINER ( frame ), sw );

	table = gtk_table_new ( 1, 1, FALSE );
	gtk_widget_show ( table );
	gtk_scrolled_window_add_with_viewport ( GTK_SCROLLED_WINDOW ( sw ),
		table );
	global.cursor_left_table = table;

///	RIGHT NOTEBOOK

	book = gtk_notebook_new ();
	gtk_widget_show ( book );
	gtk_box_pack_start ( GTK_BOX ( hbox ), book, TRUE, TRUE, 0 );

///	MAP TAB

	label = gtk_label_new ( "Map" );
	gtk_widget_show ( label );

	vbox = gtk_vbox_new ( FALSE, 0 );
	gtk_widget_show ( vbox );
	gtk_notebook_append_page ( GTK_NOTEBOOK(book), vbox, label );

	slider = mtgex_spinslide_new ( -2, -2, global.map_zoom,
		POLYMAP_ZOOM_MIN, POLYMAP_ZOOM_MAX, 2, 1 );
	global.gw[_GW_SPINSLIDER_MAPZOOM] = slider;
	gtk_box_pack_start ( GTK_BOX ( vbox ), slider, FALSE, FALSE, 2 );
	mtgex_spinslide_connect ( slider, GTK_SIGNAL_FUNC ( map_zoom_changed ),
		NULL );

	area = mtgex_sdraw_new ();
	gtk_box_pack_start ( GTK_BOX ( vbox ), area, TRUE, TRUE, 0 );

	if (	! mtgex_sdraw_get ( area, &sdraw ) &&
		sdraw
		)
	{
		global.gw[ _GW_SDRAW_MAP ] = area;
		global.gw[ _GW_SDRAW_AREA_MAP ] = sdraw->drawing_area;

		gtk_signal_connect (
			GTK_OBJECT ( global.gw[ _GW_SDRAW_AREA_MAP ]),
			"expose_event", GTK_SIGNAL_FUNC ( map_expose_canvas ),
			(gpointer)sdraw );

		gtk_signal_connect (
			GTK_OBJECT ( global.gw[ _GW_SDRAW_AREA_MAP ]),
			"button_press_event",
			GTK_SIGNAL_FUNC ( map_click_button ),
			(gpointer)sdraw );

		gtk_widget_set_events ( global.gw[ _GW_SDRAW_AREA_MAP ],
			GDK_ALL_EVENTS_MASK );
	}

///	CARTOGRAM TAB

	label = gtk_label_new ( "Cartogram" );
	gtk_widget_show ( label );

	vbox = gtk_vbox_new ( FALSE, 0 );
	gtk_widget_show ( vbox );
	gtk_notebook_append_page ( GTK_NOTEBOOK ( book ), vbox, label );

	slider = gtk_hscale_new_with_range ( 1, 32, 1 );
	gtk_widget_show ( slider );
	gtk_box_pack_start ( GTK_BOX ( vbox ), slider, FALSE, FALSE, 2 );

	adj = gtk_range_get_adjustment ( GTK_RANGE ( slider ) );
	gtk_adjustment_set_value ( adj, global.carto_zoom );
	gtk_scale_set_value_pos ( GTK_SCALE ( slider ), GTK_POS_RIGHT );
	g_signal_connect ( G_OBJECT ( adj ), "value_changed",
		G_CALLBACK ( carto_zoom_changed ), NULL );

	sw = mtgex_image_new ();
	gtk_box_pack_start ( GTK_BOX ( vbox ), sw, TRUE, TRUE, 0 );
	if ( mtgex_image_get_geximage ( sw, &global.gimage_carto ) )
	{
		fprintf ( stderr, "Unable to create gexImage\n" );
		exit ( 1 );
	}
	else
	{
		mim = mtpixel_image_new ( MAP_WIDTH, MAP_HEIGHT,
			MTPIXEL_IMAGE_RGB, MTPIXEL_CHANNEL_MASK_IMAGE );

		if ( mtgex_image_set_image ( sw, mim, 1 ) )
		{
			mtpixel_image_destroy ( mim );
		}
	}

	init_carto_area ();

///	DIAGRAM TAB

	if ( global.diagram_image )
	{
		label = gtk_label_new ( "Diagram" );
		gtk_widget_show ( label );

		vbox = gtk_vbox_new ( FALSE, 0 );
		gtk_widget_show ( vbox );
		gtk_notebook_append_page ( GTK_NOTEBOOK ( book ), vbox, label );

//	CARTOGRAM ZOOM SLIDER

		slider = gtk_hscale_new_with_range ( 1, 16, 1 );
		gtk_widget_show ( slider );
		gtk_box_pack_start ( GTK_BOX ( vbox ), slider, FALSE, FALSE,
			2 );

		adj = gtk_range_get_adjustment ( GTK_RANGE ( slider ) );
		gtk_adjustment_set_value ( adj, global.diag_zoom );
		gtk_scale_set_value_pos ( GTK_SCALE ( slider ), GTK_POS_RIGHT );
		g_signal_connect ( G_OBJECT ( adj ), "value_changed",
			G_CALLBACK ( diag_zoom_changed ), NULL );

		sw = mtgex_image_new ();
		gtk_box_pack_start ( GTK_BOX ( vbox ), sw, TRUE, TRUE, 0 );
		if ( mtgex_image_get_geximage ( sw, &global.gimage_diagram ) )
		{
			fprintf ( stderr, "Unable to create gexImage\n" );
			exit ( 1 );
		}
		else
		{
			mim = mtpixel_image_duplicate ( global.diagram_image );
			if ( mtgex_image_set_image ( sw, mim, 1 ) )
			{
				mtpixel_image_destroy ( mim );
			}
		}

		init_diagram_area ();
	}

//	GRAPH TAB

	label = gtk_label_new ( "Graph" );
	gtk_widget_show ( label );

	vbox = gtk_vbox_new ( TRUE, 0 );
	gtk_widget_show ( vbox );
	gtk_notebook_append_page ( GTK_NOTEBOOK ( book ), vbox, label );

	sw = mtgex_image_new ();
	gtk_box_pack_start ( GTK_BOX ( vbox ), sw, TRUE, TRUE, 0 );
	global.gw[_GW_GIMAGE_GRAPH] = sw;

/////////	End of main window widget setup

	gtk_signal_connect ( GTK_OBJECT ( global.main_window ), "delete_event",
		GTK_SIGNAL_FUNC ( delete_event ), NULL );

	font_name = NULL;
	mtkit_prefs_get_str ( global.prefs_file, _INIFILE_FONT_FILENAME,
		&font_name );

	if ( font_name && ! font_name[0] )
	{
		font_name = NULL;
	}

	font_size = 16;
	mtkit_prefs_get_int ( global.prefs_file, _INIFILE_FONT_SIZE, &font_size
		);

	if ( project_set_font ( font_name, font_size ) )
	{
		project_set_font ( NULL, 16 );
	}

	update_titlebar ();

	gtk_widget_show ( global.main_window );

	gtk_window_set_icon ( (GtkWindow *)global.main_window,
		gdk_pixbuf_new_from_xpm_data ( (const char **)icon_xpm ) );

	project_load ( DATA_DIR "eleana_index.zip" );

	if ( ! global.eleana_index )
	{
		clear_all ();		// Start with empty file structure
	}

	reset_map_zoom_pos ();

	gtk_main ();
}


/*
	File operations - return 0 on success
*/

static void register_project (
	char		* filename
	)
{
	if ( ! filename )
	{
		return;
	}

	mtkit_strncpy0 ( global.mem_filename, filename, MTGEX_PATHBUF );
	update_titlebar ();
}

int project_load (
	char		* filename
	)
{
	int		i = 0,
			year = 0;
	char		real_filename[MTGEX_PATHBUF];
	eleanaINDEX	* newindex;
	CedCell		* cell;


	if ( ! filename || ! filename[0] )
	{
		return 1;
	}

	if ( filename[0] != MTKIT_DIR_SEP )
	{
		// Expand the filename to include the full path
		if ( getcwd ( real_filename, MTGEX_PATHBUF - 1 ) )
		{
			i = strlen ( real_filename );
			real_filename[i] = MTKIT_DIR_SEP;
			i++;
		}
	}

	real_filename[i] = 0;
	mtkit_strnncat ( real_filename, filename, MTGEX_PATHBUF );

	newindex = eleana_index_open ( real_filename );
	if ( ! newindex )
	{
		error_message_file ( _("load index"), real_filename );

		return 1;
	}

	// Index was loaded correctly so purge all other data
	purge_index_loaded ();
	purge_data_loaded ();

	// Remove the old years in the picker
	mtgex_bmenu_add_item ( global.gw[_GW_BMENU_PICK_YEAR], NULL );

	for ( i = ELEANA_INDEX_ROW_START; ; i++ )
	{
		cell = ced_sheet_get_cell ( newindex->sheet_index, i,
			ELEANA_INDEX_COL_YEAR );

		if ( ! cell )
		{
			break;
		}

		mtgex_bmenu_add_item ( global.gw[_GW_BMENU_PICK_YEAR],
			cell->text );
	}

	pressed_reset_swings ( NULL, NULL );

	global.eleana_index = newindex;

	// Default to the previous year that was used
	mtkit_prefs_get_int ( global.prefs_file, _INIFILE_LAST_YEAR, &year );
	mtgex_bmenu_set_value ( global.gw[_GW_BMENU_PICK_YEAR], year );

	update_menus ();
	register_project ( real_filename );

	reset_map_zoom_pos ();

	return 0;
}

int project_save_graph (
	char		* filename
	)
{
	gexImage	* gimage = NULL;


	mtgex_image_get_geximage ( global.gw[_GW_GIMAGE_GRAPH], &gimage );

	if (	! gimage ||
		mtpixel_image_save ( gimage->image, filename,
			MTPIXEL_FILE_TYPE_PNG, 5 )
		)
	{
		error_message_file ( _("save graph"), filename );

		return 1;
	}

	return 0;
}

int project_save_polymap (
	char		* filename
	)
{
	char		* s[5] = {
				_INIFILE_MAP_COL_A,
				_INIFILE_MAP_COL_B,
				_INIFILE_MAP_COL_C,
				_INIFILE_MAP_COL_D,
				_INIFILE_MAP_COL_INK
				};
	int		cols[5] = {0},
			i;
	mtImage		* image = NULL;


	image = mtpixel_image_new_rgb ( POLYMAP_WIDTH * global.map_zoom + 1,
		POLYMAP_HEIGHT * global.map_zoom + 1 );

	if ( ! image )
	{
		goto error;
	}

	for ( i = 0; i < 5; i++ )
	{
		mtkit_prefs_get_int ( global.prefs_file, s[i], &cols[i] );
	}

	memset ( image->channel[MTPIXEL_CHANNEL_IMAGE], 255,
		image->width * image->height * 3 );
	eleana_poly_render ( image, 0, 0, global.polymap_tree, global.table,
		global.map_zoom, cols );

	if ( mtpixel_image_save ( image, filename, MTPIXEL_FILE_TYPE_PNG, 5 ) )
	{
		goto error;
	}

	mtpixel_image_destroy ( image );

	return 0;

error:
	mtpixel_image_destroy ( image );
	error_message_file ( _("save map"), filename );

	return 1;
}

int project_set_font (
	char		* filename,
	int		size
	)
{
	mtFont		* newfont;
	mtFontRender	* ren;


	newfont = mtpixel_font_new ( filename, filename ? 0 : size );
	if ( ! newfont )
	{
		char		* msg,
				* f8;


		f8 = mtgex_gtkuncpy ( NULL, filename, 0 );
		msg = g_strdup_printf (
			_("Unable to open font file %s size %i."), f8, size );

		mtgex_alert_box ( _("Error"), msg, _("OK"), NULL, NULL,
			global.main_window );

		g_free ( msg );
		g_free ( f8 );

		return 1;
	}

	if ( global.font )
	{
		mtpixel_font_destroy ( global.font );
	}

	global.font = newfont;

	if ( filename )
	{
		mtpixel_font_set_size ( newfont, size );
		mtkit_prefs_set_str ( global.prefs_file,
			_INIFILE_FONT_FILENAME, filename );
	}
	else
	{
		mtkit_prefs_set_str ( global.prefs_file,
			_INIFILE_FONT_FILENAME, "" );
	}

	mtkit_prefs_set_int ( global.prefs_file, _INIFILE_FONT_SIZE, size );

	ren = mtpixel_font_render_new ( newfont, "01234567890", NULL,
		MTPIXEL_IMAGE_RGB );

	if ( ren )
	{
		if ( ! mtpixel_font_extent ( ren ) )
		{
			global.font_w = (ren->width + 5) / 11;
			global.font_h = ren->height;
		}

		mtpixel_font_render_destroy ( ren );
	}

	return 0;
}

int main (
	int		argc,
	char		* argv[]
	)
{
	char		txt[2048],
			* s;


	// Change directory to where the binary is
	s = strrchr ( argv[0], MTKIT_DIR_SEP );

	if ( s )
	{
		s[0] = 0;

		if ( chdir ( argv[0] ) )
		{
			// No action required on error
		}

		s[0] = MTKIT_DIR_SEP;
	}

	if ( argc > 1 )
	{
		if ( strcmp ( argv[1], "--version" ) == 0 )
		{
			printf ("%s\n\n", VERSION);

			exit ( 0 );
		}
		else if ( strcmp ( argv[1], "--help" ) == 0 )
		{
			printf ( "%s\n\n"
				"Usage: %s [OPTIONS]\n\n"
				"Options:\n"
				"  --help          Output this help\n"
				"  --version       Output version information\n"
			, VERSION, BIN_NAME );

			exit ( 0 );
		}
	}

	global.argc = argc;
	global.argv = argv;

	snprintf ( txt, 2048, "%s/.%s", mtkit_file_home (), BIN_NAME );
	global.prefs_file = mtkit_prefs_new ( prefs_table );
	mtgex_fpick_init_prefs ( global.prefs_file, _INIFILE_FPICK_PRE );
	mtgex_prefs_init_prefs ( global.prefs_file );
	mtkit_prefs_load ( global.prefs_file, txt );

	ced_init ();
	mtpixel_init ();

	global.map = malloc ( MAP_WIDTH * MAP_HEIGHT * 3 );

	if ( ! global.map )
	{
		printf ( "Unable to allocate map memory\n" );
	}
	else
	{
		global.diagram_image = mtpixel_image_load (
			DATA_DIR "diagrami.png" );

		gui_main_init ();	// Create main window and run the GUI
	}

	mtkit_prefs_save ( global.prefs_file, txt );
	mtkit_prefs_destroy ( global.prefs_file );
	global.prefs_file = NULL;

	mtpixel_image_destroy ( global.diagram_image );
	global.diagram_image = NULL;

	mtkit_tree_destroy ( global.polymap_tree );
	global.polymap_tree = NULL;

	mtpixel_quit ();

	return 0;
}
