/*
	Copyright (C) 2009-2010 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <mtcelledit.h>



static CedSheet		*sheet;
static CedCell		*cell;
//static unsigned int	r, c;


static void print_version()
{
	printf( "Reads full election data and writes all party vote totals.\n"
		"\n"
		"Usage:\n"
		"\n"
		"_partytots FILE1 FILE2 PARTY\n"
		"\n"
		"FILE1 is the input TSV file (FULL results).\n"
		"FILE2 is the output TSV file.\n"
		"PARTY is the name of the party to sort by.\n"
		"\n"
		);
}

static int find_cb( CedSheet *s, CedCell *cell, unsigned int r, unsigned int c, void *ud )
{
	CedCellRef *cellref = (CedCellRef *)ud;

	cellref->row_d = r;
	cellref->col_d = c;

	return 1;
}

static void add_grid( char *output_filename, char *sort_party_name )
{
	char		txt[128], *party_name;
	CedSheet	*grid;
	unsigned int	con_out_row, prow, votes, newvotes, next_new_party,// pr, pc,
			sort_rows[] = { 1, 2, 0 }, sort_cols[] = { 2, 3, 0 };
	int		res;
	CedCellRef	cellref, pcr;


	grid = ced_sheet_new();
	if ( !grid ) return;

	con_out_row = 3;
	next_new_party = 2;
	cellref.row_d = 3;
	cellref.col_d = 1;
	while (1)	// Add for each constituency
	{
		res = ced_sheet_find_text( sheet, "", CED_FIND_MODE_CASE,
			cellref.row_d+1, cellref.col_d,
			0, 1, find_cb, &cellref );
		if ( res != 2 ) break;

		cell = ced_sheet_get_cell( sheet, cellref.row_d, cellref.col_d );
		if ( !cell ) break;

		ced_sheet_set_cell( grid, con_out_row, 1, cell->text );

		for ( prow = cellref.row_d; ; prow++ )
		{
			cell = ced_sheet_get_cell( sheet, prow, 5 );
			if ( !cell ) break;
			votes = cell->value;

			cell = ced_sheet_get_cell( sheet, prow, 4 );
			if ( !cell ) party_name = "No label";
			else party_name = cell->text;

			// Does this party already have a column?
			res = ced_sheet_find_text( grid, party_name, CED_FIND_MODE_ALLCHARS,
				2, 2, 1, 0, find_cb, &pcr );

			if ( res != 2 )
			{
				pcr.row_d = 2;
				pcr.col_d = next_new_party;
				ced_sheet_set_cell( grid, 2, next_new_party++, party_name );
			}

			// Does this party already have votes in this seat?  if not put value straight in, else add to current
			cell = ced_sheet_get_cell( grid, con_out_row, pcr.col_d );
			if ( !cell ) newvotes = votes;
			else newvotes = cell->value + votes;

			ced_sheet_set_cell_value( grid, con_out_row, pcr.col_d, newvotes );
		}

		con_out_row++;
	}

	ced_sheet_get_geometry( grid, (unsigned int *)&cellref.row_d, NULL );
	sprintf( txt, "=sum( r[2]c:r%uc )", cellref.row_d );
	cellref.row_d = 2;
	cellref.col_d = 1;
	while (1)	// Add total for each party
	{
		res = ced_sheet_find_text( grid, "", CED_FIND_MODE_CASE,
			cellref.row_d, cellref.col_d+1, 1, 0, find_cb, &cellref );
		if ( res < 1 ) break;

		ced_sheet_set_cell( grid, 1, cellref.col_d, txt );
	}

	ced_sheet_get_geometry( grid, (unsigned int *)&cellref.row_d, NULL );
	sprintf( txt, "=counta( r[2]c:r[%u]c )", cellref.row_d );
	ced_sheet_set_cell( grid, 1, 1, txt );

	ced_sheet_sort_columns( grid, 2, 0, sort_rows, CED_SORT_MODE_DESCENDING, NULL );

	if ( ced_sheet_find_text( grid, sort_party_name, CED_FIND_MODE_ALLCHARS,
		2, 2, 1, 0, find_cb, &pcr ) == 2 )
	{
		sort_cols[0] = pcr.col_d;
		sort_cols[1] = 0;
		ced_sheet_sort_rows( grid, 3, 0, sort_cols, CED_SORT_MODE_DESCENDING, NULL );
	}
	else
	{
		fprintf( stderr, "Unable to find '%s'\n", sort_party_name );
	}

	ced_sheet_save( grid, output_filename, CED_FILE_TYPE_TSV_CONTENT );
	ced_sheet_destroy( grid );
}


int main( int argc, char **argv ) 
{
	if ( argc != 4 )
	{
		print_version();
		return 0;
	}

	sheet = ced_sheet_load( argv[1], "ISO-8859-1", NULL );
	if ( !sheet )
	{
		fprintf( stderr, "Unable to load TSV file %s\n", argv[1] );
		return 1;
	}

	add_grid( argv[2], argv[3] );

	ced_sheet_destroy( sheet );

	return 0;
}
