/*
	Copyright (C) 2009-2012 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <mtcelledit.h>

#include "eleana.h"




static void print_version()
{
	printf( "cli_index Load an index and validate it.\n"
		"\n"
		"Usage:\n"
		"\n"
		"./cli_index FILE1\n"
		"\n"
		"FILE1 is the input TSV index file.\n"
		"\n"
		);
}


int main( int argc, char **argv )
{
	eleanaINDEX	*index;
	char		*filename;
	int		i, seats;
	CedSheet	*full;
	mtTree		*polymap;
	double		*table;

	if ( argc != 2 )
	{
		print_version();
		exit(0);
	}

	filename = argv[1];

	index = eleana_index_open( filename );
	if ( !index )
	{
		fprintf( stderr, "Unable to load index file %s\n", filename );
		exit(1);
	}

	printf( "\nIndex loaded at %i.  Path = '%s'\n\n", (int)(intptr_t)index, index->path );

	for ( i=0; i<index->records; i++ )
	{
		if ( eleana_index_record_load( index, i, 1, &full,
			&polymap, &table, &seats, NULL, NULL ) )
		{
			printf( "ERROR on record %i\n", i );
			continue;
		}

		printf("Record %i '%s' = %i %i %i %i\n", i,
			eleana_index_get( index, i, ELEANA_INDEX_COL_YEAR),
			(int)(intptr_t)full, (int)(intptr_t)polymap, (int)(intptr_t)table,
			seats );

		eleana_index_record_close( full, polymap, table, NULL, NULL );
	}

	eleana_index_close( index );

	return 0;
}

