/*
	Copyright (C) 2010-2012 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <mtcelledit.h>

#include "eleana.h"



static CedSheet		*res_old, *res_new, *sheet_out;
static CedIndex		*index_old;



static void print_version()
{
	printf( "res2res - Add county/region details for a new set of results from a complete old set.\n"
		"\n"
		"Usage:\n"
		"\n"
		"./res OLDRESULT NEWRESULT NEWFILE \n"
		"\n"
		);
}

static int callback(
	CedSheet *sheet,
	CedCell *cell,
	unsigned int row,
	unsigned int col,
	void *user_data
	)
{
	CedIndexItem	*item;
	CedCell		*cc;


	if ( !cell->text ) return 0;

	if ( ced_index_query( index_old, 0, cell->text, &item ) != 1 )
	{
		printf( "%s\n", cell->text );
		return 0;
	}

	cc = ced_sheet_get_cell( res_old, item->row, FULL_COL_COUNTY );
	if ( cc && cc->text )
	{
		ced_sheet_set_cell_text( sheet_out, row, FULL_COL_COUNTY, cc->text );
	}

	cc = ced_sheet_get_cell( res_old, item->row, FULL_COL_REGION );
	if ( cc && cc->text )
	{
		ced_sheet_set_cell_text( sheet_out, row, FULL_COL_REGION, cc->text );
	}

	return 0;		// Continue
}


int main( int argc, char **argv )
{
	int res = 1;


	if ( argc != 4 )
	{
		print_version();
		return 1;
	}

	res_old = ced_sheet_load( argv[1], "ISO-8859-1", NULL );
	if ( !res_old )
	{
		fprintf( stderr, "Unable to open %s\n", argv[1] );
		goto error;
	}

	res_new = ced_sheet_load( argv[2], "ISO-8859-1", NULL );
	if ( !res_new )
	{
		fprintf( stderr, "Unable to open %s\n", argv[2] );
		goto error;
	}

	sheet_out = ced_sheet_duplicate( res_new );
	if ( !sheet_out )
	{
		fprintf( stderr, "Unable to duplicate sheet\n" );
		goto error;
	}

	index_old = ced_index_new( CED_INDEX_TYPE_TEXT );
	if ( !index_old || ced_index_add_items( index_old, res_old,
		FULL_ROW_PARTY1, FULL_COL_SEAT_NAME, 0, 1 ) )
	{
		fprintf( stderr, "Unable to create old index\n" );
		goto error;
	}

	ced_sheet_scan_area( res_new, FULL_ROW_PARTY1, FULL_COL_SEAT_NAME, 0, 1,
		callback, NULL );

	ced_sheet_save( sheet_out, argv[3], CED_FILE_TYPE_TSV_CONTENT );

	res = 0;
error:
	ced_index_destroy( index_old );

	ced_sheet_destroy( res_old );
	ced_sheet_destroy( res_new );
	ced_sheet_destroy( sheet_out );

	return res;
}

