/*
	Copyright (C) 2008-2009 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#include <mtcelledit.h>
#include <mtkit.h>

#include "eleana.h"



static eleanaINDEX	*el_index = NULL;
static int		target_seats[4] = { 0 };
static double		target_p[4] = { 0 };


#define EL_TAB_VALUE( row, col )       table[EL_TAB_TOTAL * row + col]

/*
static int recurse_index( mtTreeNode *node, int step )
{
	CedCell 	*cell;
	CedIndexItem	*item;

	if ( node->left ) step = recurse_index( node->left, step );

	cell = node->key;
	item = node->data;
	step++;

	printf( "%4i -  r%4u c%4u  - '%s'\n",
		step, item->row, item->col, cell->text );

	if ( node->right ) step = recurse_index( node->right, step );

	return step;
}

static void display_index( CedIndex *index )
{
	recurse_index( index->tree->root, 0 );

	printf("\n\n");
}
*/

static int stage_1( char *filename, int year )
{
	CedSheet	*full;
	mtTree		*polymap;
	double		*table, swing[6] = { 0 };
	int		seats, i, tab[4][3] =
			{
			{ EL_TAB_PROJ_A, EL_TAB_PROJ_A_SEAT, EL_TAB_A },
			{ EL_TAB_PROJ_B, EL_TAB_PROJ_B_SEAT, EL_TAB_B },
			{ EL_TAB_PROJ_C, EL_TAB_PROJ_C_SEAT, EL_TAB_C },
			{ EL_TAB_PROJ_O, EL_TAB_PROJ_O_SEAT, EL_TAB_O }
			};


	el_index = eleana_index_open( filename );
	if ( !el_index )
	{
		fprintf( stderr, "Unable to load TSV index file %s\n", filename );
		return 1;
	}

	if ( eleana_index_record_load( el_index, year, 0, &full, &polymap, &table, &seats, NULL, NULL ) )
	{
		fprintf( stderr, "stage_1: Unable to open year %i\n", year );
		return 1;
	}

	if ( eleana_calculate_eleana_swing( table, seats, swing, target_seats ) )
	{
		goto error;
	}

	for ( i=0; i<4; i++ )
	{
		target_p[i] = 100 * rint( EL_TAB_VALUE( seats, tab[i][0] ) ) / EL_TAB_VALUE( seats, EL_TAB_TURNOUT );
	}

	printf( "Target S = %4i %4i %4i %4i\n", target_seats[0], target_seats[1], target_seats[2], target_seats[3] );
	printf( "Target P = %4.1f %4.1f %4.1f %4.1f\n", target_p[0], target_p[1], target_p[2], target_p[3] );

	eleana_index_record_close( full, polymap, table, NULL, NULL );
	return 0;

error:
	eleana_index_record_close( full, polymap, table, NULL, NULL );
	return 1;
}

static int stage_2345( int year, double swings[6], double swing_ranges[6], char *output_filename )
{
#define K_ARRAY_SIZE	6
//#define K_ARRAY_MAX	11
#define K_ARRAY_MAX	7
#define MAX_SEAT_ERROR	0
#define MAX_PERC_ERROR	1.3
#define RANDY_LIMIT1	10000000
#define RANDY_LIMIT2	20000000

	CedSheet	*full;
	mtTree		*polymap;
	double		*table, swing[6] = { 0 }, start_p[4];
	int		/*ka[K_ARRAY_SIZE] = { 0 }, */i, j, tot = 0,
			totmax = K_ARRAY_MAX * K_ARRAY_MAX * K_ARRAY_MAX * K_ARRAY_MAX * K_ARRAY_MAX * K_ARRAY_MAX ,
			year_seats[4], /*k2 = (K_ARRAY_MAX-1) / 2,*/ diff, diffd,
			seats, tab[4][3] =
			{
			{ EL_TAB_PROJ_A, EL_TAB_PROJ_A_SEAT, EL_TAB_A },
			{ EL_TAB_PROJ_B, EL_TAB_PROJ_B_SEAT, EL_TAB_B },
			{ EL_TAB_PROJ_C, EL_TAB_PROJ_C_SEAT, EL_TAB_C },
			{ EL_TAB_PROJ_O, EL_TAB_PROJ_O_SEAT, EL_TAB_O }
			};

	if ( eleana_index_record_load( el_index, year, 0, &full, &polymap, &table, &seats, NULL, NULL ) )
	{
		fprintf( stderr, "stage_2345: Unable to open year %i\n", year );
		return 1;
	}

	if ( eleana_calculate_eleana_swing( table, seats, swing, year_seats ) )
	{
		goto error;
	}

	for ( i=0; i<4; i++ )
	{
		start_p[i] = 100 * rint( EL_TAB_VALUE( seats, tab[i][0] ) ) / EL_TAB_VALUE( seats, EL_TAB_TURNOUT );
	}

	printf( "START  S = %4i %4i %4i %4i\n", year_seats[0], year_seats[1], year_seats[2], year_seats[3] );
	printf( "START  P = %4.1f %4.1f %4.1f %4.1f\n", start_p[0], start_p[1], start_p[2], start_p[3] );
	printf( "\n\n" );

	do
	{
		// Set up and calculate swings for this iteration
		for ( j=0; j<K_ARRAY_SIZE; j++ )
		{
//			swing[j] = (swings[j] + swing_range * (ka[j] - k2) / k2);
			swing[j] = (swings[j] + swing_ranges[j] * ((rand() % RANDY_LIMIT2) - RANDY_LIMIT1) / RANDY_LIMIT1 );
//printf("%f ", swing[j]);
		}
//printf("\n");

		if ( eleana_calculate_eleana_swing( table, seats, swing, NULL ) )
			goto next;

		diff = 0;
		for ( j=0; j<4; j++ )
		{
			year_seats[j] = (int)EL_TAB_VALUE( seats, tab[j][1] );
			diffd = abs( year_seats[j] - target_seats[j] );

			// Ignore if not close to true result
			if ( diffd > MAX_SEAT_ERROR ) goto next;
			diff += diffd;

// FIXME - calculate new % totals, compare to target, if >5% for any of the 4 parties continue to next

//printf("%i ", abs( year_seats[j] - target_seats[j] ));
//printf("%i ", year_seats[j] );
		}
//printf("\n");

		for ( i=0; i<4; i++ )
		{
			start_p[i] = 100 * rint( EL_TAB_VALUE( seats, tab[i][0] ) ) / EL_TAB_VALUE( seats, EL_TAB_TURNOUT );
			if ( fabs( start_p[i] - target_p[i] ) > MAX_PERC_ERROR ) goto next;
		}

		printf( "Valid result: %i %i %i %i    %6.2f %6.2f %6.2f %6.2f    ",
			year_seats[0] , year_seats[1] , year_seats[2] , year_seats[3],
			start_p[0], start_p[1], start_p[2], start_p[3]
			);
		for ( j=0; j<6; j++ )
		{
			printf("%6.2f ", swing[j]);
		}
		printf("\n");

// FIXME - add this swing to the tree

		tot++;
next:
		if (0)
		{
		}
/*
		// This neat little trick simulates a bunch of nested FOR loops,
		// but without the code bloat
		for ( i=0; i<K_ARRAY_SIZE; i++ )
		{
			ka[i]++;
			if ( ka[i] < K_ARRAY_MAX ) break;
			ka[i] = 0;
		}
*/
//	} while ( i<K_ARRAY_SIZE );
	} while ( tot<100 );

	printf( "Total valid swings = %i/%i = %.3f%%\n\n", tot, totmax, 100 * (double)tot / (double)totmax );

// FIXME - output results to a TSV file

	eleana_index_record_close( full, polymap, table, NULL, NULL );
	return 0;

error:
	eleana_index_record_close( full, polymap, table, NULL, NULL );
	return 1;
}

int main( int argc, char *argv[] )
{
	int		year1, year2;
	double		swings[6], swrang[6];


	srand( (unsigned int)time(NULL) );              // Ensure random numbers really are random
printf("argc = %i\n", argc);

	if ( argc != 17 ||
		mtkit_strtoi( argv[2], &year1, NULL, 1 ) ||
		mtkit_strtoi( argv[3], &year2, NULL, 1 ) ||
		mtkit_strtod( argv[5], &swings[0], NULL, 1 ) ||
		mtkit_strtod( argv[6], &swings[1], NULL, 1 ) ||
		mtkit_strtod( argv[7], &swings[2], NULL, 1 ) ||
		mtkit_strtod( argv[8], &swings[3], NULL, 1 ) ||
		mtkit_strtod( argv[9], &swings[4], NULL, 1 ) ||
		mtkit_strtod( argv[10], &swings[5], NULL, 1 ) ||
		mtkit_strtod( argv[11], &swrang[0], NULL, 1 ) ||
		mtkit_strtod( argv[12], &swrang[1], NULL, 1 ) ||
		mtkit_strtod( argv[13], &swrang[2], NULL, 1 ) ||
		mtkit_strtod( argv[14], &swrang[3], NULL, 1 ) ||
		mtkit_strtod( argv[15], &swrang[4], NULL, 1 ) ||
		mtkit_strtod( argv[16], &swrang[5], NULL, 1 ) ||
		year1 < 0 ||
		year2 < 0
		)
	{
		fprintf( stderr, "Usage: %s FILE_INDEX INT_YEAR1 INT_YEAR2 FILE_RESULTS "
			"NUM_MIDSWING_CON_LAB NUM_MIDSWING_CON_LIB NUM_MIDSWING_CON_OTH "
			"NUM_MIDSWING_LAB_LIB NUM_MIDSWING_LAB_OTH NUM_MIDSWING_LIB_OTH "
			"NUM_SWING_RANGE_CON_LAB NUM_SWING_RANGE_CON_LIB NUM_SWING_RANGE_CON_OTH "
			"NUM_SWING_RANGE_LAB_LIB NUM_SWING_RANGE_LAB_OTH NUM_SWING_RANGE_LIB_OTH "
			"\n\n"
			"\n",
			argv[0] );
		return 0;	// No argument passed
	}

	printf(	"Index file      %s\n"
		"Year 1          %2i - %s\n"
		"Year 2          %2i - %s\n"
		"Results file    %s\n"
		"Con - Lab       %f\n"
		"Con - Lib       %f\n"
		"Con - Oth       %f\n"
		"Lab - Lib       %f\n"
		"Lab - Oth       %f\n"
		"Lib - Oth       %f\n"
		"Con - Lab R     %f\n"
		"Con - Lib R     %f\n"
		"Con - Oth R     %f\n"
		"Lab - Lib R     %f\n"
		"Lab - Oth R     %f\n"
		"Lib - Oth R     %f\n"
		"\n",
		argv[1],
		year1, eleana_index_get( el_index, year1, ELEANA_INDEX_COL_YEAR),
		year2, eleana_index_get( el_index, year2, ELEANA_INDEX_COL_YEAR),
		argv[4],
		swings[0], swings[1], swings[2], swings[3], swings[4], swings[5],
		swrang[0], swrang[1], swrang[2], swrang[3], swrang[4], swrang[5] );

	if ( stage_1( argv[1], year2 ) ) goto error;
	if ( stage_2345( year1, swings, swrang, argv[4] ) ) goto error;

	eleana_index_close( el_index );
	return 0;

error:
	printf( "ERROR!!!\n\n" );
	eleana_index_close( el_index );

	return 1;
}
