/*
	Copyright (C) 2004-2013 Mark Tyler
	Copyright (C) 2006-2008 Mark Tyler and Dmitry Groshev

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "private.h"



char * mtgex_gtkncpy (
	char		* dest,
	const char	* src,
	int		cnt
	)
{
	char		* c;


	c = g_locale_from_utf8 ( (gchar *)src, -1, NULL, NULL, NULL );

	if ( c )
	{
		if ( ! dest )
		{
			return c;
		}

		mtkit_strncpy0 ( dest, c, cnt );
		g_free ( c );
	}
	else
	{
		if ( ! dest )
		{
			return g_strdup ( src );
		}

		mtkit_strncpy0 ( dest, src, cnt );
	}

	return dest;
}

char * mtgex_gtkuncpy (
	char		* dest,
	const char	* src,
	int		cnt
	)
{
	char		* c;


	c = g_locale_to_utf8 ( (gchar *)src, -1, NULL, NULL, NULL );

	if ( c )
	{
		if ( ! dest )
		{
			return c;
		}

		mtkit_strncpy0 ( dest, c, cnt );
		g_free ( c );
	}
	else
	{
		if ( ! dest )
		{
			return g_strdup ( src );
		}

		// As a final fallback use ISO-8859-1 - we must always return
		// UTF-8

		c = mtkit_iso8859_to_utf8 ( (char *)src, 0, NULL );

		if ( c )
		{
			mtkit_strncpy0 ( dest, c, cnt );
			free ( c );
		}
		else
		{
			// Miserable failure so return an empty string
			dest[0] = 0;
		}
	}

	return dest;
}



////	ALERT BOX

static int	alert_result;

static gboolean alert_reply (
	GtkWidget	* widget,
	gpointer	data
	)
{
	if ( alert_result == 0 )
	{
		alert_result = (intptr_t)data;
	}

	if ( alert_result == 10 )
	{
		gtk_widget_destroy ( widget );
	}

	return FALSE;
}

int mtgex_alert_box (
	char		* title,
	char		* message,
	char		* text1,
	char		* text2,
	char		* text3,
	GtkWidget	* main_window
	)
{
	GtkWidget	* alert,
			* buttons[3],
			* label;
	char		* butxt[3] = { text1, text2, text3 };
	gint		i;
	GtkAccelGroup	* ag;


	if ( ! text1 )
	{
		return 1;
	}

	ag = gtk_accel_group_new ();

	alert = gtk_dialog_new ();
	gtk_window_set_title ( GTK_WINDOW ( alert ), title );
	gtk_window_set_modal ( GTK_WINDOW ( alert ), TRUE );
	gtk_window_set_position ( GTK_WINDOW ( alert ), GTK_WIN_POS_CENTER );
	gtk_container_set_border_width ( GTK_CONTAINER ( alert ), 6 );
	gtk_signal_connect ( GTK_OBJECT ( alert ), "destroy",
			GTK_SIGNAL_FUNC ( alert_reply ), (gpointer) 10 );

	label = gtk_label_new ( message );
	gtk_label_set_line_wrap ( GTK_LABEL ( label ), TRUE );
	gtk_box_pack_start ( GTK_BOX ( GTK_DIALOG ( alert )->vbox), label,
		TRUE, FALSE, 8 );
	gtk_widget_show ( label );

	for ( i = 0; i <= 2; i++ )
	{
		if ( ! butxt[i] )
		{
			continue;
		}

		buttons[i] = mtgex_add_a_button ( butxt[i], 2,
			GTK_DIALOG ( alert )->action_area, TRUE );
		gtk_signal_connect ( GTK_OBJECT ( buttons[i] ), "clicked",
			GTK_SIGNAL_FUNC ( alert_reply ),
			(gpointer)(intptr_t)(i + 1));
	}

	gtk_widget_add_accelerator ( buttons[0], "clicked", ag, GDK_Escape, 0,
		(GtkAccelFlags) 0);

	if ( main_window )
	{
		gtk_window_set_transient_for ( GTK_WINDOW ( alert ),
			GTK_WINDOW ( main_window ) );
	}

	gtk_widget_show ( alert );
	gdk_window_raise ( alert->window );
	alert_result = 0;
	gtk_window_add_accel_group ( GTK_WINDOW ( alert ), ag );

	while ( alert_result == 0 )
	{
		gtk_main_iteration ();
	}

	if ( alert_result != 10 )
	{
		gtk_widget_destroy ( alert );
	}

	return alert_result;
}

void mtgex_store_window_position (
	GtkWidget	* window,
	mtPrefs		* prefs,
	char		* key_prefix
	)
{
	char		name[128];
	gint		xywh[4];
	int		i,
			l = strlen ( key_prefix );


	if ( l > 120 )
	{
		return;
	}

	memcpy ( name, key_prefix, l );
	name[l++] = '_';
	name[l + 1] = '\0';
	gdk_window_get_size ( window->window, xywh + 2, xywh + 3 );
	gdk_window_get_root_origin ( window->window, xywh + 0, xywh + 1 );

	for ( i = 0; i < 4; i++ )
	{
		name[l] = "xywh"[i];
		mtkit_prefs_set_int ( prefs, name, xywh[i] );
	}
}

void mtgex_restore_window_position (
	GtkWidget	* window,
	mtPrefs		* prefs,
	char		* key_prefix
	)
{
	char		name[128];
	gint		xywh[4] = {0};
	int		i,
			l = strlen ( key_prefix );


	if ( l > 120 )
	{
		return;
	}

	memcpy ( name, key_prefix, l );
	name[l++] = '_';
	name[l + 1] = '\0';

	for ( i = 0; i < 4; i++ )
	{
		name[l] = "xywh"[i];
		mtkit_prefs_get_int ( prefs, name, &xywh[i] );
	}

	gtk_window_set_default_size ( GTK_WINDOW ( window ), xywh[2], xywh[3]);
	gtk_widget_set_uposition ( window, xywh[0], xywh[1] );
}

GtkWidget * mtgex_add_a_window (
	GtkWindowType	type,
	char		* title,
	GtkWindowPosition pos,
	gboolean	modal
	)
{
	GtkWidget	* win;


	win = gtk_window_new ( type );

	gtk_window_set_title ( GTK_WINDOW ( win ), title );
	gtk_window_set_position ( GTK_WINDOW ( win ), pos );
	gtk_window_set_modal ( GTK_WINDOW ( win ), modal );

	return win;
}

void mtgex_destroy_dialog (
	GtkWidget	* window
	)
{
	/* Needed in Windows to stop GTK+ lowering the main window */
	gtk_window_set_transient_for ( GTK_WINDOW ( window ), NULL );
	gtk_widget_destroy ( window );
}

GtkWidget * mtgex_add_a_button (
	char		* text,
	int		bord,
	GtkWidget	* box,
	gboolean	filler
	)
{
	GtkWidget	* button;


	button = gtk_button_new_with_label ( text );

	gtk_widget_show ( button );
	gtk_box_pack_start ( GTK_BOX ( box ), button, filler, filler, 0 );
	gtk_container_set_border_width ( GTK_CONTAINER ( button ), bord );

	return button;
}

static GtkWidget * spin_new_x (
	GtkObject	* adj,
	int		fpart
	)
{
	GtkWidget	* spin;


	spin = gtk_spin_button_new ( GTK_ADJUSTMENT ( adj ), 1, fpart );
	gtk_widget_show ( spin );
	gtk_spin_button_set_numeric ( GTK_SPIN_BUTTON ( spin ), TRUE );

	return spin;
}

GtkWidget * mtgex_spinbutton_new (
	double		value,
	double		min,
	double		max,
	double		increment,
	int		digits
	)
{
	return spin_new_x ( gtk_adjustment_new ( value, min, max, increment,
		10, 0 ), digits );
}

void mtgex_spinbutton_connect (
	GtkWidget	* spin,
	GtkSignalFunc	handler,
	gpointer	user_data
	)
{
	GtkAdjustment	* adj;


	adj = gtk_spin_button_get_adjustment ( GTK_SPIN_BUTTON ( spin ) );
	gtk_signal_connect ( GTK_OBJECT ( adj ), "value_changed", handler,
		user_data );
}

GtkWidget * mtgex_add_a_spin (
	int		value,
	int		min,
	int		max
	)
{
	return spin_new_x ( gtk_adjustment_new ( value, min, max, 1, 10, 0 ),
		0 );
}

GtkWidget * mtgex_add_a_table (
	int		rows,
	int		columns,
	int		bord,
	GtkWidget	* box
	)
{
	GtkWidget	* table;


	table = mtgex_pack ( box, gtk_table_new ( rows, columns, FALSE ) );

	gtk_widget_show ( table );
	gtk_container_set_border_width ( GTK_CONTAINER ( table ), bord );

	return table;
}

GtkWidget * mtgex_add_to_table (
	char		* text,
	GtkWidget	* table,
	int		row,
	int		column,
	int		hspacing,
	int		vspacing
	)
{
	GtkWidget	* label;


	label = gtk_label_new ( text );
	gtk_widget_show ( label );
	gtk_table_attach ( GTK_TABLE ( table ), label, column, column + 1,
		row, row + 1, (GtkAttachOptions) GTK_FILL,
		(GtkAttachOptions) 0, hspacing, vspacing );

	gtk_label_set_justify ( GTK_LABEL ( label ), GTK_JUSTIFY_LEFT );
	gtk_misc_set_alignment ( GTK_MISC ( label ), 0.0, 0.5 );

	return label;
}

int mtgex_table_clear (
	GtkWidget	* table
	)
{
	GList		* item;


	if ( ! table )
	{
		return 1;
	}

	for (	item = GTK_TABLE ( table )->children;
		item;
		item = GTK_TABLE ( table )->children
		)
	{
		gtk_container_remove ( GTK_CONTAINER ( table ),
			( (GtkTableChild *)item->data )->widget );
	}

	return 0;
}

void mtgex_add_hseparator (
	GtkWidget	* widget,
	int		xs,
	int		ys
	)
{
	GtkWidget	* sep;


	sep = mtgex_pack ( widget, gtk_hseparator_new () );

	gtk_widget_show ( sep );
	gtk_widget_set_usize ( sep, xs, ys );
}

GtkWidget * mtgex_pack (
	GtkWidget	* box,
	GtkWidget	* widget
	)
{
	gtk_box_pack_start ( GTK_BOX ( box ), widget, FALSE, FALSE, 0 );

	return widget;
}

GtkWidget * mtgex_xpack (
	GtkWidget	* box,
	GtkWidget	* widget
	)
{
	gtk_box_pack_start ( GTK_BOX ( box ), widget, TRUE, TRUE, 0 );

	return widget;
}

GtkWidget * mtgex_pack_end (
	GtkWidget	* box,
	GtkWidget	* widget
	)
{
	gtk_box_pack_end ( GTK_BOX ( box ), widget, FALSE, FALSE, 0 );

	return widget;
}

GtkWidget * mtgex_pack5 (
	GtkWidget	* box,
	GtkWidget	* widget
	)
{
	gtk_box_pack_start ( GTK_BOX ( box ), widget, FALSE, FALSE, 5 );

	return widget;
}

GtkWidget * mtgex_xpack5 (
	GtkWidget	* box,
	GtkWidget	* widget
	)
{
	gtk_box_pack_start ( GTK_BOX ( box ), widget, TRUE, TRUE, 5 );

	return widget;
}

GtkWidget * mtgex_pack_end5 (
	GtkWidget	* box,
	GtkWidget	* widget
	)
{
	gtk_box_pack_end ( GTK_BOX ( box ), widget, FALSE, FALSE, 5 );

	return widget;
}



// Focusable pixmap widget

#define FPIXMAP_KEY "mtPaint.fpixmap"

typedef struct {
	int		xp,
			yp,
			width,
			height,
			xc,
			yc,
			focused_cursor;
	GdkRectangle	pm,
			cr;
	GdkPixmap	* pixmap,
			* cursor;
	GdkBitmap	* cmask;
} fpixmap_data;



static guint		fpixmap_key;



static fpixmap_data * wj_fpixmap_data (
	GtkWidget	* widget
	)
{
	return gtk_object_get_data_by_id ( GTK_OBJECT ( widget ), fpixmap_key );
}

static void wj_fpixmap_paint (
	GtkWidget	* widget,
	GdkRectangle	* area
	)
{
	GdkRectangle	pdest,
			cdest;
	fpixmap_data	* d;
	int		draw_pmap;


	if ( ! GTK_WIDGET_DRAWABLE ( widget ) )
	{
		return;
	}

	if ( ! (d = wj_fpixmap_data ( widget ) ) )
	{
		return;
	}

	draw_pmap = d->pixmap && gdk_rectangle_intersect ( &d->pm, area,
		&pdest );

	/* Frame */
	if ( d->pixmap )
	{
		gdk_draw_rectangle ( widget->window, widget->style->black_gc,
			FALSE, d->pm.x - 1, d->pm.y - 1, d->pm.width + 1,
			d->pm.height + 1 );
	}

	while ( draw_pmap )
	{
		/* Contents pixmap */
		gdk_draw_pixmap ( widget->window, widget->style->black_gc,
			d->pixmap, pdest.x - d->pm.x, pdest.y - d->pm.y,
			pdest.x, pdest.y, pdest.width, pdest.height );

		/* Cursor pixmap */
		if (	d->focused_cursor &&
			! GTK_WIDGET_HAS_FOCUS ( widget )
			)
		{
			break;
		}

		if (	! d->cursor ||
			! gdk_rectangle_intersect ( &d->cr, &pdest, &cdest )
			)
		{
			break;
		}

		if ( d->cmask )
		{
			gdk_gc_set_clip_mask ( widget->style->black_gc,
				d->cmask );
			gdk_gc_set_clip_origin ( widget->style->black_gc,
				d->cr.x, d->cr.y );
		}

		gdk_draw_pixmap ( widget->window, widget->style->black_gc,
			d->cursor, cdest.x - d->cr.x, cdest.y - d->cr.y,
			cdest.x, cdest.y, cdest.width, cdest.height );

		if ( d->cmask )
		{
			gdk_gc_set_clip_mask ( widget->style->black_gc, NULL );
			gdk_gc_set_clip_origin ( widget->style->black_gc, 0, 0
				);
		}

		break;
	}

	/* Focus rectangle */
	if ( GTK_WIDGET_HAS_FOCUS ( widget ) )
	{
		gtk_paint_focus ( widget->style, widget->window,
			GTK_WIDGET_STATE ( widget ), area, widget, NULL, 0, 0,
			widget->allocation.width - 1,
			widget->allocation.height - 1 );
	}
}

static gboolean wj_fpixmap_expose (
	GtkWidget	* widget,
	GdkEventExpose	* event
	)
{
	wj_fpixmap_paint ( widget, &event->area );

	return FALSE;
}

static void wj_fpixmap_size_req (
	GtkWidget	* widget,
	GtkRequisition	* requisition,
	gpointer	user_data
	)
{
	fpixmap_data	* d = user_data;
	gint		xp,
			yp;


	gtk_widget_style_get ( GTK_WIDGET ( widget ),
		"focus-line-width", &xp, "focus-padding", &yp, NULL );
	yp = (xp + yp) * 2 + 2;
	xp = widget->style->xthickness * 2 + yp;
	yp = widget->style->ythickness * 2 + yp;

	requisition->width = d->width + (d->xp = xp);
	requisition->height = d->height + (d->yp = yp);
}

static void wj_fpixmap_size_alloc (
	GtkWidget	* widget,
	GtkAllocation	* allocation,
	gpointer	user_data
	)
{
	fpixmap_data	* d = user_data;
	GdkRectangle	opm = d->pm;
	int		w,
			h,
			x = d->xp,
			y = d->yp;


	w = allocation->width - d->xp;
	h = allocation->height - d->yp;

	if ( w > d->width )
	{
		x = allocation->width - d->width , w = d->width;
	}

	if ( y > d->height )
	{
		y = allocation->height - d->height , h = d->height;
	}

	x /= 2; y /= 2;
	w = w < 0 ? 0 : w;
	h = h < 0 ? 0 : h;
	d->pm.x = x;
	d->pm.y = y;
	d->pm.width = w;
	d->pm.height = h;
	d->cr.x += x - opm.x;
	d->cr.y += y - opm.y;
}

static void wj_fpixmap_destroy (
	GtkObject	* object,
	gpointer	user_data
	)
{
	fpixmap_data	* d = user_data;


	if ( d->pixmap )
	{
		gdk_pixmap_unref ( d->pixmap );
	}

	if ( d->cursor )
	{
		gdk_pixmap_unref ( d->cursor );
	}

	if ( d->cmask )
	{
		gdk_bitmap_unref ( d->cmask );
	}

	free ( d );
}

GtkWidget * mtgex_wj_fpixmap (
	int		width,
	int		height
	)
{
	GtkWidget	* w;
	fpixmap_data	* d;


	if ( ! fpixmap_key )
	{
		fpixmap_key = g_quark_from_static_string ( FPIXMAP_KEY );
	}

	d = calloc ( 1, sizeof ( fpixmap_data ) );

	if ( ! d )
	{
		return NULL;
	}

	d->width = width;
	d->height = height;
	w = gtk_drawing_area_new ();
	GTK_WIDGET_SET_FLAGS ( w, GTK_CAN_FOCUS );
	gtk_widget_set_events ( w, GDK_ALL_EVENTS_MASK );
	gtk_widget_show ( w );
	gtk_object_set_data_by_id ( GTK_OBJECT ( w ), fpixmap_key, d );

	gtk_signal_connect ( GTK_OBJECT ( w ), "destroy",
		GTK_SIGNAL_FUNC ( wj_fpixmap_destroy ), d );

	gtk_signal_connect ( GTK_OBJECT ( w ), "expose_event",
		GTK_SIGNAL_FUNC ( wj_fpixmap_expose ), NULL );

	gtk_signal_connect_after ( GTK_OBJECT ( w ), "size_request",
		GTK_SIGNAL_FUNC ( wj_fpixmap_size_req ), d );

	gtk_signal_connect ( GTK_OBJECT ( w ), "size_allocate",
		GTK_SIGNAL_FUNC ( wj_fpixmap_size_alloc ), d );

	return w;
}

GdkPixmap * mtgex_wj_fpixmap_pixmap (
	GtkWidget	* widget
	)
{
	fpixmap_data	* d;


	d = wj_fpixmap_data ( widget );

	if ( ! d )
	{
		return NULL;
	}

	if (	! d->pixmap &&
		GTK_WIDGET_REALIZED ( widget )
		)
	{
		d->pixmap = gdk_pixmap_new ( widget->window, d->width,
			d->height, -1 );
	}

	return d->pixmap;
}

void mtgex_wj_fpixmap_draw_rgb (
	GtkWidget	* widget,
	int		x,
	int		y,
	int		w,
	int		h,
	unsigned char	* rgb,
	int		step
	)
{
	fpixmap_data	* d;


	d = wj_fpixmap_data ( widget );

	if ( ! d )
	{
		return;
	}

	if ( ! d->pixmap )
	{
		return;
	}

	gdk_draw_rgb_image ( d->pixmap, widget->style->black_gc,
		x, y, w, h, GDK_RGB_DITHER_NONE, rgb, step );
	gtk_widget_queue_draw_area ( widget, x + d->pm.x, y + d->pm.y, w, h );
}

void mtgex_wj_fpixmap_fill_rgb (
	GtkWidget	* widget,
	int		x,
	int		y,
	int		w,
	int		h,
	int		rgb
	)
{
	GdkGCValues	sv;
	fpixmap_data	* d;


	d = wj_fpixmap_data ( widget );

	if ( ! d )
	{
		return;
	}

	if ( ! d->pixmap )
	{
		return;
	}

	gdk_gc_get_values ( widget->style->black_gc, &sv );
	gdk_rgb_gc_set_foreground ( widget->style->black_gc, rgb );
	gdk_draw_rectangle ( d->pixmap, widget->style->black_gc, TRUE, x, y, w,
		h );
	gdk_gc_set_foreground ( widget->style->black_gc, &sv.foreground );
	gtk_widget_queue_draw_area ( widget, x + d->pm.x, y + d->pm.y, w, h );
}

void mtgex_wj_fpixmap_move_cursor (
	GtkWidget	* widget,
	int		x,
	int		y
	)
{
	GdkRectangle	ocr,
			tcr1,
			tcr2,
			* rcr = NULL;
	fpixmap_data	* d;


	d = wj_fpixmap_data ( widget );

	if ( ! d )
	{
		return;
	}

	if ( ( x == d->xc ) && ( y == d->yc ) )
	{
		return;
	}

	ocr = d->cr;
	d->cr.x += x - d->xc;
	d->cr.y += y - d->yc;
	d->xc = x; d->yc = y;

	if ( ! d->pixmap || ! d->cursor )
	{
		return;
	}

	if ( d->focused_cursor && ! GTK_WIDGET_HAS_FOCUS ( widget ) )
	{
		return;
	}

	/* Anything visible? */
	if ( ! GTK_WIDGET_VISIBLE ( widget ) )
	{
		return;
	}

	if ( gdk_rectangle_intersect ( &ocr, &d->pm, &tcr1 ) )
	{
		rcr = &tcr1;
	}

	if ( gdk_rectangle_intersect ( &d->cr, &d->pm, &tcr2 ) )
	{
		if ( rcr )
		{
			gdk_rectangle_union ( &tcr1, &tcr2, rcr = &ocr );
		}
		else
		{
			rcr = &tcr2;
		}
	}

	if ( ! rcr )
	{
		return;			// Both positions invisible
	}

	gtk_widget_queue_draw_area ( widget, rcr->x, rcr->y, rcr->width,
		rcr->height );
}

int mtgex_wj_fpixmap_set_cursor (
	GtkWidget	* widget,
	char		* image,
	char		* mask,
	int		width,
	int		height,
	int		hot_x,
	int		hot_y,
	int		focused
	)
{
	fpixmap_data	* d;


	if ( ! GTK_WIDGET_REALIZED ( widget ) )
	{
		return FALSE;
	}

	d = wj_fpixmap_data ( widget );

	if ( ! d )
	{
		return FALSE;
	}

	if ( d->cursor )
	{
		gdk_pixmap_unref ( d->cursor );
	}

	if ( d->cmask )
	{
		gdk_bitmap_unref ( d->cmask );
	}

	d->cursor = NULL; d->cmask = NULL;
	d->focused_cursor = focused;

	while ( image )
	{
		d->cursor = gdk_pixmap_create_from_data ( widget->window,
			image, width, height, -1,
			&widget->style->white, &widget->style->black );
		d->cr.x = d->pm.x + d->xc - hot_x;
		d->cr.y = d->pm.y + d->yc - hot_y;
		d->cr.width = width;
		d->cr.height = height;

		if ( ! mask )
		{
			break;
		}

		d->cmask = gdk_bitmap_create_from_data ( widget->window,
			mask, width, height );

		break;
	}

	if ( ! d->pixmap )
	{
		return TRUE;
	}

	/* Optimizing redraw in a rare operation is useless */
	gtk_widget_queue_draw ( widget );

	return TRUE;
}

int mtgex_wj_fpixmap_xy (
	GtkWidget	* widget,
	int		x,
	int		y,
	int		* xr,
	int		* yr
	)
{
	fpixmap_data	* d;


	d = wj_fpixmap_data ( widget );

	if ( ! d )
	{
		return FALSE;
	}

	if ( ! d->pixmap )
	{
		return FALSE;
	}

	x -= d->pm.x;
	y -= d->pm.y;
	xr[0] = x;
	yr[0] = y;

	if (	x < 0			||
		x >= d->pm.width	||
		y < 0			||
		y >= d->pm.height
		)
	{
		return FALSE;
	}

	return TRUE;
}

void mtgex_wj_fpixmap_cursor (
	GtkWidget	* widget,
	int		* x,
	int		* y
	)
{
	fpixmap_data	* d;


	d = wj_fpixmap_data ( widget );

	if ( d && d->cursor )
	{
		x[0] = d->xc;
		y[0] = d->yc;
	}
	else
	{
		x[0] = y[0] = 0;
	}
}

unsigned char * mtgex_wj_get_rgb_image (
	GdkWindow	* window,
	GdkPixmap	* pixmap,
	unsigned char	* buf,
	int		x,
	int		y,
	int		width,
	int		height
	)
{
	GdkPixbuf	* pix,
			* res;
	unsigned char	* wbuf = NULL;


	if ( ! buf )
	{
		buf = wbuf = malloc ( width * height * 3 );
	}

	if ( ! buf )
	{
		return NULL;
	}

	pix = gdk_pixbuf_new_from_data ( buf, GDK_COLORSPACE_RGB,
		FALSE, 8, width, height, width * 3, NULL, NULL );

	if ( pix )
	{
		res = gdk_pixbuf_get_from_drawable ( pix,
			pixmap ? pixmap : window, NULL,
			x, y, 0, 0, width, height );
		g_object_unref ( pix );

		if ( res )
		{
			return buf;
		}
	}

	free ( wbuf );

	return NULL;
}

int mtgex_arrow_key (
	GdkEventKey	* event,
	int		* dx,
	int		* dy,
	int		mult
	)
{
	if ( ( event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK) ) !=
		GDK_SHIFT_MASK )
	{
		mult = 1;
	}

	dx[0] = dy[0] = 0;

	switch ( event->keyval )
	{
		case GDK_KP_Left:
		case GDK_Left:
			dx[0] = -mult;
			break;

		case GDK_KP_Right:
		case GDK_Right:
			dx[0] = mult;
			break;

		case GDK_KP_Up:
		case GDK_Up:
			dy[0] = -mult;
			break;

		case GDK_KP_Down:
		case GDK_Down:
			dy[0] = mult;
			break;
	}

	return ( dx[0] || dy[0] );
}

GdkCursor * mtgex_make_cursor (
	const char	* icon,
	const char	* mask,
	int		w,
	int		h,
	int		tip_x,
	int		tip_y
	)
{
	static GdkColor	cfg = { -1, -1, -1, -1 },
			cbg = { 0, 0, 0, 0 };
	GdkPixmap	* icn,
			* msk;
	GdkCursor	* cursor;


	icn = gdk_bitmap_create_from_data ( NULL, icon, w, h );
	msk = gdk_bitmap_create_from_data ( NULL, mask, w, h );
	cursor = gdk_cursor_new_from_pixmap ( icn, msk, &cfg, &cbg, tip_x,
		tip_y );
	gdk_pixmap_unref ( icn );
	gdk_pixmap_unref ( msk );

	return cursor;
}


// Box unpacking macros
#define BOX_CHILD_0( box ) \
	( ((GtkBoxChild*)GTK_BOX( box )->children->data)->widget )
#define BOX_CHILD_1( box ) \
	( ((GtkBoxChild*)GTK_BOX( box )->children->next->data)->widget )


GtkWidget * mtgex_spinslide_new (
	gint		swidth,
	gint		sheight,
	gdouble		svalue,
	gdouble		smin,
	gdouble		smax,
	gdouble		sdigits,
	gdouble		step
	)
{
	GtkWidget	* box,
			* slider,
			* spin;
	GtkObject	* adj;


	adj = gtk_adjustment_new ( svalue, smin, smax, step, 0, 0 );
	box = gtk_hbox_new ( FALSE, 0 );

	slider = gtk_hscale_new ( GTK_ADJUSTMENT ( adj ) );
	gtk_box_pack_start ( GTK_BOX ( box ), slider, swidth < 0, TRUE, 0 );
	gtk_widget_set_usize ( slider, swidth, sheight );
	gtk_scale_set_draw_value ( GTK_SCALE ( slider ), FALSE );
	gtk_scale_set_digits ( GTK_SCALE ( slider ), sdigits );

	spin = spin_new_x ( adj, 0 );
	gtk_box_pack_start ( GTK_BOX ( box ), spin, swidth >= 0, TRUE, 2 );
	gtk_spin_button_set_digits ( GTK_SPIN_BUTTON ( spin ), sdigits );

	gtk_widget_show_all ( box );

	return box;
}

void mtgex_spinslide_set_range (
	GtkWidget	* spinslide,
	gdouble		minv,
	gdouble		maxv
	)
{
	GtkAdjustment	* adj;


	adj = gtk_range_get_adjustment ( GTK_RANGE ( BOX_CHILD_0 ( spinslide )
		) );
	adj->lower = minv;
	adj->upper = maxv;
	gtk_adjustment_changed ( adj );
}

gdouble mtgex_spinslide_get_value (
	GtkWidget	* spinslide
	)
{
	GtkSpinButton	* spin;


	spin = GTK_SPIN_BUTTON ( BOX_CHILD_1 ( spinslide ) );
	gtk_spin_button_update ( spin );

	return gtk_spin_button_get_value ( spin );
}

gdouble mtgex_spinslide_read_value (
	GtkWidget	* spinslide
	)
{
	GtkSpinButton	* spin;


	spin = GTK_SPIN_BUTTON ( BOX_CHILD_1 ( spinslide ) );

	return gtk_spin_button_get_value ( spin );
}

void mtgex_spinslide_set_value (
	GtkWidget	* spinslide,
	gdouble		value
	)
{
	GtkSpinButton	* spin;


	spin = GTK_SPIN_BUTTON ( BOX_CHILD_1 ( spinslide ) );
	gtk_spin_button_set_value ( spin, value );
}

void mtgex_spinslide_connect (
	GtkWidget	* spinslide,
	GtkSignalFunc	handler,
	gpointer	user_data
	)
{
	GtkAdjustment	* adj;


	adj = gtk_range_get_adjustment ( GTK_RANGE ( BOX_CHILD_0 ( spinslide )
		) );
	gtk_signal_connect ( GTK_OBJECT ( adj ), "value_changed", handler,
		user_data );
}

void mtgex_spinslide_block (
	GtkWidget	* spinslide,
	GtkSignalFunc	handler,
	gpointer	user_data
	)
{
	GtkAdjustment	* adj;


	adj = gtk_range_get_adjustment ( GTK_RANGE ( BOX_CHILD_0 ( spinslide )
		) );
	gtk_signal_handler_block_by_func ( GTK_OBJECT ( adj ),
		GTK_SIGNAL_FUNC ( handler ), user_data );
}

void mtgex_spinslide_unblock (
	GtkWidget	* spinslide,
	GtkSignalFunc	handler,
	gpointer	user_data
	)
{
	GtkAdjustment	* adj;


	adj = gtk_range_get_adjustment ( GTK_RANGE ( BOX_CHILD_0 ( spinslide )
		) );
	gtk_signal_handler_unblock_by_func ( GTK_OBJECT ( adj ),
		GTK_SIGNAL_FUNC ( handler ), user_data );
}

// Workaround for GtkCList reordering bug
// From mtPaint and the work of Dmitry Groshev

/* This bug is the favorite pet of GNOME developer Behdad Esfahbod
 * See http://bugzilla.gnome.org/show_bug.cgi?id=400249#c2 */

static void clist_drag_fix (
	GtkWidget	* widget,
	GdkDragContext	* drag_context,
	gpointer	user_data
	)
{
	g_dataset_remove_data ( drag_context, "gtk-clist-drag-source" );
}

void mtgex_clist_enable_drag (
	GtkWidget	* clist
	)
{
	gtk_signal_connect ( GTK_OBJECT ( clist ), "drag_begin",
		GTK_SIGNAL_FUNC ( clist_drag_fix ), NULL );
	gtk_signal_connect ( GTK_OBJECT ( clist ), "drag_end",
		GTK_SIGNAL_FUNC ( clist_drag_fix ), NULL );
	gtk_clist_set_reorderable ( GTK_CLIST ( clist ), TRUE );
}

