/*
	Copyright (C) 2009-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "gui_main.h"



static gboolean click_help_end (
	GtkWidget	* widget,
	GdkEvent	* event,
	gpointer	data
	)
{
	gtk_widget_destroy ( widget );

	return FALSE;
}

void pressed_help (
	GtkMenuItem	* menu_item,
	gpointer	user_data,
	gint		item
	)
{
	GtkWidget	* window,
			* vbox,
			* button,
			* textview,
			* sw;
	GtkAccelGroup	* ag;
	GtkTextBuffer	* buffer;
	gchar		* about_text = VERSION"\n"
	"\n"
	"Copyright (C) 2009-2013 Mark Tyler.\n"
	"\n"
	"This program is free software: you can redistribute it and/or modify "
	"it under the terms of the GNU General Public License as published by "
	"the Free Software Foundation, either version 3 of the License, or "
	"(at your option) any later version.\n"
	"\n"
	"This program is distributed in the hope that it will be useful, "
	"but WITHOUT ANY WARRANTY; without even the implied warranty of "
	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
	"GNU General Public License for more details.\n"
	"\n"
	"You should have received a copy of the GNU General Public License "
	"along with this program.  If not, see http://www.gnu.org/licenses/\n"
			;


	ag = gtk_accel_group_new ();

	window = mtgex_add_a_window ( GTK_WINDOW_TOPLEVEL, _("About"),
		GTK_WIN_POS_CENTER, TRUE );
	gtk_window_set_transient_for ( GTK_WINDOW ( window ),
		GTK_WINDOW ( global.main_window ) );

	vbox = gtk_vbox_new ( FALSE, 0 );
	gtk_container_add ( GTK_CONTAINER ( window ), vbox );
	gtk_widget_show ( vbox );

	sw = gtk_scrolled_window_new ( NULL, NULL );
	gtk_widget_show ( sw );
	gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW ( sw ),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
	gtk_box_pack_start ( GTK_BOX ( vbox ), sw, TRUE, TRUE, 0 );

	textview = gtk_text_view_new ();
	gtk_widget_show ( textview );
	gtk_container_add ( GTK_CONTAINER ( sw ), textview );
	gtk_text_view_set_editable ( GTK_TEXT_VIEW ( textview ), FALSE );
	gtk_text_view_set_cursor_visible ( GTK_TEXT_VIEW ( textview ), FALSE );
	gtk_text_view_set_wrap_mode ( GTK_TEXT_VIEW (textview ),
		GTK_WRAP_WORD );

	buffer = gtk_text_view_get_buffer ( GTK_TEXT_VIEW ( textview ) );
	gtk_text_buffer_set_text ( GTK_TEXT_BUFFER ( buffer ), about_text, -1 );

	mtgex_add_hseparator ( vbox, -2, 10 );

	button = mtgex_pack ( vbox, gtk_button_new_with_label ( _("Close") ) );

	gtk_widget_add_accelerator ( button, "clicked", ag, GDK_KP_Enter, 0,
		(GtkAccelFlags) 0 );

	gtk_widget_add_accelerator ( button, "clicked", ag, GDK_Return, 0,
		(GtkAccelFlags) 0 );

	gtk_widget_add_accelerator ( button, "clicked", ag, GDK_Escape, 0,
		(GtkAccelFlags) 0 );

	GTK_WIDGET_SET_FLAGS ( button, GTK_CAN_DEFAULT );

	gtk_signal_connect_object ( GTK_OBJECT ( button ), "clicked",
		GTK_SIGNAL_FUNC ( click_help_end ), GTK_OBJECT ( window ) );

	gtk_signal_connect ( GTK_OBJECT ( window ), "delete_event",
		GTK_SIGNAL_FUNC ( click_help_end ), NULL );

	gtk_widget_show ( button );

	gtk_window_set_default_size ( GTK_WINDOW ( window ), 512, 400 );

	gtk_widget_show ( window );
	gtk_window_add_accel_group ( GTK_WINDOW ( window ), ag );
}

///	FILE SELECTION WINDOW

static gboolean fs_destroy (
	GtkWidget	* fs
	)
{
	mtgex_store_window_position ( fs, global.prefs_file,
		_INIFILE_MAIN_FPICK"_window" );
	mtgex_fpick_destroy ( fs );

	return FALSE;
}

static void fs_ok (
	GtkWidget	* fs
	)
{
	char		fname[MTGEX_PATHBUF],
			* c,
			c2;
	int		action_type;


	action_type = (intptr_t) gtk_object_get_user_data ( GTK_OBJECT ( fs ) );

	/* Needed to show progress in Windows GTK+2 */
	gtk_window_set_modal ( GTK_WINDOW ( fs ), FALSE );

	/* Looks better if no dialog under progressbar */
	mtgex_store_window_position ( fs, global.prefs_file,
		_INIFILE_MAIN_FPICK"_window" );
	gtk_widget_hide ( fs );

	/* File extension */
	mtkit_strncpy0 ( fname, mtgex_fpick_get_filename ( fs, TRUE ),
		MTGEX_PATHBUF );
	mtgex_fpick_set_filename ( fs, fname, TRUE );

	// Get filename the proper way (convert it from UTF8 in GTK2/Windows,
	// leave it in system filename encoding on Unix)
	mtkit_strncpy0 ( fname, mtgex_fpick_get_filename ( fs, FALSE ),
		MTGEX_PATHBUF );

	switch ( action_type )
	{
	case FS_OPEN_IMAGE:
		if ( image_load ( fname ) )
		{
			goto redo;
		}
		break;
	}

	c = strrchr ( fname, MTKIT_DIR_SEP );
	if ( c )
	{
		c2 = c[0];
		c[0] = '\0';		// Strip off filename
		mtkit_prefs_set_str ( global.prefs_file, _INIFILE_LAST_DIR,
			fname );
		c[0] = c2;
	}

	mtgex_fpick_destroy ( fs );

	return;

redo:
	gtk_widget_show ( fs );
	gtk_window_set_modal ( GTK_WINDOW ( fs ), TRUE );
}

static void fs_setup (
	GtkWidget	* fs,
	GtkWidget	* xtra_hbox,
	int		action_type,
	char		* filename
	)
{
	char		txt[MTGEX_PATHBUF],
			* last;


	last = NULL;
	mtkit_prefs_get_str ( global.prefs_file, _INIFILE_LAST_DIR, &last );

	if ( ! last || ! last[0] )
	{
		last = mtkit_file_home ();
	}

	snprintf ( txt, MTGEX_PATHBUF, "%s%c", last, MTKIT_DIR_SEP );

	gtk_window_set_modal ( GTK_WINDOW ( fs ), TRUE );
	mtgex_restore_window_position ( fs, global.prefs_file,
		_INIFILE_MAIN_FPICK"_window" );

	gtk_object_set_user_data ( GTK_OBJECT ( fs ),
		(gpointer)(intptr_t)action_type );
	mtgex_fpick_setup ( fs, xtra_hbox, GTK_SIGNAL_FUNC ( fs_ok ),
		GTK_SIGNAL_FUNC ( fs_destroy ) );
	mtgex_fpick_set_filename ( fs, txt, FALSE );

	if ( xtra_hbox )
	{
		g_object_set_qdata ( G_OBJECT ( fs ), _FS_XTRA_HBOX, xtra_hbox
			);
	}

	gtk_widget_show ( fs );
	gtk_window_set_transient_for ( GTK_WINDOW ( fs ),
		GTK_WINDOW ( global.main_window ) );

	gdk_window_raise ( fs->window );
	// Needed to ensure window is at the top
}

void file_selector (
	int		action_type,
	GtkWidget	* main_window,
	mtPrefs		* prefs_file,
	char		* filename
	)
{
	GtkWidget	* hbox = NULL;
	char		* title = NULL;
	int		fpick_flags = MTGEX_FPICK_ENTRY;


	switch ( action_type )
	{
	case FS_OPEN_IMAGE:
		title = _("Open Image File");
		fpick_flags = MTGEX_FPICK_LOAD;

		break;
	}

	fs_setup ( mtgex_fpick_new ( title, fpick_flags, main_window,
		global.prefs_file, _INIFILE_MAIN_FPICK_PRE ),
		hbox, action_type, filename );
}

