/*
	Copyright (C) 2009-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "gui_main.h"

#include "icon.xpm"



GUI_Global	global = { NULL };



static void error_message_file (
	char		* message,
	char		* filename
	)
{
	char		* msg,
			* f8;


	f8 = mtgex_gtkuncpy ( NULL, filename, 0 );
	msg = g_strdup_printf ( _("Unable to %s file: %s"), message, f8 );
	mtgex_alert_box ( _("Error"), msg, _("OK"), NULL, NULL,
		global.main_window );
	g_free ( msg );
	g_free ( f8 );
}

static int confirm_warning_alert (
	char		* msg
	)
{
	int		res;


	res = mtgex_alert_box ( _("Warning"), msg, _("Yes"), _("No"), NULL,
		global.main_window );

	if ( res == 1 )
	{
		return 1;		// YES
	}

	return 0;			// NO
}

static void update_titlebar ( void )
{
	// Update filename in titlebar and the menus

	gtk_window_set_title ( GTK_WINDOW ( global.main_window ), VERSION );
}

static void form_table_filename (
	char		* buf,
	int		buflen,
	int		table
	)
{
	snprintf ( buf, buflen, "%s/.%s_%i", mtkit_file_home (), BIN_NAME,
		table );

	buf[buflen - 1] = 0;
}

static void save_current_jigsaw ( void )
{
	char		txt[2048];


	if ( ! global.jigsaw || ! global.current_table )
	{
		return;
	}

	form_table_filename ( txt, 2048, global.current_table );

	if ( mtjigsaw_save ( global.jigsaw, txt ) )
	{
		error_message_file ( "save", txt );
	}
}

static void delete_event (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	save_current_jigsaw ();
	mtgex_store_window_position ( global.main_window, global.prefs_file,
		_INIFILE_MAIN_WINDOW );
	gtk_main_quit ();
}

static void pressed_open_image (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	file_selector ( FS_OPEN_IMAGE, global.main_window, global.prefs_file,
		NULL );
}

static void redraw_canvas ( void )
{
	gtk_widget_queue_draw ( global.main_canvas );
}

static void select_jigsaw_table (
	int		new_table
	)
{
	char		txt[2048];
	mtJigsaw	* new_jigsaw;


	save_current_jigsaw ();

	form_table_filename ( txt, 2048, new_table );

	new_jigsaw = mtjigsaw_load ( txt );

	if ( ! new_jigsaw )
	{
		new_jigsaw = mtjigsaw_load ( NULL );
		// Start with an empty jigsaw
	}

	mtjigsaw_destroy ( global.jigsaw );
	global.jigsaw = new_jigsaw;
	global.current_table = new_table;

	mtkit_prefs_set_int ( global.prefs_file, _INIFILE_MAIN_JIGSAW_TABLE,
		new_table );

	if ( ! global.jigsaw->nodes_hori )
	{
		int		hori = 10,
				vert = 10;


		mtkit_prefs_get_int ( global.prefs_file,
			_INIFILE_MAIN_NODES_HORI, &hori );
		mtkit_prefs_get_int ( global.prefs_file,
			_INIFILE_MAIN_NODES_VERT, &vert );

		if (	hori < MTJIGSAW_NODES_MIN ||
			hori > MTJIGSAW_NODES_MAX ||
			vert < MTJIGSAW_NODES_MIN ||
			vert > MTJIGSAW_NODES_MAX
			)
		{
			hori = 10;
			vert = 10;
			mtkit_prefs_set_int ( global.prefs_file,
				_INIFILE_MAIN_NODES_HORI, hori );
			mtkit_prefs_set_int ( global.prefs_file,
				_INIFILE_MAIN_NODES_VERT, vert );
		}

		mtjigsaw_set_mesh ( global.jigsaw, hori, vert );
	}

	mtjigsaw_set_image ( global.jigsaw, NULL, global.canvas_width,
		global.canvas_height );
	redraw_canvas ();
}

static void pressed_set_fullscreen (
	GtkMenuItem	* menu_item,
	gpointer	user_data,
	gint		item
	)
{
	if ( gtk_check_menu_item_get_active ( GTK_CHECK_MENU_ITEM ( menu_item )
		) )
	{
		gtk_window_fullscreen ( GTK_WINDOW ( global.main_window ) );
	}
	else
	{
		gtk_window_unfullscreen ( GTK_WINDOW ( global.main_window ) );
	}
}

static void pressed_set_jigsaw (
	GtkMenuItem	* menu_item,
	gpointer	user_data,
	gint		item
	)
{
	int		old;


	if ( ! gtk_check_menu_item_get_active ( GTK_CHECK_MENU_ITEM ( menu_item
		) ) )
	{
		return;			// Filter out changes
	}

	old = 0;
	mtkit_prefs_get_int ( global.prefs_file, _INIFILE_MAIN_JIGSAW_TABLE,
		&old );

	if ( item == old )
	{
		return;			// Only action a change
	}

	select_jigsaw_table ( item );
}

static void pressed_start_new (
	GtkMenuItem	* menu_item,
	gpointer	user_data,
	gint		item
	)
{
	if ( ! confirm_warning_alert (
		_("Do you really want to start a new puzzle?") ) )
	{
		return;
	}

	mtjigsaw_set_mesh ( global.jigsaw, global.jigsaw->nodes_hori,
		global.jigsaw->nodes_vert );
	redraw_canvas ();
}

static void pressed_join_pieces (
	GtkMenuItem	* menu_item,
	gpointer	user_data,
	gint		item
	)
{
	if ( ! confirm_warning_alert (
		_("Do you really want to join all of the pieces?") ) )
	{
		return;
	}

	mtjigsaw_join ( global.jigsaw );
	redraw_canvas ();
}

static void pressed_scatter_pieces (
	GtkMenuItem	* menu_item,
	gpointer	user_data,
	gint		item
	)
{
	if ( ! confirm_warning_alert (
		_("Do you really want to scatter all of the pieces?") ) )
	{
		return;
	}

	mtjigsaw_scatter ( global.jigsaw );
	redraw_canvas ();
}


static void pressed_preferences (
	GtkMenuItem	* menu_item,
	gpointer	user_data
	)
{
	mtgex_prefs_window ( global.prefs_file, global.main_window, NULL, NULL,
		NULL, 0 );
}

static gboolean configure_canvas (
	GtkWidget	* widget,
	GdkEventConfigure * event,
	gpointer	data
	)
{
	if ( ! widget )
	{
		return FALSE;
	}

	global.canvas_width = widget->allocation.width;
	global.canvas_height = widget->allocation.height;

	if ( global.jigsaw )
	{
		mtjigsaw_set_image ( global.jigsaw, NULL, global.canvas_width,
			global.canvas_height );
		redraw_canvas ();
	}

	return TRUE;
}

static gboolean expose_canvas (
	GtkWidget	* widget,
	GdkEventExpose	* event,
	gpointer	data
	)
{
	mtImage		* image;
	int		px,
			py,
			pw,
			ph;


	if ( ! widget || ! event )
	{
		return TRUE;
	}

	px = event->area.x;
	py = event->area.y;
	pw = event->area.width;
	ph = event->area.height;

	if ( pw < 1 || ph < 1 )
	{
		return TRUE;
	}

	image = mtpixel_image_new_rgb ( pw, ph );
	if ( ! image )
	{
		return TRUE;
	}

	mtjigsaw_render ( global.jigsaw, image, px, py );

	gdk_draw_rgb_image ( widget->window, widget->style->black_gc,
		px, py, pw, ph, GDK_RGB_DITHER_NONE,
		image->channel[MTPIXEL_CHANNEL_IMAGE], pw * 3 );

	mtpixel_image_destroy ( image );

	return TRUE;
}

static gboolean clickrelease_button (
	GtkWidget	* widget,
	GdkEvent	* event
	)
{
	int		x,
			y,
			button,
			piece_ID,
			ux,
			uy,
			uw,
			uh;


	if ( ! event )
	{
		return FALSE;
	}

	x = event->button.x;
	y = event->button.y;
	button = event->button.button;

	if ( event->type == GDK_BUTTON_PRESS )
	{
		if ( global.click_button )
		{
			// Only notice one click at a time
			return TRUE;
		}

		piece_ID = mtjigsaw_grab_piece ( global.jigsaw, x, y, &ux, &uy,
			&uw, &uh );

		if ( piece_ID < 0 )
		{
			// No piece has been clicked
			return TRUE;
		}

		gtk_widget_queue_draw_area ( global.main_canvas, ux, uy, uw, uh
			);

		global.click_button = button;
		global.click_piece_ID = piece_ID;
		global.click_x = x;
		global.click_y = y;
	}
	else if ( event->type == GDK_BUTTON_RELEASE )
	{
		int		ux,
				uy,
				uw,
				uh;


		if ( button != global.click_button )
		{
			// Only notice a logged click
			return TRUE;
		}

		if ( mtjigsaw_drop_piece ( global.jigsaw,
			global.click_piece_ID, &ux, &uy, &uw, &uh )
			== -1 )
		{
			gtk_widget_queue_draw_area ( global.main_canvas,
				ux, uy, uw, uh );
		}

		global.click_button = 0;
		save_current_jigsaw ();
	}
	else
	{
		return TRUE;
	}

	return TRUE;
}

static gboolean motion_canvas (
	GtkWidget	* widget,
	GdkEvent	* event
	)
{
	GdkModifierType	state;
	int		x,
			y,
			dx,
			dy,
			button,
			ux,
			uy,
			uw,
			uh;


	if ( ! event )
	{
		return FALSE;
	}

	if ( event->motion.is_hint )
	{
		gdk_window_get_pointer ( event->motion.window, &x, &y, &state );
	}
	else
	{
		x = event->motion.x;
		y = event->motion.y;
		state = event->motion.state;
	}

	button = 0;

	if ( state & GDK_BUTTON1_MASK )
	{
		button = 1;
	}
	else if ( state & GDK_BUTTON3_MASK )
	{
		button = 3;
	}
	else
	{
		return TRUE;
	}

	if ( button != global.click_button )
	{
		return TRUE;
	}

	dx = x - global.click_x;
	dy = y - global.click_y;

	if ( ! mtjigsaw_drag_piece ( global.jigsaw, global.click_piece_ID, dx,
		dy, &ux, &uy, &uw, &uh )
		)
	{
		gtk_widget_queue_draw_area ( global.main_canvas, ux, uy, uw, uh
			);
	}

	global.click_x = x;
	global.click_y = y;

	return TRUE;
}

static void gui_main_init ( void )
{
	int		last_table;
	char		txt[256];
	GtkWidget	* menubar,
			* vbox_main,
			* menu_widget;
	GtkAccelGroup	* accel_group;
	GtkItemFactory	* item_factory;
	GtkItemFactoryEntry menu_items[] = {
	{ _("/_Options"),		NULL,		NULL,0, "<Branch>" },
	{ _("/Options/tear"),		NULL,		NULL,0, "<Tearoff>" },
	{ _("/Options/Start New Puzzle"), NULL,		pressed_start_new, 0, NULL },
	{ _("/Options/Join Pieces"),	NULL,		pressed_join_pieces, 0, NULL },
	{ _("/Options/Scatter Pieces"),	NULL,		pressed_scatter_pieces, 0, NULL },
	{ _("/Options/sep4"),		NULL,		NULL,0, "<Separator>" },
	{ _("/Options/Open Image ..."),	"<control>O",	pressed_open_image, 0, NULL },
	{ _("/Options/sep5"),		NULL,		NULL,0, "<Separator>" },
	{ _("/Options/Full Screen"),	"F11",		pressed_set_fullscreen, 0, "<ToggleItem>" },
	{ _("/Options/Preferences ..."), "<control>P",	pressed_preferences, 0, NULL },
	{ _("/Options/sep6"),		NULL,		NULL,0, "<Separator>" },
	{ _("/Options/Quit"),		"<control>Q",	delete_event, 0, NULL },

	{ _("/_Jigsaw"),		NULL,		NULL,0, "<Branch>" },
	{ _("/Jigsaw/tear"),		NULL,		NULL,0, "<Tearoff>" },
	{ _("/Jigsaw/Puzzle 1"),	NULL,		pressed_set_jigsaw, 1, "<RadioItem>" },
	{ _("/Jigsaw/Puzzle 2"),	NULL,		pressed_set_jigsaw, 2, _("/Jigsaw/Puzzle 1") },
	{ _("/Jigsaw/Puzzle 3"),	NULL,		pressed_set_jigsaw, 3, _("/Jigsaw/Puzzle 1") },
	{ _("/Jigsaw/Puzzle 4"),	NULL,		pressed_set_jigsaw, 4, _("/Jigsaw/Puzzle 1") },
	{ _("/Jigsaw/Puzzle 5"),	NULL,		pressed_set_jigsaw, 5, _("/Jigsaw/Puzzle 1") },
	{ _("/Jigsaw/Puzzle 6"),	NULL,		pressed_set_jigsaw, 6, _("/Jigsaw/Puzzle 1") },
	{ _("/Jigsaw/Puzzle 7"),	NULL,		pressed_set_jigsaw, 7, _("/Jigsaw/Puzzle 1") },
	{ _("/Jigsaw/Puzzle 8"),	NULL,		pressed_set_jigsaw, 8, _("/Jigsaw/Puzzle 1") },
	{ _("/Jigsaw/Puzzle 9"),	NULL,		pressed_set_jigsaw, 9, _("/Jigsaw/Puzzle 1") },
	{ _("/Jigsaw/Puzzle 10"),	NULL,		pressed_set_jigsaw, 10, _("/Jigsaw/Puzzle 1") },

	{ _("/_Help"),			NULL,		NULL,0, "<LastBranch>" },
	{ _("/Help/About ..."),		"F1",		pressed_help, 0, NULL }
	};


	// Needed to read non ASCII filenames in GTK+2
	putenv ( "G_BROKEN_FILENAMES=1" );

	gtk_init ( &global.argc, &global.argv );

	accel_group = gtk_accel_group_new ();

///	MAIN WINDOW

	global.main_window = gtk_window_new ( GTK_WINDOW_TOPLEVEL );

	// Set minimum width/height
	gtk_widget_set_usize ( global.main_window, 100, 100 );

	mtgex_restore_window_position ( global.main_window, global.prefs_file,
		_INIFILE_MAIN_WINDOW );
	gtk_window_set_title ( GTK_WINDOW ( global.main_window ), VERSION );

	vbox_main = gtk_vbox_new ( FALSE, 0 );
	gtk_widget_show ( vbox_main );
	gtk_container_add ( GTK_CONTAINER ( global.main_window ), vbox_main );

	// we need to realize the window because we use pixmaps for
	// items on the toolbar & menus in the context of it
	gtk_widget_realize ( global.main_window );

///	MENU

	item_factory = gtk_item_factory_new ( GTK_TYPE_MENU_BAR,"<main>",
		accel_group );
	gtk_item_factory_create_items_ac ( item_factory,
		sizeof ( menu_items ) / sizeof ( ( menu_items )[0] ),
		menu_items, NULL, 2 );

	menubar = gtk_item_factory_get_widget ( item_factory,"<main>" );
	gtk_widget_show ( menubar );
	gtk_box_pack_start ( GTK_BOX ( vbox_main ), menubar, FALSE, FALSE, 0 );

	// Stop dynamic allocation of accelerators during runtime
	gtk_accel_group_lock ( accel_group );

	gtk_window_add_accel_group ( GTK_WINDOW ( global.main_window ),
		accel_group );

///	MAIN DRAWING AREA

	global.main_canvas = gtk_drawing_area_new ();
	gtk_widget_show ( global.main_canvas );
	mtgex_fix_darea ( global.main_canvas );

	gtk_box_pack_start ( GTK_BOX ( vbox_main ), global.main_canvas, TRUE,
		TRUE, 0 );

	gtk_signal_connect ( GTK_OBJECT ( global.main_canvas ),
		"expose_event", GTK_SIGNAL_FUNC ( expose_canvas ), NULL );

	gtk_signal_connect ( GTK_OBJECT ( global.main_canvas ),
		"configure_event", GTK_SIGNAL_FUNC ( configure_canvas ), NULL );

	gtk_signal_connect ( GTK_OBJECT ( global.main_canvas ),
		"button_press_event", GTK_SIGNAL_FUNC ( clickrelease_button ),
		NULL );

	gtk_signal_connect ( GTK_OBJECT ( global.main_canvas ),
		"button_release_event", GTK_SIGNAL_FUNC ( clickrelease_button ),
		NULL );

	gtk_signal_connect ( GTK_OBJECT ( global.main_canvas ),
		"motion_notify_event", GTK_SIGNAL_FUNC ( motion_canvas ),
		NULL );

	gtk_widget_set_events ( global.main_canvas, GDK_ALL_EVENTS_MASK );

/////////	End of main window widget setup

	gtk_signal_connect ( GTK_OBJECT ( global.main_window ), "delete_event",
		GTK_SIGNAL_FUNC ( delete_event ), NULL );

	update_titlebar ();

	gtk_widget_show ( global.main_window );

	gtk_window_set_icon ( (GtkWindow *)global.main_window,
		gdk_pixbuf_new_from_xpm_data ( (const char **)icon_xpm ) );

	last_table = 1;
	mtkit_prefs_get_int ( global.prefs_file, _INIFILE_MAIN_JIGSAW_TABLE,
		&last_table );

	if ( last_table < 1 || last_table > 10 )
	{
		last_table = 1;
	}

	sprintf ( txt, "%s%i", _("/Jigsaw/Puzzle "), last_table );
	menu_widget = gtk_item_factory_get_item ( item_factory, txt );

	if ( menu_widget && last_table != 1 )
	{
		gtk_check_menu_item_set_active (
			GTK_CHECK_MENU_ITEM ( menu_widget ), TRUE );
	}

	select_jigsaw_table ( last_table );

	gtk_main ();

	mtjigsaw_destroy ( global.jigsaw );
	global.jigsaw = NULL;
}

int image_load (
	char		* filename
	)
{
	if ( ! global.jigsaw )
	{
		return 1;
	}

	if ( mtjigsaw_set_image ( global.jigsaw, filename, global.canvas_width,
		global.canvas_height ) )
	{
		error_message_file ( "load image", filename );

		return 1;
	}

	redraw_canvas ();

	return 0;			// Success
}

static void change_nodes (
	mtPrefItemValue	* piv,
	int		callback_data
	)
{
	int		hori,
			vert;


	hori = 10;
	vert = 10;

	mtkit_prefs_get_int ( global.prefs_file, _INIFILE_MAIN_NODES_HORI,
		&hori );
	mtkit_prefs_get_int ( global.prefs_file, _INIFILE_MAIN_NODES_VERT,
		&vert );

	mtjigsaw_set_mesh ( global.jigsaw, hori, vert );
	redraw_canvas ();
}

int main (
	int		argc,
	char		* argv[]
	)
{
	mtPrefItem	prefs_table[] = {
	{ _INIFILE_LAST_DIR,		MTKIT_PREF_TYPE_DIR },
	{ _INIFILE_MAIN_WINDOW"_x",	MTKIT_PREF_TYPE_INT, "0" },
	{ _INIFILE_MAIN_WINDOW"_y",	MTKIT_PREF_TYPE_INT, "0" },
	{ _INIFILE_MAIN_WINDOW"_w",	MTKIT_PREF_TYPE_INT, "400" },
	{ _INIFILE_MAIN_WINDOW"_h",	MTKIT_PREF_TYPE_INT, "400" },
	{ _INIFILE_MAIN_JIGSAW_TABLE,	MTKIT_PREF_TYPE_INT, "1" },
	{ _INIFILE_MAIN_NODES_HORI,	MTKIT_PREF_TYPE_INT, "10", "Horizontal Mesh Nodes", change_nodes, 0, "6	100" },
	{ _INIFILE_MAIN_NODES_VERT,	MTKIT_PREF_TYPE_INT, "10", "Vertical Mesh Nodes", change_nodes, 1, "6	100" },
			{ NULL }
			};

	char		txt[2048];


	if ( argc > 1 )
	{
		if ( strcmp ( argv[1], "--version" ) == 0 )
		{
			printf ( "%s\n\n", VERSION );

			exit ( 0 );
		}
		else if ( strcmp ( argv[1], "--help" ) == 0 )
		{
			printf ( "%s\n\n"
				"Usage: %s [option] [file]\n\n"
				"Options:\n"
				"  --help          Output this help\n"
				"  --version       Output version information\n"
			, VERSION, BIN_NAME );

			exit ( 0 );
		}
		else if ( ! global.arg_filename )
		{
			global.arg_filename = argv[1];
		}
	}

	global.argc = argc;
	global.argv = argv;
	mtpixel_init ();

	snprintf ( txt, 2048, "%s/.%s", mtkit_file_home (), BIN_NAME );
	txt[2047] = 0;
	global.prefs_file = mtkit_prefs_new ( prefs_table );
	mtgex_fpick_init_prefs ( global.prefs_file, _INIFILE_MAIN_FPICK_PRE );
	mtgex_prefs_init_prefs ( global.prefs_file );

	mtkit_prefs_load ( global.prefs_file, txt );

	ced_init ();
	gui_main_init ();		// Create main window and run the GUI

	mtkit_prefs_save ( global.prefs_file, txt );
	mtkit_prefs_destroy ( global.prefs_file );
	global.prefs_file = NULL;
	mtpixel_quit ();

	memset( &global, 0, sizeof ( global ) );

	return 0;
}
