/*
	Copyright (C) 2008-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#ifndef _mtkit_h
#define _mtkit_h

#include <stdio.h>
#include <stdint.h>
#include <inttypes.h>



typedef struct _mtArg		mtArg;
typedef struct _mtBulkDouble	mtBulkDouble;
typedef struct _mtBulkInt	mtBulkInt;
typedef struct _mtBulkStr	mtBulkStr;
typedef struct _mtCliItem	mtCliItem;
typedef struct _mtCliTab	mtCliTab;
typedef struct _mtFile		mtFile;
typedef struct _mtPrefItem	mtPrefItem;
typedef struct _mtPrefItemList	mtPrefItemList;
typedef struct _mtPrefItemValue	mtPrefItemValue;
typedef struct _mtPrefTrans	mtPrefTrans;
typedef struct _mtPrefs		mtPrefs;
typedef struct _mtTree		mtTree;
typedef struct _mtTreeNode	mtTreeNode;
typedef struct _mtUtreeNode	mtUtreeNode;
typedef struct _mtZip		mtZip;



#undef  mtMIN
#define mtMIN(a,b,c)	if ( b < c ) a = b; else a = c;
#undef  mtMAX
#define mtMAX(a,b,c)	if ( b > c ) a = b; else a = c;
#undef  MAX
#define MAX(a,b)	( ( (a) > (b) ) ? (a) : (b) )
#undef  MIN
#define MIN(a,b)	( ( (a) < (b) ) ? (a) : (b) )

#define MTKIT_DIR_SEP			'/'
#define MTKIT_PATHBUF_MAX		2048

#define MTKIT_FILESIZE_MAX		1234567890

// Generic wrapper for strncpy (), ensuring 0 termination
#define mtkit_strncpy0( A, B, C ) ( strncpy ( (A), (B), (C))[(C) - 1] = 0 )

#define MTKIT_DDT_MIN_DATE_YEAR		0
#define MTKIT_DDT_MAX_DATE_YEAR		5879609



enum
{
	MTKIT_FILE_NONE		= 0,
	MTKIT_FILE_ZERO		= 1<<0,	// Place an extra 0 after the file in
					// memory
	MTKIT_FILE_GUNZIP	= 1<<1,	// Enable/allow/actioned gzip
					// compress/decompress
};

typedef enum
{
	MTKIT_UTREE_NODE_TYPE_NONE	= 0,

	MTKIT_UTREE_NODE_TYPE_ROOT,
	MTKIT_UTREE_NODE_TYPE_ELEMENT,
	MTKIT_UTREE_NODE_TYPE_TEXT,
	MTKIT_UTREE_NODE_TYPE_COMMENT,

	MTKIT_UTREE_NODE_TOTAL
} mtUtreeNodeType;

enum
{
	MTKIT_UTREE_OUTPUT_INDENTS	= 1 << 0,
	MTKIT_UTREE_OUTPUT_NEWLINES	= 1 << 1,
	MTKIT_UTREE_OUTPUT_TEXT_NEWLINES = 1 << 2,
};

#define MTKIT_UTREE_OUTPUT_DEFAULT (MTKIT_UTREE_OUTPUT_INDENTS | MTKIT_UTREE_OUTPUT_NEWLINES)


/*
	Prefs tables terminated by name = NULL.
	If (bulk.var == NULL) it is skipped.
*/

typedef enum
{
	MTKIT_PREF_TYPE_NONE,

	MTKIT_PREF_TYPE_INT,		// If opt used: "min	max"
					// else = +/-1000000
	MTKIT_PREF_TYPE_BOOL,		// Stored as integer: 0 = false 1 = true
	MTKIT_PREF_TYPE_RGB,		// Stored as integer: (((R) << 16) +
					// ((G) << 8) + (B))
	MTKIT_PREF_TYPE_OPTION,		// Stored as integer: 0 = o1 1 = o2
					// opt = "o1	o2	..."
	MTKIT_PREF_TYPE_DOUBLE,		// If opt used: "min	max	step"
					// else = +/-1000000,0.001
	MTKIT_PREF_TYPE_STR,		// NUL terminated string.  If opt used,
					// it sets the max chars
	MTKIT_PREF_TYPE_STR_MULTI,	// NUL terminated multi line string.
	MTKIT_PREF_TYPE_FILE,		// Stored as string: implies that file
					// picker can be used
	MTKIT_PREF_TYPE_DIR,		// Same as filename but picker only
					// stores directory

	MTKIT_PREF_TYPE_TOTAL
} mtPrefType;

enum
{
	MTKIT_ARG_NONE,

	MTKIT_ARG_SWITCH,		// Switch - No data follows
	MTKIT_ARG_INT,			// Integer follows in next argument
	MTKIT_ARG_STRING,		// String follows in next argument
	MTKIT_ARG_DOUBLE,		// Double precision float follows in
					// next argument
	MTKIT_ARG_TOTAL
};

enum
{
	MTKIT_ARG_ERROR_NONE,

	MTKIT_ARG_ERROR_UNKNOWN,	// Switch in arguments not found in list
	MTKIT_ARG_ERROR_DATA,		// Data passed in extra argument not
					// recognised
	MTKIT_ARG_ERROR_TERMINATED,	// Premature termination - extra
					// argument expected
	MTKIT_ARG_ERROR_BAD_TYPE,	// Bad argument type found in list

	MTKIT_ARG_ERROR_TOTAL
};

enum
{
	MTKIT_ZIP_OK_DONT_FREE	=	-10,	// When loading, user callback
						// can keep memory
	MTKIT_ZIP_STOP		=	-1,
	MTKIT_ZIP_OK		=	0,
	MTKIT_ZIP_ERROR		=	1,
	MTKIT_ZIP_ERROR_MEM	=	11,
	MTKIT_ZIP_ERROR_FILE	=	12,
	MTKIT_ZIP_ERROR_HEADER	=	13,
	MTKIT_ZIP_ERROR_ZLIB	=	20,

	MTKIT_ZIP_ERROR_USER	=	100
	// All values over 100 are reserved for user callback errors
};



typedef int (* mtTreeFuncCmp) (		// Compare 2 node keys
	void		* k1,
	void		* k2
	);
	// -1 : k1 < k2
	//  0 : k1 = k2
	//  1 : k1 > k2

typedef void (* mtTreeFuncDel) (	// Delete key/value data before node is
	mtTreeNode	* node		// destroyed
	);

typedef int (* mtTreeFuncDup) (
	mtTreeNode	* src,		// Duplicate data & key from here ...
	mtTreeNode	* dest		// ... and put them here
	);
	// 0 = success
	// 1 = fail

typedef int (* mtTreeFuncScan) (	// Don't add/remove anything from the
	mtTreeNode	* node,		// tree during this callback
	void		* user_data
	);
	// 0 = continue
	// 1 = stop

typedef void (* mtPrefCB) (		// Called when a preference changes
	mtPrefItemValue	* piv,
	int		callback_data
	);

typedef int (* mtArgFunc) (		// Called after an argument is actioned
	mtArg		* mtarg,	// Argument encountered
	int		arg,		// Arg number
	int		argc,		// Total args
	char		* argv[]	// Arg list
	);
	// 0 = continue parsing
	// 1 = Stop

typedef int (* mtArgFileFunc) (		// Callback for when a filename is
					// encountered
	char		* filename
	);
	// 0 = continue parsing
	// 1 = Stop

typedef int (* mtArgErrorFunc) (	// Callback for when an error occurs
	int		error,		// Error number
	int		arg,		// Arg number that causes the error
	int		argc,
	char		* argv[]
	);
	// 0 = continue parsing
	// 1 = Stop

typedef int (* mtZipLoadFunc) (
	char		* name,
	void		* buf,
	int32_t		buf_len,
	int32_t		year,
	int8_t		month,
	int8_t		day,
	int8_t		hour,
	int8_t		minute,
	int8_t		second,
	void		* user_data
	);
	// MTKIT_ZIP_OK - Continue, library free's allocated memory
	// MTKIT_ZIP_OK_DONT_FREE - Continue, caller free's allocated memory
	// MTKIT_ZIP_STOP - Stop, library free's allocated memory
	// Any other value stops, and is returned from mtkit_zip_load function



struct _mtBulkInt
{
	char		* name;
	int		* var;
};

struct _mtBulkDouble
{
	char		* name;
	double		* var;
};

struct _mtBulkStr
{
	char		* name,
			** var;
};

struct _mtTreeNode
{
	void		* key,
			* data
			;
	mtTreeNode	* left,		// Child with a lower key
			* right		// Child with a higher key
			;
	int		balance		// Balance between nodes below
			;		// (left = -1 right = +1)
};

struct _mtUtreeNode
{
	mtUtreeNodeType	type;
	char		* text;
	mtTree		* attribute_tree;
	mtUtreeNode	* previous,
			* next,
			* parent,
			* child,
			* child_last;
};

struct _mtTree
{
	mtTreeNode	* root;
	mtTreeFuncCmp	cmp;		// Compares 2 node keys
	mtTreeFuncDel	del;		// Destroys key/value data
};

struct _mtPrefItem
{
	char		* key;		// Unique index, valid chars:
					// (a-z A-Z 0-9 _ - . : ) or
					// (anything < 0) checked using
					// mtkit_utree_valid_name ()
	mtPrefType	type;		// As MTKIT_PREF_TYPE_*
	char		* def,		// Default value
			* description;	// Human readable description for user
	mtPrefCB	callback;	// Called when preference changes
	int		callback_data;	// Callback data
	char		* opt;		// Used for options in an option list
};

struct _mtPrefItemValue			// Each data pointer in an mtPrefs tree
{					// node points to this
	char		* value;	// Allocated string for current value

	mtPrefItem	* pref_item;	// Item in preferences table that this
};					// value is for

struct _mtPrefTrans
{
	char		* dest,
			* src;		// Key names.  If either is NULL, stop.
};

struct _mtArg
{
	char		* argument;	// String passed as argument after '-'
	int		type;		// Type of argument
	void		* variable;	// Pointer to variable to change
	int		value;		// Value to place in integer variable
					// for ARG_TYPE_SWITCH
	mtArgFunc	callback;	// Called after an argument is actioned
};

struct _mtCliItem
{
	char		* command;
	void		* func;
	int		func_args;	// -1 = 0 or more,
					// -2 = 1 or more, else as given
	char		* help_args;
};



// 0 = success, or NULL = fail unless otherwise stated

char * mtkit_file_load (
/*
Load a whole file into memory (possibly gunzip'd). Can only load files less
than MTKIT_FILESIZE_MAX in size.
*/
	char		* filename,
	int		* bytes,	// Bytes read stored here
	int		in_flag,	// Input flags as per MTKIT_FILE_*
	int		* out_flag	// Optional output results as per
	);				// MTKIT_FILE_*

int mtkit_file_save (			// Save a memory chunk to a file
					// (possibly gzip'd)
	char		* filename,
	char		* buf,		// Bytes to save
	int		buf_size,	// Number of bytes to save.
					// 0 <= buf_size <= MTKIT_FILESIZE_MAX
	int		out_flag	// As per MTKIT_FILE_*
	);

char * mtkit_file_home (		// Get the home directory
	void
	);

int mtkit_file_valid (			// Can this file be opened for reading?
	char		* filename
	);
	// -1 = errno = ENOENT
	//  0 = Yes
	//  1 = other

char * mtkit_file_readline (		// Read a line of input and allocated to
					// new NUL terminated string.
	FILE		* fp,		// Excludes newline char(s) at end for
					// Unix/Mac/DOS, i.e. \n \r \r\n
					// fp can be stdin to read input from
					// the command line or a pipe
	int		* len,		// Optional: put length of string here
	int		* len_nl	// Optional: put length of newline
					// string here
	);

/*
The mtFile structure is designed to allow the writing of a file to disk
or memory.  It is useful for code that wants to write to either at different
times, and makes switching easy and seemless.

The memory file write avoids the need for a temp file on disk, which may be
costly or undesirable.
*/

mtFile * mtkit_file_open_disk (		// Open a new disk file
	char		* filename
	);

mtFile * mtkit_file_open_mem (		// Open a new memory file
	void
	);

int mtkit_file_close (			// Close an mtFile & destroy structure
	mtFile		* mtfp
	);

int mtkit_file_write (			// Write a chunk of data to an mtFile
	mtFile		* mtfp,
	void		* mem,		// Start of memory to write
	int64_t		mem_len		// Number of bytes to write
	);

int mtkit_file_write_string (		// Write a C string to an mtFile
	mtFile		* mtfp,
	char		* str		// NUL terminated C string
	);

int mtkit_file_get_mem (
	mtFile		* mtfp,
	void		** buf,		// Put buffer start here (NULL = don't)
	int64_t		* buf_len	// Put buffer length here (NULL = don't)
	);

int mtkit_file_header_gz (		// Is this a .gz file header?
	unsigned char	* mem,		// 31, 139
	int		mem_size	// > 20
	);
	// 0 = No
	// 1 = Yes

int mtkit_file_header_zip (		// Is this a .zip file header?
	unsigned char	* mem,		// 0x50, 0x4b, 0x03, 0x04
	int		mem_size	// > 30
	);
	// 0 = No
	// 1 = Yes

int64_t mtkit_file_size (		// Get the file size
	char		* filename
	);
	// -2 = System error (use errno to get details)
	// -1 = Arg error
	// Otherwise it returns the file size in bytes

void mtkit_print_errno (		// If errno != 0, output info
	int		e
	);

char * mtkit_strnncat (			// As per strncat () but NUL terminates
	char		* dest,
	const char	* src,
	int		max
	);

int mtkit_strtod (			// Read in a string as a double
	char		* input,	// Input string to parse NUL terminated
	double		* result,	// Put the numerical result here
					// (NULL = don't)
	char		** next,	// Put next unparsed char here
					// (NULL = don't)
	int		strict		// 1 = string must only contain
					// number/whitespaces
	);

int mtkit_strtoi (			// Read in a string as an int
	char		* input,	// Input string to parse NUL terminated
	int		* result,	// Put the numerical result here
					// (NULL = don't)
	char		** next,	// Put next unparsed char here
					// (NULL = don't)
	int		strict		// 1 = string must only contain
					// number/whitespaces
	);

char * mtkit_strtok (			// Extract the n'th token and create a
					// new string of it
	char		* input,	// Input string, NUL terminated
	char		* delim,	// List of character delimeters, NUL
					// terminated
	int		n		// n'th token (0 = first)
	);
	// Pointer to newly allocated string

int mtkit_strtok_count (		// Count the number of tokens in a
					// string
	char		* input,	// Input string, NUL terminated
	char		* delim		// List of character delimeters, NUL
					// terminated
	);
	// -1 = fail, else number of tokens

int mtkit_strtok_num (			// Extract the n'th token and convert it
					// to a double
	char		* input,	// Input string, NUL terminated
	char		* delim,	// List of character delimeters, NUL
					// terminated
	int		n,		// n'th token (0 = first)
	double		* num		// Put result here
	);

char * mtkit_strcasestr (		// Find a string in another string
					// regardless of case
	char		* haystack,
	char		* needle
	);
	// NULL = not found, else position in haystack where needle is

char * mtkit_strjoinv (			// Join some strings together to form a
					// new string
	char		* sep,		// Separator between items NULL = don't
					// use
	char		** list		// NULL terminated array of strings
	);

int mtkit_strfreeswap (			// Free an old string, swap in
					// duplicated string pointer
	char		** spp,		// Pointer to a current string pointer
					// (to be freed)
	char		* str		// New string to be swapped, or NULL
	);

char * mtkit_strtohtml (		// Create a valid HTML string from a C
					// string
	char		* input		// chars < 32 => 32, < > & " get
					// substituted
	);

int mtkit_strmatch (			// String match with wildcards
	char		* string,	// C string
	char		* pattern,	// Pattern to match, including ? *
	int		mode		// 1 = case sensitive
					// 0 = case insensitive
	);
	// -3 = OS error
	// -2 = Arg error
	// -1 = no match, else returns offset in 'string' of first non wildcard
	// character match.

/*
? = any single char, * = any number of chars.  To represent these or \ chars in
'pattern' they must be preceded by a \ character, e.g. "\*\\\?" will search for
characters "*\?".

	\n = newline
	\t = tab
	\r = carriage return

Results when putting \ before any other chars are undefined.

If pattern is "*text*" then 'text' could be anywhere in the string, "*text"
'text' must be at the end and "text*" means 'text' must be at the beginning.

	"abcdef", "*c*"		= 2
	"abcdef", "*?c*"	= 2
	"abcdef", "*c"		= -1
	"abcdef", "c"		= -1
*/

int mtkit_strnonspaces (		// Does a string contain any
					// non-whitespace characters?
	char * input			// Input string to parse NUL terminated
	);
	// 0 = String only has whitespaces
	// 1 = Has non-whitespace characters

/*
	Add thousand separator(s) to a string containing a number, e.g.
	str2thousands ( dest, "1234567", 32, ',', '-', '.', 3, 0 );
	Creates:

	1234567		->	1,234,567
	-123456		->	-123,456
	-123456.7891	->	-123,456.7891


	Can also be used to separate any text between two delimeters, e.g.
	str2thousands ( dest, "aaa>270907<bbb", 32, '-', '>', '<', 2, 0 );
	Creates:

	aaa>27-09-07<bbb

*/

int mtkit_strtothou (
	char		* dest,		// Output buffer
	char		* src,		// Input string (can be same as
					// destination)
	int		dest_size,	// Size of output buffer
	char		separator,	// Separator character to use,
					// typically ','
	char		minus,		// Minus character, typically '-'
	char		dpoint,		// Decimal point character,
					// typically '.'
	int		sep_num,	// Numbers to contain between
					// separators, typically 3
	int		right_justify	// Should output string be right
					// justified?
	);

char * mtkit_strdupstrip (		// Duplicate string and strip characters
					// from it
	char		* input,
	char		* filter,	// Chars to include or exclude from
					// input
	int mode			// 0 = strip all chars in filter
					// 1 = strip all chars EXCEPT those in
					// filter
	);

/*
Examples:
	mtkit_strdupstrip ( "-1,234,567.89", ",", 0 );
	creates "-1234567.89"

	mtkit_strdupstrip ( "-1,234,567.89", "0123456789.-", 1 );
	creates "-1234567.89"
*/


/*
	ddt = decimal day time => each 1.0 is a day.
	e.g.
	00:00			= 0.00
	00:00:00		= 0.00
	1/1/0 00:00:00		= 0.00
	2/1/0			= 1.00
	1/1/2000		= 730485.00
	1/1/2000 12:34:56	= 730485.524259

	String Formats:
	h:m			= Shortened time format
					(seconds = 0, date = 1/1/0)
	h:m:s			= Full time format (date = 1/1/0)
	d/m/y			= Full date format (time = 0:0:0)
	d-m-y			= Alternative date separators
	d/m/y h:m:s		= Complete date/time format (:s optional)
*/

int mtkit_strtoddt (			// String (above formats) to double
					// date/time
	char		* input,	// Input string to parse NUL terminated
					// ([d/m/y] [h:m:s])
					// If input day is > 31 then interpret
					// as y/m/d
	double		* result	// Put result here
	);

int mtkit_itoddt (
	int		day,		// 1..31
	int		month,		// 1..12
	int		year,		// 0..
	int		hour,		// 0..23
	int		minute,		// 0..59
	int		second,		// 0..59
	double		* result	// Put result here
	);

int mtkit_ddttoi (
	double		datetime,	// (>= 0.0)  0.* => only populate time
	int		* day,
	int		* month,
	int		* year,
	int		* hour,
	int		* minute,
	int		* second
	);

int mtkit_is_leap_year (
	int		year
	);
	// 0 = Not leap year
	// 1 = Is leap year

int mtkit_ddt_weekday (
	double		datetime
	);
	// -1 = error
	//  0 = Sun
	//  1 = Mon
	//  2 = Tue
	//  3 = Wed
	//  4 = Thu
	//  5 = Fri
	//  6 = Sat

int mtkit_string_encoding_conversion (	// Convert a string from one encoding to
					// another, allocating a new string for
					// the result
	char		* text_in,	// NUL terminated string to convert
	char		* text_in_encoding,
					// e.g. "ASCII", "ISO-8859-1", "UTF-8"
	char		** text_out,	// Pointer to storage space for result
	char		* text_out_encoding
					// e.g. "ASCII", "ISO-8859-1", "UTF-8"
	);
	// 0 = success
	// < 0 = failure

char * mtkit_string_join (
	char		** str_array,	// NULL terminated list of strings to
					// join
	char		* separator	// Separator to use between items
	);
/*
	If mtkit_string_join returns a non NULL value, the caller must
	release the result after use with:
	free ( ... );
*/

char ** mtkit_string_argv (		// Split C string into arguments
					// separated by whitespaces.
	char		* input		// Args can be contained inside
					// " quotes.
					// Backslashes imply literal char
					// follows, e.g. \\ = \, \" = "
	);
	// NULL = fail, else NULL terminated argv of allocated strings

int mtkit_string_argv_free (		// Free strings array, and its strings
	char		** args
	);

int mtkit_set_locale (			// Set locale.
	char		* st		// NULL => ASCII, "" => current system
	);

char * mtkit_utf8_from_locale (		// Convert text from current locale to
					// UTF-8
	char		* text
	);

int mtkit_utf8_remove (			// Remove some characters from a UTF-8
					// string
	char		** utf8,
	int		start,		// 1st character to remove 0,1,2,...
	int		length		// Number of characters to remove
					// 1,2,...
	);

int mtkit_utf8_insert (			// Insert 'utf8_insert' into 'utf8'
					// after 'n' chars
	char		** utf8,
	int		n,		// 0 = put 'new' at beginning of
					// resulting string
	char		* utf8_insert
	);

/*
	UTF-8 convenience functions
*/

int mtkit_utf8_string_legal (
	unsigned char	* src,		// Is this a valid UTF-8 string?  Must
					// be NUL terminated
	int		bytes		// Check this many bytes (<1 => stop
					// when NUL reached)
	);
	// 1 = legal

int mtkit_utf8_is_legal (		// Is this character legal?
	unsigned char	* src,
	int		bytes		// Length of source buffer 1-4
	);
	// 1 = legal

int mtkit_utf8_to_utf32 (
	unsigned char	* src,		// Start of UTF-8 character
	unsigned int	* unicode	// Where the code is to be put
					// (NULL = don't use)
	);
	// -1 = error, otherwise bytes used for character 1-4

int mtkit_utf8_from_utf32 (
	unsigned char	* src,		// Start of an empty 5 byte buffer
	unsigned int	unicode		// UTF32 character to convert
	);
	// 0 = fail 1-4 = bytes used to encode

int mtkit_utf8_len (			// Count the number of characters in
					// this UTF-8 string
	unsigned char	* src,		// Valid UTF-8 string
	int		bytes		// Total bytes in src to check
					// ( < 1 => stop when NUL reached )
	);
	// 0 = error

int mtkit_utf8_char_len (
	unsigned char	byte		// First byte of valid UTF-8 char
	);
	// Return the size of full char in bytes

int mtkit_utf8_offset (			// Count the number of bytes for a given
					// number of characters
	unsigned char	* src,		// Valid UTF-8 string
	int		bytes,		// Total bytes in src buffer
	int		num		// UTF-8 Characters to count
	);
	// -1 = fail, else the byte offset for the 'chars' character

/*
	Do a conversion of an ISO-8859-1 chunk to UTF-8.
	This is slightly faster and much less memory intensive than using
	mtkit_string_encoding_conversion.
*/

char * mtkit_iso8859_to_utf8 (		// Allocate new UTF-8 memory chunk
					// (+ extra trailing NUL)
	char		* input,
	int		bytes,		// 0 = use strlen ( input )
	int		* new_size	// Size of new allocated memory chunk
	);				// (NULL = don't use)

mtTree * mtkit_tree_new (
	mtTreeFuncCmp	cmp_func,	// Function to compare 2 keys
	mtTreeFuncDel	del_func	// Function to release key & data
					// structures (NULL = none)
	);

int mtkit_tree_destroy (		// Destroy a whole tree
	mtTree		* tree
	);

mtTree * mtkit_tree_duplicate (
	mtTree		* tree,
	mtTreeFuncDup	duplicate	// NULL = copy key/data as static data
	);

int mtkit_tree_node_add (
	mtTree		* tree,
	void		* key,
	void		* data
	);
	// 0 = not added
	// 1 = added
	// 2 = overwritten old node

mtTreeNode * mtkit_tree_node_find (
	mtTree		* tree,
	void		* key
	);				// Find a node

int mtkit_tree_node_remove (
	mtTree		* tree,
	void		* key
	);
	// 0 = not removed
	// 1 = removed

int mtkit_tree_scan (			// Scan each node in tree
	mtTree		* tree,
	mtTreeFuncScan	callback,
	void		* user_data,	// Passed to callback
	int		direction	// 0 = Left to right; 1 = Right to left
	);
	// 0 = success
	// 1 = error
	// 2 = user termination

mtUtreeNode * mtkit_utree_load_file (
	mtUtreeNode	* parent,	// NULL = create new root
	char		* filename,
	int		* errors,	// Put error flag here,
					// NULL = don't. Result
					// 0 = success
	int		* filetype	// MTKIT_FILE_OUT_*
	);
	// NULL = nothing loaded, else something loaded

mtUtreeNode * mtkit_utree_load_mem (
	mtUtreeNode	* parent,	// NULL = create new root
	char		* buf,		// Memory buffer containing data
	int		size,
	char		** breakpoint	// Put pointer to parsing error in buf
					// here, NULL = don't
	);
	// NULL = nothing loaded, else something loaded

int mtkit_utree_save_file (
	mtUtreeNode	* node,		// Node to save
	char		* filename,
	int		output,		// Output style
	int		filetype	// MTKIT_FILE_*
	);

mtFile * mtkit_utree_save_file_mem (
	mtUtreeNode	* node,		// Node to save
	int		output		// Output style
	);

mtUtreeNode * mtkit_utree_new_root (	// Create new root node
	void
	);

mtUtreeNode * mtkit_utree_new_element (
	mtUtreeNode	* parent,
	char		* name
	);

mtUtreeNode * mtkit_utree_new_text (
	mtUtreeNode	* parent,
	char		* text
	);

mtUtreeNode * mtkit_utree_new_comment (
	mtUtreeNode	* parent,
	char		* comment
	);

mtUtreeNode * mtkit_utree_get_node (	// Find a child of this node that has
					// this name/type
	mtUtreeNode	* parent,
	char		* text,		// NULL = Don't check
	mtUtreeNodeType	type		// 0 = Any type
	);

mtUtreeNode * mtkit_utree_get_node_next ( // Find next node after
					// mtkit_utree_get_node
	mtUtreeNode	* start,
	char		* text,		// NULL = Don't check
	mtUtreeNodeType	type		// 0 = Any type
	);

mtUtreeNode * mtkit_utree_get_node_via_address (
					// Traverse elements in the tree
	mtUtreeNode	* root,		// Start point
	char		* address	// In Unix path form, e.g.
					// "utree/tree/node/address"
	);				// NULL = not found

int mtkit_utree_destroy_node (
	mtUtreeNode	* node
	);
	// 0 = destroyed
	// 1 = not destroyed

int mtkit_utree_destroy_attribute (
	mtUtreeNode	* node,
	char		* name
	);
	// 0 = destroyed
	// 1 = not destroyed

int mtkit_utree_get_attribute_int (
	mtUtreeNode	* node,
	char		* name,
	int		* value
	);

int mtkit_utree_get_attribute_double (
	mtUtreeNode	* node,
	char		* name,
	double		* value
	);

int mtkit_utree_get_attribute_str (
	mtUtreeNode	* node,
	char		* name,
	char		** value
	);

int mtkit_utree_set_attribute_int (
	mtUtreeNode	* node,
	char		* name,
	int		value
	);

int mtkit_utree_set_attribute_double (
	mtUtreeNode	* node,
	char		* name,
	double		value
	);

int mtkit_utree_set_attribute_str (
	mtUtreeNode	* node,
	char		* name,
	char		* value
	);

int mtkit_utree_bulk_get (		// Get a bunch of attribute values
					// (missing items skipped)
	mtUtreeNode	* node,
	mtBulkInt	* table_i,	// NULL = skip
	mtBulkDouble	* table_d,	// NULL = skip
	mtBulkStr	* table_s	// NULL = skip.  All strings created
					// via mtkit_strfreeswap ()
	);

int mtkit_utree_bulk_set (		// Set a bunch of attribute values
	mtUtreeNode	* node,
	mtBulkInt	* table_i,	// NULL = skip
	mtBulkDouble	* table_d,	// NULL = skip
	mtBulkStr	* table_s	// NULL = skip
	);

mtPrefs * mtkit_prefs_new (		// Create new prefs structure
	mtPrefItem	* table		// Static Table to add (NULL = don't)
	);

int mtkit_prefs_add (			// Add a table to current prefs
	mtPrefs		* prefs,
	mtPrefItem	* table,
	int		destroy		// 0 = table is static, 1 = destroy this
					// table when prefs is destroyed
	);

int mtkit_prefs_load (			// Load a prefs file into a prefs
					// structure
	mtPrefs		* prefs,	// Ignores entries in the file not in
					// this structure
	char		* filename	// NULL = don't load anything (i.e. use
					// defaults)
	);

int mtkit_prefs_save (			// Save a prefs file
	mtPrefs		* prefs,
	char		* filename
	);

int mtkit_prefs_destroy (		// Destroy a prefs structure
	mtPrefs		* prefs
	);

mtPrefItem * mtkit_prefs_duplicate_table ( // Duplicate a NULL terminated
					// preferences table
	mtPrefItem	* table,
	char		* prefs_prefix	// NULL = no prefix
	);

int mtkit_prefs_destroy_table (		// Destory table created with
					// mtkit_prefs_duplicate_table ()
	mtPrefItem	* table
	);

int mtkit_prefs_block_callback (	// Stop all callbacks
	mtPrefs		* prefs
	);

int mtkit_prefs_unblock_callback (	// Restart callbacks after blocking
	mtPrefs		* prefs
	);

int mtkit_prefs_get_int (		// Retrieve an integer
	mtPrefs		* prefs,
	char		* key,
	int		* value		// Put the value here
	);

int mtkit_prefs_get_double (
	mtPrefs		* prefs,
	char		* key,
	double		* value		// Put the value here
	);

int mtkit_prefs_get_str (
	mtPrefs		* prefs,
	char		* key,
	char		** value	// Put the value here
	);

int mtkit_prefs_set_default (		// Set to default
	mtPrefs		* prefs,
	char		* key
	);

int mtkit_prefs_set_int (		// Set an integer
	mtPrefs		* prefs,
	char		* key,
	int		value
	);

int mtkit_prefs_set_double (
	mtPrefs		* prefs,
	char		* key,
	double		value
	);

int mtkit_prefs_set_str (
	mtPrefs		* prefs,
	char		* key,
	char		* value
	);

char * mtkit_prefs_type_text (		// Return a static string
					// representation of type
	mtPrefType type
	);

/*
This puts a value into a string.  Simple direct conversion for
string/numerical types but does fiddly work for boolean, RGB, and option types.
*/
void mtkit_prefs_get_str_val (
	mtPrefItemValue	* piv,		// Item
	char		* value,	// Value to convert
	char		* buf,		// Destination buffer
	int		buf_size	// Bytes in buffer
	);

int mtkit_prefs_bulk_get (		// Get a bunch of values (missing items
					// skipped)
	mtPrefs		* prefs,
	mtBulkInt	* table_i,	// NULL = skip
	mtBulkDouble	* table_d,	// NULL = skip
	mtBulkStr	* table_s	// NULL = skip.  All strings created via
					// mtkit_strfreeswap ()
	);

int mtkit_prefs_bulk_set (		// Set a bunch of values
	mtPrefs		* prefs,
	mtBulkInt	* table_i,	// NULL = skip
	mtBulkDouble	* table_d,	// NULL = skip
	mtBulkStr	* table_s	// NULL = skip
	);

/*
	Tables terminated by name = NULL.
	If (bulk.var == NULL) it is skipped.
*/

int mtkit_prefs_value_mirror (		// Copy values from src to dest (all
					// items in table)
	mtPrefs		* dest,
	mtPrefs		* src,
	mtPrefItem	* table
	);

int mtkit_prefs_value_copy (		// Copy values from src to dest (all
					// items in table)
	mtPrefs		* dest,
	mtPrefs		* src,
	mtPrefTrans	* table
	);

mtTree * mtkit_prefs_get_tree (		// Return core tree structure
	mtPrefs		* prefs
	);

int mtkit_arg_parse (
	int		argc,
	char		* argv[],
	mtArg		* arg_list,	// Terminated when
					// arg_list[].argument == NULL
	mtArgFileFunc	file_func,	// Callback for when a filename is
					// encountered. NULL = skip
	mtArgErrorFunc	error_func	// Callback for error. NULL = skip
	);
	// 0 = success
	// 1 = fail
	// 2 = callback termination

/*
The following code is a simple implentation of the ZIP file format.
Its goal is to provide a simply way to create and read ZIP archives for general
programming tasks.  Due to its simplicity it does have limitations.

mtkit_file_load_zip () cannot read all types of ZIP file created with other
programs.  Only those ZIP archives created with mtkit_file_save_zip () or other
programs using the old PKZIP standard (i.e. version 2 using deflate compression)
are guaranteed to work.	If you plan to manipulate ZIP archives containing large
files (e.g. >1GB) on a system with less memory than this you should not use
these functions. You will need to use code that is more memory frugal.

My goal with this work is simplicity, which means expanding each file in the
ZIP archive as a single chunk of memory.  This vastly simplifies the API here,
and also the code in the library, and works well for 99% of ZIP archives on
modern computers (i.e. most PC's have many megabytes of free RAM and very few
files inside a ZIP archive are likely to exceed this, especially as this library
is aimed at small scale data storage rather than multimedia files).

Can only access files less than 2GB in size.
*/

mtZip * mtkit_zip_save_open (		// Start saving a ZIP file
	char		* filename	// Must be a valid string until
					// mtkit_file_save_zip_close ()
	);

int mtkit_zip_save_file (		// Add this memory chunk to a ZIP file
	mtZip		* zip,
	char		* name,
	void		* mem,		// Memory chunk to save
	int32_t		mem_len,	// Bytes in chunk
	int8_t		deflate,	// 1 = Try to deflate memory chunk
	int32_t		year,
	int8_t		month,
	int8_t		day,
	int8_t		hour,
	int8_t		minute,
	int8_t		second
	);

/*
This must always be called to:
	1. End saving (or)
	2. Cleanup after a failure of mtkit_file_save_zip_file ()
*/
int mtkit_zip_save_close (
	mtZip		* zip
	);
	// 0 = successfully saved a zip file

int mtkit_zip_load (			// Read a ZIP file created by
					// mtkit_file_save_zip ()
	char		* filename,
	mtZipLoadFunc	callback,	// Called for each chunk in ZIP file,
					// return MTKIT_ZIP_STOP
					// to stop or 0 to request a new chunk
	void		* user_data	// Passed to callback
	);
	// 0 = success else MTKIT_ZIP_ERROR*

/*
NOTE: mtkit_cli_new () assumes the caller knows what he is doing.  For example
these two entries might be logically impossible to differentiate:

	{ "save",		jtf_save,		1, "<OS FILENAME>" },
	{ "save as",		jtf_save_as },

The input "save as"  could be interpreted as either, so results are undefined.
Beware!
*/

mtCliTab * mtkit_clitab_new (		// Create empty mtCliTab
	void
	);

mtCliTab * mtkit_cli_new (		// Create a new CLI table
	mtCliItem	* list		// NULL terminated static array
	);

int mtkit_cli_free (			// Destroy a CLI table
	mtCliTab	* table
	);

mtCliItem * mtkit_cli_match (		// Try to match a command line + args
	mtCliTab	* table,
	char		** argv,	// NULL terminated
	int		* cli_error,	// NULL = don't pass error, else return
					//  0 = correct match with correct
					// number of args
					// -1 = too few args
					// -2 = too many args
					// -3 = other error
					// 1,2,3,etc = unmatched command
	int		* ncargs	// Number of command arguments found if
					// matched
	);
	// NULL = No command match

int mtkit_cli_help (			// Output command list and associated
					// help
	mtCliTab	* table,
	char		** argv		// NULL terminated
	);
	// 0 = success
	// 1 = unmatched command at this point



#endif		// _mtkit_h

