/*
	Copyright (C) 2009 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <stdlib.h>
#include <stdio.h>

#include <mtkit.h>


static int file_func_term( char *filename )
{
	fprintf( stderr, "file_func_term: Argument ERROR! - Unexpected filename '%s'\n", filename );

	return 1;		// Stop parsing
}

static int error_func_term( int error, int arg, int argc, char *argv[] )
{
	fprintf( stderr, "error_func_term: Argument ERROR! - num=%i arg=%i/%i", error, arg, argc );

	if ( arg < argc )
		fprintf( stderr, " '%s'", argv[arg] );

	fprintf( stderr, "\n" );

	return 1;		// Stop parsing
}

static int file_func( char *filename )
{
	fprintf( stderr, "file_func: Argument ERROR! - Unexpected filename '%s'\n", filename );

	return 0;		// Keep parsing
}

static int error_func( int error, int arg, int argc, char *argv[] )
{
	fprintf( stderr, "error_func: Argument ERROR! - num=%i arg=%i/%i", error, arg, argc );

	if ( arg < argc )
		fprintf( stderr, " '%s'", argv[arg] );

	fprintf( stderr, "\n" );

	return 0;		// Keep parsing
}

static int callback_func( mtArg *mtarg, int arg, int argc, char *argv[] )
{
	printf( "Arg parsed: %s argn=%i type=%i\n", mtarg->argument, arg, mtarg->type );

	return 0;		// Keep parsing
}


#define VAL_TOT 6

int main( int argc, char *argv[] )
{
	double	double_vals[VAL_TOT] = { 0 };
	int	i, int_vals[VAL_TOT] = { 0 };
	char	*char_vals[VAL_TOT] = { NULL };
	mtArg	arg_list[] = {
	// NOTE - its good practice to keep these in alpha order to easily spot duplicates
	{ "-double",	MTKIT_ARG_DOUBLE,	&double_vals[4] },
	{ "-integer",	MTKIT_ARG_INT,		&int_vals[4] },
	{ "-output",	MTKIT_ARG_STRING,	&char_vals[4]  },
	{ "c0",		MTKIT_ARG_STRING,	&char_vals[0] },
	{ "c1",		MTKIT_ARG_STRING,	&char_vals[1] },
	{ "c2",		MTKIT_ARG_STRING,	&char_vals[2] },
	{ "c3",		MTKIT_ARG_STRING,	&char_vals[3],	0, callback_func },
	{ "d",		MTKIT_ARG_DOUBLE,	&double_vals[4] },
	{ "d0",		MTKIT_ARG_DOUBLE,	&double_vals[0] },
	{ "d1",		MTKIT_ARG_DOUBLE,	&double_vals[1] },
	{ "d2",		MTKIT_ARG_DOUBLE,	&double_vals[2] },
	{ "d3",		MTKIT_ARG_DOUBLE,	&double_vals[3], 0, callback_func },
	{ "i",		MTKIT_ARG_INT,		&int_vals[4] },
	{ "i0",		MTKIT_ARG_INT,		&int_vals[0] },
	{ "i1",		MTKIT_ARG_INT,		&int_vals[1] },
	{ "i2",		MTKIT_ARG_INT,		&int_vals[2] },
	{ "i3",		MTKIT_ARG_INT,		&int_vals[3],	0, callback_func },
	{ "o",		MTKIT_ARG_STRING,	&char_vals[4] },
	{ "s50",	MTKIT_ARG_SWITCH,	&int_vals[5],	0 },
	{ "s51",	MTKIT_ARG_SWITCH,	&int_vals[5],	1 },
	{ "s52",	MTKIT_ARG_SWITCH,	&int_vals[5],	2 },
	{ "s53",	MTKIT_ARG_SWITCH,	&int_vals[5],	3, callback_func },
	{ NULL }
	};

	printf( "\nTEST 1 - with terminating callbacks\n\n" );
	mtkit_arg_parse( argc, argv, arg_list, file_func_term, error_func_term );

	printf( "\nTEST 2 - with callbacks\n\n" );
	mtkit_arg_parse( argc, argv, arg_list, file_func, error_func );

	printf( "\nTEST 3 - without any callbacks\n\n" );
	mtkit_arg_parse( argc, argv, arg_list, NULL, NULL );

	printf( "\nTEST RESULTS\n\n" );
	for ( i=0; i<VAL_TOT; i++ )
	{
		printf( "%i : %8i %18f %s\n", i, int_vals[i], double_vals[i], char_vals[i] );
	}

	return 0;
}
