/*
	Copyright (C) 2008-2010 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <stdlib.h>
#include <stdio.h>

#include <mtkit.h>



#define INFILE1			"test.in.txt"
#define OUTFILE_CREATE1		"out_create.txt"
#define OUTFILE_CREATE2		"out_create2.txt"
#define OUTFILE_TYPE1		"out_type1.txt"
#define OUTFILE_TYPE2		"out_type2.txt"
#define OUTFILE_TYPE3		"out_type3.txt"
#define OUTFILE_TYPE4		"out_type4.txt"
#define OUTFILE_TYPE1b		"out_type1b.txt"
#define OUTFILE_TYPE2b		"out_type2b.txt"
#define OUTFILE_TYPE3b		"out_type3b.txt"
#define OUTFILE_TYPE4b		"out_type4b.txt"
#define OUTFILE_IN1		"out_in.txt"
#define OUTFILE_CREATE1GZ	"out_create.txt.gz"
#define OUTFILE_CREATE2GZ	"out_create2.txt.gz"
#define OUTFILE_IN1GZ		"out_in.txt.gz"

static void test_create()
{
	mtUtreeNode *root, *n1, *n2;


	root = mtkit_utree_new_root();

	n1 = mtkit_utree_new_element( root, "main" );
	mtkit_utree_set_attribute_str( n1, "a", "1" );
	mtkit_utree_set_attribute_str( n1, "bb", "1.234" );
	mtkit_utree_set_attribute_str( n1, "cdef", "-512" );
	mtkit_utree_set_attribute_str( n1, "z", "string \\ \" { } &" );
	mtkit_utree_set_attribute_str( n1, "empty", "" );		// Invalid
	mtkit_utree_new_comment( n1, " This is a comment " );
	mtkit_utree_new_text( n1, "This is an example of a \"text block\" in my UTREE file" );
	mtkit_utree_new_text( n1, "a1" );
	mtkit_utree_new_text( n1, "a2" );
	mtkit_utree_new_text( n1, "a3" );
	mtkit_utree_new_text( n1, "a4" );
	mtkit_utree_new_text( n1, "a5" );
	mtkit_utree_new_text( n1, "" );

	n2 = mtkit_utree_new_element( n1, "" );
	n2 = mtkit_utree_new_element( n1, NULL );
	n2 = mtkit_utree_new_element( n1, " " );

	n2 = mtkit_utree_new_element( n1, "subnode" );
	mtkit_utree_new_text( n2, "This program is distributed in the hope that it will be useful" );
	mtkit_utree_new_comment( n2, " This is a sub-comment " );

	n2 = mtkit_utree_new_element( n1, "new_subnode" );
	mtkit_utree_new_comment( n2, " Subnode comment 1 " );
	mtkit_utree_new_element( n2, "nested_subnode" );
	mtkit_utree_new_comment( n2, " Subnode comment */*/*/ 2 " );

	mtkit_utree_new_comment( n1, " This is yet another comment " );
	mtkit_utree_new_comment( root, " Last but not least, this is the final comment! { } & \" # $ % " );

	mtkit_utree_save_file( root, OUTFILE_CREATE1, MTKIT_UTREE_OUTPUT_DEFAULT, 0 );
	mtkit_utree_save_file( root, OUTFILE_CREATE1GZ, MTKIT_UTREE_OUTPUT_DEFAULT,
		MTKIT_FILE_GUNZIP );

	mtkit_utree_destroy_node( root );
	n1 = n2 = root = NULL;			// Return everything to original state
}

static int attr_scan( mtTreeNode *node, void *user_data )
{
	printf( "\t%s=\"%s\"", (char *)node->key, (char *)node->data );
	return 0;
}

static void test_read()
{
	char		*address = "window/dialog/frame";
	mtUtreeNode	*root, *found;


	root = mtkit_utree_load_file( NULL, INFILE1, NULL, NULL );

	found = mtkit_utree_get_node_via_address( root, address );
	if ( found )
	{
		printf("Address '%s' contains:\n", address);
		found = found->child;
		while ( found )
		{
			printf("\t%i\t%s", found->type, found->text);

			mtkit_tree_scan( found->attribute_tree, attr_scan, NULL, 0 );

			printf("\n");
			found = found->next;
		}
	}

	mtkit_utree_destroy_node( root );
	root = NULL;
}

static void test_write()
{
	mtUtreeNode *root;

	root = mtkit_utree_load_file( NULL, INFILE1, NULL, NULL );
	mtkit_utree_save_file( root, OUTFILE_IN1, MTKIT_UTREE_OUTPUT_DEFAULT, 0 );
	mtkit_utree_save_file( root, OUTFILE_IN1GZ, MTKIT_UTREE_OUTPUT_DEFAULT,
		MTKIT_FILE_GUNZIP );
	mtkit_utree_destroy_node( root );

	root = mtkit_utree_load_file( NULL, OUTFILE_CREATE1, NULL, NULL );
	mtkit_utree_save_file( root, OUTFILE_CREATE2, MTKIT_UTREE_OUTPUT_DEFAULT, 0 );
	mtkit_utree_save_file( root, OUTFILE_CREATE2GZ, MTKIT_UTREE_OUTPUT_DEFAULT,
		MTKIT_FILE_GUNZIP );
	mtkit_utree_destroy_node( root );

	root = NULL;
}

static void test_types()
{
	int		errors = -1, filetype = -1;
	mtUtreeNode	*utree;


	utree = mtkit_utree_load_file( NULL, INFILE1, &errors, &filetype );
	printf( "LOAD: errors = %i filetype = %i\n", errors, filetype );

	mtkit_utree_save_file( utree, OUTFILE_TYPE1, MTKIT_UTREE_OUTPUT_INDENTS |
		MTKIT_UTREE_OUTPUT_NEWLINES | MTKIT_UTREE_OUTPUT_TEXT_NEWLINES, 0 );
	mtkit_utree_save_file( utree, OUTFILE_TYPE2, MTKIT_UTREE_OUTPUT_INDENTS |
		MTKIT_UTREE_OUTPUT_NEWLINES, 0 );
	mtkit_utree_save_file( utree, OUTFILE_TYPE3, MTKIT_UTREE_OUTPUT_INDENTS, 0 );
	mtkit_utree_save_file( utree, OUTFILE_TYPE4, 0, 0 );
	mtkit_utree_destroy_node( utree );

	utree = mtkit_utree_load_file( NULL, OUTFILE_TYPE1, &errors, &filetype );
	mtkit_utree_save_file( utree, OUTFILE_TYPE1b, MTKIT_UTREE_OUTPUT_INDENTS |
		MTKIT_UTREE_OUTPUT_NEWLINES | MTKIT_UTREE_OUTPUT_TEXT_NEWLINES, 0 );
	mtkit_utree_destroy_node( utree );

	utree = mtkit_utree_load_file( NULL, OUTFILE_TYPE2, &errors, &filetype );
	mtkit_utree_save_file( utree, OUTFILE_TYPE2b, MTKIT_UTREE_OUTPUT_INDENTS |
		MTKIT_UTREE_OUTPUT_NEWLINES, 0 );
	mtkit_utree_destroy_node( utree );

	utree = mtkit_utree_load_file( NULL, OUTFILE_TYPE3, &errors, &filetype );
	mtkit_utree_save_file( utree, OUTFILE_TYPE3b, MTKIT_UTREE_OUTPUT_INDENTS, 0 );
	mtkit_utree_destroy_node( utree );

	utree = mtkit_utree_load_file( NULL, OUTFILE_TYPE4, &errors, &filetype );
	mtkit_utree_save_file( utree, OUTFILE_TYPE4b, 0, 0 );
	mtkit_utree_destroy_node( utree );
}

int main( int argc, char *argv[] )
{
	test_create();
	test_read();
	test_write();
	test_types();

	return 0;
}
