/*
	Copyright (C) 2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#ifndef _mtotp_h
#define _mtotp_h



typedef struct _mtOTP		mtOTP;



#define MTOTP_MASH_MAX		16
#define MTOTP_PAD_LOADED	0



// Return 0 = success, NULL = fail unless otherwise stated

mtOTP * mtotp_new (
	void
	);

int mtotp_free (
	mtOTP		* mtotp
	);

int mtotp_set_root (
	mtOTP		* mtotp,
	char		* root
	);

int mtotp_is_pad_loaded (
	mtOTP		* mtotp
	);
	// 0 = Yes - Pad loaded - MTOTP_PAD_LOADED
	// 1 = No  - Pad not loaded yet, or invalid mtotp

int mtotp_query (
	mtOTP		* mtotp,
	double		* total,	// Total remaining bytes in pad.
					// NULL = Don't use
	int		output,		// 0 = Don't output info to stdout
	int		verbose
	);

int mtotp_query_filename (
	char		* filename,
	double		* row,		// X if not NULL
	double		* offset	// Y if not NULL
	);
	// 0 = Filename is of this form: FILE.mtotp.X.Y
	// 1 = Not recognised

char * mtotp_next_char_left (
	char		* a,		// String origin
	char		* b,		// Place in string to start looking
	char		ch		// Character to find
	);
	// Return pointer to next occurence in the string (moving to left).
	// NULL = Not found

int mtotp_mash (
	mtOTP		* mtotp,
	char		* mash_list[],	// List of file inputs
	int		mash_filetot,	// Total number of filenames in list
	char		* output,	// Output filename
	int		verbose
	);

int mtotp_analyse (
	mtOTP		* mtotp,
	char		* filename,
	int		verbose
	);

int mtotp_decrypt (
	mtOTP		* mtotp,
	char		* filename,
	int		verbose
	);

int mtotp_encrypt (
	mtOTP		* mtotp,
	char		* filename,
	int		verbose
	);

int mtotp_generate (
	mtOTP		* mtotp,
	char		* filename,
	char		* comment,
	int		verbose
	);


#endif		// _mtotp_h

