#!/bin/bash


if [ "$VALG" = "Y" ]
then
	VALG="valgrind --leak-check=full --show-possibly-lost=no"
fi

ROOTDIR=~/.config/mtotp_test
MTOTP="$VALG mtotp -root $ROOTDIR"
VALG_LOGFILE=$ROOTDIR/tmp/valg_log.txt

> $VALG_LOGFILE


# On error exit
set -e

# Echo commands
set -x



# Clean up old rubbish
rm -f $ROOTDIR/*.mtotp
rm -f $ROOTDIR/conf/*
rm -f $ROOTDIR/in/*
rm -f $ROOTDIR/out/*
rm -f $ROOTDIR/tmp/*


if [ "$1" = "clean" ]
then
	exit
fi


# Create mashed input files
dd if=/dev/urandom of=$ROOTDIR/tmp/m_01.bin count=1 bs=1048575
dd if=/dev/urandom of=$ROOTDIR/tmp/m_02.bin count=1 bs=1048576
dd if=/dev/urandom of=$ROOTDIR/tmp/m_03.bin count=1 bs=1048577
dd if=/dev/urandom of=$ROOTDIR/tmp/m_04.bin count=1 bs=10485759
dd if=/dev/urandom of=$ROOTDIR/tmp/m_05.bin count=1 bs=10485760
dd if=/dev/urandom of=$ROOTDIR/tmp/m_06.bin count=1 bs=10485761
dd if=/dev/zero of=$ROOTDIR/tmp/m_07.bin count=4096 bs=1048576
dd if=/dev/zero of=$ROOTDIR/tmp/z_01.bin count=1 bs=1048575

$MTOTP -m $ROOTDIR/tmp/m_01.bin -o $ROOTDIR/tmp/f_01.bin >> $VALG_LOGFILE 2>&1

$MTOTP -m $ROOTDIR/tmp/m_01.bin $ROOTDIR/tmp/m_01.bin \
	-o $ROOTDIR/tmp/f_02.bin >> $VALG_LOGFILE 2>&1

$MTOTP -m $ROOTDIR/tmp/m_01.bin $ROOTDIR/tmp/m_02.bin \
	-o $ROOTDIR/tmp/f_03.bin >> $VALG_LOGFILE 2>&1

$MTOTP -m $ROOTDIR/tmp/m_01.bin $ROOTDIR/tmp/m_02.bin \
	$ROOTDIR/tmp/m_03.bin $ROOTDIR/tmp/m_03.bin \
	-o $ROOTDIR/tmp/f_04.bin >> $VALG_LOGFILE 2>&1

$MTOTP -m $ROOTDIR/tmp/m_01.bin $ROOTDIR/tmp/m_02.bin \
	$ROOTDIR/tmp/m_03.bin $ROOTDIR/tmp/m_04.bin \
	$ROOTDIR/tmp/m_05.bin $ROOTDIR/tmp/m_05.bin \
	-o $ROOTDIR/tmp/f_05.bin >> $VALG_LOGFILE 2>&1

$MTOTP -m $ROOTDIR/tmp/m_01.bin $ROOTDIR/tmp/m_02.bin \
	$ROOTDIR/tmp/m_03.bin $ROOTDIR/tmp/m_04.bin \
	$ROOTDIR/tmp/m_05.bin $ROOTDIR/tmp/m_06.bin \
	$ROOTDIR/tmp/m_06.bin $ROOTDIR/tmp/m_06.bin \
	-o $ROOTDIR/tmp/f_06.bin >> $VALG_LOGFILE 2>&1

# This is a simple reversal
$MTOTP -m $ROOTDIR/tmp/m_01.bin $ROOTDIR/tmp/f_02.bin \
	-o $ROOTDIR/tmp/d_01.bin >> $VALG_LOGFILE 2>&1

# Check results are identical as expected
cmp -l $ROOTDIR/tmp/f_01.bin $ROOTDIR/tmp/d_01.bin
cmp -l $ROOTDIR/tmp/f_02.bin $ROOTDIR/tmp/z_01.bin


# Create pad files

# On error do nothing
set +e

echo This should generate an error
$MTOTP -g $ROOTDIR/tmp/f_01.bin

# On error exit
set -e

$MTOTP -g $ROOTDIR/tmp/f_0[3-6].bin >> $VALG_LOGFILE 2>&1
$MTOTP -g $ROOTDIR/tmp/m_0[2-7].bin >> $VALG_LOGFILE 2>&1


# Statistically analyse pads
$MTOTP -a $ROOTDIR/*.mtotp >> $VALG_LOGFILE 2>&1


# Create dummy files
dd if=/dev/urandom of=$ROOTDIR/in/a_01.bin count=1 bs=5242880
dd if=/dev/urandom of=$ROOTDIR/in/a_02.bin count=1 bs=3145728
yes "qwertyuiop" | head --bytes=3G > $ROOTDIR/in/a_03.bin

for (( NUM=10; NUM<=19; NUM++ ))
do
	dd if=/dev/urandom of=$ROOTDIR/in/a_$NUM.bin count=1 bs=12347
done


# Encrypt dummy files
$MTOTP $ROOTDIR/in/* >> $VALG_LOGFILE 2>&1
mv $ROOTDIR/in/*.mtotp* $ROOTDIR/out/


# Decrypt dummy files
$MTOTP $ROOTDIR/out/* >> $VALG_LOGFILE 2>&1
mv $ROOTDIR/out/*.mtotp* $ROOTDIR/tmp/


# Check results are identical as start files.
# Sadly diff runs out of memory, so I use cmp via a loop instead.
for FILE in $ROOTDIR/in/*
do
	FILENAME=$(basename $FILE)

	cmp $ROOTDIR/in/$FILENAME $ROOTDIR/out/$FILENAME >> $VALG_LOGFILE 2>&1
done


if [ -n "$VALG" ]
then
	# On error do nothing
	set +e

	echo
	grep "definitely lost" $VALG_LOGFILE

	echo
	grep "ERROR SUMMARY" $VALG_LOGFILE

	# On error exit
	set -e
fi
