/*
	Copyright (C) 2009-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#ifndef _mtpixel_h
#define _mtpixel_h

#include <mtkit.h>



typedef struct _mtBrush		mtBrush;
typedef struct _mtFont		mtFont;
typedef struct _mtFontFT	mtFontFT;
typedef struct _mtFontRender	mtFontRender;
typedef struct _mtGradInfo	mtGradInfo;
typedef struct _mtGradMap	mtGradMap;
typedef struct _mtGradient	mtGradient;
typedef struct _mtImage		mtImage;
typedef struct _mtPalette	mtPalette;
typedef struct _mtPolygon	mtPolygon;



#define MTPIXEL_PALETTE_ITEMS		256
#define MTPIXEL_PALETTE_CHANNELS	3	// RGB
#define MTPIXEL_POLYGON_MAX		1000	// Maximum points
#define MTPIXEL_BRUSH_PATTERN_TOTAL	100
#define MTPIXEL_MAX_GRAD		65536
#define MTPIXEL_GRAD_POINTS		256
#define MTPIXEL_INT_2_R(A)		(((A) >> 16) & 0xFF)
#define MTPIXEL_INT_2_G(A)		(((A) >> 8) & 0xFF)
#define MTPIXEL_INT_2_B(A)		((A) & 0xFF)
#define MTPIXEL_RGB_2_INT(R,G,B)	(((R) << 16) + ((G) << 8) + (B))



enum
{
	MTPIXEL_ERRNO_NONE			= 0,

	MTPIXEL_ERRNO_MALLOC			= 1,
	MTPIXEL_ERRNO_FILE_OPEN			= 2,
	MTPIXEL_ERRNO_FILETYPE_NOT_SUPPORTED	= 5,
	MTPIXEL_ERRNO_FULL			= 10,
	MTPIXEL_ERRNO_FREETYPE			= 50,
	MTPIXEL_ERRNO_FREETYPE_NOT_SUPPORTED	= 51,
	MTPIXEL_ERRNO_CHANNEL_GEOMETRY		= 80,
	MTPIXEL_ERRNO_ARGUMENT			= 100,
	MTPIXEL_ERRNO_FILE_LOAD			= 201,
	MTPIXEL_ERRNO_FILE_SAVE			= 211,

	MTPIXEL_ERRNO_IMAGE_TYPE		= 1000
};

typedef enum	// mtImage->type, mtFontRender->image_type
{
	MTPIXEL_IMAGE_INDEXED		= 1,
	MTPIXEL_IMAGE_RGB		= 3
} mtImageType;

typedef enum
{
	MTPIXEL_FILE_TYPE_NONE,

	MTPIXEL_FILE_TYPE_BMP,		// Always available

	MTPIXEL_FILE_TYPE_PNG,
	MTPIXEL_FILE_TYPE_GIF,		// Possibly available via external libs
	MTPIXEL_FILE_TYPE_JPEG,

	MTPIXEL_FILE_TYPE_TOTAL
} mtFileType;

typedef enum
{
	MTPIXEL_CHANNEL_IMAGE		= 0,
	MTPIXEL_CHANNEL_ALPHA		= 1,
	MTPIXEL_CHANNEL_SELECTION	= 2,
	MTPIXEL_CHANNEL_MASK		= 3,

	MTPIXEL_CHANNEL_TOTAL		= 4
} mtChannelType;

typedef enum
{
	MTPIXEL_CHANNEL_MASK_NONE	= 0,

	MTPIXEL_CHANNEL_MASK_IMAGE	= 1 << MTPIXEL_CHANNEL_IMAGE,
	MTPIXEL_CHANNEL_MASK_ALPHA	= 1 << MTPIXEL_CHANNEL_ALPHA,
	MTPIXEL_CHANNEL_MASK_SELECTION	= 1 << MTPIXEL_CHANNEL_SELECTION,
	MTPIXEL_CHANNEL_MASK_MASK	= 1 << MTPIXEL_CHANNEL_MASK,

	MTPIXEL_CHANNEL_MASK_ALL	= ~0
} mtChannelMask;

typedef enum
{
	MTPIXEL_BRUSH_SHAPE_NONE	= 0,

	MTPIXEL_BRUSH_SHAPE_SQUARE,
	MTPIXEL_BRUSH_SHAPE_CIRCLE,
	MTPIXEL_BRUSH_SHAPE_SPRAY,
	MTPIXEL_BRUSH_SHAPE_SLASH,
	MTPIXEL_BRUSH_SHAPE_BACKSLASH,
	MTPIXEL_BRUSH_SHAPE_VERTICAL,
	MTPIXEL_BRUSH_SHAPE_HORIZONTAL,

	MTPIXEL_BRUSH_SHAPE_TOTAL
} mtBrushShape;

typedef enum
{
	MTPIXEL_BRUSH_MODE_NONE,

	MTPIXEL_BRUSH_MODE_PATTERN,
	MTPIXEL_BRUSH_MODE_GRADIENT,

	MTPIXEL_BRUSH_MODE_TOTAL
} mtBrushMode;

typedef enum
{
	MTPIXEL_PALETTE_SORT_NONE,

	MTPIXEL_PALETTE_SORT_HUE,
	MTPIXEL_PALETTE_SORT_SATURATION,
	MTPIXEL_PALETTE_SORT_VALUE,
	MTPIXEL_PALETTE_SORT_LUMINANCE,
	MTPIXEL_PALETTE_SORT_RED,
	MTPIXEL_PALETTE_SORT_GREEN,
	MTPIXEL_PALETTE_SORT_BLUE,
	MTPIXEL_PALETTE_SORT_FREQUENCY,

	MTPIXEL_PALETTE_SORT_TOTAL
} mtPaletteSort;

typedef enum
{
	MTPIXEL_QUANTIZE_NONE,

	MTPIXEL_QUANTIZE_PNN,
	MTPIXEL_QUANTIZE_MAXMIN,

	MTPIXEL_QUANTIZE_TOTAL
} mtQuantizeType;

typedef enum
{
	MTPIXEL_SCALE_NONE,

	MTPIXEL_SCALE_NEAREST_NEIGHBOUR,
	MTPIXEL_SCALE_BILINEAR,		// FIXME - creates combing on 50%, 25%
	MTPIXEL_SCALE_BICUBIC,
	MTPIXEL_SCALE_BICUBIC_EDGED,
	MTPIXEL_SCALE_BICUBIC_BETTER,
	MTPIXEL_SCALE_BICUBIC_SHARP,
	MTPIXEL_SCALE_BLACKMAN_HARRIS,
	MTPIXEL_SCALE_MT,		// Original mtPaint nearest neighbour
					// (image channel only)
	MTPIXEL_SCALE_MT_SMOOTH,	// Original mtPaint smooth (image
					// channel only)

	MTPIXEL_SCALE_TOTAL
} mtScaleType;

typedef enum
{
	MTPIXEL_GRADIENT_MODE_NONE,
	MTPIXEL_GRADIENT_MODE_LINEAR,
	MTPIXEL_GRADIENT_MODE_BILINEAR,
	MTPIXEL_GRADIENT_MODE_RADIAL,
	MTPIXEL_GRADIENT_MODE_SQUARE,

	MTPIXEL_GRADIENT_MODE_TOTAL
} mtGradientMode;

typedef enum
{
	MTPIXEL_GRADIENT_BOUND_STOP,
	MTPIXEL_GRADIENT_BOUND_LEVEL,
	MTPIXEL_GRADIENT_BOUND_REPEAT,
	MTPIXEL_GRADIENT_BOUND_MIRROR,

	MTPIXEL_GRADIENT_BOUND_TOTAL
} mtGradientBound;

typedef enum
{
	MTPIXEL_GRADIENT_TYPE_RGB,
	MTPIXEL_GRADIENT_TYPE_HSV,
	MTPIXEL_GRADIENT_TYPE_BK_HSV,
	MTPIXEL_GRADIENT_TYPE_CONST,
	MTPIXEL_GRADIENT_TYPE_CUSTOM,

	MTPIXEL_GRADIENT_TYPE_TOTAL
} mtGradientType;

typedef enum
{
	MTPIXEL_DITHER_TYPE_NONE,
	MTPIXEL_DITHER_TYPE_FLOYD1,
	MTPIXEL_DITHER_TYPE_FLOYD2,
	MTPIXEL_DITHER_TYPE_STUCKI,
	MTPIXEL_DITHER_TYPE_DITHER,
	MTPIXEL_DITHER_TYPE_SCATTER,

	MTPIXEL_DITHER_TYPE_TOTAL
} mtDitherType;

typedef enum
{
	MTPIXEL_DITHER_CS_NONE,

	MTPIXEL_DITHER_CS_RGB,
	MTPIXEL_DITHER_CS_SRGB,
	MTPIXEL_DITHER_CS_LXN,

	MTPIXEL_DITHER_CS_TOTAL
} mtDitherCS;

typedef enum
{
	MTPIXEL_DITHER_DIFF_NONE,

	MTPIXEL_DITHER_DIFF_LINF,
	MTPIXEL_DITHER_DIFF_L1,
	MTPIXEL_DITHER_DIFF_L2,

	MTPIXEL_DITHER_DIFF_TOTAL
} mtDitherDiff;

typedef enum
{
	MTPIXEL_DITHER_BL_NONE,

	MTPIXEL_DITHER_BL_GAMUT,
	MTPIXEL_DITHER_BL_WEAK,
	MTPIXEL_DITHER_BL_STRONG,

	MTPIXEL_DITHER_BL_TOTAL
} mtDitherBL;

typedef enum
{
	MTPIXEL_DITHER_SEP_NONE,

	MTPIXEL_DITHER_SEP_OFF,
	MTPIXEL_DITHER_SEP_SUM,
	MTPIXEL_DITHER_SEP_SPLIT,
	MTPIXEL_DITHER_SEP_LSUM,
	MTPIXEL_DITHER_SEP_LSPLIT,

	MTPIXEL_DITHER_SEP_TOTAL
} mtDitherSep;

typedef enum	// mtFont->type
{
	MTPIXEL_FONT_NONE		= 0,

	MTPIXEL_FONT_FIXED,
	MTPIXEL_FONT_FT_FIXED,
	MTPIXEL_FONT_FT_SCALABLE,

	MTPIXEL_FONT_TOTAL
} mtFontType;

typedef enum
{
	MTPIXEL_POLYGON_UNREADY		= 0,
	MTPIXEL_POLYGON_READY		= 1 << 0	// Max/Min calculated
} mtPolygonMode;



struct _mtPalette
{
	unsigned char	col[MTPIXEL_PALETTE_ITEMS][MTPIXEL_PALETTE_CHANNELS];
	int		trans,		// Transparent colour (-1 = none)
			size;		// Number of cols in palette
};

struct _mtImage
{
	int		width,
			height;
	mtImageType	type;		// As MTPIXEL_IMAGE_*
	unsigned char	* channel[MTPIXEL_CHANNEL_TOTAL];
					// Memory allocations per channel
	mtPalette	palette;
};

struct _mtGradInfo
{
	/* Base values */
	int		status,
			x1,
			y1,
			x2,
			y2,		// Gradient placement tool
			len,
			rep,
			ofs,		// Gradient length, repeat, and offset
			gmode,
			rmode;		// Gradient mode and repeat mode
	/* Derived values */
	double		wrep,
			wil1,
			wil2,
			xv,
			yv;
	int		wmode;
};

struct _mtGradMap
{
	char		gtype,
			otype,		// Main and opacity gradient types
			grev,
			orev;		// Main and opacity reversal flags
	int		vslen,
			oplen,		// Current gradient lengths
			cvslen,
			coplen;		// Custom gradient lengths
	unsigned char	* vs,
			* vsmap,	// Current gradient data
			* op,
			* opmap;	// Current gradient opacities
};

typedef unsigned char
	mtGradStore[(6 + MTPIXEL_CHANNEL_TOTAL * 4) * MTPIXEL_GRAD_POINTS];

struct _mtGradient
{
	mtGradInfo	gradient[MTPIXEL_CHANNEL_TOTAL];
					// Per-channel gradient placement
	mtGradMap	graddata[MTPIXEL_CHANNEL_TOTAL + 1];
					// RGB + per-channel gradient data
	mtGradStore	gradbytes;	// Storage space for custom gradients
	unsigned char	grad_def[4 + 8 + MTPIXEL_CHANNEL_TOTAL * 4];
};

struct _mtBrush
{
	int		col_rgb[3][2],	// RGB colour A/B
			col_i[MTPIXEL_CHANNEL_TOTAL][2],
					// Indexed/channel colour A/B
			pattern,	// Current brush pattern ID
			size,		// Current brush size
			flow,		// Flow of spray
			opacity;	// For RGB operations - 255 = opaque
					// 0 = transparent
	mtBrushShape	shape;		// Current brush shape ID
	mtBrushMode	mode;		// Painting a pattern/gradient
	unsigned char	pattern_i[8 * 8],
			pattern_rgb[8 * 8 * 3],
			chan_dis[MTPIXEL_CHANNEL_TOTAL];
					// 1 = Disable channel operations
	mtGradient	gradient;
};

struct _mtFont
{
	int		height;		// Font height in pixels. 8/16 for
					// MTPIXEL_FONT_FIXED
	mtFontType	type;
	mtFontFT	* ft;		// Used if font is a FreeType font
};

struct _mtFontRender
{
	// Used on input, i.e. Set/Created/Destroyed by calling program,
	// read by library

	mtFont		* font;		// Prepared font to use
	char		* utf8;		// Prepared string to use
	mtBrush		* brush;	// Brush to use NULL = don't create
					// image channel
	mtImageType	image_type;	// MTPIXEL_IMAGE_RGB/INDEXED

	int		max_chars,	// Maximum characters to render
					// (0 = no limit)
			max_width;	// Maximum pixel width (0 = no limit)

	// Used as output, i.e. Set/Created/Destroyed by Library, read by
	// calling program

	int		width,
			height,		// Pixel geometry of the output
			chars,		// Characters rendered
			* posx		// X Pixel positions of each character
					// (or NULL)
			;

	char		* utf8_next;	// Next UTF-8 character to use (if
					// truncated)
};

struct _mtPolygon
{
	int		size,			// Points defined
			point[MTPIXEL_POLYGON_MAX][2],
			min_x, max_x,
			min_y, max_y;
	mtPolygonMode	mode;
};



// Return 0 = success, NULL = fail unless otherwise stated

int mtpixel_init (			// Call first before anything else
	void
	);

int mtpixel_quit (			// Call once finished with the library
	void
	);

int mtpixel_filetype (			// Is this filetype supported?
	int		type,
	int		* flags,	// Flags (RGB/Indexed)
	char		** desc,	// Description
	char		** ext_short,	// Extension
	char		** ext_long	// Possible long extension
	);
	// 0 = no
	// 1 = yes

mtPolygon * mtpixel_polygon_new (
	void
	);

int mtpixel_polygon_add_point (
	mtPolygon	* polygon,
	int		x,
	int		y
	);

int mtpixel_polygon_clear (		// Empty a polygon
	mtPolygon	* polygon
	);

mtPolygon * mtpixel_polygon_duplicate (
	mtPolygon	* polygon
	);

int mtpixel_polygon_destroy (
	mtPolygon	* polygon
	);

int mtpixel_palette_load (
	mtPalette	* palette,
	char		* filename
	);

int mtpixel_palette_save (
	mtPalette	* palette,
	char		* filename
	);

int mtpixel_palette_copy (		// Copy palette
	mtPalette	* dest,
	mtPalette	* src
	);

int mtpixel_palette_invert (
	mtPalette	* palette
	);

int mtpixel_palette_sort (
	mtPalette	* palette,
	mtImage		* image,	// Indexed image to change according to
					// pallete
	int		start,
	int		finish,
	mtPaletteSort	type,
	int		reverse
	);

int mtpixel_palette_remove_unused (
	mtImage		* image,
	int		* removed	// Put number of cols removed here
					// (NULL = don't)
	);

int mtpixel_palette_shift (
	mtPalette	* palette,
	int		start,		// 0..254
	int		finish,		// 1..255
	int		shift		// 1..255
	);

int mtpixel_palette_move (		// Move a palette item
	mtPalette	* palette,
	mtImage		* image,	// Define to change colours in indexed
					// image
	int		c1,		// Current position
	int		c2		// New position
	);

int mtpixel_palette_scale (		// Create a scale in the palette
	mtPalette	* palette,
	int		i1,		// Start 0..255
	int		r1,
	int		g1,
	int		b1,
	int		i2,		// Finish 0..255
	int		r2,
	int		g2,
	int		b2
	);

void mtpixel_rgb2hsv (
	unsigned char	* rgb,		// 3 RGB bytes (input)
	double		* hsv		// 3 HSV values (output)
	);

void mtpixel_hsv2rgb (
	unsigned char	* rgb,		// 3 RGB bytes (output)
	double		* hsv		// 3 HSV values (input)
	);

mtBrush * mtpixel_brush_new (
	void
	);

int mtpixel_brush_set_RGB (
	mtBrush		* brush,
	int		col_a,		// Packed RGB or <0 for don't set
	int		col_b		// Packed RGB or <0 for don't set
	);

int mtpixel_brush_set_index (
	mtBrush		* brush,
	mtPalette	* palette,
	int		index_a,	// <0 = don't set
	int		index_b,	// <0 = don't set
	mtChannelType	channel
	);

int mtpixel_brush_set_style (
	mtBrush		* brush,
	mtBrushShape	shape,		// MTPIXEL_BRUSH_SHAPE_NONE = Don't set
	mtBrushMode	mode,		// MTPIXEL_BRUSH_MODE_NONE = Don't set
	int		pattern,	// <0 = Don't set
	int		size,		// <0 = Don't set
	int		flow,		// <0 = Don't set
	int		opacity		// <0 = Don't set
	);

int mtpixel_brush_set_size (
	mtBrush		* brush,
	int		size
	);

mtBrush * mtpixel_brush_duplicate (
	mtBrush		* brush
	);

int mtpixel_brush_destroy (
	mtBrush		* brush
	);

int mtpixel_brush_channel_mask_set (
	mtBrush		* brush,
	mtChannelMask	mask
	);

int mtpixel_brush_channel_mask_clear (
	mtBrush		* brush,
	mtChannelMask	mask
	);

int mtpixel_brush_gradient_position (	// Set gradient position for the given
					// channel
	mtBrush		* brush,
	mtChannelType	channel,
	int		x1,
	int		y1,
	int		x2,
	int		y2
	);

int mtpixel_brush_gradient (		// Define gradient mode for the current
					// channel
	mtBrush		* brush,
	mtChannelType	channel,
	int		length,
	int		repeat_length,
	int		offset,
	mtGradientMode	grad_type,
	mtGradientBound	ext_type,
	mtGradientType	grad_col_type,
	int		grad_col_reverse,
	mtGradientType	grad_opac_type,	// Custom/Const
	int		grad_opac_reverse
	);

int mtpixel_brush_gradient_custom (	// Define custom gradient
					// colours/opacity for given channel
	mtBrush		* brush,
	mtChannelType	channel,
	int		points,
	int		table[256][2],
	int		opacity
	);

/*
	PAINTING FUNCTIONS

	Actions apply to all non-disabled channels
*/

int mtpixel_brush_plot (		// Plot as shape/size
	mtImage		* image,
	mtBrush		* brush,
	int		x,
	int		y
	);

int mtpixel_brush_line (
	mtImage		* image,
	mtBrush		* brush,
	int		x1,
	int		y1,
	int		x2,
	int		y2
	);

int mtpixel_brush_rectangle (		// Brush size 0 = filled, else as
					// outline of size pixels
	mtImage		* image,
	mtBrush		* brush,
	int		x1,
	int		y1,
	int		x2,
	int		y2
	);

int mtpixel_brush_ellipse (		// Brush size 0 = filled, else as
					// outline of size pixels
	mtImage		* image,
	mtBrush		* brush,
	int		x1,
	int		y1,
	int		x2,
	int		y2
	);

int mtpixel_brush_polygon (		// Brush size 0 = filled, else as
					// outline of size pixels
	mtImage		* image,
	mtBrush		* brush,
	mtPolygon	* polygon
	);

int mtpixel_brush_flood_fill (		// Flood fill
	mtImage		* image,
	mtBrush		* brush,
	int		x,
	int		y
	);

mtImage * mtpixel_image_new (
	int		width,		// 1 or more (Pixels)
	int		height,		// 1 or more (Pixels)
	mtImageType	type,		// Image type MTPIXEL_IMAGE_*
	mtChannelMask	chan_mask	// Channels to be created
	);

mtImage * mtpixel_image_new_rgb (
	int		width,		// 1 or more (Pixels)
	int		height		// 1 or more (Pixels)
	);

mtImage * mtpixel_image_new_indexed (
	int		width,		// 1 or more (Pixels)
	int		height		// 1 or more (Pixels)
	);

mtImage * mtpixel_image_load (
	char		* filename
	);

int mtpixel_image_save (
	mtImage		* image,
	char		* filename,
	mtFileType	filetype,	// MTPIXEL_FILE_TYPE_*
	int		fileinfo	// e.g. PNG/JPEG compression level
	);

mtImage * mtpixel_image_new_from_image (// Can be used to crop/expand canvas
	mtImage		* image,
	int		x,
	int		y,
	int		width,		// -1 = Use current image width
	int		height		// -1 = Use current image height
	);

// Cut a shape from an image (cropping/expanding to max/min of polygon)
mtImage * mtpixel_image_new_from_polygon (
	mtImage		* image,
	mtPolygon	* polygon
	);

mtImage * mtpixel_image_duplicate (
	mtImage		* image
	);

int mtpixel_image_destroy (
	mtImage		* image
	);

int mtpixel_image_paste (
	mtImage		* dest,
	mtImage		* src,
	mtBrush		* brush,	// Paste mode, channel disable etc.
	int		x,
	int		y
	);

int mtpixel_image_quantize (		// Create a quantized palette from an
					// RGB image
	mtImage		* image,	// Source RGB image
	mtPalette	* palette,	// Destination of quantized palette
					// (NULL = image)
	int		cols,		// 2..256
	mtQuantizeType	type		// As per MTPIXEL_QUANTIZE_*
	);

int mtpixel_image_merge_duplicates (	// Merge duplicate coloured pixels on an
					// indexed canvas
	mtImage		* image,	// Duplicate = pixel who shares an RGB
					// with a different index
	int		* duplicates	// Put number of cols merged here
					// (NULL = don't use)
	);

int mtpixel_image_count_colors (	// Count different colours used in this
					// image
	mtImage		* image
	);
	// Return colours found (0 = fail)

int mtpixel_image_color_transform (
	mtImage		* image,	// NULL = do nothing
	mtPalette	* palette,	// NULL = do nothing
	int		brightness,	// -255..255	0 = don't change
	int		contrast,	// -255..255	0 = don't change
	int		saturation,	// -100..100	0 = don't change
	int		posterize,	// 1..8		8 = don't change
	int		gamma,		// 20..500	100 = don't change
	int		hue,		// -1529..1529	0 = don't change
	int		red,		// 1 = enable channel transform
					// 0 = disable
	int		green,		// 1 = enable channel transform
					// 0 = disable
	int		blue		// 1 = enable channel transform
					// 0 = disable
	);

int mtpixel_image_invert (
	mtImage		* image,	// NULL = do nothing
	mtPalette	* palette,	// NULL = do nothing
	mtBrush		* brush		// Used for channel disabling
	);

int mtpixel_image_grayscale (
	mtImage		* image,	// NULL = do nothing
	mtPalette	* palette	// NULL = do nothing
	);

int mtpixel_image_posterize (
	mtImage		* image,	// NULL = do nothing
	mtPalette	* palette,	// NULL = do nothing
	mtBrush		* brush,	// Used for channel disabling
	int		level,		// 2..128
	int		red,		// 1 = enable channel transform
					// 0 = disable
	int		green,		// 1 = enable channel transform
					// 0 = disable
	int		blue		// 1 = enable channel transform
					// 0 = disable
	);

/*
Image manipulatons here all create new images.
The original image is unchanged.
Any brush is used to protect certain channels from change.
Using a brush of NULL implies that all available channels will be changed.
*/

mtImage * mtpixel_image_flip_vertical (
	mtImage		* image
	);

mtImage * mtpixel_image_flip_horizontal (
	mtImage		* image
	);

mtImage * mtpixel_image_to_rgb (	// Convert indexed image to RGB
	mtImage		* image
	);

mtImage * mtpixel_image_to_indexed (
	mtImage		* image,	// Palette used must have at least 2
					// colours
	mtDitherType	type,
	mtDitherCS	cspace,
	mtDitherDiff	diff,
	mtDitherBL	bleed,
	int		serp,		// 0..1
	int		errp,		// 0..100
	mtDitherSep	sep,
	int		fullerr		// 0..1
	);

mtImage * mtpixel_image_scale (
	mtImage		* image,
	mtScaleType	type,
	int		width,
	int		height,
	int		gamma_correction,
	int		sharper_reduce
	);

mtImage * mtpixel_image_rotate (
	mtImage		* image,
	double		angle,		// Degrees
	int		smooth,		// 0/1	1 is only for RGB images
	int		gamma_correction// 0/1	1 is only for RGB images
	);

mtImage * mtpixel_image_sharpen (
	mtImage		* image,
	mtBrush		* brush,
	int		value		// 1..100
	);

mtImage * mtpixel_image_soften (
	mtImage		* image,
	mtBrush		* brush,
	int		value		// 1..100
	);

mtImage * mtpixel_image_edge_detect (
	mtImage		* image,
	mtBrush		* brush
	);

mtImage * mtpixel_image_emboss (
	mtImage		* image,
	mtBrush		* brush
	);

int mtpixel_channel_new (
	mtImage		* image,
	mtChannelType	channel		// Channel to be created
	);

int mtpixel_channel_load (
	mtImage		* image,
	char		* filename,
	mtChannelType	channel
	);

int mtpixel_channel_save (
	mtImage		* image,
	char		* filename,
	mtChannelType	channel,
	mtFileType	filetype,	// MTPIXEL_FILE_TYPE_*
	int		fileinfo	// e.g. PNG/JPEG compression level
	);

int mtpixel_channel_copy (
	mtImage		* image,
	mtChannelType	dest,
	mtChannelType	src
	);

int mtpixel_channel_move (
	mtImage		* image,
	mtChannelType	dest,
	mtChannelType	src
	);

int mtpixel_channel_destroy (
	mtImage		* image,
	mtChannelType	channel		// Channel to destroy
	);

mtChannelMask mtpixel_channel_get_mask (// Return mask of channels in this image
	mtImage		* image
	);

mtFont * mtpixel_font_new (		// Create new font structure, defaults
					// to 12 point size
	char		* filename,	// Font file for FreeType to open.
					// NULL = Fixed
	int		face_index	// Usually 0, but maybe higher for
					// bitmap fonts like *.FON.  Fixed fonts
					// are 8 or 16 for the height
	);

int mtpixel_font_destroy (		// Destroy and free a font
	mtFont		* font
	);

int mtpixel_font_set_size (		// Set font size
	mtFont		* font,
	int		size
	);

mtFontRender * mtpixel_font_render_new (// Create an empty render structure
	mtFont		* font,		// Prepared font to use
	char		* utf8,		// Prepared string to use
	mtBrush		* brush,	// Brush to use NULL = don't create
					// image channel
	mtImageType	image_type	// MTPIXEL_IMAGE_RGB/INDEXED
	);

int mtpixel_font_render_destroy (	// Destroy a render structure
	mtFontRender	* ren
	);

mtImage * mtpixel_font_render (		// Create image from text
	mtFontRender	* ren
	);

mtImage * mtpixel_font_render_posx (	// Create image from text, noting X
					// positions of glyphs
	mtFontRender	* ren
	);

int mtpixel_font_extent (		// Calculate the extent of the render
	mtFontRender	* ren
	);



#endif		// _mtpixel_h
