/*
	A basic demo of using fonts
	font.c

	Copyright (C) 2008-2009 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <stdlib.h>
#include <stdio.h>
#include <mtpixel.h>

#define TOTAL_FONTS 4
#define TEXTFILE_NAME "../../COPYING.txt"
#define TEXTFILE_SIZE 4000
#define WIN_W 640
#define WIN_H 640

/*
static void show_cache( mtFont *font )
{
	int		i,j,k;
	mtFontCache	*cp;


	if ( !font->cache_index || font->cache_size<1 )
	{
		return;
	}

printf("\nCache in use:\n");
	for ( k=0, i=0, j=0; i<256; i++ )
	{
		cp = &font->cache_index[i];
		if ( cp->advance )
		{
			j++;
			k += cp->w * cp->h;
			printf("%4i : %8i - %i x %i ad=%i xy = %i,%i\n",
				i, cp->unicode, cp->w, cp->h, cp->advance, cp->x, cp->y);
		}
	}
printf("%i glyphs cached, %i pixels\n", j, k);
}

static void display_render_info( mtFontRender *ren )
{
	int i;

	printf("X positions:\n\n");

	if ( ren->char_posx )
	{
		for ( i=0; i<ren->chars; i++ )
		{
			if ( ren->char_posx[i] == 0 ) printf("\n");
			printf("%i,", ren->char_posx[i]);
		}
	}

	printf("\n\n");
}
*/

int main( int argc, char *argv[] )
{
	FILE		*fp;
	mtFont		*myfonts[TOTAL_FONTS];
	mtFontRender	*ren;
	mtImage		*image, *ti;
	mtBrush		*brush, *brush_white, *brush_grey;
	int		i, j;
	char		st_ascii[16], textfile[TEXTFILE_SIZE], *ds;


	mtpixel_init();

	brush = mtpixel_brush_new();
	brush_white = mtpixel_brush_new();
	brush_grey = mtpixel_brush_new();
	ren = mtpixel_font_render_new( NULL, NULL, brush, MTPIXEL_IMAGE_RGB );

	myfonts[3] = mtpixel_font_new( NULL, 8 );
	myfonts[2] = mtpixel_font_new( NULL, 16 );

	myfonts[1] =
			mtpixel_font_new("/usr/share/fonts/misc/6x10-ISO8859-1.pcf.gz", 0);
//			mtpixel_font_new("/usr/share/fonts/TTF/DejaVuSans.ttf", 0);
//			mtpixel_font_new("/usr/X11R6/lib/X11/fonts/misc/9x15-ISO8859-1.pcf.gz", 0);
//			mtpixel_font_new("/usr/X11R6/lib/X11/fonts/misc/9x18-ISO8859-1.pcf.gz", 0);
//			mtpixel_font_new("/usr/X11R6/lib/X11/fonts/misc/10x20-ISO8859-1.pcf.gz", 0);

	myfonts[0] =
			mtpixel_font_new("/usr/share/fonts/TTF/DejaVuSans.ttf", 0);
//			mtpixel_font_new("/usr/share/fonts/truetype/arial.ttf", 0);

//	mtpixel_font_set_size( myfonts[0], 8 );

//	mtpixel_font_cache_init( myfonts[0], 256 );
//	mtpixel_font_cache_init( myfonts[1], 256 );


#define ASCII_FONT_TEST
#define TEXTFILE_FONT_TEST

#ifdef ASCII_FONT_TEST
printf("\n\nASCII Text\n");

	image = mtpixel_image_new( WIN_W, WIN_H, MTPIXEL_IMAGE_RGB, MTPIXEL_CHANNEL_MASK_IMAGE );

	mtpixel_brush_set_index( brush_white, &image->palette, 7, 7, MTPIXEL_CHANNEL_IMAGE );
	mtpixel_brush_set_index( brush_grey, &image->palette, 10, 10, MTPIXEL_CHANNEL_IMAGE );

	mtkit_set_locale( "" );

	for ( i=0; i<TOTAL_FONTS; i++ )
	{
		for ( j=0; j<256; j++ )
		{
			st_ascii[0] = j;
			st_ascii[1] = 0;
			ds = mtkit_iso8859_to_utf8( st_ascii, 1, NULL );
			if ( !ds ) continue;

			ren->font	= myfonts[i];
			ren->utf8	= ds;
			ren->brush	= brush_white;
			ren->max_chars	= 1;
			ren->max_width	= 0;
//			ren->flags	= 0;

			ti = mtpixel_font_render( ren );
			mtpixel_image_paste( image, ti, brush, 20 * (j % 32), 80 * (j / 32) + 17*i );
			mtpixel_image_destroy( ti );

			free( ds );
		}
	}

	mtpixel_image_save( image, "test_ascii.bmp", MTPIXEL_FILE_TYPE_BMP, 5 );
	mtpixel_image_destroy( image );

#endif		// ASCII_FONT_TEST


#ifdef TEXTFILE_FONT_TEST
printf("\n\nFull Screen FT & Fixed Text\n");

	fp = fopen(TEXTFILE_NAME, "rb");		// Load text from file
	if (!fp)
	{
		fprintf(stderr, "Unable to open %s\n", TEXTFILE_NAME);
		exit(1);
	}
	fread( textfile, 1, TEXTFILE_SIZE, fp );
	fclose(fp);

	textfile[TEXTFILE_SIZE-1] = 0;
	ds = mtkit_iso8859_to_utf8( textfile, TEXTFILE_SIZE-1, NULL );

	image = mtpixel_image_new( WIN_W, WIN_H, MTPIXEL_IMAGE_RGB, MTPIXEL_CHANNEL_MASK_IMAGE );

	mtpixel_brush_set_index( brush_white, &image->palette, 7, 7, MTPIXEL_CHANNEL_IMAGE );
	mtpixel_brush_set_index( brush_grey, &image->palette, 10, 10, MTPIXEL_CHANNEL_IMAGE );

	if (ds) for ( i=0; i<TOTAL_FONTS; i++ )
	{
		ren->font	= myfonts[i];
		ren->utf8	= ds;
		ren->brush	= brush_white;
		ren->max_chars	= 640;
		ren->max_width	= 600;
//		ren->flags	= MTPIXEL_FONT_WORDWRAP | MTPIXEL_FONT_CHAR_POSX;

		ti = mtpixel_font_render( ren );
		mtpixel_image_paste( image, ti, brush, 20, 5+i*160 );

		mtpixel_brush_rectangle( image, brush_grey, 20 - 2, 5+i*160 - 2,
				20 + ti->width + 1, 5+i*160 + ti->height + 1 );

		mtpixel_image_destroy( ti );

//display_render_info( ren );

//		show_cache(myfonts[i]);
	}

	mtpixel_image_save( image, "test_file.bmp", MTPIXEL_FILE_TYPE_BMP, 5 );
	mtpixel_image_destroy( image );

#define MYFONT_CHOICE 0

	for ( i=0; i<2; i++ )
	{
		image = mtpixel_image_new( WIN_W, WIN_H, MTPIXEL_IMAGE_RGB, MTPIXEL_CHANNEL_MASK_IMAGE );

		if ( i==0 )
		{
			printf("\n\nFull Screen FT Text - NO cache\n");
		}
		if ( i==1 )
		{
			printf("\n\nFull Screen FT Text - cache is 256\n");
		}

		ren->font	= myfonts[MYFONT_CHOICE];
		ren->utf8	= ds;
		ren->brush	= brush_white;
		ren->max_chars	= 0;
		ren->max_width	= 600;
//		ren->max_height	= 600;
//		ren->flags	= MTPIXEL_FONT_WORDWRAP | MTPIXEL_FONT_CHAR_POSX;

		ti = mtpixel_font_render( ren );
		mtpixel_image_paste( image, ti, brush, 20, 5 );

		mtpixel_brush_rectangle( image, brush_grey, 20 - 2, 5 - 2,
			20 + ti->width + 4, 5 + ti->height + 1 );

		mtpixel_image_destroy( ti );

//display_render_info( ren );

		if ( i==0 )
			mtpixel_image_save( image, "test_file2_nocache.bmp", MTPIXEL_FILE_TYPE_BMP, 5 );
		if ( i==1 )
			mtpixel_image_save( image, "test_file2_cache.bmp", MTPIXEL_FILE_TYPE_BMP, 5 );

		mtpixel_image_destroy( image );

//		show_cache( myfonts[MYFONT_CHOICE] );
	}
//printf( "surface created: w=%i h=%i\n", surf->w, surf->h );

	free( ds );



#endif		// TEXTFILE_FONT_TEST



	mtpixel_font_destroy( myfonts[0] );
	mtpixel_font_destroy( myfonts[1] );
	mtpixel_font_destroy( myfonts[2] );
	mtpixel_font_destroy( myfonts[3] );

	mtpixel_brush_destroy( brush_grey );
	mtpixel_brush_destroy( brush_white );
	mtpixel_brush_destroy( brush );
	mtpixel_font_render_destroy( ren );

system("mtv *.bmp");

	mtpixel_quit();

	return 0;
}
