/*	magif.c
	Copyright (C) 2007-2009 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <mtpixel.h>



#ifdef WIN32
	#ifndef WEXITSTATUS
	#define WEXITSTATUS(A) ((A) & 0xFF)
	#endif
#else
	#include <sys/wait.h>
#endif

#define GIFSICLE_CREATE "gifsicle --colors 256 -w -O2 -D 2 -l0 --careful"
	// global colourmaps, suppress warning, high optimizations, background removal method, infinite loops, ensure result works with Java & MS IE

static int gifsicle_animated_gif(char *input, char *output, int delay)		// Create animated GIF
{
	int res;
	char txt[500];

	snprintf(txt, 500, "%s -d %i %s -o \"%s\"", GIFSICLE_CREATE, delay, input, output);

	res = system(txt);
	if ( res != 0 )		// Error occured
	{
		if ( res>0 ) res = WEXITSTATUS(res);
	}

	return res;
}

#define TOT_ENV 8

void do_animation()
{
	mtImage	*sprites, *frame[2], *env[TOT_ENV];
	mtBrush	*brush;
	int	i, j, x, y, yo[8] = {0, 5, 2, 10, 7, 3, 12, 4 };
	char	txt[128];

	sprites = mtpixel_image_load("img/sprites.png");
	brush = mtpixel_brush_new();

	// Create envelope sprites
	for ( i=0; i<TOT_ENV; i++ )
	{
		env[i] = mtpixel_image_new_from_image( sprites, 200, i*20, 20, 20 );
	}

	frame[0] = mtpixel_image_new_from_image( sprites, 1, 149, 180, 100 );
	mtpixel_channel_destroy( frame[0], MTPIXEL_CHANNEL_ALPHA );
	frame[1] = mtpixel_image_duplicate( frame[0] );

	for ( j=0; j<100; j++ )
	{
		mtpixel_image_paste( frame[1], frame[0], brush, 0, 0 );

		for ( i=0; i<TOT_ENV; i++ )
		{
			x = 5 + i*22;
			y = 2*j - 25 - 4*yo[i];
			mtpixel_image_paste( frame[1], env[(i+j)%8], brush, x, y );
		}

		snprintf( txt, 128, "frame%03i.gif", j );
		mtpixel_image_save( frame[1], txt, MTPIXEL_FILE_TYPE_GIF, 0 );
	}
	mtpixel_image_destroy( frame[0] );
	mtpixel_image_destroy( frame[1] );

	// Destroy envelope sprites
	for ( i=0; i<TOT_ENV; i++ )
	{
		mtpixel_image_destroy( env[i] );
	}

	gifsicle_animated_gif("frame???.gif", "frames.gif", 10);
	system("gifview -a frames.gif &");

	mtpixel_brush_destroy( brush );
	mtpixel_image_destroy( sprites );
}

int main( int argc, char *argv[] )
{
	mtpixel_init();
	do_animation();
	mtpixel_quit();

	return 0;
}
