/*	micmp.c
	Copyright (C) 2007-2008 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <mtpixel.h>




static void compare_images( mtImage *ia, mtImage *ib )
{
	int		i, j, k, rgb[3][3] = { {0,0,0}, {0,0,0}, {0,0,0} }, identical = 0;
	unsigned char	*a, *b;
	float		jf;


	a = ia->channel[MTPIXEL_CHANNEL_IMAGE];
	b = ib->channel[MTPIXEL_CHANNEL_IMAGE];

	if ( ia->width == ib->width && ia->height == ib->height && ia->type == ib->type )
	{
		if ( ia->type == MTPIXEL_IMAGE_INDEXED )
		{
			j = ia->width * ia->height;
			for ( i=0; i<j; i++ )
			{
				if ( a[0] == b[0] ) identical++;
				rgb[0][0] = abs( a[0] - b[0] );
				if ( rgb[0][0] > rgb[0][1] ) rgb[0][1] = rgb[0][0];
				rgb[0][2] += rgb[0][0];

				a ++;
				b ++;
			}
			jf = j;
			if ( identical < jf )
			{
				printf("\n%i x %i x %i (%i pixels)\n\n",
					ia->width, ia->height, ia->type, j);
				printf("Identical pixels = %i (%.2f%%)\n", identical, 100*identical/jf);
				printf("Max index error  = %i\n", rgb[0][1]);
				printf("Ave index error  = %.2f\n\n", (rgb[0][2])/jf );
			}
		}

		if ( ia->type == MTPIXEL_IMAGE_RGB )
		{
			j = ia->width * ia->height;
			for ( i=0; i<j; i++ )
			{
				for ( k=0; k<3; k++ )
				{
					rgb[k][0] = abs( a[k] - b[k] );
					if ( rgb[k][0] > rgb[k][1] ) rgb[k][1] = rgb[k][0];
					rgb[k][2] += rgb[k][0];
				}
				if ( a[0] == b[0] && a[1] == b[1] && a[2] == b[2] ) identical++;
				a += 3;
				b += 3;
			}
			jf = j;
			if ( identical < jf )
			{
				printf("\n%i x %i x %i (%i pixels)\n\n",
					ia->width, ia->height, ia->type, j);
				printf("Identical pixels = %i (%.2f%%)\n",
					identical, 100*identical/jf);
				printf("Max RGB error    = %i, %i, %i\n",
					rgb[0][1], rgb[1][1], rgb[2][1]);
				printf("Ave RGB error    = %.2f, %.2f, %.2f\n\n",
					(rgb[0][2])/jf, (rgb[1][2])/jf, (rgb[2][2])/jf );
			}
		}
	}
	else fprintf(stderr, "\nImage geometry not identical.\n\n");
}


int main( int argc, char *argv[] )
{
	mtImage	*ia, *ib;
	int	res = 0;

	mtpixel_init();

	if ( argc != 3 )
	{
		printf(	"micmp - Image Compare Program\n"
		"\n"
		"Usage:\n"
		"\n"
		"micmp <file1> <file2>\n"
		"\n"
		"Compares 2 image files of the same geometry, and if they are\n"
		"different displays the differences.  This is useful for judging\n"
		"how accurately a lossy codec has compressed an image.\n"
			"\n" );
		res = 2;
	}
	else
	{
		ia = mtpixel_image_load( argv[1] );
		if ( !ia )
		{
			fprintf(stderr, "\nUnable to load %s\n\n", argv[1]);
			res = 1;
		}
		else
		{
			ib = mtpixel_image_load( argv[2] );
			if ( !ib )
			{
				fprintf(stderr, "\nUnable to load %s\n\n", argv[2]);
				res = 1;
			}
			else
			{
				compare_images( ia, ib );
				mtpixel_image_destroy( ib );
			}

			mtpixel_image_destroy( ia );
		}
	}

	mtpixel_quit();

	return res;
}
