/*	testcard.c
	Copyright (C) 2007-2008,2010-2011 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include <mtpixel.h>


static int width = 1024, height = 1024;


static void test_patterns( mtImage *image, mtBrush *brush )
{
	int	x = 0, y = 100, gap = 32, i;

	for ( i=0; i<MTPIXEL_BRUSH_PATTERN_TOTAL; i++ )
	{
		mtpixel_brush_set_index( brush, &image->palette, i, 0, MTPIXEL_CHANNEL_IMAGE );
		mtpixel_brush_set_style( brush, MTPIXEL_BRUSH_SHAPE_SQUARE, MTPIXEL_BRUSH_MODE_PATTERN,
			i, 0, 0, 255 );

		mtpixel_brush_rectangle( image, brush, x, y, x+gap, y+gap );

		x += gap;
		if ( x >= (width-gap) )
		{
			x = 0;
			y += gap;
		}
	}
}


#define BR_TOT 11
static void test_brushes( mtImage *image, mtBrush *brush )
{
	int size[BR_TOT] = { 1, 2, 3, 4, 8, 12, 16, 20, 24, 28, 32},
		x = 20, y = 20, gap = 35, i, tot = (MTPIXEL_BRUSH_SHAPE_TOTAL-1) * BR_TOT;

	mtpixel_brush_set_index( brush, &image->palette, 1, 1, MTPIXEL_CHANNEL_IMAGE );

	for ( i=0; i<tot; i++ )
	{
		mtpixel_brush_set_style( brush, 1 + (i / BR_TOT), MTPIXEL_BRUSH_MODE_PATTERN,
			0, size[i % BR_TOT], size[i % BR_TOT], 255 );

		mtpixel_brush_plot( image, brush, x, y );

		x += gap;
		if ( x >= (width-gap) )
		{
			x = 20;
			y += gap;
		}
	}
}

static void test_gradients( mtImage *image, mtBrush *brush )
{
	int	custom[256][2], mibpp;


	mibpp = image->type;

	mtpixel_brush_set_style( brush, MTPIXEL_BRUSH_SHAPE_SQUARE, MTPIXEL_BRUSH_MODE_GRADIENT,
			0, 0, 0, 255 );
	mtpixel_brush_set_index( brush, &image->palette, 1, 7, MTPIXEL_CHANNEL_IMAGE );

	mtpixel_brush_gradient_position( brush, MTPIXEL_CHANNEL_IMAGE, 70, 270, 330, 380 );
//	mtpaint_gradient_position( 0, 0, width, height );
	mtpixel_brush_gradient( brush, MTPIXEL_CHANNEL_IMAGE, 0, 0, 0,
			MTPIXEL_GRADIENT_MODE_LINEAR, MTPIXEL_GRADIENT_BOUND_STOP,
			MTPIXEL_GRADIENT_TYPE_RGB, 0, MTPIXEL_GRADIENT_TYPE_CONST, 0 );

	mtpixel_brush_rectangle( image, brush, 10, 250, 400, 400 );

	mtpixel_brush_gradient( brush, MTPIXEL_CHANNEL_IMAGE, 0, 0, 0,
			MTPIXEL_GRADIENT_MODE_RADIAL, MTPIXEL_GRADIENT_BOUND_MIRROR,
			MTPIXEL_GRADIENT_TYPE_CUSTOM, 0, MTPIXEL_GRADIENT_TYPE_CONST, 0 );
	custom[0][1] = 0;
	custom[1][1] = 0;
	custom[2][1] = 0;
	custom[3][1] = 0;
	custom[4][1] = 0;
	custom[5][1] = 0;
	custom[6][1] = 0;
	custom[7][1] = 0;
	custom[8][1] = 0;
	if ( mibpp == 1 )
	{
		custom[0][0] = 0;
		custom[1][0] = 1;
		custom[2][0] = 3;
		custom[3][0] = 2;
		custom[4][0] = 6;
		custom[5][0] = 4;
		custom[6][0] = 5;
		custom[7][0] = 1;
		custom[8][0] = 0;
	}
	if ( mibpp == 3 )
	{
		custom[0][0] = MTPIXEL_RGB_2_INT(0,0,0);
		custom[1][0] = MTPIXEL_RGB_2_INT(255,0,0);
		custom[2][0] = MTPIXEL_RGB_2_INT(255,255,0);
		custom[3][0] = MTPIXEL_RGB_2_INT(0,255,0);
		custom[4][0] = MTPIXEL_RGB_2_INT(0,255,255);
		custom[5][0] = MTPIXEL_RGB_2_INT(0,0,255);
		custom[6][0] = MTPIXEL_RGB_2_INT(255,0,255);
		custom[7][0] = MTPIXEL_RGB_2_INT(255,0,0);
		custom[8][0] = MTPIXEL_RGB_2_INT(0,0,0);
	}
	mtpixel_brush_gradient_custom( brush, MTPIXEL_CHANNEL_IMAGE, 9, custom, 0 );	// 255 at end?
	mtpixel_brush_gradient_position( brush, MTPIXEL_CHANNEL_IMAGE, 600, 350, 450, 300 );
	mtpixel_brush_rectangle( image, brush, 400,250, 700,400 );

	custom[0][1] = MTPIXEL_GRADIENT_TYPE_CONST;
	custom[1][1] = MTPIXEL_GRADIENT_TYPE_CONST;
	custom[2][1] = MTPIXEL_GRADIENT_TYPE_CONST;
	custom[3][1] = MTPIXEL_GRADIENT_TYPE_CONST;
	custom[4][1] = MTPIXEL_GRADIENT_TYPE_CONST;
	custom[5][1] = MTPIXEL_GRADIENT_TYPE_CONST;
	custom[6][1] = MTPIXEL_GRADIENT_TYPE_CONST;
	custom[7][1] = MTPIXEL_GRADIENT_TYPE_CONST;
	custom[8][1] = MTPIXEL_GRADIENT_TYPE_CONST;

	mtpixel_brush_gradient_custom( brush, MTPIXEL_CHANNEL_IMAGE, 9, custom, 0 );
	mtpixel_brush_gradient_position( brush, MTPIXEL_CHANNEL_IMAGE, 900, 350, 750, 300 );
	mtpixel_brush_ellipse( image, brush, 700,250, 1000,400 );
}

static void test_text( mtImage *image, mtBrush *brush )
{
	mtFont		*font;
	mtFontRender	*ren;
	mtImage		*ti;
	char		*font_filename = "/usr/share/fonts/TTF/arial.ttf",
			*txt = "libmtpixel";
	int		i, x, y, /*angle, size,*/ xmin = width/2;


	srand(100);
	font = mtpixel_font_new( font_filename, 0 );
	ren = mtpixel_font_render_new( font, txt, brush, image->type );

	mtpixel_brush_set_style( brush, MTPIXEL_BRUSH_SHAPE_SQUARE, MTPIXEL_BRUSH_MODE_PATTERN,
			0, 32, 32, 255 );
	mtpixel_font_set_size( font, 20 );
	for ( i=0; i<250; i++ )
	{
		mtpixel_brush_set_index( brush, &image->palette, 1 + rand() % 11, -1,
				MTPIXEL_CHANNEL_IMAGE );

		x = xmin + rand() % (width - xmin);
		y = 410 + rand() % 200;
//		angle = rand() % 360;
//		size = 8 + rand() % 20;

//		mtpixel_font_set_matrix( font, 0, angle );
		ti = mtpixel_font_render( ren );

//printf("ti=%i image=%i brush=%i\n", (int)ti, (int)image, (int)brush);
//printf("ti=%i image=%i brush=%i ren=%i\n", (int)NULL, (int)image, (int)brush, (int)ren);
//printf("ti=%i w=%i h=%i\n", (int)ti, ti->width, ti->height);
		mtpixel_image_paste( image, ti, brush, x, y );
		mtpixel_image_destroy( ti );
	}

	mtpixel_font_render_destroy( ren );
	mtpixel_font_destroy( font );
}

static void test_dots( mtImage *image, mtBrush *brush )
{
	int	i, xmin=0, ymin=720, xmax=320, ymax=1024;

	mtpixel_brush_set_style( brush, MTPIXEL_BRUSH_SHAPE_SQUARE, MTPIXEL_BRUSH_MODE_PATTERN,
			0, 1, 32, 255 );

	for ( i=0; i<1000; i++ )
	{
		mtpixel_brush_set_index( brush, &image->palette, 1 + rand() % 250, -1,
			MTPIXEL_CHANNEL_IMAGE );
		mtpixel_brush_plot( image, brush,
			xmin + rand() % (xmax-xmin), ymin + rand() % (ymax-ymin) );
	}
}

static void test_lines( mtImage *image, mtBrush *brush )
{
	int i, xmin=320, ymin=720, xmax=640, ymax=1024;

	for ( i=0; i<100; i++ )
	{
		mtpixel_brush_set_index( brush, &image->palette, 1 + rand() % 250, -1,
			MTPIXEL_CHANNEL_IMAGE );
		mtpixel_brush_set_style( brush, MTPIXEL_BRUSH_SHAPE_SQUARE, MTPIXEL_BRUSH_MODE_PATTERN,
				0, rand() % 10, 32, 255 );

		mtpixel_brush_line( image, brush,
			xmin + rand() % (xmax-xmin), ymin + rand() % (ymax-ymin),
			xmin + rand() % (xmax-xmin), ymin + rand() % (ymax-ymin)
			);
	}
}

static void test_triangles( mtImage *image, mtBrush *brush )
{
	mtPolygon	*polygon;
	int		i, j, xmin=640, ymin=720, xmax=1024, ymax=1024;

	mtpixel_brush_set_style( brush, MTPIXEL_BRUSH_SHAPE_SQUARE, MTPIXEL_BRUSH_MODE_PATTERN,
			0, 0, 32, 255 );

	polygon = mtpixel_polygon_new();

	for ( i=0; i<50; i++ )
	{
		mtpixel_brush_set_index( brush, &image->palette, 1 + rand() % 7, -1,
			MTPIXEL_CHANNEL_IMAGE );

		mtpixel_polygon_clear( polygon );
		for ( j=0; j<3; j++ )
		{
			mtpixel_polygon_add_point( polygon,
				xmin + rand() % (xmax-xmin), ymin + rand() % (ymax-ymin)
				);
		}
		mtpixel_brush_polygon( image, brush, polygon );
	}

	mtpixel_polygon_destroy( polygon );
}

static void test_motif( mtImage *image, mtBrush *brush )
{
	char		*st1 = "libmtpixel", *st2 = "Pixel Art for C Programs";
	mtFont		*font;
	mtFontRender	*ren1, *ren2;
	mtImage		*ti;
	mtPolygon	*polygon;
	char		*font_filename = "/usr/share/fonts/truetype/ariblk.ttf";
	int		/*custom[256][2],*/ xmin = 20, xmax = 532, xlines = 32,
			ymin = 410, ymax = 666, i, ylines = 16, x, y;

	font = mtpixel_font_new( font_filename, 0 );
	ren1 = mtpixel_font_render_new( font, st1, brush, image->type );
	ren2 = mtpixel_font_render_new( font, st2, brush, image->type );
	polygon = mtpixel_polygon_new();

	mtpixel_brush_set_style( brush, MTPIXEL_BRUSH_SHAPE_SQUARE, MTPIXEL_BRUSH_MODE_PATTERN,
			0, 0, 32, 255 );

	mtpixel_brush_set_index( brush, &image->palette, 7, 7, MTPIXEL_CHANNEL_IMAGE );
	mtpixel_brush_rectangle( image, brush, xmin, ymin, xmax, ymax ); // White background

	mtpixel_brush_set_index( brush, &image->palette, 0, 0, MTPIXEL_CHANNEL_IMAGE );
	for ( i=0; i<xlines; i++ )				// Grid lines
	{
		x = xmin + (xmax - xmin) * i / xlines;
		mtpixel_brush_line( image, brush, x, ymin, x, ymax );
	}
	for ( i=0; i<ylines; i++ )
	{
		y = ymin + (ymax - ymin) * i / ylines;
		mtpixel_brush_line( image, brush, xmin, y, xmax, y );
	}

	mtpixel_brush_set_style( brush, MTPIXEL_BRUSH_SHAPE_SQUARE, MTPIXEL_BRUSH_MODE_GRADIENT,
			0, 0, 32, 255 );
	mtpixel_brush_set_index( brush, &image->palette, 1, 3, MTPIXEL_CHANNEL_IMAGE );
	mtpixel_brush_gradient( brush, MTPIXEL_CHANNEL_IMAGE, 0, 0, 0,
			MTPIXEL_GRADIENT_MODE_LINEAR, MTPIXEL_GRADIENT_BOUND_LEVEL,
			MTPIXEL_GRADIENT_TYPE_RGB, 0, MTPIXEL_GRADIENT_TYPE_CONST, 0 );

/*	custom[0][0] = 255;
	custom[1][0] = 0;
	custom[0][1] = 0;
	custom[1][1] = 0;
	mtpixel_brush_gradient_custom( brush, MTPIXEL_CHANNEL_IMAGE, 2, custom, 1 );*/
	mtpixel_brush_gradient_position( brush, MTPIXEL_CHANNEL_IMAGE, 70, 460, 70, 550 );

	mtpixel_polygon_add_point( polygon, 86,  406 );
	mtpixel_polygon_add_point( polygon, 586, 491 );
	mtpixel_polygon_add_point( polygon, 10,  556 );
	mtpixel_brush_polygon( image, brush, polygon );

	mtpixel_brush_set_index( brush, &image->palette, 4, 7, MTPIXEL_CHANNEL_IMAGE );
	mtpixel_brush_gradient_position( brush, MTPIXEL_CHANNEL_IMAGE, 110, 500, 600, 500 );
	mtpixel_polygon_clear( polygon );
	mtpixel_polygon_add_point( polygon, 86,  506 );
	mtpixel_polygon_add_point( polygon, 586, 591 );
	mtpixel_polygon_add_point( polygon, 10,  656 );
	mtpixel_brush_polygon( image, brush, polygon );

	mtpixel_brush_set_style( brush, MTPIXEL_BRUSH_SHAPE_SQUARE, MTPIXEL_BRUSH_MODE_PATTERN,
			0, 0, 32, 255 );

	mtpixel_font_set_size( font, 28 );

	mtpixel_brush_set_index( brush, &image->palette, 0, -1, MTPIXEL_CHANNEL_IMAGE );
	ti = mtpixel_font_render( ren1 );
	mtpixel_image_paste( image, ti, brush, 90, 455 );
	mtpixel_image_destroy( ti );

	mtpixel_brush_set_index( brush, &image->palette, 3, -1, MTPIXEL_CHANNEL_IMAGE );
	ti = mtpixel_font_render( ren1 );
	mtpixel_image_paste( image, ti, brush, 90-3, 455-3 );
	mtpixel_image_destroy( ti );

	mtpixel_font_set_size( font, 20 );

	mtpixel_brush_set_index( brush, &image->palette, 0, -1, MTPIXEL_CHANNEL_IMAGE );
	ti = mtpixel_font_render( ren2 );
	mtpixel_image_paste( image, ti, brush, 70, 580 );
	mtpixel_image_destroy( ti );

	mtpixel_brush_set_index( brush, &image->palette, 7, -1, MTPIXEL_CHANNEL_IMAGE );
	ti = mtpixel_font_render( ren2 );
	mtpixel_image_paste( image, ti, brush, 70-2, 580-2 );
	mtpixel_image_destroy( ti );

	mtpixel_polygon_destroy( polygon );
	mtpixel_font_render_destroy( ren1 );
	mtpixel_font_render_destroy( ren2 );
	mtpixel_font_destroy( font );
}

static void test_floodfill( mtImage *image, mtBrush *brush )
{
	mtpixel_brush_set_style( brush, MTPIXEL_BRUSH_SHAPE_SQUARE, MTPIXEL_BRUSH_MODE_PATTERN,
			0, 0, 32, 255 );
	mtpixel_brush_set_index( brush, &image->palette, 1, 1, MTPIXEL_CHANNEL_IMAGE );

	mtpixel_brush_flood_fill( image, brush, 444, 642 );
	mtpixel_brush_flood_fill( image, brush, 371, 800 );
}

static void test_polypaste( mtImage *image, mtBrush *brush )
{
	mtPolygon	*polygon;
	mtImage		*im2;

	mtpixel_brush_set_style( brush, MTPIXEL_BRUSH_SHAPE_SQUARE, MTPIXEL_BRUSH_MODE_PATTERN,
			0, 0, 32, 255 );
	mtpixel_brush_set_index( brush, &image->palette, 0, 0, MTPIXEL_CHANNEL_IMAGE );

	polygon = mtpixel_polygon_new();
	mtpixel_polygon_add_point( polygon, 432, 326 );
	mtpixel_polygon_add_point( polygon, 688, 351 );
	mtpixel_polygon_add_point( polygon, 477, 344 );

	im2 = mtpixel_image_new_from_polygon( image, polygon );
	mtpixel_image_paste( image, im2, brush, 710, 210 );
	mtpixel_image_destroy( im2 );

	mtpixel_brush_polygon( image, brush, polygon );

	mtpixel_polygon_destroy( polygon );
}

static void test( mtImageType type, char *filename )
{
	mtImage *image;
	mtBrush	*brush;

	image = mtpixel_image_new( width, height, type, MTPIXEL_CHANNEL_MASK_IMAGE );
	brush = mtpixel_brush_new();

	test_brushes( image, brush );
	test_patterns( image, brush );
	test_gradients( image, brush );
	test_text( image, brush );
	test_dots( image, brush );
	test_lines( image, brush );
	test_triangles( image, brush );
	test_motif( image, brush );
	test_floodfill( image, brush );
	test_polypaste( image, brush );

	mtpixel_image_save( image, filename, MTPIXEL_FILE_TYPE_PNG, 5 );

	mtpixel_brush_destroy( brush );
	mtpixel_image_destroy( image );
}

int main( int argc, char *argv[] )
{
	mtpixel_init();

	test( MTPIXEL_IMAGE_INDEXED, "test_indexed.png" );
	test( MTPIXEL_IMAGE_RGB, "test_rgb.png" );

	mtpixel_quit();

system("mtv *.png &");

	return 0;
}
