/*
	Copyright (C) 2004-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include <mtcelledit.h>



enum	// Column numbers for each field in results sheet
{
	RAFT_COL_NAME		= 1,
	RAFT_COL_FILES		= 2,
	RAFT_COL_FILES_PERCENT	= 3,
	RAFT_COL_BYTES		= 4,
	RAFT_COL_MB		= 5,
	RAFT_COL_MB_PERCENT	= 6,
	RAFT_COL_SUBDIRS	= 7,
	RAFT_COL_OTHER		= 8,

	RAFT_COL_TOTAL		= 9,
};



#define PATH_LEN		2048
#define PREFS_WINDOW_X		"window_x"
#define PREFS_WINDOW_Y		"window_y"
#define PREFS_WINDOW_W		"window_w"
#define PREFS_WINDOW_H		"window_h"
#define PREFS_DIR_PREFIX	"Dir"
#define PREFS_DIR_LAST		"last_dir"



typedef int (* raftScanFunc) (
	CedSheet	* sheet,
	int		row,		// Row calculated in sheet
	void		* user_data
	);
	// 0 = Keep scanning
	// 1 = Stop



void prefs_init ( void );	// Setup memory, filename, etc.
void prefs_load ( void );	// Load prefs file from disk into memory
void prefs_save ( void );	// Save current prefs from memory to disk
void prefs_close ( void );	// Free all memory created in prefs_init ()

void prefs_set_int (
	char		* key,
	int		value
	);

void prefs_set_string (
	char		* key,
	char		* value
	);

void prefs_set_double (
	char		* key,
	double		value
	);

int prefs_get_int (
	char		* key
	);

char * prefs_get_string (
	char		* key
	);

double prefs_get_double (
	char		* key
	);

void raft_gui_init (			// Set up GUI
	void
	);

void raft_gui_start (			// Pass control to GUI
	void
	);

void raft_gui_analyse (
	const char	* directory	// Analyse this directory
	);

int raft_scan_sheet (
	const char	* path,		// Path to scan all files & subdirs
	CedSheet	** sheet,	// Put results here
	raftScanFunc	callback,	// Option callback for each subdir
	void		* user_data	// Passed to raftScanFunc
	);
	// -1 = Error
	//  0 = Success - sheet created
	//  1 = User termination

