/*
	Copyright (C) 2004-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "private.h"



static const char	* scan_directory = NULL;



static int arg_file_func (
	char		* filename
	)
{
	if ( ! scan_directory )
	{
		scan_directory = filename;
	}

	return 0;			// Keep parsing
}

static int arg_error_func (
	int		error,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	fprintf ( stderr, "error_func: Argument ERROR! - num=%i arg=%i/%i",
		error, arg, argc );

	if ( arg < argc )
	{
		fprintf ( stderr, " '%s'", argv[arg] );
	}

	fprintf ( stderr, "\n" );

	return 0;			// Keep parsing
}

int main (
	int		argc,
	char		* argv[]
	)
{
	int		show_version = 0;
	mtArg		arg_list[] = {
			{ "-help",	MTKIT_ARG_SWITCH, &show_version, 2 },
			{ "-version",	MTKIT_ARG_SWITCH, &show_version, 1 },
			{ NULL }
			};


	mtkit_arg_parse ( argc, argv, arg_list, arg_file_func, arg_error_func );

	switch ( show_version )
	{
	case 1:
		printf ( "%s\n\n", VERSION );

		exit ( 0 );

	case 2:
		printf ( "%s\n\n"
		"Usage: %s [OPTION] [PATHNAME]\n\n"
		"Pass a pathname to recursively analyse it.\n"
		"\n"
		"Options:\n"
		"  --help               Output this help\n"
		"  --version            Output version information\n"
		"\n"
		, VERSION, BIN_NAME );

		exit ( 0 );
	}

	prefs_init ();
	prefs_load ();

	raft_gui_init ();

	if ( scan_directory )
	{
		raft_gui_analyse ( scan_directory );
	}

	raft_gui_start ();

	prefs_save ();
	prefs_close ();

	return 0;
}

