/*
	Copyright (C) 2011-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "private.h"



static mtPrefs		* prefs_mem;
static char		prefs_filename[2048];
static mtPrefItem	prefs_table[] = {
	{ PREFS_WINDOW_X,	MTKIT_PREF_TYPE_INT,	"0" },
	{ PREFS_WINDOW_Y,	MTKIT_PREF_TYPE_INT,	"0" },
	{ PREFS_WINDOW_W,	MTKIT_PREF_TYPE_INT,	"500" },
	{ PREFS_WINDOW_H,	MTKIT_PREF_TYPE_INT,	"500" },
	{ PREFS_DIR_LAST,	MTKIT_PREF_TYPE_STR,	"/" },
	{ PREFS_DIR_PREFIX"00",	MTKIT_PREF_TYPE_STR,	"/home/" },
	{ PREFS_DIR_PREFIX"01",	MTKIT_PREF_TYPE_STR,	"/bin/" },
	{ PREFS_DIR_PREFIX"02",	MTKIT_PREF_TYPE_STR,	"/boot/" },
	{ PREFS_DIR_PREFIX"03",	MTKIT_PREF_TYPE_STR,	"/etc/" },
	{ PREFS_DIR_PREFIX"04",	MTKIT_PREF_TYPE_STR,	"/lib/" },
	{ PREFS_DIR_PREFIX"05",	MTKIT_PREF_TYPE_STR,	"/root/" },
	{ PREFS_DIR_PREFIX"06",	MTKIT_PREF_TYPE_STR,	"/tmp/" },
	{ PREFS_DIR_PREFIX"07",	MTKIT_PREF_TYPE_STR,	"/var/" },
	{ PREFS_DIR_PREFIX"08",	MTKIT_PREF_TYPE_STR,	"/usr/local/" },
	{ PREFS_DIR_PREFIX"09",	MTKIT_PREF_TYPE_STR,	"/usr/share/" },
	{ PREFS_DIR_PREFIX"10",	MTKIT_PREF_TYPE_STR,	"/usr/doc/" },
	{ PREFS_DIR_PREFIX"11",	MTKIT_PREF_TYPE_STR,	"/usr/include/" },
	{ PREFS_DIR_PREFIX"12",	MTKIT_PREF_TYPE_STR,	"/usr/sbin/" },
	{ PREFS_DIR_PREFIX"13",	MTKIT_PREF_TYPE_STR,	"/usr/X11R6/" },
	{ PREFS_DIR_PREFIX"14",	MTKIT_PREF_TYPE_STR,	"/usr/lib/" },
	{ PREFS_DIR_PREFIX"15",	MTKIT_PREF_TYPE_STR,	"/usr/bin/" },
	{NULL}
	};



void prefs_init ( void )
{
	prefs_mem = mtkit_prefs_new ( prefs_table );

	snprintf ( prefs_filename, 2048, "%s/.%s", mtkit_file_home (),
		BIN_NAME );
	prefs_filename[2047] = 0;
}

void prefs_load ( void )
{
	mtkit_prefs_load ( prefs_mem, prefs_filename );
}

void prefs_save ( void )
{
	mtkit_prefs_save ( prefs_mem, prefs_filename );
}

void prefs_close ( void )
{
	mtkit_prefs_destroy ( prefs_mem );
	prefs_mem = NULL;
}

void prefs_set_int (
	char		* key,
	int		value
	)
{
	mtkit_prefs_set_int ( prefs_mem, key, value );
}

void prefs_set_string (
	char		* key,
	char		* value
	)
{
	mtkit_prefs_set_str ( prefs_mem, key, value );
}

void prefs_set_double (
	char		* key,
	double		value
	)
{
	mtkit_prefs_set_double ( prefs_mem, key, value );
}

int prefs_get_int (
	char		* key
	)
{
	int		res = 0;


	mtkit_prefs_get_int ( prefs_mem, key, &res );

	return res;
}

char * prefs_get_string (
	char		* key
	)
{
	static char	* nothing = "";
	char		* res = nothing;


	mtkit_prefs_get_str ( prefs_mem, key, &res );

	return res;
}

double prefs_get_double (
	char		* key
	)
{
	double		res = 0;


	mtkit_prefs_get_double ( prefs_mem, key, &res );

	return res;
}
